--[[
MapsterEnhanced - Coordinates (Sindragosa)

Based on Mapster
Copyright (c) 2009, Hendrik "Nevcairiel" Leppkes < h.leppkes@gmail.com >
All rights reserved.
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local L = LibStub("AceLocale-3.0"):GetLocale("Mapster")

local MapsterEnhanced = LibStub("AceAddon-3.0"):GetAddon("MapsterEnhanced")

local MODNAME = "Coords"
local Coords = Mapster:GetModule(MODNAME)
local MapsterEnhanced_Coords = MapsterEnhanced:NewModule(MODNAME, "AceHook-3.0")

local Astrolabe = DongleStub("Astrolabe-0.7-Global")


local GetCursorPosition = GetCursorPosition
local GetPlayerMapPosition = GetPlayerMapPosition
local WorldMapDetailFrame = WorldMapDetailFrame
local display, cursortext, playertext, yardstext
local texttemplate, text = "%%s: %%.%df, %%.%df"
local yards = "%.0f yd"

local string_sub = string.sub;
local format = format;
local floor = floor;
local abs = abs;

local OnUpdate

local db
local defaults = {
	profile = {
		metric = true,
	}
}

local function ModifyModuleOptions()
	local modOptions = LibStub("AceConfigRegistry-3.0"):GetOptionsTable("Mapster", "dialog", "MapsterEnhanced-1.54").args[MODNAME] --Mapster.moduleOptions[MODNAME]
	local options = {
		nl = {
			order = 5,
			type = "description",
			name = "",
		},
		metric = {
			order = 6,
			type = "toggle",
			name = L["Metric"],
			desc = L["Show the distance in meters."],
		},
	}
	for k,v in pairs(defaults.profile) do
		if db[k] == nil then
			db[k] = defaults.profile[k]
		end
	end
	for k,v in pairs(options) do
		modOptions.args[k] = v
	end
end

function MapsterEnhanced_Coords:OnInitialize()
	self.db = Mapster.db:GetNamespace(MODNAME)
	db = self.db.profile

	ModifyModuleOptions()
end

function MapsterEnhanced_Coords:OnEnable()
	self:SecureHook(Coords, "OnEnable", function() MapsterEnhanced_Coords:EnableMod() end)
	self:SecureHook(Coords, "OnDisable", function() MapsterEnhanced_Coords:DisableMod() end)
	if Coords:IsEnabled() then MapsterEnhanced_Coords:EnableMod() end
end

function MapsterEnhanced_Coords:EnableMod()
	if not Mapster_CoordsFrame then return end

	display = Mapster_CoordsFrame
	if display then
		if not self.initialized then
			self.initialized = true
			cursortext, playertext = display:GetRegions()
			yardstext = display:CreateFontString(nil, "ARTWORK", "GameFontNormal")

			display:HookScript("OnUpdate", OnUpdate)
		end
		if not self.hooked then
			self.hooked = true
			self:SecureHook(Coords, "UpdateMapsize", function(self, ...) MapsterEnhanced_Coords:UpdateMapsize(...) end)
			self:SecureHook(Coords, "Refresh", function() MapsterEnhanced_Coords:Refresh() end)
		end
		self:Refresh()
	end
end

function MapsterEnhanced_Coords:DisableMod()
	if self.hooked then
		self:Unhook(Coords, "UpdateMapsize")
		self:Unhook(Coords, "Refresh")
		self.hooked = nil
	end
end

function MapsterEnhanced_Coords:Refresh()
	local acc = tonumber(db.accuracy) or 1
	text = texttemplate:format(acc, acc)
	yards = (db.metric and "%.0f m" or "%.0f yd")
	Coords.miniMap = Mapster.miniMap
	self:UpdateMapsize(Coords.miniMap)
end

function MapsterEnhanced_Coords:UpdateMapsize(mini)
	-- map was minimized, fix display position
	cursortext:ClearAllPoints()
	playertext:ClearAllPoints()
	yardstext:ClearAllPoints()
	if mini then
		cursortext:SetPoint("BOTTOMRIGHT", WorldMapPositioningGuide, "BOTTOM", -50, -2)
		playertext:SetPoint("BOTTOMLEFT", WorldMapPositioningGuide, "BOTTOM", 35, -2)
		yardstext:SetPoint("BOTTOM", WorldMapPositioningGuide, "BOTTOM", -7, -2)
	else
		cursortext:SetPoint("BOTTOMRIGHT", WorldMapPositioningGuide, "BOTTOM", -80, 12)
		playertext:SetPoint("BOTTOMLEFT", WorldMapPositioningGuide, "BOTTOM", 80, 12)
		yardstext:SetPoint("BOTTOM", WorldMapPositioningGuide, "BOTTOM", 0, 12)
	end
end

local function MouseXY()
	local left, top = WorldMapDetailFrame:GetLeft(), WorldMapDetailFrame:GetTop()
	local width, height = WorldMapDetailFrame:GetWidth(), WorldMapDetailFrame:GetHeight()
	local scale = WorldMapDetailFrame:GetEffectiveScale()

	local x, y = GetCursorPosition()
	local cx = (x/scale - left) / width
	local cy = (top - y/scale) / height

	if cx < 0 or cx > 1 or cy < 0 or cy > 1 then
		return
	end

	return cx, cy
end

local cursor, player = L["Cursor"], L["Player"]
local cachedPos = {}
function OnUpdate()
	local cx, cy = MouseXY()
	local px, py = GetPlayerMapPosition("player")

	if cx then
		cursortext:SetFormattedText(text, cursor, 100 * cx, 100 * cy)
	else
		cursortext:SetText("")
	end

	if px == 0 then
		playertext:SetText("")

		local C, Z = Astrolabe:GetCurrentVirtualMapContinent(), Astrolabe:GetCurrentVirtualMapZone()
		if cachedPos.Z and cachedPos.px then
			local distance = Astrolabe:ComputeDistance(cachedPos.C, cachedPos.Z, cachedPos.px, cachedPos.py, C, Z, cx, cy)
			if distance then
				yardstext:SetFormattedText(yards, distance)
			else
				yardstext:SetText("")
			end
		else
			local c, z, px, py = Astrolabe:GetCurrentPlayerPosition(true)
			local distance = Astrolabe:ComputeDistance(c, z, px, py, C, Z, cx, cy)
			if distance then
				cachedPos.C, cachedPos.Z, cachedPos.px, cachedPos.py = c, z, px, py
				yardstext:SetFormattedText(yards, distance)
			else
				yardstext:SetText("")
			end
		end
	else
		playertext:SetFormattedText(text, player, 100 * px, 100 * py)
		local C, Z = Astrolabe:GetCurrentVirtualMapContinent(), Astrolabe:GetCurrentVirtualMapZone()
		cachedPos.C, cachedPos.Z, cachedPos.px, cachedPos.py = C, Z, px, py

		if cx then
			local distance = Astrolabe:ComputeDistance(C, Z, px, py, C, Z, cx, cy)
			if distance then
				yardstext:SetFormattedText(yards, distance)
			else
				yardstext:SetText("")
			end
		else
			yardstext:SetText("")
		end
	end
end
