--[[
MapsterEnhanced - ElvUI Skin (Sindragosa)
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local MapsterEnhanced = LibStub("AceAddon-3.0"):GetAddon("MapsterEnhanced")

local MODNAME = "ElvUI"
local MapsterEnhanced_ElvUI = Mapster:NewModule(MODNAME, "AceHook-3.0")

local function FixElvUI()
	if ElvUI then
		if MapsterNonOverlayHolder then
			MapsterNonOverlayHolder:StripTextures()
		end
		if WorldMapFrame.backdrop then
			if WORLDMAP_SETTINGS.size == WORLDMAP_FULLMAP_SIZE then
				WorldMapFrame.backdrop:ClearAllPoints()
				WorldMapFrame.backdrop:Point("TOPLEFT", WorldMapDetailFrame, "TOPLEFT", -25, 70)
				WorldMapFrame.backdrop:Point("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT", 25, -30)
			elseif WORLDMAP_SETTINGS.size == WORLDMAP_WINDOWED_SIZE then
				WorldMapFrame.backdrop:ClearAllPoints()
				WorldMapFrame.backdrop:Point("TOPLEFT", 2, -12)
				WorldMapFrame.backdrop:Point("BOTTOMRIGHT", -7, -8)
			elseif WORLDMAP_SETTINGS.size == WORLDMAP_QUESTLIST_SIZE then
				WorldMapFrame.backdrop:ClearAllPoints()
				WorldMapFrame.backdrop:Point("TOPLEFT", WorldMapDetailFrame, "TOPLEFT", -25, 70)
				WorldMapFrame.backdrop:Point("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT", 334, -236)
			end
		end
		if not MapsterEnhanced_ElvUI.skinnedSatMap then
			local SatMap = Mapster:GetModule("SatMap")
			if SatMap then
				if SatMap_MapButton then
					MapsterEnhanced_ElvUI.skinnedSatMap = true
					MapsterEnhanced_ElvUI.ElvUI.S:HandleButton(SatMap_MapButton)
					SatMap_MapButton:ClearAllPoints()
					SatMap_MapButton:SetPoint("RIGHT", WorldMapContinentDropDown, "LEFT", 0, 4)
					MapsterEnhanced_ElvUI:SecureHook(SatMap, "UpdateMapsize", "SatMap_UpdateMapsize")
				end
			end
		end
		if not MapsterEnhanced_ElvUI.skinnedMaps then
			local Maps = Mapster:GetModule("InstanceMaps")
			if Maps then
				if MapsterMapSelectButton then
					MapsterEnhanced_ElvUI.skinnedMaps = true
					MapsterEnhanced_ElvUI.ElvUI.S:HandleButton(MapsterMapSelectButton)
					MapsterEnhanced_ElvUI:Unhook(Maps, "Refresh")
				end
			end
		end
	end
end

function MapsterEnhanced_ElvUI:OnInitialize()
end

function MapsterEnhanced_ElvUI:OnEnable()
	if ElvUI then
		if not self.ElvUI then
			self.ElvUI = {}
			self.ElvUI.E = unpack(ElvUI)
			self.ElvUI.S = self.ElvUI.E:GetModule("Skins")
		end

		self.ElvUI.S:HandleDropDownBox(MapsterQuestObjectivesDropDown)
		self.ElvUI.S:HandleButton(MapsterOptionsButton)
		if not self.skinnedSatMap then
			local SatMap = Mapster:GetModule("SatMap")
			if SatMap then
				if SatMap_MapButton then
					self.skinnedSatMap = true
					self.ElvUI.S:HandleButton(SatMap_MapButton)
					SatMap_MapButton:ClearAllPoints()
					SatMap_MapButton:SetPoint("RIGHT", WorldMapContinentDropDown, "LEFT", 0, 4)
					self:SecureHook(SatMap, "UpdateMapsize", "SatMap_UpdateMapsize")
				end
			end
		end
		if not self.skinnedMaps then
			local Maps = Mapster:GetModule("InstanceMaps")
			if Maps then
				self:SecureHook(Maps, "Refresh", function()
					if MapsterMapSelectButton then
						self.skinnedMaps = true
						MapsterEnhanced_ElvUI.ElvUI.S:HandleButton(MapsterMapSelectButton)
						MapsterEnhanced_ElvUI:Unhook(Maps, "Refresh")
					end
				end)
			end
		end
		WorldMapFrame:HookScript("OnShow", FixElvUI)
		self:SecureHook(Mapster, "RefreshQuestObjectivesDisplay", "RefreshQuestObjectivesDisplay")
		if MapsterNonOverlayHolder then
			MapsterNonOverlayHolder:StripTextures()
		end
		function self.ElvUI.E.WorldMap:AdjustMapSize() end
		function self.ElvUI.E.WorldMap:SetLargeWorldMap() end
		function self.ElvUI.E.WorldMap:SetQuestWorldMap() end
		function self.ElvUI.E.WorldMap:SetSmallWorldMap() end
		function self.ElvUI.E.WorldMap:PLAYER_REGEN_ENABLED() end
		function self.ElvUI.E.WorldMap:PLAYER_REGEN_DISABLED() end
		function self.ElvUI.E.WorldMap:UpdateCoords()
			if CoordsHolder then
				CoordsHolder:Hide()
				if not CoordsHolder:IsShown() then
					function MapsterEnhanced_ElvUI.ElvUI.E.WorldMap:UpdateCoords() end
				end
			end
		end
		function self.ElvUI.E.WorldMap:ToggleTinyWorldMapSetting() end
	end
end

function MapsterEnhanced_ElvUI:SatMap_UpdateMapsize(f, mini)
	if ElvUI then
		if not mini then
			SatMap_MapButton:ClearAllPoints()
			SatMap_MapButton:SetPoint("RIGHT", WorldMapContinentDropDown, "LEFT", 0, 4)
		end
		if WorldMapFrame.backdrop then
			if WORLDMAP_SETTINGS.size == WORLDMAP_FULLMAP_SIZE then
				WorldMapFrame.backdrop:ClearAllPoints()
				WorldMapFrame.backdrop:Point("TOPLEFT", WorldMapDetailFrame, "TOPLEFT", -25, 70)
				WorldMapFrame.backdrop:Point("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT", 25, -30)
			elseif WORLDMAP_SETTINGS.size == WORLDMAP_WINDOWED_SIZE then
				WorldMapFrame.backdrop:ClearAllPoints()
				WorldMapFrame.backdrop:Point("TOPLEFT", 2, -12)
				WorldMapFrame.backdrop:Point("BOTTOMRIGHT", -7, -8)
			elseif WORLDMAP_SETTINGS.size == WORLDMAP_QUESTLIST_SIZE then
				WorldMapFrame.backdrop:ClearAllPoints()
				WorldMapFrame.backdrop:Point("TOPLEFT", WorldMapDetailFrame, "TOPLEFT", -25, 70)
				WorldMapFrame.backdrop:Point("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT", 334, -236)
			end
		end
	end
end

function MapsterEnhanced_ElvUI:RefreshQuestObjectivesDisplay()
	if ElvUI then
		if WorldMapFrame.backdrop then
			if WORLDMAP_SETTINGS.size == WORLDMAP_FULLMAP_SIZE then
				WorldMapFrame.backdrop:ClearAllPoints()
				WorldMapFrame.backdrop:Point("TOPLEFT", WorldMapDetailFrame, "TOPLEFT", -25, 70)
				WorldMapFrame.backdrop:Point("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT", 25, -30)
			elseif WORLDMAP_SETTINGS.size == WORLDMAP_WINDOWED_SIZE then
				WorldMapFrame.backdrop:ClearAllPoints()
				WorldMapFrame.backdrop:Point("TOPLEFT", 2, -12)
				WorldMapFrame.backdrop:Point("BOTTOMRIGHT", -7, -8)
			elseif WORLDMAP_SETTINGS.size == WORLDMAP_QUESTLIST_SIZE then
				WorldMapFrame.backdrop:ClearAllPoints()
				WorldMapFrame.backdrop:Point("TOPLEFT", WorldMapDetailFrame, "TOPLEFT", -25, 70)
				WorldMapFrame.backdrop:Point("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT", 334, -236)
			end
		end
	end
end
