--[[
Copyright (c) 2010, Sindragosa
All rights reserved.
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local L = LibStub("AceLocale-3.0"):GetLocale("Mapster")

local MODNAME = "Encounters"
local Encounters = Mapster:NewModule(MODNAME, "AceHook-3.0")

local _G = _G
local WorldMapBossButtonFrame = WorldMapBossButtonFrame

local db
local defaults = {
	profile = {
		show = true,
		scale = 0.5,
	}
}

local optGetter, optSetter
do
	local mod = Encounters
	function optGetter(info)
		local key = info[#info]
		return db[key]
	end

	function optSetter(info, value)
		local key = info[#info]
		db[key] = value
		mod:Refresh()
	end
end

local options
local function getOptions()
	if not options then
		options = {
			type = "group",
			name = L["Encounters"],
			arg = MODNAME,
			get = optGetter,
			set = optSetter,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["The Encounters module lets you scale or hide the Encounter Journal Buttons on the World Map."],
				},
				enabled = {
					order = 2,
					type = "toggle",
					width = "full",
					name = L["Enable Encounters"],
					get = function() return Mapster:GetModuleEnabled(MODNAME) end,
					set = function(info, value) Mapster:SetModuleEnabled(MODNAME, value) end,
				},
				show = {
					order = 3,
					type = "toggle",
					name = L["Show Encounters"],
					desc = L["Show the Encounter Journal Buttons."],
				},
				scale = {
					order = 4,
					type = "range",
					name = L["Scale"],
					desc = L["Scale of the buttons."],
					min = 0.1, max = 2, step = 0.01, bigStep = 0.05,
				},
			},
		}
	end

	return options
end

function Encounters:OnInitialize()
	self.db = Mapster.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	self:SetEnabledState(Mapster:GetModuleEnabled(MODNAME))
	Mapster:RegisterModuleOptions(MODNAME, getOptions, L["Encounters"])
end

function Encounters:OnEnable()
	if not self.hooked then
		self.hooked = true
		self.scale = 1
		self:SecureHook("EncounterJournal_AddMapButtons", "ScaleButtons")
	end
	self:Refresh()
end

function Encounters:OnDisable()
	if not WorldMapBossButtonFrame:IsShown() then
		WorldMapBossButtonFrame:Show()
	end
	if self.scale ~= 1 then
		self:ScaleButtons(1, self.scale)
	end
end

function Encounters:Refresh()
	db = self.db.profile
	if not self:IsEnabled() then return end
	if db.show and not WorldMapBossButtonFrame:IsShown() then
		WorldMapBossButtonFrame:Show()
	elseif not db.show and WorldMapBossButtonFrame:IsShown() then
		WorldMapBossButtonFrame:Hide()
	end

	if db.scale ~= self.scale then
		self:ScaleButtons(db.scale, self.scale)
	end
end

function Encounters:ScaleButtons(scaleTo, scaleFrom)
	if not self:IsEnabled() and self.scale == 1 then return end
	local scale
	if WorldMapBossButtonFrame and WorldMapBossButtonFrame.ready then
		scale = WorldMapBossButtonFrame:GetScale()
	end
	if not scale or not EJMapButton1 then return end

	local index = 1
	local scaleTo = scaleTo or db.scale
	local currentScale = scaleFrom or 1
	while _G["EJMapButton"..index] do
		local _, _ ,_ , x, y = _G["EJMapButton"..index]:GetPoint()
		_G["EJMapButton"..index]:SetScale(scale * scaleTo)

		if scaleTo ~= currentScale then
			x = x * (currentScale / scaleTo)
			y = y * (currentScale / scaleTo)
			_G["EJMapButton"..index]:SetPoint("CENTER", WorldMapBossButtonFrame, "BOTTOMLEFT", x, y)
		end
		index = index + 1
	end
	self.scale = scaleTo
end
