--[[
MapsterEnhanced - Exploration (Sindragosa)
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local L = LibStub("AceLocale-3.0"):GetLocale("Mapster")

local MODNAME = "Exploration"
local Exploration = Mapster:NewModule(MODNAME, "AceEvent-3.0")

local Notes

local GetMapInfo = GetMapInfo
local GetCurrentMapZone = GetCurrentMapZone
local GetCurrentMapContinent = GetCurrentMapContinent
local GetAchievementCriteriaInfo = GetAchievementCriteriaInfo
local WorldMapDetailFrame = WorldMapDetailFrame
local wipe = table.wipe

local db
local defaults = {
	profile = {
		exploration = true,
		showExplored = false,
	},
	char = {
		pois = {},
	},
}

local optGetter, optSetter
do
	local mod = Exploration
	function optGetter(info)
		local key = info[#info]
		return db[key]
	end

	function optSetter(info, value)
		local key = info[#info]
		db[key] = value
		mod:Refresh()
	end
end

local options
local function getOptions()
	if not options then
		options = {
			type = "group",
			name = L["Exploration"],
			arg = MODNAME,
			get = optGetter,
			set = optSetter,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["The Exploration module adds points of exploration to the World Map frame."],
				},
				enabled = {
					order = 2,
					type = "toggle",
					name = L["Enable Exploration"],
					get = function() return Mapster:GetModuleEnabled(MODNAME) end,
					set = function(info, value) Mapster:SetModuleEnabled(MODNAME, value) end,
					width = "double",
				},
				exploration = {
					order = 3,
					type = "toggle",
					name = L["Exploration locations"],
					desc = L["Show the locations needed to explore a certain area."],
					get = function(info) return db.exploration end,
					set = function(info, value)
						db.exploration = value
						if not value then
							wipe(Exploration.db.char.pois)
						else
							Exploration:AddExplorationNotes()
						end
						if Notes then
							Notes:RefreshMap()
						end
					end,
				},
				showExplored = {
					order = 4,
					type = "toggle",
					name = L["Already explored"],
					desc = L["Show areas that have already been explored."],
					get = function(info) return db.showExplored end,
					set = function(info, value)
						db.showExplored = value
						if not value then
							wipe(Exploration.db.char.pois)
							Exploration:AddExplorationNotes()
						else
							Exploration:AddExplorationNotes()
						end
						if Notes then
							Notes:RefreshMap()
						end
					end,
				},
			},
		}
	end

	return options
end

local path = "Interface\\AddOns\\MapsterEnhanced\\Textures\\Exploration\\"
local icons = {
	["NotExplored"] = {
		text = "POI_NotExplored",
		path = path .. "icon-red",
		width = 12,
		height = 12,
	},
	["Explored"] = {
		text = "POI_Explored",
		path = path .. "icon-green",
		width = 12,
		height = 12,
	},
}

local POI_Exploration = {
	[1] =  -- Kalimdor
	{
		[1] =  -- Ahn'Qiraj: The Fallen Kingdom
		{
		},
		[2] =  -- Ashenvale
		{
			[0] = 845,
			["Warsong Lumber Camp"] = { 0.8748, 0.5868 },
			["The Howling Vale"] = { 0.5324, 0.3278 },
			["Raynewood Retreat"] = { 0.5968, 0.5007 },
			["Thistlefur Village"] = { 0.3708, 0.3301 },
			["Fallen Sky Lake"] = { 0.6637, 0.8159 },
			["The Shrine of Aessina"] = { 0.2146, 0.5539 },
			["Thunder Peak"] = { 0.4890, 0.4656 },
			["Bough Shadow"] = { 0.9341, 0.3830 },
			["Satyrnaar"] = { 0.8049, 0.4978 },
			["Lake Falathim"] = { 0.2026, 0.4117 },
			["Silverwind Refuge"] = { 0.4885, 0.6609 },
			["Felfire Hill"] = { 0.8310, 0.6546 },
			["Maestra's Post"] = { 0.2750, 0.3787 },
			["The Ruins of Stardust"] = { 0.3268, 0.6579 },
			["Astranaar"] = { 0.3593, 0.5037 },
			["Orendil's Retreat"] = { 0.2600, 0.1984 },
			["Splintertree Post"] = { 0.7100, 0.4806 },
			["The Zoram Strand"] = { 0.1400, 0.2200 },
		},
		[3] =  -- Azshara
		{
			[0] = 852,
			["The Secret Lab"] = { 0.4351, 0.7545 },
			["Southridge Beach"] = { 0.5511, 0.6979 },
			["Ruins of Arkkoran"] = { 0.7116, 0.3316 },
			["Tower of Eldara"] = { 0.8024, 0.2934 },
			["Gallywix Pleasure Palace"] = { 0.2051, 0.5516 },
			["Blackmaw Hold"] = { 0.3194, 0.3263 },
			["Lake Mennar"] = { 0.3408, 0.7455 },
			["Bitter Reaches"] = { 0.6442, 0.2388 },
			["The Ruined Reaches"] = { 0.5893, 0.9072 },
			["Bilgewater Harbor"] = { 0.5998, 0.4993 },
			["Orgrimmar Rear Gate"] = { 0.2440, 0.8046 },
			["Bear's Head"] = { 0.2470, 0.3735 },
			["Ruins of Eldarath"] = { 0.3448, 0.5165 },
			["Storm Cliffs"] = { 0.5464, 0.7861 },
			["The Shattered Strand"] = { 0.4351, 0.4955 },
			["Darnassian Base Camp"] = { 0.4915, 0.2500 },
			["Ravencrest Monument"] = { 0.6597, 0.8144 },
		},
		[4] =  -- Azuremyst Isle
		{
			[0] = 860,
			["Silting Shore"] = { 0.3538, 0.1257 },
			["Fairbridge Strand"] = { 0.4765, 0.0689 },
			["Geezle's Camp"] = { 0.5828, 0.6714 },
			["Ammen Ford"] = { 0.6133, 0.5374 },
			["Moongraze Woods"] = { 0.5269, 0.4192 },
			["Pod Wreckage"] = { 0.5269, 0.6138 },
			["Odesyus' Landing"] = { 0.4661, 0.7246 },
			["The Exodar"] = { 0.2929, 0.3525 },
			["Pod Cluster"] = { 0.3698, 0.5853 },
			["Stillpine Hold"] = { 0.4601, 0.2021 },
			["Ammen Vale"] = { 0.7759, 0.4379 },
			["Azure Watch"] = { 0.4925, 0.5082 },
			["Bristlelimb Village"] = { 0.2650, 0.6527 },
			["Wrathscale Point"] = { 0.3199, 0.7949 },
			["Silvermyst Isle"] = { 0.1272, 0.7867 },
			["Valaar's Berth"] = { 0.2365, 0.5419 },
			["Emberglade"] = { 0.5873, 0.1774 },
		},
		[5] =  -- Bloodmyst Isle
		{
			[0] = 861,
			["Wrathscale Lair"] = { 0.6901, 0.6677 },
			["Bladewood"] = { 0.4591, 0.4521 },
			["The Hidden Reef"] = { 0.3278, 0.1999 },
			["Veridian Point"] = { 0.7440, 0.0763 },
			["Blood Watch"] = { 0.5469, 0.5554 },
			["Wyrmscar Island"] = { 0.7211, 0.2972 },
			["Ragefeather Ridge"] = { 0.5604, 0.3503 },
			["Axxarien"] = { 0.4082, 0.3308 },
			["Blacksilt Shore"] = { 0.3328, 0.9042 },
			["Bristlelimb Enclave"] = { 0.6697, 0.7837 },
			["The Bloodcursed Reef"] = { 0.8094, 0.2028 },
			["The Cryo-Core"] = { 0.3852, 0.5966 },
			["Middenvale"] = { 0.5165, 0.7680 },
			["Nazzivian"] = { 0.3787, 0.7590 },
			["The Warp Piston"] = { 0.5414, 0.1714 },
			["The Vector Coil"] = { 0.1801, 0.5352 },
			["The Bloodwash"] = { 0.3877, 0.2208 },
			["Talon Stand"] = { 0.7340, 0.2111 },
			["The Lost Fold"] = { 0.5744, 0.8114 },
			["Bloodcurse Isle"] = { 0.8543, 0.5292 },
			["Amberweb Pass"] = { 0.1886, 0.3099 },
			["Tel'athion's Camp"] = { 0.2445, 0.4132 },
			["The Foul Pool"] = { 0.2919, 0.3690 },
			["The Crimson Reach"] = { 0.6262, 0.2552 },
			["Vindicator's Rest"] = { 0.3019, 0.4633 },
			["Ruins of Loreth'Aran"] = { 0.6163, 0.4528 },
			["Kessel's Crossing"] = { 0.6188, 0.9019 },
			["Mystwood"] = { 0.4386, 0.8481 },
		},
		[6] =  -- Darkshore
		{
			[0] = 844,
			["Shatterspear Vale"] = { 0.7255, 0.1744 },
			["Lor'danel"] = { 0.5125, 0.1939 },
			["Ruins of Auberdine"] = { 0.3792, 0.4424 },
			["The Master's Glaive"] = { 0.4037, 0.8630 },
			["Ameth'Aran"] = { 0.4371, 0.6063 },
			["The Eye of the Vortex"] = { 0.4286, 0.5397 },
			["Ruins of Mathystra"] = { 0.6392, 0.2208 },
			["Wildbend River"] = { 0.4252, 0.6964 },
			["Nazj'vel"] = { 0.3209, 0.8413 },
			["Shatterspear War Camp"] = { 0.6597, 0.0720 },
			["Withering Thicket"] = { 0.4400, 0.4000 },
		},
		[7] =  -- Darnassus
		{
		},
		[8] =  -- Desolace
		{
			[0] = 848,
			["Ranazjar Isle"] = { 0.2874, 0.0921 },
			["Gelkis Village"] = { 0.3618, 0.8877 },
			["Mannoroc Coven"] = { 0.5140, 0.7844 },
			["Magram Territory"] = { 0.7465, 0.4566 },
			["Shadowprey Village"] = { 0.2425, 0.7088 },
			["Thunder Axe Fortress"] = { 0.5454, 0.2537 },
			["Shok'Thokar"] = { 0.7061, 0.7485 },
			["Cenarion Wildlands"] = { 0.5614, 0.4723 },
			["Shadowbreak Ravine"] = { 0.8024, 0.7942 },
			["Slitherblade Shore"] = { 0.2999, 0.2972 },
			["Valley of Spears"] = { 0.3533, 0.5719 },
			["Tethris Aran"] = { 0.5155, 0.1010 },
			["Kodo Graveyard"] = { 0.5080, 0.5913 },
			["Thargad's Camp"] = { 0.3653, 0.6991 },
			["Sargeron"] = { 0.7829, 0.2171 },
			["Nijel's Point"] = { 0.6672, 0.0883 },
		},
		[9] =  -- Durotar
		{
			[0] = 728,
			["Southfury Watershed"] = { 0.4042, 0.4000 },
			["Valley of Trials"] = { 0.4440, 0.6212 },
			["Skull Rock"] = { 0.5379, 0.0921 },
			["Sen'jin Village"] = { 0.5599, 0.7567 },
			["Echo Isles"] = { 0.6622, 0.8293 },
			["Drygulch Ravine"] = { 0.5240, 0.2440 },
			["Razormane Grounds"] = { 0.4471, 0.5075 },
			["Tiragarde Keep"] = { 0.5793, 0.6033 },
			["Thunder Ridge"] = { 0.4027, 0.2478 },
			["Northwatch Foothold"] = { 0.4840, 0.7912 },
			["Orgrimmar"] = { 0.4546, 0.0846 },
			["Razor Hill"] = { 0.5349, 0.4341 },
		},
		[10] =  -- Dustwallow Marsh
		{
			[0] = 850,
			["Shady Rest Inn"] = { 0.2964, 0.4918 },
			["Mudsprocket"] = { 0.4162, 0.7530 },
			["Theramore Isle"] = { 0.6717, 0.5007 },
			["Blackhoof Village"] = { 0.4147, 0.1235 },
			["Wyrmbog"] = { 0.5334, 0.7567 },
			["Brackenwall Village"] = { 0.3623, 0.3069 },
			["Dreadmurk Shore"] = { 0.6162, 0.3027 },
			["Alcaz Island"] = { 0.7610, 0.1766 },
			["Direhorn Post"] = { 0.4676, 0.4633 },
		},
		[11] =  -- Felwood
		{
			[0] = 853,
			["Felpaw Village"] = { 0.6123, 0.1168 },
			["Jaedenar"] = { 0.3708, 0.5913 },
			["Morlos'Aran"] = { 0.5654, 0.8690 },
			["Emerald Sanctuary"] = { 0.5230, 0.7837 },
			["Ruins of Constellas"] = { 0.3802, 0.7238 },
			["Bloodvenom Falls"] = { 0.4100, 0.4700 },
			["Jadefire Run"] = { 0.4366, 0.1871 },
			["Deadwood Village"] = { 0.4805, 0.8930 },
			["Shatter Scar Vale"] = { 0.4321, 0.4117 },
			["Talonbranch Glade"] = { 0.6262, 0.2687 },
			["Jadefire Glen"] = { 0.4177, 0.8548 },
			["Irontree Woods"] = { 0.5040, 0.2605 },
		},
		[12] =  -- Feralas
		{
			[0] = 849,
			["Dire Maul"] = { 0.6033, 0.3570 },
			["Feral Scar Vale"] = { 0.5527, 0.5624 },
			["Grimtotem Compound"] = { 0.6866, 0.4042 },
			["Lower Wilds"] = { 0.8303, 0.4102 },
			["Ruins of Isildien"] = { 0.5858, 0.7305 },
			["Gordunni Outpost"] = { 0.7560, 0.2987 },
			["The Twin Colossals"] = { 0.4636, 0.1804 },
			["Darkmist Ruins"] = { 0.6477, 0.5861 },
			["The Forgotten Coast"] = { 0.4976, 0.4942 },
			["Ruins of Feathermoon"] = { 0.2854, 0.4925 },
			["The Writhing Deep"] = { 0.7590, 0.6257 },
			["Feathermoon Stronghold"] = { 0.4586, 0.4963 },
			["Camp Mojache"] = { 0.7645, 0.4461 },
		},
		[13] =  -- Moonglade
		{
			[0] = 855,
			["Shrine of Remulos"] = { 0.3600, 0.3890 },
			["Nighthaven"] = { 0.4800, 0.3800 },
			["Lake Elune'ara"] = { 0.5450, 0.5450 },
			["Stormrage Barrow Dens"] = { 0.6700, 0.5400 },
		},
		[14] =  -- Mount Hyjal
		{
			[0] = 4863,
			["Nordrassil"] = { 0.6342, 0.2111 },
			["The Flamewake"] = { 0.3293, 0.5135 },
			["Rim of the World"] = { 0.2559, 0.6467 },
			["Sethria's Roost"] = { 0.3140, 0.8503 },
			["Darkwhisper Gorge"] = { 0.7600, 0.6500 },
			["Shrine of Goldrinn"] = { 0.2859, 0.2769 },
			["Ashen Lake"] = { 0.2176, 0.5359 },
			["The Throne of Flame"] = { 0.4526, 0.8072 },
			["The Circle of Cinders"] = { 0.4511, 0.2612 },
			["The Scorched Plain"] = { 0.5334, 0.5516 },
			["Gates of Sothann"] = { 0.7246, 0.7687 },
		},
		[15] =  -- Mulgore
		{
			[0] = 736,
			["Red Rocks"] = { 0.5993, 0.1991 },
			["Windfury Ridge"] = { 0.5114, 0.0925 },
			["Wildmane Water Well"] = { 0.4316, 0.1445 },
			["Winterhoof Water Well"] = { 0.5374, 0.6684 },
			["Bael'dun Digsite"] = { 0.3234, 0.5067 },
			["Red Cloud Mesa"] = { 0.5378, 0.8596 },
			["Ravaged Caravan"] = { 0.5419, 0.4813 },
			["The Golden Plains"] = { 0.5416, 0.2104 },
			["Palemane Rock"] = { 0.3558, 0.6130 },
			["The Venture Co. Mine"] = { 0.6272, 0.4274 },
			["Bloodhoof Village"] = { 0.4785, 0.5853 },
			["The Rolling Plains"] = { 0.6109, 0.6061 },
			["Thunderhorn Water Well"] = { 0.4486, 0.4356 },
		},
		[16] =  -- Northern Barrens
		{
			[0] = 750,
			["Thorn Hill"] = { 0.5868, 0.5007 },
			["The Mor'shan Rampart"] = { 0.3952, 0.1415 },
			["The Dry Hills"] = { 0.2565, 0.3189 },
			["The Sludge Fen"] = { 0.5579, 0.1864 },
			["Ratchet"] = { 0.6637, 0.7178 },
			["The Stagnant Oasis"] = { 0.5569, 0.7852 },
			["Grol'dom Farm"] = { 0.5484, 0.4117 },
			["Lushwater Oasis"] = { 0.4047, 0.7420 },
			["Boulder Lode Mine"] = { 0.6732, 0.1205 },
			["The Forgotten Pools"] = { 0.3688, 0.4596 },
			["Far Watch Post"] = { 0.6662, 0.4222 },
			["Dreadmist Peak"] = { 0.4306, 0.3548 },
			["The Crossroads"] = { 0.5025, 0.5734 },
			["The Merchant Coast"] = { 0.7026, 0.8413 },
		},
		[17] =  -- Orgrimmar
		{
		},
		[18] =  -- Silithus
		{
			[0] = 856,
			["Cenarion Hold"] = { 0.5430, 0.3436 },
			["The Crystal Vale"] = { 0.3089, 0.1617 },
			["Southwind Village"] = { 0.6532, 0.4753 },
			["Valor's Rest"] = { 0.8129, 0.1789 },
			["Hive'Ashi"] = { 0.4940, 0.2260 },
			["Hive'Zora"] = { 0.3134, 0.5397 },
			["Hive'Regal"] = { 0.5998, 0.7186 },
			["The Scarab Wall"] = { 0.3200, 0.7900 },
		},
		[19] =  -- Southern Barrens
		{
			[0] = 4996,
			["Frazzlecraz Motherlode"] = { 0.4097, 0.7837 },
			["Honor's Stand"] = { 0.3658, 0.1115 },
			["Hunter's Hill"] = { 0.3942, 0.2253 },
			["Bael Modan"] = { 0.5075, 0.8406 },
			["Vendetta Point"] = { 0.4247, 0.4416 },
			["The Overgrowth"] = { 0.4785, 0.3391 },
			["Ruins of Taurajo"] = { 0.4496, 0.5442 },
			["Forward Command"] = { 0.5215, 0.4865 },
			["Northwatch Hold"] = { 0.6870, 0.4933 },
			["Razorfen Kraul"] = { 0.4127, 0.9456 },
			["Battlescar"] = { 0.4552, 0.6992 },
		},
		[20] =  -- Stonetalon Mountains
		{
			[0] = 847,
			["Boulderslide Ravine"] = { 0.6352, 0.8877 },
			["Webwinder Path"] = { 0.5967, 0.7906 },
			["Sun Rock Retreat"] = { 0.4940, 0.6347 },
			["Mirkfallon Lake"] = { 0.5160, 0.4740 },
			["The Charred Vale"] = { 0.3159, 0.7163 },
			["Malaka'jin"] = { 0.7285, 0.9229 },
			["Krom'gar Fortress"] = { 0.6692, 0.6617 },
			["Stonetalon Peak"] = { 0.4092, 0.1999 },
			["Ruins of Eldre'thar"] = { 0.4890, 0.7642 },
			["Battlescar Valley"] = { 0.3698, 0.5444 },
			["Unearthed Grounds"] = { 0.7680, 0.7597 },
			["Webwinder Hollow"] = { 0.5719, 0.7283 },
			["Cliffwalker Post"] = { 0.4520, 0.3031 },
			["Windshear Hold"] = { 0.5903, 0.5786 },
			["Thal'darah Overlook"] = { 0.3573, 0.3166 },
			["Windshear Crag"] = { 0.6697, 0.4940 },
			["Greatwood Vale"] = { 0.6959, 0.8508 },
		},
		[21] =  -- Tanaris
		{
			[0] = 851,
			["Gadgetzan"] = { 0.5140, 0.2928 },
			["Caverns of Time"] = { 0.6417, 0.5000 },
			["Lost Rigger Cove"] = { 0.7152, 0.4948 },
			["Abyssal Sands"] = { 0.5515, 0.4078 },
			["Southmoon Ruins"] = { 0.4080, 0.7066 },
			["The Noxious Lair"] = { 0.3443, 0.4566 },
			["Sandsorrow Watch"] = { 0.4102, 0.2740 },
			["The Gaping Chasm"] = { 0.5374, 0.6766 },
			["Valley of the Watchers"] = { 0.3767, 0.7799 },
			["Broken Pillar"] = { 0.5235, 0.4558 },
			["Zul'Farrak"] = { 0.3733, 0.1452 },
			["Thistleshrub Valley"] = { 0.3039, 0.6639 },
			["Dunemaul Compound"] = { 0.4107, 0.5569 },
			["Land's End Beach"] = { 0.5359, 0.9177 },
			["Eastmoon Ruins"] = { 0.4721, 0.6490 },
			["Southbreak Shore"] = { 0.6300, 0.5934 },
		},
		[22] =  -- Teldrassil
		{
			[0] = 842,
			["Starbreeze Village"] = { 0.6580, 0.4925 },
			["Shadowglen"] = { 0.5843, 0.3320 },
			["Dolanaar"] = { 0.5554, 0.5135 },
			["Pools of Arlithrien"] = { 0.4187, 0.5711 },
			["Ban'ethil Hollow"] = { 0.4626, 0.4993 },
			["Rut'theran Village"] = { 0.5500, 0.8900 },
			["Gnarlpine Hold"] = { 0.4476, 0.6752 },
			["Lake Al'Ameth"] = { 0.5500, 0.6123 },
			["The Cleft"] = { 0.5070, 0.3817 },
			["The Oracle Glade"] = { 0.4002, 0.2695 },
			["Wellspring Lake"] = { 0.4446, 0.3466 },
			["Darnassus"] = { 0.3049, 0.5022 },
		},
		[23] =  -- The Exodar
		{
		},
		[24] =  -- Thousand Needles
		{
			[0] = 846,
			["The Great Lift"] = { 0.3219, 0.1841 },
			["Splithoof Heights"] = { 0.8827, 0.4800 },
			["Darkcloud Pinnacle"] = { 0.3378, 0.3892 },
			["Westreach Summit"] = { 0.1337, 0.1048 },
			["Razorfen Downs"] = { 0.5175, 0.3009 },
			["The Shimmering Deep"] = { 0.7800, 0.6200 },
			["Highperch"] = { 0.1312, 0.3772 },
			["Freewind Post"] = { 0.4701, 0.5067 },
			["Twilight Bulwark"] = { 0.3104, 0.5883 },
			["The Twilight Withering"] = { 0.5499, 0.6347 },
			["Sunken Dig Site"] = { 0.6712, 0.8585 },
			["Southsea Holdfast"] = { 0.9247, 0.7889 },
		},
		[25] =  -- Thunder Bluff
		{
		},
		[26] =  -- Uldum
		{
			[0] = 4865,
			["Ramkahen"] = { 0.5479, 0.3249 },
			["Ruins of Ammon"] = { 0.3209, 0.6564 },
			["The Cursed Landing"] = { 0.8473, 0.5292 },
			["Tahret Grounds"] = { 0.6040, 0.4000 },
			["Cradle of the Ancients"] = { 0.4376, 0.7036 },
			["Lost City of the Tol'vir"] = { 0.6392, 0.7343 },
			["Akhenet Fields"] = { 0.5434, 0.5232 },
			["Orsis"] = { 0.4032, 0.3975 },
			["Nahom"] = { 0.6717, 0.4072 },
			["Neferset City"] = { 0.5125, 0.8219 },
			["Obelisk of the Sun"] = { 0.4556, 0.5629 },
			["Mar'at"] = { 0.4895, 0.3840 },
			["The Trail of Devastation"] = { 0.8030, 0.6020 },
			["Tombs of the Precursors"] = { 0.7200, 0.4400 },
			["Vir'naal Dam"] = { 0.5830, 0.4090 },
			["Ruins of Ahmtul"] = { 0.4611, 0.1564 },
			["Obelisk of the Stars"] = { 0.6462, 0.3181 },
			["Khartut's Tomb"] = { 0.6452, 0.1624 },
			["Schnottz's Landing"] = { 0.2076, 0.6198 },
			["The Gate of Unending Cycles"] = { 0.6900, 0.2200 },
			["Obelisk of the Moon"] = { 0.4077, 0.2305 },
			["Temple of Uldum"] = { 0.3378, 0.3114 },
		},
		[27] =  -- Un'Goro Crater
		{
			[0] = 854,
			["Lakkari Tar Pits"] = { 0.5145, 0.2567 },
			["The Slithering Scar"] = { 0.5110, 0.7747 },
			["Fire Plume Ridge"] = { 0.5165, 0.4813 },
			["The Marshlands"] = { 0.6786, 0.5472 },
			["The Screaming Reaches"] = { 0.2899, 0.3593 },
			["The Roiling Gardens"] = { 0.6926, 0.3443 },
			["Terror Run"] = { 0.3338, 0.7081 },
			["Mossy Pile"] = { 0.4217, 0.4184 },
			["Golakka Hot Springs"] = { 0.3144, 0.5419 },
			["Ironstone Plateau"] = { 0.7713, 0.3935 },
			["Fungal Rock"] = { 0.6577, 0.1437 },
			["Marshal's Stand"] = { 0.5404, 0.6100 },
		},
		[28] =  -- Winterspring
		{
			[0] = 857,
			["Starfall Village"] = { 0.4775, 0.4000 },
			["Lake Kel'Theril"] = { 0.5110, 0.5524 },
			["Ice Thistle Hills"] = { 0.6786, 0.6445 },
			["Frostfire Hot Springs"] = { 0.3189, 0.4948 },
			["Owl Wing Thicket"] = { 0.6457, 0.7725 },
			["Mazthoril"] = { 0.5479, 0.6310 },
			["Winterfall Village"] = { 0.6800, 0.4800 },
			["Frostwhisper Gorge"] = { 0.5978, 0.8563 },
			["The Hidden Grove"] = { 0.6272, 0.2478 },
			["Timbermaw Post"] = { 0.3643, 0.5689 },
			["Frostsaber Rock"] = { 0.4541, 0.1564 },
			["Everlook"] = { 0.5998, 0.4903 },
		},
	},
	[2] =  -- Eastern Kingdoms
	{
		[1] =  -- Abyssal Depths
		{
			[0] = 4825,
			["L'ghorek"] = { 0.3224, 0.5427 },
			["The Scalding Chasm"] = { 0.4700, 0.2400 },
			["Underlight Canyon"] = { 0.4227, 0.7010 },
			["Abyssal Breach"] = { 0.7156, 0.2979 },
			["Abandoned Reef"] = { 0.2210, 0.8030 },
			["Seabrush"] = { 0.5484, 0.4356 },
			["Deepfin Ridge"] = { 0.4000, 0.2000 },
			["Korthun's End"] = { 0.5399, 0.6392 },
		},
		[2] =  -- Arathi Highlands
		{
			[0] = 761,
			["Thandol Span"] = { 0.4127, 0.9102 },
			["Boulderfist Hall"] = { 0.4905, 0.7904 },
			["Refuge Pointe"] = { 0.4000, 0.4640 },
			["Boulder'gor"] = { 0.2754, 0.4440 },
			["Witherbark Village"] = { 0.6153, 0.7043 },
			["Dabyrie's Farmstead"] = { 0.4990, 0.4132 },
			["Circle of Inner Binding"] = { 0.3004, 0.5958 },
			["Circle of West Binding"] = { 0.1881, 0.3054 },
			["Circle of Outer Binding"] = { 0.4736, 0.5150 },
			["Circle of East Binding"] = { 0.5933, 0.3293 },
			["Hammerfall"] = { 0.6936, 0.3735 },
			["Galen's Fall"] = { 0.1302, 0.3645 },
			["Northfold Manor"] = { 0.2695, 0.2725 },
			["Faldir's Cove"] = { 0.2380, 0.8200 },
			["Go'Shek Farm"] = { 0.5640, 0.5900 },
			["Stromgarde Keep"] = { 0.1800, 0.6400 },
		},
		[3] =  -- Badlands
		{
			[0] = 765,
			["New Kargath"] = { 0.1657, 0.4326 },
			["Agmond's End"] = { 0.4601, 0.5943 },
			["Camp Kosh"] = { 0.5993, 0.2050 },
			["Bloodwatcher Point"] = { 0.5115, 0.5157 },
			["Camp Cagg"] = { 0.1680, 0.6340 },
			["Uldaman"] = { 0.4396, 0.1205 },
			["Angor Fortress"] = { 0.4032, 0.2440 },
			["Scar of the Worldbreaker"] = { 0.3393, 0.5262 },
			["The Dustbowl"] = { 0.2640, 0.3675 },
			["Lethlor Ravine"] = { 0.6926, 0.4843 },
		},
		[4] =  -- Blasted Lands
		{
			[0] = 766,
			["The Red Reaches"] = { 0.6577, 0.7814 },
			["Serpent's Coil"] = { 0.6038, 0.2852 },
			["The Tainted Scar"] = { 0.3224, 0.4588 },
			["Nethergarde Keep"] = { 0.6437, 0.1579 },
			["The Dark Portal"] = { 0.5444, 0.5299 },
			["Altar of Storms"] = { 0.3703, 0.2650 },
			["The Tainted Forest"] = { 0.3423, 0.7238 },
			["Surwich"] = { 0.4451, 0.8615 },
			["Dreadmaul Post"] = { 0.4711, 0.4019 },
			["Rise of the Defiler"] = { 0.4506, 0.2687 },
			["Shattershore"] = { 0.6911, 0.3353 },
			["Nethergarde Supply Camps"] = { 0.5060, 0.1400 },
			["Sunveil Excursion"] = { 0.5070, 0.7088 },
			["Dreadmaul Hold"] = { 0.4000, 0.1310 },
		},
		[5] =  -- Burning Steppes
		{
			[0] = 775,
			["Blackrock Mountain"] = { 0.2066, 0.2934 },
			["Blackrock Stronghold"] = { 0.3229, 0.3660 },
			["Altar of Storms"] = { 0.0933, 0.2762 },
			["Black Tooth Hovel"] = { 0.3610, 0.5367 },
			["Morgan's Vigil"] = { 0.7340, 0.6800 },
			["The Whelping Downs"] = { 0.2425, 0.5786 },
			["Ruins of Thaurissan"] = { 0.5659, 0.3750 },
			["Terror Wing Path"] = { 0.7390, 0.5350 },
			["Dreadmaul Rock"] = { 0.6971, 0.4072 },
			["Blackrock Pass"] = { 0.6677, 0.7757 },
		},
		[6] =  -- Deadwind Pass
		{
			[0] = 777,
			["The Vice"] = { 0.5863, 0.6475 },
			["Karazhan"] = { 0.4266, 0.6886 },
			["Deadman's Crossing"] = { 0.4516, 0.3593 },
		},
		[7] =  -- Dun Morogh
		{
			[0] = 627,
			["Iceflow Lake"] = { 0.4142, 0.4034 },
			["Shimmer Ridge"] = { 0.4830, 0.3810 },
			["Helm's Bed Lake"] = { 0.8408, 0.5195 },
			["Frostmane Front"] = { 0.5859, 0.5783 },
			["Kharanos"] = { 0.5359, 0.5075 },
			["Frostmane Hold"] = { 0.3154, 0.5195 },
			["Ironforge Airfield"] = { 0.7799, 0.2440 },
			["Gol'Bolar Quarry"] = { 0.7650, 0.5726 },
			["North Gate Outpost"] = { 0.9027, 0.3780 },
			["New Tinkertown"] = { 0.3378, 0.3764 },
			["Coldridge Pass"] = { 0.4386, 0.6467 },
			["Amberstill Ranch"] = { 0.7141, 0.4626 },
			["Coldridge Valley"] = { 0.3463, 0.7575 },
			["The Grizzled Den"] = { 0.4835, 0.5292 },
			["The Tundrid Hills"] = { 0.6725, 0.5407 },
			["Gates of Ironforge"] = { 0.5965, 0.3393 },
		},
		[8] =  -- Duskwood
		{
			[0] = 778,
			["Darkshire"] = { 0.7455, 0.4633 },
			["Addle's Stead"] = { 0.2136, 0.6998 },
			["Vul'Gol Ogre Mound"] = { 0.3588, 0.7283 },
			["Manor Mistmantle"] = { 0.7720, 0.3608 },
			["Raven Hill"] = { 0.1981, 0.5531 },
			["Raven Hill Cemetery"] = { 0.1961, 0.4139 },
			["Tranquil Gardens Cemetery"] = { 0.7969, 0.6617 },
			["Brightwood Grove"] = { 0.6467, 0.3787 },
			["The Hushed Bank"] = { 0.1003, 0.4446 },
			["Twilight Grove"] = { 0.4600, 0.3862 },
			["The Yorgen Farmstead"] = { 0.5095, 0.7410 },
			["The Rotting Orchard"] = { 0.6377, 0.7201 },
			["The Darkened Bank"] = { 0.5464, 0.2133 },
		},
		[9] =  -- Eastern Plaguelands
		{
			[0] = 771,
			["Light's Hope Chapel"] = { 0.7675, 0.5404 },
			["Tyr's Hand"] = { 0.7559, 0.7520 },
			["Corin's Crossing"] = { 0.5569, 0.6302 },
			["Pestilent Scar"] = { 0.6874, 0.5667 },
			["Northpass Tower"] = { 0.5055, 0.2073 },
			["Terrordale"] = { 0.1248, 0.2635 },
			["Zul'Mashar"] = { 0.6647, 0.1044 },
			["The Undercroft"] = { 0.2395, 0.7897 },
			["Thondroril River"] = { 0.0860, 0.6620 },
			["Blackwood Lake"] = { 0.4631, 0.4356 },
			["The Noxious Glade"] = { 0.7814, 0.3593 },
			["Quel'Lithien Lodge"] = { 0.4815, 0.1325 },
			["The Fungal Vale"] = { 0.3613, 0.4491 },
			["Stratholme"] = { 0.2670, 0.1003 },
			["Ruins of the Scarlet Enclave"] = { 0.8713, 0.7852 },
			["Lake Mereldar"] = { 0.5748, 0.7425 },
			["Plaguewood"] = { 0.3357, 0.2462 },
			["Darrowshire"] = { 0.3508, 0.8436 },
			["Eastwall Tower"] = { 0.6233, 0.4177 },
			["Crown Guard Tower"] = { 0.3568, 0.6864 },
			["Northdale"] = { 0.6617, 0.2507 },
			["The Infectis Scar"] = { 0.5017, 0.6380 },
			["The Marris Stead"] = { 0.2260, 0.6609 },
		},
		[10] =  -- Elwynn Forest
		{
			[0] = 776,
			["Northshire Valley"] = { 0.5040, 0.4250 },
			["Goldshire"] = { 0.4212, 0.6475 },
			["Ridgepoint Tower"] = { 0.8478, 0.7957 },
			["Eastvale Logging Camp"] = { 0.8353, 0.6707 },
			["Westbrook Garrison"] = { 0.2450, 0.7350 },
			["Brackwell Pumpkin Patch"] = { 0.6956, 0.7950 },
			["Jerod's Landing"] = { 0.4855, 0.8600 },
			["Stone Cairn Lake"] = { 0.7400, 0.5200 },
			["Tower of Azora"] = { 0.6462, 0.6946 },
			["Fargodeep Mine"] = { 0.4000, 0.8046 },
			["Crystal Lake"] = { 0.5284, 0.6632 },
		},
		[11] =  -- Eversong Woods
		{
			[0] = 859,
			["Sunstrider Isle"] = { 0.3600, 0.2400 },
			["Silvermoon City"] = { 0.5700, 0.4400 },
			["Ruins of Silvermoon"] = { 0.4400, 0.4000 },
			["Lake Elrendar"] = { 0.6592, 0.7867 },
			["Duskwither Grounds"] = { 0.6800, 0.4700 },
			["Saltheril's Haven"] = { 0.3827, 0.7365 },
			["Tranquil Shore"] = { 0.2844, 0.5793 },
			["The Scorched Grove"] = { 0.3623, 0.8608 },
			["East Sanctum"] = { 0.5225, 0.7300 },
			["Thuron's Livery"] = { 0.6208, 0.5352 },
			["North Sanctum"] = { 0.4400, 0.5270 },
			["Goldenbough Pass"] = { 0.3338, 0.7769 },
			["Sunsail Anchorage"] = { 0.2939, 0.6939 },
			["Runestone Shan'dor"] = { 0.5569, 0.8428 },
			["Elrendar Falls"] = { 0.6457, 0.7328 },
			["Stillwhisper Pond"] = { 0.5574, 0.5651 },
			["Runestone Falithas"] = { 0.4411, 0.8585 },
			["Farstrider Retreat"] = { 0.6108, 0.6377 },
			["Tor'Watha"] = { 0.7196, 0.7549 },
			["Golden Strand"] = { 0.2350, 0.7507 },
			["Azurebreeze Coast"] = { 0.7161, 0.4543 },
			["Zeb'Watha"] = { 0.6233, 0.7942 },
			["West Sanctum"] = { 0.3528, 0.5771 },
			["Fairbreeze Village"] = { 0.4300, 0.7373 },
			["The Living Wood"] = { 0.5419, 0.7201 },
		},
		[12] =  -- Ghostlands
		{
			[0] = 858,
			["Sanctum of the Sun"] = { 0.5614, 0.4800 },
			["Sanctum of the Moon"] = { 0.3343, 0.3226 },
			["Deatholme"] = { 0.3313, 0.7994 },
			["Suncrown Village"] = { 0.6043, 0.1200 },
			["Tranquillien"] = { 0.4710, 0.3286 },
			["Howling Ziggurat"] = { 0.4117, 0.4985 },
			["Elrendar Crossing"] = { 0.4800, 0.1100 },
			["Zeb'Nowa"] = { 0.6617, 0.6115 },
			["Bleeding Ziggurat"] = { 0.3418, 0.4678 },
			["Farstrider Enclave"] = { 0.7226, 0.3211 },
			["Windrunner Spire"] = { 0.1362, 0.5591 },
			["Dawnstar Spire"] = { 0.7859, 0.1879 },
			["Goldenmist Village"] = { 0.2490, 0.1527 },
			["Thalassian Pass"] = { 0.4800, 0.8443 },
			["Amani Pass"] = { 0.7730, 0.6452 },
			["Windrunner Village"] = { 0.1796, 0.4154 },
		},
		[13] =  -- Hillsbrad Foothills
		{
			-- http://www.wowhead.com/?achievement=772
			[0] = 772,
			["Brazie Farmstead"] = { 0.3378, 0.4656 },
			["The Uplands"] = { 0.5245, 0.1272 },
			["Corrahn's Dagger"] = { 0.4965, 0.4686 },
			["Purgation Isle"] = { 0.2695, 0.8600 },
			["Misty Shore"] = { 0.3506, 0.2614 },
			["The Sludge Fields"] = { 0.3757, 0.6025 },
			["Sofera's Naze"] = { 0.5564, 0.3847 },
			["Durnholde Keep"] = { 0.6800, 0.6000 },
			["Darrow Hill"] = { 0.4657, 0.5430 },
			["Ruins of Southshore"] = { 0.4945, 0.7021 },
			["Dalaran Crater"] = { 0.3044, 0.3638 },
			["Ruins of Alterac"] = { 0.4521, 0.2912 },
			["Dun Garok"] = { 0.6352, 0.8466 },
			["Dandred's Fold"] = { 0.4480, 0.0928 },
			["Azurelode Mine"] = { 0.3398, 0.7388 },
			["Nethander Stead"] = { 0.5863, 0.7395 },
			["Tarren Mill"] = { 0.5669, 0.4701 },
			["Crushridge Hold"] = { 0.5014, 0.2436 },
			["Gallows' Corner"] = { 0.4988, 0.3260 },
			["Chillwind Point"] = { 0.6800, 0.3300 },
			["Slaughter Hollow"] = { 0.4943, 0.1888 },
			["Strahnbrad"] = { 0.5838, 0.2350 },
			["Southpoint Gate"] = { 0.2949, 0.6370 },
			["The Headland"] = { 0.4416, 0.4858 },
			["Growless Cave"] = { 0.4326, 0.3862 },
			["Gavin's Naze"] = { 0.4007, 0.4768 },
		},
		[14] =  -- Ironforge
		{
		},
		[15] =  -- Isle of Quel'Danas
		{
			-- http://www.wowhead.com/?achievement=868
			[0] = 868,
			["Dawnstar Village"] = { 0.4100, 0.3200 },
			["Greengill Coast"] = { 0.6300, 0.4800 },
			{ 0.6000, 0.3000, "Magisters' Terrace"},
			["Sun's Reach Harbor"] = { 0.5400, 0.3300 },
			["Sunwell Plateau"] = { 0.4000, 0.4500 },
			["The Dead Scar"] = { 0.5300, 0.8400 },
		},
		[16] =  -- Kelp'thar Forest
		{
			[0] = 4825,
			["Gurboggle's Ledge"] = { 0.5320, 0.5630 },
			["Legion's Fate"] = { 0.4000, 0.2972 },
			["Seafarer's Tomb"] = { 0.5035, 0.2433 },
			["Gnaws' Boneyard"] = { 0.6143, 0.5898 },
			["The Clutch"] = { 0.5878, 0.8136 },
			["The Skeletal Reef"] = { 0.6000, 0.4400 },
		},
		[17] =  -- Loch Modan
		{
			[0] = 779,
			["North Gate Pass"] = { 0.2024, 0.1720 },
			["Thelsamar"] = { 0.3518, 0.4775 },
			["Mo'grosh Stronghold"] = { 0.7101, 0.2395 },
			["Valley of Kings"] = { 0.2026, 0.7859 },
			["Stonesplinter Valley"] = { 0.3244, 0.7844 },
			["Stonewrought Dam"] = { 0.4800, 0.1200 },
			["The Farstrider Lodge"] = { 0.8239, 0.6519 },
			["Ironband's Excavation Site"] = { 0.6936, 0.6549 },
			["Silver Stream Mine"] = { 0.3483, 0.2171 },
			["The Loch"] = { 0.4800, 0.2000 },
			["Grizzlepaw Ridge"] = { 0.3792, 0.6085 },
		},
		[18] =  -- Northern Stranglethorn
		{
			[0] = 781,
			["Kurzen's Compound"] = { 0.6023, 0.1991 },
			["Mizjah Ruins"] = { 0.4611, 0.5307 },
			["Mosh'Ogg Ogre Mound"] = { 0.6697, 0.5097 },
			["Bambala"] = { 0.6462, 0.3975 },
			["Bal'lal Ruins"] = { 0.3423, 0.3638 },
			["Zul'Gurub"] = { 0.8383, 0.3263 },
			["Lake Nazferiti"] = { 0.5200, 0.3400 },
			["Zuuldaia Ruins"] = { 0.1926, 0.2575 },
			["Fort Livingston"] = { 0.5329, 0.6609 },
			["Rebel Camp"] = { 0.4671, 0.0958 },
			["Kal'ai Ruins"] = { 0.4227, 0.3847 },
			["Ruins of Zul'Kunda"] = { 0.2645, 0.2028 },
			["The Vile Reef"] = { 0.3300, 0.4263 },
			["Nesingwary's Expedition"] = { 0.4417, 0.1991 },
			["Grom'gol Base Camp"] = { 0.3777, 0.4800 },
			["Balia'mah Ruins"] = { 0.5898, 0.5531 },
		},
		[19] =  -- Redridge Mountains
		{
			[0] = 780,
			["Stonewatch Falls"] = { 0.7125, 0.5457 },
			["Lakeridge Highway"] = { 0.3822, 0.6849 },
			["Render's Valley"] = { 0.6462, 0.7133 },
			["Render's Camp"] = { 0.3512, 0.1302 },
			["Stonewatch Keep"] = { 0.6093, 0.5254 },
			["Three Corners"] = { 0.1971, 0.5928 },
			["Alther's Mill"] = { 0.4721, 0.4000 },
			["Galardell Valley"] = { 0.7030, 0.3466 },
			["Shalewind Canyon"] = { 0.8099, 0.6250 },
			["Redridge Canyons"] = { 0.3014, 0.2620 },
			["Lakeshire"] = { 0.2545, 0.4341 },
			["Camp Everstill"] = { 0.5344, 0.5472 },
			["Lake Everstill"] = { 0.4182, 0.5269 },
		},
		[20] =  -- Ruins of Gilneas
		{
		},
		[21] =  -- Ruins of Gilneas City
		{
		},
		[22] =  -- Searing Gorge
		{
			[0] = 774,
			["Thorium Point"] = { 0.3767, 0.2904 },
			["The Cauldron"] = { 0.5269, 0.4970 },
			["Blackrock Mountain"] = { 0.3987, 0.8301 },
			["Grimesilt Dig Site"] = { 0.6352, 0.6048 },
			["Firewatch Ridge"] = { 0.2400, 0.3204 },
			["The Sea of Cinders"] = { 0.6000, 0.7178 },
			["Dustfire Valley"] = { 0.7176, 0.2695 },
			["Blackchar Cave"] = { 0.2126, 0.8016 },
		},
		[23] =  -- Shimmering Expanse
		{
			[0] = 4825,
			["Nespirah"] = { 0.5918, 0.5659 },
			["Ruins of Thelserai Temple"] = { 0.6637, 0.4184 },
			["Ruins of Vashj'ir"] = { 0.3368, 0.7246 },
			["Glimmerdeep Gorge"] = { 0.4361, 0.4800 },
			["Silver Tide Hollow"] = { 0.4785, 0.3787 },
			["Shimmering Grotto"] = { 0.5095, 0.2171 },
			["Beth'mora Ridge"] = { 0.5534, 0.8301 },
		},
		[24] =  -- Silvermoon City
		{
		},
		[25] =  -- Silverpine Forest
		{
			[0] = 769,
			["The Skittering Dark"] = { 0.3478, 0.1250 },
			["Shadowfang Keep"] = { 0.4217, 0.6340 },
			["The Decrepit Fields"] = { 0.5654, 0.3481 },
			["North Tide's Beachhead"] = { 0.3738, 0.2807 },
			["Forsaken High Command"] = { 0.6208, 0.0876 },
			["Deep Elem Mine"] = { 0.5928, 0.4551 },
			["The Forsaken Front"] = { 0.5095, 0.6677 },
			["Valgan's Field"] = { 0.5319, 0.2582 },
			["The Battlefront"] = { 0.4611, 0.7964 },
			["Ambermill"] = { 0.6123, 0.6243 },
			["Fenris Isle"] = { 0.6911, 0.2687 },
			["Forsaken Rear Guard"] = { 0.4491, 0.1894 },
			["The Sepulcher"] = { 0.4496, 0.4012 },
			["North Tide's Run"] = { 0.3800, 0.3300 },
			["Olsen's Farthing"] = { 0.4606, 0.5112 },
		},
		[26] =  -- Stormwind City
		{
		},
		[27] =  -- Stranglethorn Vale
		{
		},
		[28] =  -- Swamp of Sorrows
		{
			[0] = 782,
			["The Harborage"] = { 0.2884, 0.3219 },
			["Stagalbog"] = { 0.6776, 0.7500 },
			["Misty Reed Strand"] = { 0.7730, 0.1380 },
			["Stonard"] = { 0.4700, 0.5434 },
			["Sorrowmurk"] = { 0.8418, 0.3907 },
			["Marshtide Watch"] = { 0.6836, 0.3585 },
			["The Shifting Mire"] = { 0.4880, 0.4220 },
			["Pool of Tears"] = { 0.6260, 0.5010 },
			["Bogpaddle"] = { 0.7236, 0.1302 },
			["Splinterspear Junction"] = { 0.2226, 0.4955 },
			["Purespring Cavern"] = { 0.1861, 0.6836 },
			["Misty Valley"] = { 0.1472, 0.3585 },
		},
		[29] =  -- The Cape of Stranglethorn
		{
			[0] = 4995,
			["Hardwrench Hideaway"] = { 0.3410, 0.3256 },
			["Mistvale Valley"] = { 0.5050, 0.5831 },
			["Gurubashi Arena"] = { 0.4631, 0.2268 },
			["Crystalvein Mine"] = { 0.6243, 0.2814 },
			["Ruins of Jubuwal"] = { 0.5444, 0.3039 },
			["Ruins of Aboraz"] = { 0.6218, 0.4379 },
			["The Wild Shore"] = { 0.4346, 0.8159 },
			["Booty Bay"] = { 0.4000, 0.7100 },
			["Nek'mani Wellspring"] = { 0.4306, 0.4918 },
			["Jaquero Isle"] = { 0.6078, 0.8166 },
		},
		[30] =  -- The Hinterlands
		{
			[0] = 773,
			["Quel'Danil Lodge"] = { 0.3200, 0.4800 },
			["Seradane"] = { 0.6272, 0.2433 },
			["The Overlook Cliffs"] = { 0.7266, 0.6617 },
			["Aerie Peak"] = { 0.1392, 0.4528 },
			["Jintha'Alor"] = { 0.6307, 0.7433 },
			["Shadra'Alor"] = { 0.3458, 0.7216 },
			["Shaol'watha"] = { 0.7325, 0.5434 },
			["Agol'watha"] = { 0.4711, 0.3922 },
			["Plaguemist Ravine"] = { 0.2310, 0.3360 },
			["Skulk Rock"] = { 0.5758, 0.4281 },
			["The Altar of Zul"] = { 0.4840, 0.6707 },
			["The Creeping Ruin"] = { 0.4905, 0.5217 },
			["Valorwind Lake"] = { 0.4052, 0.5943 },
			["Zun'watha"] = { 0.2346, 0.5821 },
		},
		[31] =  -- Tirisfal Glades
		{
			[0] = 768,
			["Ruins of Lordaeron"] = { 0.6200, 0.6755 },
			["Solliden Farmstead"] = { 0.3847, 0.4895 },
			["Garren's Haunt"] = { 0.5694, 0.3578 },
			["Brill"] = { 0.5973, 0.5105 },
			["Venomweb Vale"] = { 0.8703, 0.4731 },
			["Balnir Farmstead"] = { 0.7365, 0.6020 },
			["Scarlet Monastery Entrance"] = { 0.8508, 0.3134 },
			["The Bulwark"] = { 0.8488, 0.6946 },
			["Agamand Mills"] = { 0.4561, 0.3308 },
			["Brightwater Lake"] = { 0.6836, 0.4506 },
			["Nightmare Vale"] = { 0.4511, 0.6549 },
			["Cold Hearth Manor"] = { 0.5384, 0.5876 },
			["Deathknell"] = { 0.3224, 0.6392 },
			["Crusader Outpost"] = { 0.7909, 0.5487 },
			["Calston Estate"] = { 0.4621, 0.5150 },
			["Scarlet Watch Post"] = { 0.7844, 0.2702 },
		},
		[32] =  -- Tol Barad
		{
		},
		[33] =  -- Tol Barad Peninsula
		{
		},
		[34] =  -- Twilight Highlands
		{
			[0] = 4866,
			["Twilight Shore"] = { 0.7100, 0.7100 },
			["Bloodgulch"] = { 0.5404, 0.4289 },
			["Slithering Cove"] = { 0.7100, 0.4000 },
			["The Black Breach"] = { 0.5793, 0.3046 },
			["Humboldt Conflagration"] = { 0.4436, 0.2741 },
			["Victor's Point"] = { 0.4037, 0.5763 },
			["Ruins of Drakgor"] = { 0.4361, 0.1834 },
			["Kirthaven"] = { 0.5778, 0.1430 },
			["Wyrms' Bend"] = { 0.2900, 0.4800 },
			["The Gullet"] = { 0.3797, 0.3772 },
			["Obsidian Forest"] = { 0.6432, 0.7897 },
			["Gorshak War Camp"] = { 0.6262, 0.4611 },
			["Glopgut's Hollow"] = { 0.3822, 0.2919 },
			["Highbank"] = { 0.8069, 0.7702 },
			["The Krazzworks"] = { 0.7690, 0.1467 },
			["Firebeard's Patrol"] = { 0.6123, 0.5674 },
			["Dragonmaw Port"] = { 0.7605, 0.5322 },
			["Crushblow"] = { 0.4616, 0.7725 },
			["Crucible of Carnage"] = { 0.5070, 0.5696 },
			["The Maw of Madness"] = { 0.4785, 0.1078 },
			["Grim Batol"] = { 0.1881, 0.5157 },
			["Thundermar"] = { 0.5025, 0.2934 },
			["Highland Forest"] = { 0.5400, 0.6300 },
			["Dragonmaw Pass"] = { 0.2630, 0.3787 },
			["Vermillion Redoubt"] = { 0.2530, 0.2111 },
			["Dunwald Ruins"] = { 0.4960, 0.6894 },
			["The Twilight Breach"] = { 0.3997, 0.4663 },
		},
		[35] =  -- Undercity
		{
		},
		[36] =  -- Vashj'ir
		{
		},
		[37] =  -- Western Plaguelands
		{
			[0] = 770,
			["Thondroril River"] = { 0.6960, 0.5100 },
			["Northridge Lumber Camp"] = { 0.4721, 0.3353 },
			["Hearthglen"] = { 0.4491, 0.1714 },
			["Darrowmere Lake"] = { 0.6000, 0.7500 },
			["Redpine Dell"] = { 0.4592, 0.4573 },
			["The Bulwark"] = { 0.2710, 0.5778 },
			["Andorhal"] = { 0.4356, 0.6774 },
			["Caer Darrow"] = { 0.6833, 0.8117 },
			["Felstone Field"] = { 0.3643, 0.5464 },
			["The Weeping Cave"] = { 0.6437, 0.4027 },
			["Sorrow Hill"] = { 0.5461, 0.8561 },
			["Dalson's Farm"] = { 0.4516, 0.5127 },
			["Gahrron's Withering"] = { 0.6397, 0.5831 },
			["The Writhing Haunt"] = { 0.5489, 0.6639 },
		},
		[38] =  -- Westfall
		{
			[0] = 802,
			["Saldean's Farm"] = { 0.5444, 0.3241 },
			["The Jansen Stead"] = { 0.5773, 0.1587 },
			["The Raging Chasm"] = { 0.3917, 0.4341 },
			["The Dagger Hills"] = { 0.4486, 0.8226 },
			["The Dust Plains"] = { 0.6337, 0.7260 },
			["Demont's Place"] = { 0.3393, 0.7365 },
			["The Molsen Farm"] = { 0.4531, 0.3503 },
			["The Dead Acre"] = { 0.6148, 0.5928 },
			["Jangolode Mine"] = { 0.4486, 0.2395 },
			["Sentinel Hill"] = { 0.5584, 0.4940 },
			["Moonbrook"] = { 0.4237, 0.6564 },
			["Furlbrow's Pumpkin Farm"] = { 0.5105, 0.2156 },
			["Westfall Lighthouse"] = { 0.3000, 0.8700 },
			["Alexston Farmstead"] = { 0.3817, 0.5240 },
		},
		[39] =  -- Wetlands
		{
			[0] = 841,
			["Menethil Harbor"] = { 0.1023, 0.5846 },
			["Black Channel Marsh"] = { 0.1930, 0.4710 },
			["Dun Algaz"] = { 0.4995, 0.7657 },
			["Slabchisel's Survey"] = { 0.5838, 0.7096 },
			["Whelgar's Excavation Site"] = { 0.3583, 0.4903 },
			["Direforge Hill"] = { 0.6023, 0.2627 },
			["Thelgen Rock"] = { 0.5220, 0.6153 },
			["Raptor Ridge"] = { 0.6791, 0.3496 },
			["Saltspray Glen"] = { 0.3255, 0.1761 },
			["Greenwarden's Grove"] = { 0.5788, 0.4079 },
			["Bluegill Marsh"] = { 0.1899, 0.3608 },
			["Mosshide Fen"] = { 0.6188, 0.5666 },
			["Sundown Marsh"] = { 0.2446, 0.2432 },
			["Ironbeard's Tomb"] = { 0.4356, 0.2597 },
			["Angerfang Encampment"] = { 0.4855, 0.4813 },
			["Dun Modr"] = { 0.4691, 0.1662 },
		},
		[40] =  -- Zandalari Isle
		{
		},
	},
	[3] =  -- Outland
	{
		[1] =  -- Blade's Edge Mountains
		{
			[0] = 865,
			["Bash'ir Landing"] = { 0.5175, 0.1684 },
			["Sylvanaar"] = { 0.3693, 0.6549 },
			["Bladed Gulch"] = { 0.6600, 0.3700 },
			["Raven's Wood"] = { 0.3189, 0.2754 },
			["Gruul's Lair"] = { 0.6771, 0.2358 },
			["Veil Ruuan"] = { 0.6512, 0.3174 },
			["Bladespire Hold"] = { 0.4077, 0.5314 },
			["Bloodmaul Outpost"] = { 0.4631, 0.7710 },
			["Death's Door"] = { 0.6407, 0.6108 },
			["Jagged Ridge"] = { 0.4800, 0.6500 },
			["Razor Ridge"] = { 0.5600, 0.6900 },
			["Skald"] = { 0.7200, 0.2300 },
			["Crystal Spine"] = { 0.6796, 0.1093 },
			["Vortex Summit"] = { 0.3044, 0.5951 },
			["Vekhaar Stand"] = { 0.7710, 0.7530 },
			["Grishnath"] = { 0.4000, 0.2000 },
			["Forge Camp: Anger"] = { 0.7460, 0.4199 },
			["Broken Wilds"] = { 0.7800, 0.2600 },
			["Veil Lashh"] = { 0.3638, 0.7882 },
			["Thunderlord Stronghold"] = { 0.5309, 0.5472 },
			["Bloodmaul Camp"] = { 0.5554, 0.2635 },
			["Circle of Blood"] = { 0.5354, 0.4401 },
			["Ruuan Weald"] = { 0.6118, 0.3660 },
			["Forge Camp: Wrath"] = { 0.3348, 0.4117 },
			["Forge Camp: Terror"] = { 0.2909, 0.8114 },
			["Mok'Nathal Village"] = { 0.7385, 0.6272 },
		},
		[2] =  -- Hellfire Peninsula
		{
			[0] = 862,
			["The Warp Fields"] = { 0.4531, 0.8263 },
			["Temple of Telhamat"] = { 0.2355, 0.4012 },
			["Void Ridge"] = { 0.7894, 0.7305 },
			["Pools of Aggonar"] = { 0.4117, 0.3278 },
			["The Legion Front"] = { 0.7006, 0.5045 },
			["Fallen Sky Ridge"] = { 0.1422, 0.4109 },
			["Ruins of Sha'naar"] = { 0.1332, 0.5951 },
			["Forge Camp: Mageddon"] = { 0.5893, 0.3144 },
			["Expedition Armory"] = { 0.5429, 0.8443 },
			["The Stair of Destiny"] = { 0.8777, 0.5052 },
			["Den of Haal'esh"] = { 0.2745, 0.7710 },
			["Falcon Watch"] = { 0.2690, 0.6138 },
			["Zeth'Gor"] = { 0.7011, 0.7365 },
			["Hellfire Citadel"] = { 0.4721, 0.5299 },
			["Thrallmar"] = { 0.5534, 0.3885 },
			["Mag'har Post"] = { 0.3134, 0.2710 },
			["Honor Hold"] = { 0.5484, 0.6467 },
			["Throne of Kil'jaeden"] = { 0.6203, 0.1796 },
		},
		[3] =  -- Nagrand
		{
			[0] = 866,
			["Garadar"] = { 0.5614, 0.3645 },
			["Windyreed Village"] = { 0.7420, 0.5299 },
			["Kil'sorrow Fortress"] = { 0.6890, 0.7994 },
			["Burning Blade Ruins"] = { 0.7510, 0.6692 },
			["Forge Camp: Fear"] = { 0.2061, 0.5150 },
			["Forge Camp: Hate"] = { 0.2595, 0.3700 },
			["Telaar"] = { 0.5279, 0.6930 },
			["The Ring of Trials"] = { 0.6582, 0.5412 },
			["Clan Watch"] = { 0.6243, 0.6452 },
			["Windyreed Pass"] = { 0.7236, 0.3645 },
			["Laughing Skull Ruins"] = { 0.4691, 0.1909 },
			["Spirit Fields"] = { 0.4200, 0.7200 },
			["Throne of the Elements"] = { 0.6083, 0.2118 },
			["The Twilight Ridge"] = { 0.1078, 0.4000 },
			["Zangar Ridge"] = { 0.3388, 0.1849 },
			["Warmaul Hill"] = { 0.2964, 0.2463 },
			["Southwind Cleft"] = { 0.4980, 0.5591 },
			["Sunspring Post"] = { 0.3348, 0.4484 },
			["Halaa"] = { 0.4250, 0.4409 },
		},
		[4] =  -- Netherstorm
		{
			[0] = 843,
			["Sunfury Hold"] = { 0.5594, 0.7792 },
			["Ruins of Enkaat"] = { 0.3298, 0.5502 },
			["Area 52"] = { 0.3353, 0.6602 },
			["Celestial Ridge"] = { 0.7250, 0.3885 },
			["Eco-Dome Farfield"] = { 0.4606, 0.0973 },
			["Eco-Dome Midrealm"] = { 0.4546, 0.5367 },
			["The Stormspire"] = { 0.4506, 0.3593 },
			["Manaforge B'naar"] = { 0.2355, 0.7305 },
			["Manaforge Ara"] = { 0.2685, 0.4000 },
			["Manaforge Coruu"] = { 0.4870, 0.8458 },
			["Arklon Ruins"] = { 0.3977, 0.7358 },
			["Socrethar's Seat"] = { 0.2994, 0.1602 },
			["The Heap"] = { 0.2839, 0.7762 },
			["Kirin'Var Village"] = { 0.5704, 0.8855 },
			["Ruins of Farahlon"] = { 0.5444, 0.2148 },
			["Forge Base: Oblivion"] = { 0.3752, 0.2650 },
			["Ethereum Staging Grounds"] = { 0.5509, 0.4364 },
			["Tempest Keep"] = { 0.7570, 0.6250 },
			["Manaforge Duro"] = { 0.5950, 0.6800 },
			["Manaforge Ultris"] = { 0.6163, 0.3832 },
			["Gyro-Plank Bridge"] = { 0.2315, 0.5606 },
			["Netherstone"] = { 0.4955, 0.1707 },
		},
		[5] =  -- Shadowmoon Valley
		{
			[0] = 864,
			["Legion Hold"] = { 0.2405, 0.4000 },
			["The Deathforge"] = { 0.4000, 0.4000 },
			["Coilskar Point"] = { 0.4530, 0.2635 },
			["The Hand of Gul'dan"] = { 0.4980, 0.4132 },
			["Illidari Point"] = { 0.2984, 0.5240 },
			["Netherwing Fields"] = { 0.6400, 0.5860 },
			["Eclipse Point"] = { 0.4491, 0.6692 },
			["Altar of Sha'tar"] = { 0.6143, 0.2620 },
			["Netherwing Ledge"] = { 0.7080, 0.8585 },
			["Wildhammer Stronghold"] = { 0.3548, 0.6033 },
			["Shadowmoon Village"] = { 0.2934, 0.2620 },
			["Warden's Cage"] = { 0.5734, 0.5022 },
			["The Black Temple"] = { 0.7764, 0.4080 },
		},
		[6] =  -- Shattrath City
		{
		},
		[7] =  -- Terokkar Forest
		{
			[0] = 867,
			["Raastok Glade"] = { 0.6063, 0.4027 },
			["Refugee Caravan"] = { 0.3752, 0.4970 },
			["Razorthorn Shelf"] = { 0.5998, 0.1707 },
			["Cenarion Thicket"] = { 0.4381, 0.2036 },
			["Bleeding Hollow Ruins"] = { 0.1786, 0.6549 },
			["Bonechewer Ruins"] = { 0.6652, 0.5232 },
			["Stonebreaker Hold"] = { 0.4666, 0.4326 },
			["Firewing Point"] = { 0.7171, 0.3481 },
			["Carrion Hill"] = { 0.4300, 0.5100 },
			["Tuurem"] = { 0.5299, 0.2934 },
			["Writhing Mound"] = { 0.4900, 0.6600 },
			["Auchenai Grounds"] = { 0.3300, 0.7200 },
			["Ring of Observance"] = { 0.3957, 0.6572 },
			["Grangol'var Village"] = { 0.3952, 0.3952 },
			["Veil Rhaze"] = { 0.2440, 0.5990 },
			["Skettis"] = { 0.5893, 0.7642 },
			["Shattrath City"] = { 0.2914, 0.2343 },
			["Shadow Tomb"] = { 0.3129, 0.5254 },
			["Derelict Caravan"] = { 0.4390, 0.7610 },
			["Allerian Stronghold"] = { 0.5700, 0.5700 },
			["The Barrier Hills"] = { 0.2236, 0.1063 },
		},
		[8] =  -- Zangarmarsh
		{
			[0] = 863,
			["Cenarion Refuge"] = { 0.8109, 0.6340 },
			["The Dead Mire"] = { 0.8263, 0.3765 },
			["Darkcrest Shore"] = { 0.7066, 0.8000 },
			["Twin Spire Ruins"] = { 0.4780, 0.5180 },
			["Telredor"] = { 0.6850, 0.4933 },
			["Umbrafen Village"] = { 0.8348, 0.8211 },
			["Bloodscale Grounds"] = { 0.6262, 0.4124 },
			["Quagg Ridge"] = { 0.2899, 0.6093 },
			["Orebor Harborage"] = { 0.4486, 0.2537 },
			["Sporeggar"] = { 0.1780, 0.4970 },
			["Feralfen Village"] = { 0.4476, 0.6639 },
			["Hewn Bog"] = { 0.3353, 0.3548 },
			["Marshlight Lake"] = { 0.2171, 0.3975 },
			["Ango'rosh Stronghold"] = { 0.1861, 0.0823 },
			["Ango'rosh Grounds"] = { 0.1801, 0.2320 },
			["The Spawning Glen"] = { 0.1447, 0.6213 },
			["Zabra'jin"] = { 0.3179, 0.5247 },
			["The Lagoon"] = { 0.5858, 0.6198 },
		},
	},
	[4] =  -- Northrend
	{
		[1] =  -- Borean Tundra
		{
			[0] = 1264,
			["Riplash Strand"] = { 0.4746, 0.8121 },
			["The Geyser Fields"] = { 0.6632, 0.2867 },
			["Bor'gorok Outpost"] = { 0.5020, 0.1055 },
			["Amber Ledge"] = { 0.4611, 0.3496 },
			["Death's Stand"] = { 0.8248, 0.4656 },
			["Warsong Hold"] = { 0.4341, 0.5329 },
			["The Dens of the Dying"] = { 0.7635, 0.1654 },
			["Valiance Keep"] = { 0.5788, 0.6939 },
			["Steeljaw's Caravan"] = { 0.5015, 0.2440 },
			["Temple City of En'kilah"] = { 0.8757, 0.2522 },
			["Kaskala"] = { 0.6622, 0.5120 },
			["Garrosh's Landing"] = { 0.2989, 0.5180 },
			["Coldarra"] = { 0.2575, 0.3855 },
		},
		[2] =  -- Crystalsong Forest
		{
			[0] = 1457,
			["The Azure Front"] = { 0.2350, 0.5760 },
			["The Decrepit Flow"] = { 0.1820, 0.1560 },
			["Sunreaver's Command"] = { 0.7690, 0.4830 },
			["Forlorn Woods"] = { 0.4710, 0.4460 },
			["Windrunner's Overlook"] = { 0.7470, 0.8040 },
			["The Great Tree"] = { 0.1420, 0.3420 },
			["Violet Stand"] = { 0.1570, 0.4240 },
			["The Unbound Thicket"] = { 0.6080, 0.6180 },
		},
		[3] =  -- Dalaran
		{
		},
		[4] =  -- Dragonblight
		{
			[0] = 1265,
			["Light's Trust"] = { 0.8358, 0.2575 },
			["Emerald Dragonshrine"] = { 0.6347, 0.7283 },
			["Westwind Refugee Camp"] = { 0.1387, 0.4626 },
			["Angrathar the Wrath Gate"] = { 0.3603, 0.1512 },
			["Coldwind Heights"] = { 0.5005, 0.1766 },
			["Naxxramas"] = { 0.8887, 0.4424 },
			["Agmar's Hammer"] = { 0.3688, 0.4783 },
			["Obsidian Dragonshrine"] = { 0.3847, 0.3211 },
			["The Forgotten Shore"] = { 0.8248, 0.7081 },
			["Galakrond's Rest"] = { 0.5484, 0.3286 },
			["Venomspite"] = { 0.7710, 0.6003 },
			["New Hearthglen"] = { 0.7106, 0.7522 },
			["Wyrmrest Temple"] = { 0.5973, 0.4985 },
			["The Crystal Vice"] = { 0.6200, 0.2000 },
			["Icemist Village"] = { 0.2495, 0.3930 },
			["Lake Indu'le"] = { 0.3967, 0.6714 },
			["Scarlet Point"] = { 0.7265, 0.2575 },
			["Ruby Dragonshrine"] = { 0.4736, 0.4873 },
		},
		[5] =  -- Grizzly Hills
		{
			[0] = 1266,
			["Camp Oneqwah"] = { 0.6497, 0.4566 },
			["Drak'Tharon Keep"] = { 0.1752, 0.2358 },
			["Voldrune"] = { 0.2894, 0.7657 },
			["Dun Argol"] = { 0.7794, 0.5936 },
			["Conquest Hold"] = { 0.2131, 0.6542 },
			["Drakil'jin Ruins"] = { 0.7176, 0.2799 },
			["Amberpine Lodge"] = { 0.3134, 0.5801 },
			["Rage Fang Shrine"] = { 0.5025, 0.5726 },
			["Granite Springs"] = { 0.1587, 0.4955 },
			["Venture Bay"] = { 0.1447, 0.8451 },
			["Grizzlemaw"] = { 0.4800, 0.4237 },
			["Thor Modan"] = { 0.6800, 0.1490 },
			["Westfall Brigade Encampment"] = { 0.5674, 0.2852 },
			["Blue Sky Logging Grounds"] = { 0.3603, 0.3802 },
		},
		[6] =  -- Howling Fjord
		{
			[0] = 1263,
			["The Twisted Glade"] = { 0.5419, 0.1699 },
			["Camp Winterhoof"] = { 0.4780, 0.1212 },
			["Apothecary Camp"] = { 0.2645, 0.2148 },
			["Cauldros Isle"] = { 0.5773, 0.3600 },
			["Nifflevar"] = { 0.6786, 0.5292 },
			["Baelgun's Excavation Site"] = { 0.7250, 0.7140 },
			["Utgarde Keep"] = { 0.5828, 0.4603 },
			["Vengeance Landing"] = { 0.7809, 0.2747 },
			["Westguard Keep"] = { 0.3024, 0.4162 },
			["Giant's Run"] = { 0.6856, 0.2672 },
			["Baleheim"] = { 0.6657, 0.3922 },
			["Kamagua"] = { 0.2485, 0.5786 },
			["Ember Clutch"] = { 0.3987, 0.5082 },
			["Ivald's Ruin"] = { 0.7819, 0.4633 },
			["Halgrind"] = { 0.5000, 0.5419 },
			["Gjalerbron"] = { 0.3663, 0.0951 },
			["Skorn"] = { 0.4511, 0.3331 },
			["Fort Wildervar"] = { 0.6068, 0.1295 },
			["Scalawag Point"] = { 0.3438, 0.7852 },
			["Steel Gate"] = { 0.3134, 0.2605 },
			["New Agamand"] = { 0.5279, 0.6916 },
		},
		[7] =  -- Hrothgar's Landing
		{
		},
		[8] =  -- Icecrown
		{
			[0] = 1270,
			["Jotunheim"] = { 0.3000, 0.3600 },
			["Valhalas"] = { 0.3643, 0.2425 },
			["Valley of Echoes"] = { 0.7949, 0.6924 },
			["Icecrown Citadel"] = { 0.5414, 0.9000 },
			["Sindragosa's Fall"] = { 0.7000, 0.3600 },
			["Onslaught Harbor"] = { 0.0893, 0.4296 },
			["The Broken Front"] = { 0.6921, 0.6452 },
			["The Bombardment"] = { 0.6367, 0.4416 },
			["The Shadow Vault"] = { 0.4436, 0.2260 },
			["The Conflagration"] = { 0.4366, 0.5659 },
			["Scourgeholme"] = { 0.7834, 0.6033 },
			["Ymirheim"] = { 0.5484, 0.5644 },
			["The Fleshwerks"] = { 0.3313, 0.6647 },
			["Corp'rethar: The Horror Gate"] = { 0.4835, 0.7073 },
			["Aldur'thar: The Desolation Gate"] = { 0.5284, 0.3076 },
		},
		[9] =  -- Sholazar Basin
		{
			[0] = 1268,
			["The Savage Thicket"] = { 0.4680, 0.2627 },
			["Kartak's Hold"] = { 0.2420, 0.8200 },
			["The Glimmering Pillar"] = { 0.4950, 0.3638 },
			["The Mosslight Pillar"] = { 0.3598, 0.7537 },
			["The Suntouched Pillar"] = { 0.3323, 0.5232 },
			["The Stormwright's Shelf"] = { 0.2445, 0.3361 },
			["The Lifeblood Pillar"] = { 0.6587, 0.6020 },
			["Makers' Perch"] = { 0.2964, 0.4012 },
			["Makers' Overlook"] = { 0.8094, 0.5554 },
			["The Avalanche"] = { 0.7450, 0.3466 },
			["River's Heart"] = { 0.4800, 0.6340 },
			["Rainspeaker Canopy"] = { 0.5369, 0.5262 },
		},
		[10] =  -- The Storm Peaks
		{
			[0] = 1269,
			["Frosthold"] = { 0.2900, 0.7400 },
			["Garm's Bane"] = { 0.4710, 0.8200 },
			["Ulduar"] = { 0.4132, 0.1752 },
			["Thunderfall"] = { 0.7280, 0.4800 },
			["Terrace of the Makers"] = { 0.5040, 0.4491 },
			["Bor's Breath"] = { 0.3533, 0.6894 },
			["Dun Niffelem"] = { 0.6427, 0.5928 },
			["Nidavelir"] = { 0.2400, 0.5022 },
			["Sparksocket Minefield"] = { 0.3512, 0.8421 },
			["Brunnhildar Village"] = { 0.4785, 0.6834 },
			["Engine of the Makers"] = { 0.4087, 0.5741 },
			["Narvir's Cradle"] = { 0.3224, 0.3967 },
			["Temple of Life"] = { 0.6472, 0.4484 },
			["Valkyrion"] = { 0.2400, 0.6063 },
			["Temple of Storms"] = { 0.3283, 0.5808 },
			["Snowdrift Plains"] = { 0.2735, 0.4207 },
		},
		[11] =  -- Wintergrasp
		{
		},
		[12] =  -- Zul'Drak
		{
			[0] = 1267,
			["Gundrak"] = { 0.8400, 0.1700 },
			["Light's Breach"] = { 0.3199, 0.7657 },
			["Ampitheater of Anguish"] = { 0.4800, 0.5651 },
			["Altar of Sseratus"] = { 0.4042, 0.3900 },
			["Altar of Rhunok"] = { 0.5334, 0.3668 },
			["Altar of Har'koa"] = { 0.6447, 0.6901 },
			["Altar of Mam'toth"] = { 0.7325, 0.4573 },
			["Altar of Quetz'lun"] = { 0.7690, 0.5943 },
			["Drak'Sotra Fields"] = { 0.4177, 0.7702 },
			["Voltarus"] = { 0.2819, 0.4528 },
			["Zeramas"] = { 0.2116, 0.7537 },
			["Kolramas"] = { 0.6203, 0.7769 },
			["Zim'Torga"] = { 0.5938, 0.5793 },
			["Thrym's End"] = { 0.1687, 0.5741 },
		},
	},
	[5] =  -- Maelstrom
	{
		[1] =  -- Deepholm
		{
			[0] = 4864,
			["Stonehearth"] = { 0.2625, 0.6969 },
			["Twilight Overlook"] = { 0.6830, 0.7792 },
			["Needlerock Slag"] = { 0.2176, 0.4775 },
			["Masters' Gate"] = { 0.3917, 0.6924 },
			["Deathwing's Fall"] = { 0.6282, 0.5853 },
			["The Pale Roost"] = { 0.3982, 0.1871 },
			["Storm's Fury Wreckage"] = { 0.5574, 0.7560 },
			["Temple of Earth"] = { 0.4915, 0.4775 },
			["Needlerock Chasm"] = { 0.2600, 0.3300 },
			["The Quaking Fields"] = { 0.3528, 0.8040 },
			["Therazane's Throne"] = { 0.5654, 0.1100 },
			["Crimson Expanse"] = { 0.7430, 0.4087 },
		},
		[2] =  -- Kezan
		{
		},
		[3] =  -- The Lost Isles
		{
		},
		[4] =  -- The Maelstrom
		{
		},
	},
}

function Exploration:OnInitialize()
	self.db = Mapster.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	self:SetEnabledState(Mapster:GetModuleEnabled(MODNAME))
	Mapster:RegisterModuleOptions(MODNAME, getOptions, L["Exploration"])
end

function Exploration:OnEnable()
	self:RegisterEvent("WORLD_MAP_UPDATE")
	self:RegisterEvent("CRITERIA_UPDATE")
	Notes = Mapster:GetModule("Notes")
	if Notes and Notes:IsEnabled() then
		Notes:RegisterNotesDatabase("Exploration", self.db.char.pois, self)
		for k, v in pairs(icons) do
			Notes:RegisterIcon("POI_Exploration_"..k, {
				text = v.text,
				path = v.path,
			})
		end
	else
		Mapster:SetModuleEnabled(MODNAME, false)
	end
	self:RefreshMap()
end

function Exploration:OnDisable()
	self:UnregisterEvent("WORLD_MAP_UPDATE")
	self:UnregisterEvent("CRITERIA_UPDATE")
	if Notes and Notes.externalDBs["Exploration"] then
		Notes:UnregisterIcon("POI_Exploration_NotExplored")
		Notes:UnregisterIcon("POI_Exploration_Explored")
		Notes:UnregisterNotesDatabase("Exploration")
	end
end

function Exploration:Refresh()
	db = self.db.profile
	Notes = Mapster:GetModule("Notes")
end

function Exploration:WORLD_MAP_UPDATE()
	self:AddExplorationNotes()
end

function Exploration:CRITERIA_UPDATE()
	wipe(self.db.char.pois)
	self:AddExplorationNotes()
	Notes:RefreshMap()
end

function Exploration:RefreshMap()
	wipe(self.db.char.pois)
	self:AddExplorationNotes()
	Notes:RefreshMap()
end

function Exploration:AddExplorationNotes()
	if not self:IsEnabled() or not Notes:IsEnabled() then
		return
	end
	local continentIndex, zoneIndex = GetCurrentMapContinent(), GetCurrentMapZone()

	if (continentIndex < 1) or (zoneIndex == 0) then
		return
	end

	local zone = Notes:GetCurrentMapTexture()
	if zone then
		if db.exploration then
			local catID = POI_Exploration[continentIndex][zoneIndex][0]
			if not catID then return end
			local numCriteria = GetAchievementNumCriteria(catID)
			for index = 1, numCriteria do
				local criteria, _, completed = GetAchievementCriteriaInfo(catID, index)
				if (not completed) or (db.showExplored) then
					local value = POI_Exploration[continentIndex][zoneIndex][criteria]
					if (value and value[1] and value[2] and value[1] ~= 0 and value[2] ~= 0) then
						if completed then
							Notes:SetNote(zone, value[1], value[2], "POI_Exploration_Explored",
								"Exploration",
								"title", criteria,
								"titleR", 0.0,
								"titleG", 0.5,
								"titleB", 1.0
							)
						else
							Notes:SetNote(zone, value[1], value[2], "POI_Exploration_NotExplored",
								"Exploration",
								"title", criteria,
								"titleR", 0.0,
								"titleG", 0.5,
								"titleB", 1.0
							)
						end
					end
				end
			end
		end
	end
end
