--[[
MapsterEnhanced - Instance Maps (Sindragosa)

Based on Mapster
Copyright (c) 2009, Hendrik "Nevcairiel" Leppkes < h.leppkes@gmail.com >
All rights reserved.
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local L = LibStub("AceLocale-3.0"):GetLocale("Mapster")

local MODNAME = "InstanceMaps"
local Maps = Mapster:GetModule(MODNAME)
local Maps = LibStub("AceEvent-3.0"):Embed(Maps)

local MapsterEnhanced = LibStub("AceAddon-3.0"):GetAddon("MapsterEnhanced")
local MapsterEnhanced_Maps = MapsterEnhanced:NewModule(MODNAME, "AceHook-3.0")

local LBZ = LibStub("LibBabble-Zone-3.0", true)
local BZ = LBZ and LBZ:GetUnstrictLookupTable() or setmetatable({}, {__index = function(t,k) return k end})

local Tourist = LibStub("LibTourist-3.0-Enhanced")
local Dewdrop = AceLibrary("Dewdrop-2.0")

-- Data mostly from http://www.wowwiki.com/API_SetMapByID
local data = {
	-- Northrend Instances
	instances = {
		classic = {
			["Ragefire Chasm"] = 680,
			["Zul'Farrak"] = 686,
			["The Temple of Atal'Hakkar"] = 687,
			["Blackfathom Deeps"] = 688,
			["The Stockade"] = 690,
			["Gnomeregan"] = 691,
			["Uldaman"] = 692,
			["Dire Maul"] = 699,
			["Blackrock Depths"] = 704,
			["Blackrock Spire"] = 721,
			["Wailing Caverns"] = 749,
			["Maraudon"] = 750,
			["The Deadmines"] = 756,
			["Razorfen Downs"] = 760,
			["Razorfen Kraul"] = 761,
			["Scarlet Monastery"] = 762,
			["Scholomance"] = 763,
			["Shadowfang Keep"] = 764,
			["Stratholme"] = 765,
		},
		bc = {
			["The Shattered Halls"] = 710,
			["Auchenai Crypts"] = 722,
			["Sethekk Halls"] = 723,
			["Shadow Labyrinth"] = 724,
			["The Blood Furnace"] = 725,
			["The Underbog"] = 726,
			["The Steamvault"] = 727,
			["The Slave Pens"] = 728,
			["The Botanica"] = 729,
			["The Mechanar"] = 730,
			["The Arcatraz"] = 731,
			["Mana-Tombs"] = 732,
			["The Black Morass"] = 733,
			["Old Hillsbrad Foothills"] = 734,
			["Hellfire Ramparts"] = 797,
			["Magisters' Terrace"] = 798,
		},
		wrath = {
			["The Nexus"] = 520,
			["The Culling of Stratholme"] = 521,
			["Ahn'kahet: The Old Kingdom"] = 522,
			["Utgarde Keep"] = 523,
			["Utgarde Pinnacle"] = 524,
			["Halls of Lightning"] = 525,
			["Halls of Stone"] = 526,
			["The Oculus"] = 528,
			["Gundrak"] = 530,
			["Azjol-Nerub"] = 533,
			["Drak'Tharon Keep"] = 534,
			["The Violet Hold"] = 536,
			["Trial of the Champion"] = 542,
			["The Forge of Souls"] = 601,
			["Pit of Saron"] = 602,
			["Halls of Reflection"] = 603,
		},
		cataclysm = {
			["Lost City of the Tol'vir"] = 747,
			["Blackrock Caverns"] = 753,
			["The Deadmines"] = 756,
			["Grim Batol"] = 757,
			["Halls of Origination"] = 759,
			["Shadowfang Keep"] = 764,
			["Throne of the Tides"] = 767,
			["The Stonecore"] = 768,
			["The Vortex Pinnacle"] = 769,
			["Zul'Aman"] = 781,
			["Zul'Gurub"] = 793,
			["End Time"] = 820,
			["Well of Eternity"] = 816,
			["Hour of Twilight"] = 819,
		},
	},

	-- Northrend Raids
	raids = {
		classic = {
			["Molten Core"] = 696,
			["Blackwing Lair"] = 755,
			["Ruins of Ahn'Qiraj"] = 717,
			["Temple of Ahn'Qiraj"] = 766,
		},
		bc = {
			["Hyjal Summit"] = 775,
			["Gruul's Lair"] = 776,
			["Magtheridon's Lair"] = 779,
			["Serpentshrine Cavern"] = 780,
			["The Eye"] = 782,
			["Sunwell Plateau"] = 789,
			["Black Temple"] = 796,
			["Karazhan"] = 799,
		},
		wrath = {
			["The Eye of Eternity"] = 527,
			["Ulduar"] = 529,
			["The Obsidian Sanctum"] = 531,
			["Vault of Archavon"] = 532,
			["Naxxramas"] = 535,
			["Trial of the Crusader"] = 543,
			["Icecrown Citadel"] = 604,
			["The Ruby Sanctum"] = 609,
			["Onyxia's Lair"] = 718,
		},
		cataclysm = {
			["Baradin Hold"] = 752,
			["Blackwing Descent"] = 754,
			["The Bastion of Twilight"] = 758,
			["Throne of the Four Winds"] = 773,
			["Firelands"] = 800,
			["Dragon Soul"] = 824,
		},
	},
	bgrounds = {
		all = {
			["Alterac Valley"] = 401,
			["Warsong Gulch"] = 443,
			["Arathi Basin"] = 461,
			["Eye of the Storm"] = 482,
			["Strand of the Ancients"] = 512,
			["Isle of Conquest"] = 540,
			["The Battle for Gilneas"] = 736,
			["Twin Peaks"] = 626,
		},
	},
	other = {
		all = {
			["Molten Front"] = 795,
			["The Nexus Legendary"] = 803,
			["Darkmoon Island"] = 823,
		},
	},
}

local db
local defaults = {
	profile = {
		mapSelectButton = false,
	}
}

local optGetter, optSetter
do
	local mod = Maps
	function optGetter(info)
		local key = info[#info]
		return db[key]
	end

	function optSetter(info, value)
		local key = info[#info]
		db[key] = value
		mod:Refresh()
	end
end

local options
local function ModifyModuleOptions()
	local modOptions = LibStub("AceConfigRegistry-3.0"):GetOptionsTable("Mapster", "dialog", "MapsterEnhanced-1.54").args[MODNAME] --Mapster.moduleOptions[MODNAME]
	local options = {
		mapSelectButton = {
			order = 4,
			type = "toggle",
			name = L["Zone Select Button"],
			get = optGetter,
			set = optSetter,
		},
	}
	for k,v in pairs(defaults.profile) do
		if db[k] == nil then
			db[k] = defaults.profile[k]
		end
	end
	for k,v in pairs(options) do
		modOptions.args[k] = v
	end
end

local CreateMapSelectButton
local enteredWorld

function MapsterEnhanced_Maps:OnInitialize()
	Maps.db = Mapster.db:RegisterNamespace(MODNAME, defaults)
	db = Maps.db.profile

	ModifyModuleOptions()

	Maps.zone_names = {}
	Maps.zone_data = {}

	for category, catdata in pairs(data) do
		for subcat, subdata in pairs(catdata) do
			local name_data = {}
			local key = format("%s:%s", category, subcat)
			Maps.zone_names[key], Maps.zone_data[key] = {}, {}
			for name, id in pairs(subdata) do
				tinsert(Maps.zone_names[key], BZ[name] or name)
				name_data[BZ[name] or name] = id
			end
			table.sort(Maps.zone_names[key])
			for index, name in pairs(Maps.zone_names[key]) do
				Maps.zone_data[key][index] = name_data[name]
			end
		end
	end

	data = nil
	Maps:RegisterEvent("PLAYER_ENTERING_WORLD")

	self:SecureHook(Maps, "OnEnable", function() MapsterEnhanced_Maps:Maps_OnEnable() end)
	self:SecureHook(Maps, "OnDisable", function() MapsterEnhanced_Maps:Maps_OnDisable() end)
end

function MapsterEnhanced_Maps:Maps_OnEnable()
	if db.mapSelectButton then
		WorldMapContinentDropDown:Hide()
		WorldMapZoneDropDown:Hide()
		CreateMapSelectButton()
		WorldMapFrame:HookScript("OnHide", function()
			if Maps.mapSelectButton then
				if Dewdrop:IsOpen(Maps.mapSelectButton) then
					Dewdrop:Close()
				end
			end
		end)
		Maps:RegisterEvent("WORLD_MAP_UPDATE", "WorldMapUpdate")
		Maps:RegisterEvent("PLAYER_LEVEL_UP")
		Maps:PLAYER_LEVEL_UP()
	end

	Maps.miniMap = Mapster.miniMap
	Maps:UpdateMapsize(Maps.miniMap)
end

function MapsterEnhanced_Maps:Maps_OnDisable()
	if Maps.mapSelectButton then
		Maps.mapSelectButton:Hide()
		if (not Mapster.miniMap) then
			WorldMapContinentDropDown:Show()
			WorldMapZoneDropDown:Show()
		end
		WorldMapFrame:HookScript("OnHide", function() end)
		Maps:UnregisterEvent("WORLD_MAP_UPDATE")
		Maps:UnregisterEvent("PLAYER_LEVEL_UP")
	end
end

local function DewdropClose()
	if Maps.mapSelectButton then
		if Dewdrop:IsOpen(Maps.mapSelectButton) then
			Dewdrop:Close()
		end
	end
end

local function DoNothing() end

function Maps:Refresh()
	db = self.db.profile
	if not self:IsEnabled() then return end

	if db.mapSelectButton then
		WorldMapContinentDropDown:Hide()
		WorldMapZoneDropDown:Hide()
		CreateMapSelectButton()
		WorldMapFrame:HookScript("OnHide", DewdropClose)
		self:RegisterEvent("WORLD_MAP_UPDATE", "WorldMapUpdate")
		self:RegisterEvent("PLAYER_LEVEL_UP")
		self:PLAYER_LEVEL_UP()
	elseif self.mapSelectButton then
		self.mapSelectButton:Hide()
		if (not Mapster.miniMap) then
			WorldMapContinentDropDown:Show()
			WorldMapZoneDropDown:Show()
		end
		WorldMapFrame:HookScript("OnHide", DoNothing)
		self:UnregisterEvent("WORLD_MAP_UPDATE")
		self:UnregisterEvent("PLAYER_LEVEL_UP")
	end

	self.miniMap = Mapster.miniMap
	self:UpdateMapsize(self.miniMap)
end

function Maps:WorldMapContinentsDropDown_Update()
	local contId = UIDropDownMenu_GetSelectedID(WorldMapContinentDropDown) or self.mapContId
	if not contId or contId <= select("#", GetMapContinents()) then
		self.mapCont, self.mapContId, self.mapZone = nil, nil, nil
		UIDropDownMenu_ClearAll(WorldMapZoneDropDown)
	elseif self.mapCont then
		UIDropDownMenu_SetSelectedID(WorldMapContinentDropDown, self.mapContId)
	end
end

local function MapsterContinentButton_OnClick(frame)
	UIDropDownMenu_SetSelectedID(WorldMapContinentDropDown, frame:GetID())
	if Maps.mapContId ~= frame:GetID() then
		UIDropDownMenu_ClearAll(WorldMapZoneDropDown)
	end
	Maps.mapCont = frame.arg1
	Maps.mapContId = frame:GetID()
	Maps.mapZone = nil
end

function Maps:WorldMapFrame_LoadContinents()
	local info = UIDropDownMenu_CreateInfo()
	info.text =  L["Other Zones"]
	info.func = MapsterContinentButton_OnClick
	info.checked = nil
	info.arg1 = "other:all"
	UIDropDownMenu_AddButton(info)

	info.text =  L["Classic Instances"]
	info.func = MapsterContinentButton_OnClick
	info.checked = nil
	info.arg1 = "instances:classic"
	UIDropDownMenu_AddButton(info)

	info.text =  L["Classic Raids"]
	info.func = MapsterContinentButton_OnClick
	info.checked = nil
	info.arg1 = "raids:classic"
	UIDropDownMenu_AddButton(info)

	info.text =  L["Burning Crusade Instances"]
	info.func = MapsterContinentButton_OnClick
	info.checked = nil
	info.arg1 = "instances:bc"
	UIDropDownMenu_AddButton(info)

	info.text =  L["Burning Crusade Raids"]
	info.func = MapsterContinentButton_OnClick
	info.checked = nil
	info.arg1 = "raids:bc"
	UIDropDownMenu_AddButton(info)

	info.text =  L["Wrath Instances"]
	info.func = MapsterContinentButton_OnClick
	info.checked = nil
	info.arg1 = "instances:wrath"
	UIDropDownMenu_AddButton(info)

	info.text =  L["Wrath Raids"]
	info.func = MapsterContinentButton_OnClick
	info.checked = nil
	info.arg1 = "raids:wrath"
	UIDropDownMenu_AddButton(info)

	info.text =  L["Cataclysm Instances"]
	info.func = MapsterContinentButton_OnClick
	info.checked = nil
	info.arg1 = "instances:cataclysm"
	UIDropDownMenu_AddButton(info)

	info.text =  L["Cataclysm Raids"]
	info.func = MapsterContinentButton_OnClick
	info.checked = nil
	info.arg1 = "raids:cataclysm"
	UIDropDownMenu_AddButton(info)

	info.text =  L["Battlegrounds"]
	info.func = MapsterContinentButton_OnClick
	info.checked = nil
	info.arg1 = "bgrounds:all"
	UIDropDownMenu_AddButton(info)
end

function Maps:WorldMapZoneDropDown_Update()
	if self.mapZone then
		UIDropDownMenu_SetSelectedID(WorldMapZoneDropDown, self.mapZone)
	end
end

local function MapsterZoneButton_OnClick(frame)
	UIDropDownMenu_SetSelectedID(WorldMapZoneDropDown, frame:GetID())
	Maps.mapZone = frame:GetID()
	SetMapByID(Maps:GetZoneData())
end

local defaultZoneCache = setmetatable({}, {__index = function(t, k)
	rawset(t, k, {GetMapZones(k)})
	return t[k]
end})

do
	local function Mapster_LoadCustomZones(data)
		local info = UIDropDownMenu_CreateInfo()
		for i=1, #data, 1 do
			info.text = data[i]
			info.func = MapsterZoneButton_OnClick
			info.checked = nil
			UIDropDownMenu_AddButton(info)
		end
	end

	local function Mapster_LoadDefaultZones(data)
		local info = UIDropDownMenu_CreateInfo()
		for i=1, #data, 1 do
			info.text = data[i]
			info.func = WorldMapZoneButton_OnClick
			info.checked = nil
			UIDropDownMenu_AddButton(info)
		end
	end

	function Maps:WorldMapZoneDropDown_Initialize()
		if self.mapCont then
			Mapster_LoadCustomZones(self.zone_names[self.mapCont])
		else
			Mapster_LoadDefaultZones(defaultZoneCache[GetCurrentMapContinent()])
		end
	end
end

function Maps:SetMapZoom()
	self.mapCont, self.mapContId, self.mapZone = nil, nil, nil
end

function Maps:UpdateMapsize(mini)
	-- map was minimized, fix display position
	if self.mapSelectButton then
		if mini then
			self.mapSelectButton:ClearAllPoints()
			self.mapSelectButton:SetPoint("TOPLEFT", WorldMapFrameMiniBorderLeft, "TOPLEFT", 4, -2)
			self.mapSelectButton:SetHeight(18)
		else
			self.mapSelectButton:ClearAllPoints()
			self.mapSelectButton:SetPoint("RIGHT", WorldMapZoomOutButton, "LEFT", -6, 0)
			self.mapSelectButton:SetHeight(22)
			if db.mapSelectButton then
				WorldMapZoneDropDown:Hide()
				WorldMapContinentDropDown:Hide()
			end
		end
	end
end

function Maps:WorldMapUpdate()
	if db.mapSelectButton and self.mapSelectButton and enteredWorld then
		local map = GetMapInfo()
		if not map then
			if GetCurrentMapContinent() == 0 then
				self.mapSelectButton:SetText("Azeroth")
			elseif GetCurrentMapContinent() == -1 then
				self.mapSelectButton:SetText("Cosmic map")
			end
		elseif GetCurrentMapAreaID() == 543 then
			self.mapSelectButton:SetText("Trial of the Crusader")
		elseif GetCurrentMapAreaID() == 542 then
			self.mapSelectButton:SetText("Trial of the Champion")
		elseif GetCurrentMapAreaID() == 803 then
			self.mapSelectButton:SetText("The Nexus Legendary")
		else
			self.mapSelectButton:SetText(Tourist:GetZoneFromTexture(map))
		end
		local realZoneText = GetRealZoneText() or ""
		self.mapSelectOptions.args.player.name = format("|cffffd000%s|r", realZoneText)
		self.mapSelectOptions.args.player.desc = format("|cffffd000%s|r", realZoneText)
	end
end

function Maps:PLAYER_ENTERING_WORLD()
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	enteredWorld = true
end

local function tcount(t)
	local n = #t
	if (n == 0) then
		for _ in pairs(t) do
			n = n + 1
		end
	end
	return n
end

function CreateMapSelectButton()
	if not Maps.mapSelectButton then
		local realZoneText = GetRealZoneText() or ""
		Maps.mapSelectOptions = {
			type = "group",
			args = {
				player = {
					order = 1,
					name = format("|cffffd000%s|r", realZoneText),
					desc = format("|cffffd000%s|r", realZoneText),
					type = "execute",
					func = function()
						SetMapToCurrentZone()
					end,
				}
			},
		}
		local mapSelectButton = CreateFrame("Button", "MapsterMapSelectButton", WorldMapFrame, "UIPanelButtonTemplate2")
		mapSelectButton:SetWidth(200)
		mapSelectButton:SetText("")
		mapSelectButton:ClearAllPoints()
		mapSelectButton:SetPoint("RIGHT", WorldMapZoomOutButton, "LEFT", -8, 0)
		mapSelectButton:SetScript("OnClick", function(self)
			if Dewdrop:IsOpen(self) then
				Dewdrop:Close()
			else
				Dewdrop:Open(self)
			end
		end)
		Dewdrop:Register(mapSelectButton,
			'children', Maps.mapSelectOptions,
			'dontHook', true,
			'point', "TOPRIGHT",
			'relativePoint', "BOTTOMRIGHT"
		)
		Maps.mapSelectButton = mapSelectButton
		Maps:WorldMapUpdate()
	end

	if db.mapSelectButton then
		Maps.mapSelectButton:Show()
	else
		Maps.mapSelectButton:Hide()
	end
end

function Maps:PLAYER_LEVEL_UP()
	self.mapSelectOptions.args.cities = {
		order = 2,
		type = "group",
		name = L["Cities"],
		desc = L["Cities"],
		args = {},
	}
	self.mapSelectOptions.args.zones = {
		order = 3,
		type = "group",
		name = L["Zones"],
		desc = L["Zones"],
		args = {},
	}
	local continents = { GetMapContinents() }
	for i, continent in ipairs(continents) do
		local validate = { GetMapZones(i) }
		local zones = {}
		local cities = {}
		for k, v in ipairs(validate) do
			local text = v
			local low, high = Tourist:GetLevel(v)
			local r, g, b = Tourist:GetLevelColor(v)
			local levelText
			if low == 0 then
				levelText = ""
			elseif low == high then
				levelText = format(" |cff%02x%02x%02x[%d]|r", r * 255, g * 255, b * 255, high)
			else
				levelText = format(" |cff%02x%02x%02x[%d-%d]|r", r * 255, g * 255, b * 255, low, high)
			end
			local r, g, b = Tourist:GetFactionColor(v)
			text = format("|cff%02x%02x%02x%s|r%s", r*255, g*255, b*255, text, levelText)
			if Tourist:IsCity(v) then
				cities[v] = text
			else
				zones[v] = text
			end
			validate[v] = k
		end
		if tcount(zones) > 0 then
			self.mapSelectOptions.args.zones.args[continent] = {
				order = i,
				type = "text",
				name = continent,
				desc = continent,
				validate = zones,
				get = function() return GetCurrentMapContinent() == i and validate[GetCurrentMapZone()] end,
				set = function(text)
					SetMapZoom(i, validate[text])
					Dewdrop:Close()
				end,
			}
		end
		if tcount(cities) > 0 then
			self.mapSelectOptions.args.cities.args[continent] = {
				order = i,
				type = "text",
				name = continent,
				desc = continent,
				validate = cities,
				get = function() return GetCurrentMapContinent() == i and validate[GetCurrentMapZone()] end,
				set = function(text)
					SetMapZoom(i, validate[text])
					Dewdrop:Close()
				end,
			}
		end
	end
	local validate = self.zone_names["other:all"]
	local other = {}
	for k, v in ipairs(validate) do
		local text = v
		local low, high = Tourist:GetLevel(v)
		local r, g, b = Tourist:GetLevelColor(v)
		local levelText
		if low == 0 then
			levelText = ""
		elseif low == high then
			levelText = format(" |cff%02x%02x%02x[%d]|r", r * 255, g * 255, b * 255, high)
		else
			levelText = format(" |cff%02x%02x%02x[%d-%d]|r", r * 255, g * 255, b * 255, low, high)
		end
		local r, g, b = Tourist:GetFactionColor(v)
		text = format("|cff%02x%02x%02x%s|r%s", r*255, g*255, b*255, text, levelText)
		other[v] = text
		validate[v] = k
	end
	self.mapSelectOptions.args.zones.args.other = {
		order = 6,
		type = "text",
		name = L["Other Zones"],
		desc = L["Other Zones"],
		validate = other,
		get = function() return Maps.mapContId == 6 and validate[Maps.mapZone] end,
		set = function(text)
			Maps.mapCont, Maps.mapContId, Maps.mapZone = "other:all", 6, validate[text]
			SetMapByID(Maps:GetZoneData())
			Dewdrop:Close()
		end,
	}
	self.mapSelectOptions.args.instances = {
		order = 4,
		type = "group",
		name = L["Instances"],
		desc = L["Instances"],
		args = {},
	}
	self.mapSelectOptions.args.raids = {
		order = 5,
		type = "group",
		name = L["Raids"],
		desc = L["Raids"],
		args = {},
	}
	local titles = { L["Azeroth"], L["Outland"], L["Northrend"], L["Cataclysm"] }
	local categories = { "instances:classic", "instances:bc", "instances:wrath", "instances:cataclysm" }
	for i, category in ipairs(categories) do
		local validate = self.zone_names[category]
		local instances = {}
		for k, v in ipairs(validate) do
			local text = v
			local low, high = Tourist:GetLevel(v)
			local r, g, b = Tourist:GetLevelColor(v)
			local levelText
			if low == 0 then
				levelText = ""
			elseif low == high then
				levelText = format(" |cff%02x%02x%02x[%d]|r", r * 255, g * 255, b * 255, high)
			else
				levelText = format(" |cff%02x%02x%02x[%d-%d]|r", r * 255, g * 255, b * 255, low, high)
			end
			local r, g, b = Tourist:GetFactionColor(v)
			local complex = Tourist:GetComplex(v)
			--if complex then
			--	text = complex .. " - " .. text
			--end
			text = format("|cff%02x%02x%02x%s|r%s", r*255, g*255, b*255, text, levelText)
			local groupSize = Tourist:GetInstanceGroupSize(v)
			local altGroupSize = Tourist:GetInstanceAltGroupSize(v)
			if groupSize > 0 then
				if altGroupSize > 0 then
					text = text .. " " .. format(L["%d/%d-man"], groupSize, altGroupSize)
				else
					text = text .. " " .. format(L["%d-man"], groupSize)
				end
			end
			instances[v] = text
			validate[v] = k
		end
		self.mapSelectOptions.args.instances.args[category] = {
			order = i,
			type = "text",
			name = titles[i],
			desc = titles[i],
			validate = instances,
			get = function() return Maps.mapContId == i + 6 and validate[Maps.mapZone] end,
			set = function(text)
				Maps.mapCont, Maps.mapContId, Maps.mapZone = category, i + 6, validate[text]
				SetMapByID(Maps:GetZoneData())
				Dewdrop:Close()
			end,
		}
	end

	local titles = { L["Azeroth"], L["Outland"], L["Northrend"], L["Cataclysm"] }
	local categories = { "raids:classic", "raids:bc", "raids:wrath", "raids:cataclysm" }
	for i, category in ipairs(categories) do
		local validate = self.zone_names[category]
		local raids = {}
		for k, v in ipairs(validate) do
			local text = v
			local low, high = Tourist:GetLevel(v)
			local r, g, b = Tourist:GetLevelColor(v)
			local levelText
			if low == 0 then
				levelText = ""
			elseif low == high then
				levelText = format(" |cff%02x%02x%02x[%d]|r", r * 255, g * 255, b * 255, high)
			else
				levelText = format(" |cff%02x%02x%02x[%d-%d]|r", r * 255, g * 255, b * 255, low, high)
			end
			local r, g, b = Tourist:GetFactionColor(v)
			local complex = Tourist:GetComplex(v)
			--if complex then
			--	text = complex .. " - " .. text
			--end
			text = format("|cff%02x%02x%02x%s|r%s", r*255, g*255, b*255, text, levelText)
			local groupSize = Tourist:GetInstanceGroupSize(v)
			local altGroupSize = Tourist:GetInstanceAltGroupSize(v)
			if groupSize > 0 then
				if altGroupSize > 0 then
					text = text .. " " .. format(L["%d/%d-man"], groupSize, altGroupSize)
				else
					text = text .. " " .. format(L["%d-man"], groupSize)
				end
			end
			raids[v] = text
			validate[v] = k
		end
		self.mapSelectOptions.args.raids.args[category] = {
			order = i,
			type = "text",
			name = titles[i],
			desc = titles[i],
			validate = raids,
			get = function() return Maps.mapContId == i + 10 and validate[Maps.mapZone] end,
			set = function(text)
				Maps.mapCont, Maps.mapContId, Maps.mapZone = category, i + 10, validate[text]
				SetMapByID(Maps:GetZoneData())
				Dewdrop:Close()
			end,
		}
	end

	local validate = self.zone_names["bgrounds:all"]
	local bgrounds = {}
	for k, v in ipairs(validate) do
		local text = v
		local low, high = Tourist:GetLevel(v)
		local r, g, b = Tourist:GetLevelColor(v)
		local levelText
		if low == 0 then
			levelText = ""
		elseif low == high then
			levelText = format(" |cff%02x%02x%02x[%d]|r", r * 255, g * 255, b * 255, high)
		else
			levelText = format(" |cff%02x%02x%02x[%d-%d]|r", r * 255, g * 255, b * 255, low, high)
		end
		local r, g, b = Tourist:GetFactionColor(v)
		local complex = Tourist:GetComplex(v)
		text = format("|cff%02x%02x%02x%s|r%s", r*255, g*255, b*255, text, levelText)
		local groupSize = Tourist:GetInstanceGroupSize(v)
		local altGroupSize = Tourist:GetInstanceAltGroupSize(v)
		if groupSize > 0 then
			if altGroupSize > 0 then
				text = text .. " " .. format(L["%d/%d-man"], groupSize, altGroupSize)
			else
				text = text .. " " .. format(L["%d-man"], groupSize)
			end
		end
		bgrounds[v] = text
		validate[v] = k
	end
	self.mapSelectOptions.args.bgrounds = {
		order = 6,
		type = "text",
		name = L["Battlegrounds"],
		desc = L["Battlegrounds"],
		validate = bgrounds,
		get = function() return Maps.mapContId == 15 and validate[Maps.mapZone] end,
		set = function(text)
			Maps.mapCont, Maps.mapContId, Maps.mapZone = "bgrounds:all", 15, validate[text]
			SetMapByID(Maps:GetZoneData())
			Dewdrop:Close()
		end,
	}
end
