--[[
MapsterEnhanced (Sindragosa)

Based on Mapster
Copyright (c) 2009, Hendrik "Nevcairiel" Leppkes < h.leppkes@gmail.com >
All rights reserved.
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local MapsterEnhanced = LibStub("AceAddon-3.0"):NewAddon("MapsterEnhanced", "AceEvent-3.0", "AceHook-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("MapsterEnhanced")

local LBZ = LibStub("LibBabble-Zone-3.0", true)
local BZ = LBZ and LBZ:GetUnstrictLookupTable() or setmetatable({}, {__index = function(t,k) return k end})

local _G = getfenv(0)
local GetCursorPosition = GetCursorPosition
local GetPlayerMapPosition = GetPlayerMapPosition
local GetMapInfo = GetMapInfo
local GetCurrentMapZone = GetCurrentMapZone
local GetCurrentMapContinent = GetCurrentMapContinent
local GetCurrentMapDungeonLevel = GetCurrentMapDungeonLevel
local GetCurrentMapAreaID = GetCurrentMapAreaID
local SetMapByID = SetMapByID
local GetNumMapOverlays = GetNumMapOverlays
local GetNumberOfDetailTiles = GetNumberOfDetailTiles
local GetCVar = GetCVar
local GetCVarBool = GetCVarBool
local QuestPOIGetIconInfo = QuestPOIGetIconInfo
local WORLDMAP_SETTINGS = WORLDMAP_SETTINGS

-- Variables that are changed on "mini" mode
local miniList = { x = true, y = true, point = true, scale = true, alpha = true, hideBorder = true, disableMouse = true }

local db_
local db = setmetatable({}, {
	__index = function(t, k)
		if Mapster.miniMap and miniList[k] then
			return db_.mini[k]
		else
			return db_[k]
		end
	end,
	__newindex = function(t, k, v)
		if Mapster.miniMap and miniList[k] then
			db_.mini[k] = v
		else
			db_[k] = v
		end
	end
})

local format = string.format
local find = string.find
local gsub = string.gsub
local tostring = tostring
local type = type
local ipairs = ipairs
local pairs = pairs
local select = select
local setmetatable = setmetatable
local tconcat = table.concat

local wmfOnShow, wmfOnHide, dropdownScaleFix

function MapsterEnhanced:OnInitialize()
	db_ = Mapster.db.profile
	self:SetupOptions()
end

function MapsterEnhanced:OnEnable()
	if not self.optionsModified then
		self.optionsModified = true
		self:ModifyOptions()
	end
	WorldMapFrame:HookScript("OnShow", wmfOnShow)
	WorldMapFrame:HookScript("OnHide", wmfOnHide)

	WorldMapContinentDropDownButton:HookScript("OnClick", dropdownScaleFix)
	WorldMapZoneDropDownButton:HookScript("OnClick", dropdownScaleFix)
	WorldMapZoneMinimapDropDownButton:HookScript("OnClick", dropdownScaleFix)

	-- Init Overlay
	Mapster:SetOverlay()
	Mapster:SetAlpha()

	self:SecureHook(Mapster, "PLAYER_REGEN_ENABLED")

	if db.miniMap then
		Mapster:SizeDown()
	end

	self:RegisterEvent("WORLD_MAP_UPDATE", "WorldMapUpdate")
	self:SecureHook("WorldMapQuestPOI_OnClick")
	self:SecureHook("SetCVar")
end

--[[do
	if not MiniMapWorldMapButton2 then
		local normalTex, pushedTex, highlightTex = MiniMapWorldMapButton:GetRegions()
		MiniMapWorldMapButton:SetNormalTexture("Interface\\Icons\\Ability_Druid_Eclipse")
		MiniMapWorldMapButton:SetPushedTexture("Interface\\Icons\\Ability_Druid_Eclipse")
		normalTex:SetTexCoord(0, 1, 0, 1)
		pushedTex:SetTexCoord(0, 1, 0, 1)
		MiniMapWorldMapButton:SetHeight(18)
		MiniMapWorldMapButton:SetWidth(18)
		normalTex:SetHeight(18)
		normalTex:SetWidth(18)
		pushedTex:SetHeight(18)
		pushedTex:SetWidth(18)
		MiniMapWorldMapButton:ClearAllPoints()
		MiniMapWorldMapButton:SetPoint("TOPRIGHT", MinimapBackdrop, "TOPRIGHT", -7, 17)
		highlightTex:ClearAllPoints()
		highlightTex:SetPoint("CENTER", MiniMapWorldMapButton, "CENTER", 0, 0)

		MiniMapWorldMapButton:Hide()
		MiniMapWorldMapButton2 = MiniMapWorldMapButton
		MiniMapWorldMapButton = nil
		local mapButton = CreateFrame("Button", "MiniMapWorldMapButton", MinimapBackdrop, "MiniMapButtonTemplate")
		mapButton:SetPoint("TOPRIGHT", MinimapBackdrop, "TOPRIGHT", -21, -1)
		mapButton:SetHeight(33)
		mapButton:SetWidth(33)
		local iconTexture = mapButton:CreateTexture("MiniMapWorldMapButtonIcon", "BORDER")
		iconTexture:SetTexture("Interface\\Icons\\Ability_Druid_Eclipse")
		iconTexture:SetPoint("CENTER", mapButton, "CENTER", -2, 2)
		iconTexture:SetHeight(20)
		iconTexture:SetWidth(20)
		local mapTexture = mapButton:CreateTexture("MiniMapWorldBorder", "OVERLAY");
		mapTexture:SetTexture("Interface\\Minimap\\MiniMap-TrackingBorder")
		mapTexture:SetPoint("TOPLEFT", mapButton, "TOPLEFT")
		mapTexture:SetHeight(52)
		mapTexture:SetWidth(52)
		mapButton.tooltipText = MicroButtonTooltipText(WORLDMAP_BUTTON, "TOGGLEWORLDMAP")
		mapButton.newbieText = NEWBIE_TOOLTIP_WORLDMAP
		mapButton:RegisterEvent("UPDATE_BINDINGS")
		mapButton:SetScript("OnEnter", function(self)
			GameTooltip_AddNewbieTip(self, self.tooltipText, 1.0, 1.0, 1.0, self.newbieText)
		end)
		mapButton:SetScript("OnLeave", function(self)
			GameTooltip:Hide()
		end)
		mapButton:SetScript("OnEvent", function(self)
			self.tooltipText = MicroButtonTooltipText(WORLDMAP_BUTTON, "TOGGLEWORLDMAP");
			self.newbieText = NEWBIE_TOOLTIP_WORLDMAP
		end)
		mapButton:SetScript("OnClick", function(self)
			ToggleFrame(WorldMapFrame)
		end)
		mapButton:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight", "ADD")
	end
end]]--

local tmp = {}
local n_ipairs = function(...)
	for k in pairs(tmp) do
		tmp[k] = nil
	end
	for i = 1, select('#', ...) do
		tmp[i] = select(i, ...)
	end
	return ipairs(tmp)
end

local function GetCursorMapPosition()
	if not WorldMapFrame:IsVisible() then return end
	local left, top = WorldMapDetailFrame:GetLeft(), WorldMapDetailFrame:GetTop()
	local width, height = WorldMapDetailFrame:GetWidth(), WorldMapDetailFrame:GetHeight()
	local scale = WorldMapDetailFrame:GetEffectiveScale()

	local x, y = GetCursorPosition()
	local cx = (x/scale - left) / width
	local cy = (top - y/scale) / height

	if cx < 0 or cx > 1 or cy < 0 or cy > 1 then
		return
	end

	return cx, cy
end

local cities = {
	["ThunderBluff"] = true,
	["Orgrimmar"] = true,
	["TheExodar"] = true,
	["Darnassus"] = true,
	["Undercity"] = true,
	["Ironforge"] = true,
	["StormwindCity"] = true,
	["SilvermoonCity"] = true,
	["ShattrathCity"] = true,
	["Dalaran"] = true,
}

local function refixParent(frame)
	local self = Mapster
	if frame == self.overlayHolder or frame == self.nonOverlayHolder then
		return
	end
	local parent = self.nonOverlayHolder
	if frame == PlayerArrowFrame or frame == PlayerArrowEffectFrame or frame == WorldMapButton or frame == WorldMapPOIFrame or frame == WorldMapBlobFrame or frame == WorldMapArchaeologyDigSites then
		parent = self.overlayHolder
	elseif frame == WorldMapDetailFrame then
		if ((GetCurrentMapZone() == 0 or cities[GetMapInfo()]) and (db.overlayAlpha or 1) > (db.alpha or 0.8)) and GetMapInfo() ~= "AlteracValley" then
			parent = self.overlayHolder
		end
	elseif frame:GetName() and find(frame:GetName(), "Tooltip$") then
		return
	elseif frame:GetParent() == WorldMapDetailFrame then
		parent = self.tileHolder
	end
	frame:SetParent(parent)
end

function Mapster:SetOverlay()
	self:SecureHook(WorldMapDetailFrame, "CreateTexture", "WorldMapDetailFrame_CreateTexture")
	self:SecureHook("WorldMapButton_OnClick")
	self:SecureHookScript(WorldMapButton, "OnMouseUp", "WorldMapButton_OnClick")
	self:SecureHookScript(WorldMapButton, "OnUpdate", "FixAddedFrames")
	if not self.overlayHolder then
		self.overlayHolder = CreateFrame("Frame", "MapsterOverlayHolder", WorldMapFrame)
		self.overlayHolder:SetPoint("TOPLEFT", WorldMapDetailFrame, "TOPLEFT")
		self.overlayHolder:SetPoint("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT")
		self.overlayHolder:SetFrameStrata(WorldMapDetailFrame:GetFrameStrata())
		self.overlayHolder:SetFrameLevel(WorldMapDetailFrame:GetFrameLevel() + 1)
	end
	if not self.nonOverlayHolder then
		self.nonOverlayHolder = CreateFrame("Frame", "MapsterNonOverlayHolder", WorldMapFrame)
		self.nonOverlayHolder:SetPoint("TOPLEFT", WorldMapDetailFrame, "TOPLEFT")
		self.nonOverlayHolder:SetPoint("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT")
		self.nonOverlayHolder:SetFrameStrata(WorldMapFrame:GetFrameStrata())
		self.nonOverlayHolder:SetFrameLevel(WorldMapFrame:GetFrameLevel())
	end
	if not self.tileHolder then
		self.tileHolder = CreateFrame("Frame", "MapsterTileHolder", self.overlayHolder)
		self.tileHolder:SetPoint("TOPLEFT", WorldMapDetailFrame, "TOPLEFT")
		self.tileHolder:SetPoint("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT")
		self.tileHolder:SetFrameStrata(WorldMapDetailFrame:GetFrameStrata())
		self.tileHolder:SetFrameLevel(WorldMapDetailFrame:GetFrameLevel() + 1)
	end
	self.overlayHolder:Show()
	self.nonOverlayHolder:Show()

	self.tileHolder:Show()
	self.tileHolder:SetAlpha(db.overlayAlpha or 1)

	refixParent(WorldMapButton)
	for _,v in n_ipairs(WorldMapButton:GetChildren()) do
		v:SetParent(nil)
		v:SetParent(WorldMapButton)
	end

	for i,v in n_ipairs(WorldMapFrame:GetChildren()) do
		if v ~= self.overlayHolder and v ~= self.nonOverlayHolder then
			refixParent(v)
		end
	end
	for i,v in n_ipairs(WorldMapFrame:GetRegions()) do
		refixParent(v, self.nonOverlayHolder)
	end

	refixParent(PlayerArrowFrame)
	refixParent(PlayerArrowEffectFrame)
	PlayerArrowFrame:SetFrameLevel(WorldMapButton:GetFrameLevel() + 1)
	PlayerArrowEffectFrame:SetFrameLevel(WorldMapButton:GetFrameLevel() + 1)
	WorldMapPing:SetFrameLevel(WorldMapButton:GetFrameLevel() + 2)
	WorldMapPlayer:SetFrameLevel(WorldMapButton:GetFrameLevel() + 2)

	refixParent(WorldMapDetailFrame)
	WorldMapPOIFrame:SetFrameLevel(WorldMapButton:GetFrameLevel() + 4)
	WorldMapFrameCloseButton:SetScript("OnClick", function() HideUIPanel(WorldMapFrame) end)

	for i = 1, 1000 do
		local texture = _G["WorldMapOverlay" .. i]
		if not texture then
			break
		end
		texture:SetParent(self.tileHolder)
	end
end


local lastUpdate = 0
function Mapster:FixAddedFrames(self, elapsed)
	if (GetCurrentMapAreaID() == 504) then	-- Dalaran
		local cx, cy = GetCursorMapPosition()
		if cx and cx >= 0.83 and cx <= 1 and cy and cy >= 0.36 and cy <= 1 then
			WorldMapFrameAreaLabel:SetTextColor(1, 1, 1)
			WorldMapFrameAreaLabel:SetText(BZ["Crystalsong Forest"] or "Crystalsong Forest")
		else
			WorldMapFrameAreaLabel:SetText("")
		end
	end

	lastUpdate = lastUpdate + elapsed
	if lastUpdate >= 0.1 then
		lastUpdate = 0

		local detail_framelevel = WorldMapDetailFrame:GetFrameLevel()
		for i,v in n_ipairs(WorldMapDetailFrame:GetChildren()) do
			if v ~= self.overlayHolder and v ~= self.nonOverlayHolder then
				local framelevel = v:GetFrameLevel()
				refixParent(v)
				v:SetFrameLevel(v:GetParent():GetFrameLevel() - detail_framelevel + framelevel + 3)
			end
		end
	end
end

function Mapster:WorldMapDetailFrame_CreateTexture(WorldMapDetailFrame, name, layer)
	if name and find(name, "^WorldMapOverlay%d+$") then
		local frame = _G[name]
		frame:SetParent(self.tileHolder)
	end
end

local lastMapAreaID, lastMapAreaID2	-- So we don't keep processing zones in every Update
function Mapster:WorldMapButton_OnClick(button, mouseButton)
	if (mouseButton == "LeftButton") then
		if (WorldMapFrameAreaLabel:GetText() == "Dalaran") then
			SetMapByID(504)
		elseif (GetCurrentMapAreaID() == 504) then	-- Dalaran
			local cx, cy = GetCursorMapPosition()
			if cx and cx >= 0.83 and cx <= 1 and cy and cy >= 0.36 and cy <= 1 then
				SetMapByID(510)
			end
		end
		self.selectedQuestId = nil
		if db.questObjectives > 0 and (WatchFrame.showObjectives and WorldMapFrame.numQuests > 0) then
			WorldMapFrame_UpdateQuests()
		end
	elseif (mouseButton == "RightButton" and not IsControlKeyDown()) then
		if (lastMapAreaID2 == 504) then	-- Dalaran
			SetMapByID(510)	-- Crystalsong Forest
		elseif (lastMapAreaID2 == 481) then	-- Shattrath
			SetMapByID(478)	-- Terokkar Forest
		elseif (lastMapAreaID2 == 382) then	-- Shattrath
			SetMapByID(20)	-- Terokkar Forest
		elseif (lastMapAreaID2 == 341) then	-- Ironforge
			SetMapByID(27)	-- Dun Morogh
		elseif (lastMapAreaID2 == 301) then	-- Stormwind
			SetMapByID(30)	-- Elwynn Forest
		elseif (lastMapAreaID2 == 480) then	-- Silvermoon
			SetMapByID(462)	-- Eversong Woods
		elseif (lastMapAreaID2 == 321) then	-- Ogrimmar
			SetMapByID(4)	-- Durotar
		elseif (lastMapAreaID2 == 362) then	-- Ogrimmar
			SetMapByID(9)	-- Durotar
		elseif (lastMapAreaID2 == 381) then	-- Darnassus
			SetMapByID(41)	-- Teldrassil
		elseif (lastMapAreaID2 == 471) then	-- The Exodar
			SetMapByID(464)	-- Azuremyst Isle
		end
	end
end

function MapsterEnhanced:WorldMapUpdate()
	if lastMapAreaID ~= GetCurrentMapAreaID() then
		lastMapAreaID2 = lastMapAreaID
		lastMapAreaID = GetCurrentMapAreaID()
		Mapster.selectedQuestId = nil
	end
	refixParent(WorldMapDetailFrame)
	--self:CallMethodOnAllModules("Mapster_ChangeZone")
	self:SendMessage("MAPSTER_CHANGE_ZONE")
end

function MapsterEnhanced:PLAYER_REGEN_ENABLED()
	WorldMapBlobFrame:SetParent(Mapster.overlayHolder)
	WorldMapArchaeologyDigSites:SetParent(Mapster.overlayHolder)
end

function Mapster:ToggleMapSize()
	self.miniMap = not self.miniMap
	db.miniMap = self.miniMap

	local continentID, dungeonLevel
	local mapID = GetCurrentMapAreaID()
	if (mapID < 0) then
		continentID = GetCurrentMapContinent()
	else
		dungeonLevel = GetCurrentMapDungeonLevel()
	end

	ToggleFrame(WorldMapFrame)
	if self.miniMap then
		self:SizeDown()
	else
		self:SizeUp()
	end
	self:SetAlpha()
	self:SetPosition()

	-- Notify the modules about the map size change,
	-- so they can re-anchor frames or stuff like that.
	--self:UpdateModuleMapsizes()

	self:UpdateBorderVisibility()
	self:UpdateMouseInteractivity()

	ToggleFrame(WorldMapFrame)
	self:UpdateModuleMapsizes()
	if (continentID) then
		SetMapZoom(continentID)
	else
		SetMapByID(mapID)
		if (dungeonLevel > 0) then
			SetDungeonMapLevel(dungeonLevel)
		end
	end
	WorldMapFrame_UpdateQuests()
end

function Mapster:SizeUp()
	WORLDMAP_SETTINGS.size = WORLDMAP_QUESTLIST_SIZE
	-- adjust main frame
	WorldMapFrame:SetWidth(1024)
	WorldMapFrame:SetHeight(768)
	-- adjust map frames
	WorldMapPositioningGuide:ClearAllPoints()
	WorldMapPositioningGuide:SetPoint("CENTER")
	WorldMapDetailFrame:SetScale(WORLDMAP_QUESTLIST_SIZE)
	WorldMapDetailFrame:SetPoint("TOPLEFT", WorldMapPositioningGuide, "TOP", -726, -99)
	WorldMapButton:SetScale(WORLDMAP_QUESTLIST_SIZE)
	WorldMapFrameAreaFrame:SetScale(WORLDMAP_QUESTLIST_SIZE)
	WorldMapBlobFrame:SetScale(WORLDMAP_QUESTLIST_SIZE)
	WorldMapBlobFrame.xRatio = nil		-- force hit recalculations
	WorldMapArchaeologyDigSites:SetScale(WORLDMAP_QUESTLIST_SIZE)
	WorldMapArchaeologyDigSites.xRatio = nil		-- force hit recalculations
	if self.tileHolder then
		self.tileHolder:SetScale(1)
	end
	self:SetArrow()
	-- show big window elements
	WorldMapZoneMinimapDropDown:Show()
	WorldMapZoomOutButton:Show()
	WorldMapZoneDropDown:Show()
	WorldMapContinentDropDown:Show()
	WorldMapQuestScrollFrame:Show()
	WorldMapQuestDetailScrollFrame:Show()
	WorldMapQuestRewardScrollFrame:Show()
	WorldMapFrameSizeDownButton:Show()
	-- hide small window elements
	WorldMapFrameMiniBorderLeft:Hide()
	WorldMapFrameMiniBorderRight:Hide()
	WorldMapFrameSizeUpButton:Hide()
	-- floor dropdown
	WorldMapLevelDropDown:ClearAllPoints()
	WorldMapLevelDropDown:SetPoint("TOPRIGHT", WorldMapPositioningGuide, "TOPRIGHT", -50, -35)
	WorldMapLevelDropDown.header:Show()
	-- tiny adjustments
	WorldMapFrameCloseButton:SetPoint("TOPRIGHT", WorldMapPositioningGuide, 4, 4)
	WorldMapFrameSizeDownButton:SetPoint("TOPRIGHT", WorldMapPositioningGuide, -16, 4)
	WorldMapTrackQuest:SetParent(WorldMapFrame)
	WorldMapTrackQuest:ClearAllPoints()
	WorldMapTrackQuest:SetPoint("BOTTOMLEFT", WorldMapPositioningGuide, "BOTTOMLEFT", 16, 4)
	WorldMapTrackQuest:Show()
	WorldMapFrameTitle:ClearAllPoints()
	WorldMapFrameTitle:SetPoint("CENTER", 0, 372)

	MapsterQuestObjectivesDropDown:Show()
	WorldMapShowDigSites:SetScript("OnShow", nil)
	WorldMapShowDigSites:Show()

	WorldMapFrame_SetPOIMaxBounds()
	WorldMapQuestShowObjectives_AdjustPosition()
	self:WorldMapFrame_DisplayQuests()

	self.optionsButton:SetPoint("TOPRIGHT", WorldMapPositioningGuide, "TOPRIGHT", -43, -2)
end

function Mapster:SizeDown()
	WORLDMAP_SETTINGS.size = WORLDMAP_WINDOWED_SIZE
	-- adjust main frame
	WorldMapFrame:SetWidth(623)
	WorldMapFrame:SetHeight(437)
	-- adjust map frames
	if self.nonOverlayHolder then
		self.nonOverlayHolder:ClearAllPoints()
		self.nonOverlayHolder:SetAllPoints()
	end
	WorldMapPositioningGuide:ClearAllPoints()
	WorldMapPositioningGuide:SetAllPoints()
	WorldMapDetailFrame:SetScale(WORLDMAP_WINDOWED_SIZE)
	WorldMapButton:SetScale(WORLDMAP_WINDOWED_SIZE)
	WorldMapFrameAreaFrame:SetScale(WORLDMAP_WINDOWED_SIZE)
	WorldMapBlobFrame:SetScale(WORLDMAP_WINDOWED_SIZE)
	WorldMapBlobFrame.xRatio = nil		-- force hit recalculations
	WorldMapArchaeologyDigSites:SetScale(WORLDMAP_WINDOWED_SIZE)
	WorldMapArchaeologyDigSites.xRatio = nil		-- force hit recalculations
	WorldMapFrameMiniBorderLeft:SetPoint("TOPLEFT", 10, -14)
	if self.overlayHolder then
		self.overlayHolder:ClearAllPoints()
		self.overlayHolder:SetAllPoints()
	end
	if self.tileHolder then
		self.tileHolder:SetScale(WORLDMAP_WINDOWED_SIZE)
	end
	WorldMapDetailFrame:SetPoint("TOPLEFT", 37, -66)
	PlayerArrowFrame:SetModelScale(1)
	PlayerArrowEffectFrame:SetModelScale(1)
	-- hide big window elements
	WorldMapZoneMinimapDropDown:Hide()
	WorldMapZoomOutButton:Hide()
	WorldMapZoneDropDown:Hide()
	WorldMapContinentDropDown:Hide()
	WorldMapLevelDropDown:Hide()
	WorldMapLevelUpButton:Hide()
	WorldMapLevelDownButton:Hide()
	WorldMapQuestScrollFrame:Hide()
	WorldMapQuestDetailScrollFrame:Hide()
	WorldMapQuestRewardScrollFrame:Hide()
	WorldMapFrameSizeDownButton:Hide()
	-- show small window elements
	WorldMapFrameMiniBorderLeft:Show()
	WorldMapFrameMiniBorderRight:Show()
	WorldMapFrameSizeUpButton:Show()
	-- floor dropdown
	WorldMapLevelDropDown:ClearAllPoints()
	WorldMapLevelDropDown:SetPoint("TOPLEFT", WorldMapDetailFrame, "TOPLEFT", -19, 3)
	WorldMapLevelDropDown:SetFrameLevel(WORLDMAP_POI_FRAMELEVEL + 2)
	WorldMapLevelDropDown.header:Hide()
	-- tiny adjustments
	WorldMapFrameCloseButton:SetPoint("TOPRIGHT", WorldMapFrameMiniBorderRight, "TOPRIGHT", -44, 5)
	WorldMapFrameSizeDownButton:SetPoint("TOPRIGHT", WorldMapFrameMiniBorderRight, "TOPRIGHT", -66, 5)
	WorldMapTrackQuest:SetPoint("BOTTOMLEFT", WorldMapDetailFrame, "BOTTOMLEFT", 2, -26)
	WorldMapFrameTitle:ClearAllPoints()
	WorldMapFrameTitle:SetPoint("TOP", WorldMapDetailFrame, 0, 20)

	MapsterQuestObjectivesDropDown:Hide()
	WorldMapShowDigSites:Hide()
	WorldMapShowDigSites:SetScript("OnShow", WorldMapShowDigSites.Hide)

	WorldMapFrame_SetPOIMaxBounds()
	WorldMapQuestShowObjectives_AdjustPosition()

	self.optionsButton:SetPoint("TOPRIGHT", WorldMapFrameMiniBorderRight, "TOPRIGHT", -93, -2)
end

local function getZoneId()
	return (GetCurrentMapZone() + GetCurrentMapContinent() * 100)
end

function Mapster:ZONE_CHANGED_NEW_AREA()
end

function wmfOnShow(frame)
	if not Mapster.wmfShown then
		Mapster.wmfShown = true
		--Mapster:CallMethodOnAllModules("Mapster_MapOpened")
		Mapster:SendMessage("MAPSTER_MAP_OPENED")
	end
end

function wmfOnHide(frame)
	if Mapster.wmfShown then
		Mapster.wmfShown = false
		--Mapster:CallMethodOnAllModules("Mapster_MapClosed")
		Mapster:SendMessage("MAPSTER_MAP_CLOSED")
	end
end

function dropdownScaleFix(self)
	local point, relativeTo, relativePoint, xOfs, yOfs = DropDownList1:GetPoint()
	DropDownList1:ClearAllPoints()
	DropDownList1:SetPoint("TOPLEFT", relativeTo, relativePoint, xOfs, yOfs)
end

function Mapster:SetAlpha()
	if self.overlayHolder then
		self.overlayHolder:SetAlpha(db.overlayAlpha or 1)
		self.nonOverlayHolder:SetAlpha(db.alpha or 0.8)
	end
end

function Mapster:WorldMapFrame_DisplayQuests()
	if WORLDMAP_SETTINGS.size == WORLDMAP_WINDOWED_SIZE then return end
	if db.questObjectives == 0 or not (WatchFrame.showObjectives and WorldMapFrame.numQuests > 0) then
		self.tileHolder:SetScale(WORLDMAP_FULLMAP_SIZE)
		WorldMapButton:SetScale(WORLDMAP_FULLMAP_SIZE)
		WorldMapFrameAreaFrame:SetScale(WORLDMAP_FULLMAP_SIZE)
		WorldMapArchaeologyDigSites:SetScale(WORLDMAP_FULLMAP_SIZE)
		WorldMapArchaeologyDigSites.xRatio = nil		-- force hit recalculations
	else
		if db.questObjectives == 1 then
			WorldMapFrame_SetFullMapView()

			self.tileHolder:SetScale(WORLDMAP_FULLMAP_SIZE)
			WorldMapButton:SetScale(WORLDMAP_FULLMAP_SIZE)
			WorldMapFrameAreaFrame:SetScale(WORLDMAP_FULLMAP_SIZE)

			WorldMapBlobFrame:SetScale(WORLDMAP_FULLMAP_SIZE)
			WorldMapBlobFrame.xRatio = nil		-- force hit recalculations
			WorldMapArchaeologyDigSites:SetScale(WORLDMAP_FULLMAP_SIZE)
			WorldMapArchaeologyDigSites.xRatio = nil		-- force hit recalculations
			WorldMapFrame_SetPOIMaxBounds()
			WorldMapFrame_UpdateQuests()
			if self.selectedQuestId then
				WorldMapFrame_SelectQuestById(self.selectedQuestId)
			end
		elseif db.questObjectives == 2 then
			WorldMapFrame_SetQuestMapView()

			self.tileHolder:SetScale(WORLDMAP_QUESTLIST_SIZE)
			WorldMapButton:SetScale(WORLDMAP_QUESTLIST_SIZE)
			WorldMapFrameAreaFrame:SetScale(WORLDMAP_QUESTLIST_SIZE)

			WorldMapBlobFrame:SetScale(WORLDMAP_QUESTLIST_SIZE)
			WorldMapBlobFrame.xRatio = nil		-- force hit recalculations
			WorldMapArchaeologyDigSites:SetScale(WORLDMAP_QUESTLIST_SIZE)
			WorldMapArchaeologyDigSites.xRatio = nil		-- force hit recalculations
			WorldMapFrame_SetPOIMaxBounds()
			WorldMapFrame_UpdateQuests()
			if self.selectedQuestId then
				WorldMapFrame_SelectQuestById(self.selectedQuestId)
			end
		end
	end
	self:UpdateModuleMapsizes()
end

function MapsterEnhanced:WorldMapQuestPOI_OnClick(questPOI)
	if not Mapster.selectedQuestId and WORLDMAP_SETTINGS.selectedQuest then
		WorldMapBlobFrame:DrawBlob(WORLDMAP_SETTINGS.selectedQuestId, false)
		WorldMapFrame_SelectQuestFrame(questPOI.quest)
	end
	Mapster.selectedQuestId = WORLDMAP_SETTINGS.selectedQuestId
end

function MapsterEnhanced:SetCVar(key, value)
	if key == "rotateMinimap" then
		if value == 1 then value = "1" end
		Mapster:CallMethodOnAllModules("OnRotateMinimapUpdate", value == "1")
	end
end

function MapsterEnhanced:SetModuleEnabled(module, value)
	local old = db.modules[module]
	db.modules[module] = value
	if old ~= value then
		if value then
			self:EnableModule(module)
		else
			self:DisableModule(module)
		end
	end
end

function Mapster:CallMethodOnAllModules(method, ...)
	for name, module in self:IterateModules() do
		if type(module[method]) == "function" then
			module[method](module, ...)
		end
	end
end

function MapsterEnhanced:ShowModNotFound()
	if not StaticPopupDialogs["MAPSTERENHANCED_MODNOTFOUND"] then
		StaticPopupDialogs["MAPSTERENHANCED_MODNOTFOUND"] = {
			text = "Module options not found, please re-install MapsterEnhanced.",
			button1 = "OK",
			timeout = 0,
			preferredIndex = STATICPOPUP_NUMDIALOGS,
			exclusive = 1,
			hideOnEscape = 1,
			whileDead = 1,
		}
	end
	StaticPopup_Show("MAPSTERENHANCED_MODNOTFOUND")
end

function Mapster:GetPrint(module)
	local mod = Mapster:GetModule(module)
	if mod then
		mod.Print = self.Print
	else
		mod = MapsterEnhanced:GetModule(module)
		if mod then
			mod.Print = self.Print
		end
	end
end

function Mapster:Print(...)
	local frame = ...
	if not frame then return end
	local count, text = select("#",...)
	if type(frame) == "table" then
		if count == 1 then return end
		if count > 2 then
			for k in pairs(tmp) do
				tmp[k] = nil
			end
			for i = 2, count do
				tmp[i - 1] = select(i,...)
				if tmp[i - 1] == nil then
					tmp[i - 1] = "nil"
				end
			end
			text = tconcat(tmp, " ", 1, count - 1)
		end
		text = text or select(2,...)
	else
		if count > 1 then
			for k in pairs(tmp) do
				tmp[k] = nil
			end
			for i = 1, count do
				tmp[i] = select(i,...)
				if tmp[i] == nil then
					tmp[i] = "nil"
				end
			end
			text = tconcat(tmp, " ", 1, count)
		end
		text = text or (...)
	end

	if self == Mapster then
		if type(frame) == "table" and frame.AddMessage then
			frame:AddMessage("|cff7f7fffMapster|r: "..text)
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cff7f7fffMapster|r: "..text)
		end
	else
		local mod = gsub(tostring(self), "Mapster_", "")
		mod = gsub(mod, "MapsterEnhanced_", "")
		if type(frame) == "table" and frame.AddMessage then
			frame:AddMessage("|cff7f7fffMapster|r|cffff8000"..mod.."|r: "..text)
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cff7f7fffMapster|r|cffff8000"..mod.."|r: "..text)
		end
	end
end
