--[[
MapsterEnhanced - Notes (Sindragosa)

Based on Cartographer
Credits: ckknight
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local L = LibStub("AceLocale-3.0"):GetLocale("Mapster")

local MODNAME = "Notes"
local Notes = Mapster:NewModule(MODNAME, "AceEvent-3.0", "AceHook-3.0", "AceComm-3.0", "AceConsole-3.0", "AceSerializer-3.0")

MapsterNotes = Notes


local Astrolabe = DongleStub("Astrolabe-0.7-Global")
local Tablet = AceLibrary("Tablet-2.0")
local Dewdrop = AceLibrary("Dewdrop-2.0")
local LibGuild = LibStub("LibGuild-1.0")

local LBZ = LibStub("LibBabble-Zone-3.0", true)
local BZL = LBZ and LBZ:GetLookupTable() or setmetatable({}, {__index = function(t,k) return k end})
local BZH = LBZ and LBZ:GetUnstrictLookupTable() or setmetatable({}, {__index = function(t,k) return k end})
local BZR = LBZ and LBZ:GetReverseLookupTable() or setmetatable({}, {__index = function(t,k) return k end})

local _G = getfenv(0)
local COMM_PREFIX = "MpN"
local GetCursorPosition = GetCursorPosition
local GetPlayerMapPosition = GetPlayerMapPosition
local GetScreenWidth, GetScreenHeight = GetScreenWidth, GetScreenHeight
local MouseIsOver = MouseIsOver
local UnitName = UnitName
local UnitLevel = UnitLevel
local UnitRace = UnitRace
local UnitClass = UnitClass
local GetGuildInfo = GetGuildInfo
local GetRealmName = GetRealmName
local GetLocale = GetLocale
local GetRealZoneText = GetRealZoneText
local GetMapInfo = GetMapInfo
local GetCurrentMapZone = GetCurrentMapZone
local GetCurrentMapContinent = GetCurrentMapContinent
local GetCurrentMapDungeonLevel = GetCurrentMapDungeonLevel
local GetCurrentMapAreaID = GetCurrentMapAreaID
local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers
local UnitExists = UnitExists
local UnitIsFriend = UnitIsFriend
local UnitIsPlayer = UnitIsPlayer
local GetCVar = GetCVar
local IsControlKeyDown = IsControlKeyDown
local IsAltKeyDown = IsAltKeyDown
local IsShiftKeyDown = IsShiftKeyDown
local IsInInstance = IsInInstance
local GetTrackingTexture = GetTrackingTexture
local GetMinimapShape = GetMinimapShape
local GetPlayerFacing = GetPlayerFacing
local GetQuestDifficultyColor = GetQuestDifficultyColor
local WorldMapDetailFrame = WorldMapDetailFrame
local PLAYER = PLAYER
local PARTY = PARTY
local RAID = RAID
local RANK = RANK
local GUILD = GUILD
local CLASS = CLASS
local RACE = RACE
local LEVEL = LEVEL
local ZONE = ZONE
local NOTE_COLON = NOTE_COLON
local OFFICER_NOTE_COLON = OFFICER_NOTE_COLON
if not OFFICER_NOTE_COLON:find(":") then
	OFFICER_NOTE_COLON = OFFICER_NOTE_COLON .. ":"
end

local RAID_CLASS_COLORS = RAID_CLASS_COLORS
--[[local RAID_CLASS_COLORS = {
	["HUNTER"] = { r = 0.67, g = 0.83, b = 0.45 },
	["WARLOCK"] = { r = 0.58, g = 0.51, b = 0.79 },
	["PRIEST"] = { r = 1.0, g = 1.0, b = 1.0 },
	["PALADIN"] = { r = 0.96, g = 0.55, b = 0.73 },
	["MAGE"] = { r = 0.41, g = 0.8, b = 0.94 },
	["ROGUE"] = { r = 1.0, g = 0.96, b = 0.41 },
	["DRUID"] = { r = 1.0, g = 0.49, b = 0.04 },
	["SHAMAN"] = { r = 0.14, g = 0.35, b = 1.0 },
	["WARRIOR"] = { r = 0.78, g = 0.61, b = 0.43 },
	["DEATHKNIGHT"] = { r = 0.77, g = 0.12, b = 0.23 },
}]]--

local table = table
local table_concat = table.concat
local table_insert = table.insert
local table_remove = table.remove
local table_sort = table.sort
local wipe = table.wipe
local string = string
local string_format = string.format
local string_find = string.find
local string_match = string.match
local string_gsub = string.gsub
local string_sub = string.sub
local math = math
local math_floor = math.floor
local math_pow = math.pow
local math_random = math.random
local math_huge = math.huge
local math_sin = math.sin
local math_cos = math.cos
local math_min = math.min
local math_max = math.max
local format = format
local strmatch = strmatch
local type = type
local ipairs = ipairs
local pairs = pairs
local next = next
local rawget = rawget
local select = select
local setmetatable = setmetatable
local tonumber = tonumber
local tostring = tostring

local noteFrame
local yards = L["%.0f yd"]

local Waypoints, Coords
local pois, minimapPois
local isInInstance
local forceNextMinimapUpdate = false
local rotateMinimap = GetCVar("rotateMinimap") == "1"
local MapsterNotesTip
local dummyFrame
local setupUnitPOIs
Notes.UnitPOIs = CreateFrame("Frame", "MapsterNotes_UnitPOIsFrame")

-- defaults --
local db, global_db
local defaults = {
	profile = {
		zone = true,
		location = true,
		showDistance = true,
		showCreator = true,
		metric = true,
		minimapIcons = true,
		minimapIconSize = 0.7,
		minimapOtherZone = true,
		minimapTooltip = true,
		minimapTooltipScale = 1,
		minimapCreateClick = true,
		minimapMenu = true,
		minimapNotesPerDB = 25,
		worldmapIcons = true,
		worldmapIconSize = 1,
		worldmapOtherZone = true,
		worldmapTooltip = true,
		worldmapTooltipScale = 1,
		worldmapCreateClick = true,
		worldmapMenu = true,
		createModifier = "C",
		showNotifications = true,
		commActive = true,
		commFilter = "handmade",
	},
	global = {
		pois = {
			["*"] = {
				["*"] = {
					creator = "",
					manual = true,
				},
			},
		},
	},
}

local optGetter, optSetter
do
	local mod = Notes
	function optGetter(info)
		local key = info[#info]
		return db[key]
	end

	function optSetter(info, value)
		local key = info[#info]
		db[key] = value
		mod:Refresh()
	end
end

-- OptionsTable --
local options
local function getOptions()
	if not options then
		options = {
			type = "group",
			name = L["Notes"],
			arg = MODNAME,
			handler = Notes,
			get = optGetter,
			set = optSetter,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["The Notes module adds a POI system to the World Map."],
				},
				enabled = {
					order = 2,
					type = "toggle",
					name = L["Enable Notes"],
					get = function() return Mapster:GetModuleEnabled(MODNAME) end,
					set = function(info, value) Mapster:SetModuleEnabled(MODNAME, value) end,
				},
				notesDesc = {
					order = 3,
					type = "description",
					name = "\n" .. L["You can show various information about notes, when they are moused over."],
				},
				zone = {
					order = 4,
					type = "toggle",
					name = L["Show zone"],
					desc = L["Show zone in notes."],
				},
				location = {
					order = 5,
					type = "toggle",
					name = L["Show location"],
					desc = L["Show location coordinates in notes."],
				},
				showCreator = {
					order = 6,
					type = "toggle",
					name = L["Show note creator"],
					desc = L["Show the 'Created by' line in notes."],
				},
				showDistance = {
					order = 7,
					type = "toggle",
					name = L["Show distance"],
					desc = L["Show the distance in notes."],
				},
				metric = {
					order = 8,
					type = "toggle",
					name = L["Metric"],
					desc = L["Show the distance in meters."],
				},

				minimapDesc = {
					order = 20,
					type = "description",
					name = "\n\n" .. L["You can control the display of the notes on the minimap."],
				},
				minimapIcons = {
					order = 21,
					type = "toggle",
					name = L["Show minimap icons"],
					desc = L["Show icons on the minimap."],
					get = function(info) return db.minimapIcons end,
					set = function(info, value) Notes:ToggleShowingMinimapIcons(value) end,
				},
				minimapIconSize = {
					order = 22,
					type = "range",
					name = L["Minimap icon size"],
					desc = L["Size of the icons on the minimap."],
					min = 0.5, max = 2, step = 0.01, bigStep = 0.05,
					isPercent = true,
					get = function(info) return db.minimapIconSize end,
					set = function(info, value)
						db.minimapIconSize = value
						forceNextMinimapUpdate = true
					end,
				},
				minimapOtherZone = {
					order = 23,
					type = "toggle",
					name = L["Display icons from other zones"],
					desc = L["Show icons in other zones."],
					width = "double",
					get = function(info) return db.minimapOtherZone end,
					set = function(info, value)
						db.minimapOtherZone = value
						forceNextMinimapUpdate = true
					end,
				},
				minimapTooltip = {
					order = 24,
					type = "toggle",
					name = L["Mouseover tooltips"],
					desc = L["Display a tooltip containing information about notes, when they are moused over."],
				},
				minimapTooltipScale = {
					order = 25,
					name = L["Tooltip scale"],
					desc = L["Scale of the minimap tooltip"],
					type = "range",
					min = 0.2, max = 2, step = 0.01, bigStep = 0.05,
					isPercent = true,
				},
				minimapMenu = {
					order = 26,
					type = "toggle",
					name = L["Contextual menu"],
					desc = L["Show a menu when right-clicking on a note allowing you to edit or clear notes."],
				},
				minimapCreateClick = {
					order = 27,
					type = "toggle",
					name = L["Control-Rightclick to create"],
					desc = L["Allow control-right clicking on map to create new notes."],
					width = "double",
				},
				minimapNotesPerDB = {
					order = 28,
					type = "range",
					name = L["Max minimap icons per database"],
					desc = L["The maximum amount of minimap icons to show per database."],
					width = "double",
					min = 5, max = 100, step = 1, bigStep = 5,
					get = function(info) return db.minimapNotesPerDB end,
					set = function(info, value)
						db.minimapNotesPerDB = value
						forceNextMinimapUpdate = true
					end,
				},

				worldmapDesc = {
					order = 40,
					type = "description",
					name = "\n\n" .. L["You can control the display of the Notes on the world map."],
				},
				worldmapIcons = {
					order = 41,
					type = "toggle",
					name = L["Show world map icons"],
					desc = L["Show icons on the world map."],
				},
				worldmapIconSize = {
					order = 42,
					type = "range",
					name = L["World map icon size"],
					desc = L["Size of the icons on the world map."],
					min = 0.5, max = 2, step = 0.01, bigStep = 0.05,
					isPercent = true,
					get = function(info) return db.worldmapIconSize end,
					set = function(info, value)
						db.worldmapIconSize = value
						if Notes:IsEnabled() then
							Notes:RefreshMap()
						end
					end,
				},
				worldmapOtherZone = {
					order = 43,
					type = "toggle",
					name = L["Display icons from other zones"],
					desc = L["Show icons in other zones."],
					width = "double",
					get = function(info) return db.worldmapOtherZone end,
					set = function(info, value)
						db.worldmapOtherZone = value
						if Notes:IsEnabled() then
							Notes:RefreshMap()
						end
					end,
				},
				worldmapTooltip = {
					order = 44,
					type = "toggle",
					name = L["Mouseover tooltips"],
					desc = L["Display a tooltip containing information about notes, when they are moused over."],
				},
				worldmapTooltipScale = {
					order = 45,
					name = L["Tooltip scale"],
					desc = L["Scale of the World Map tooltip."],
					type = "range",
					min = 0.2, max = 2, step = 0.01, bigStep = 0.05,
					isPercent = true,
				},
				worldmapMenu = {
					order = 46,
					type = "toggle",
					name = L["Contextual menu"],
					desc = L["Show a menu when right-clicking on a note allowing you to edit or clear notes."],
				},
				worldmapCreateClick = {
					order = 47,
					type = "toggle",
					name = L["Control-Rightclick to create"],
					desc = L["Allow control-right clicking on the world map to create new notes."],
					width = "double",
				},
				createModifier = {
					order = 48,
					type = "select",
					name = L["Create note modifier"],
					desc = L["This setting changes the modifier used by Notes when right-clicking on the world map to create a note."],
					width = "double",
					values = {
						["A"] = "Alt",
						["C"] = "Ctrl",
						["S"] = "Shift",
						["AC"] = "Alt-Ctrl",
						["AS"] = "Alt-Shift",
						["CS"] = "Ctrl-Shift",
						["ACS"] = "Alt-Ctrl-Shift",
					},
				},

				commActive = {
					order = 90,
					type = "toggle",
					name = L["Note reception active"],
					desc = L["Enabling this will allow notes to be sent and received."],
				},
				commFilter = {
					order = 91,
					type = "select",
					name = L["Incoming note filtering"],
					desc = L["Setup filtering of note sent to you by other players."],
					hidden = function() return not Notes:IsBuiltinIncomingNoteFilter() end,
					disabled = function() return not db.commActive end,
					values = {
						["all"] = L["Accept all notes"],
						["handmade"] = L["Accept only handmade notes"],
						["none"] = L["Reject all notes"],
					},
				},
				showNotifications = {
					order = 92,
					type = "toggle",
					name = L["Show chat notifications"],
					desc = L["Show notification in the chat window."],
					width = "double",
				},
				clearnotes = {
					order = 99,
					type = "execute",
					name = L["Clear all notes on map"],
					desc = L["This will clear all notes on map."],
					func = "ClearAllNotes",
					confirm = function(info)
						return L["This will remove all notes on the map. This can't be undone, are your sure?"]
					end,
				},
			},
		}
	end

	return options
end

---------------------
-- LOCAL FUNCTIONS --
---------------------

local icons = {}
local iconCats = {}

-- getIconTitle --
local function getIconTitle(icon)
	if type(icon) ~= "string" then return "Unknown" end
	if icons[icon] then
		return icons[icon].text
	elseif icon.find and string_find(icon, "^Interface\\") then
		return L["Custom icon"]
	else
		return icon
	end
end

-- getZoneAndLevel --
local function getZoneAndLevel(zoneTexture)
	if not zoneTexture then return end
	if string_match(zoneTexture, ":%d+$") then
		local zone, level = string_match(zoneTexture, "^(.+):(%d+)$")
		local C, Z = Astrolabe:GetMapCZ(zone)
		level = level + 0
		if C then
			return zone, level
		else
			Notes:Print(zoneTexture.." doesn't exist!")
			return nil, nil
		end
	end
	return zoneTexture, 0
end

-- isCurrentLevel --
local function isCurrentLevel(level)
	local current = GetCurrentMapDungeonLevel()
	if current < 0 then
		current = 0
	end

	if level < 0 then
		level = 0
	end

	return level == current
end

-- purifyZone --
local function purifyZone(zone)
	local zone, level = getZoneAndLevel(zone)
	if level == 0 then
		return zone
	else
		return zone .. ":" .. level
	end
end

-- round --
local function round2(num, digits)
	if not digits then digits = 0 end
	local mantissa = 10^digits
	local norm = num*mantissa
	norm = norm + 0.5
	local norm_f = math_floor(norm)
	if norm == norm_f and (norm_f % 2) ~= 0 then
		return (norm_f-1)/mantissa
	end
	return norm_f/mantissa
end

-- round --
local function round(num, digits)
	if not digits then digits = 0 end
	local mantissa = 10^digits
	local norm = num*mantissa
	norm = norm + 0.5
	local norm_f = math_floor(norm)
	return norm_f/mantissa
end

-- GetCursorMapLocation --
local function GetCursorMapLocation(button)
	local x, y = GetCursorPosition()
	local left, top = button:GetLeft(), button:GetTop()
	local width = button:GetWidth()
	local height = button:GetHeight()
	local scale = button:GetEffectiveScale()

	return (x/scale - left) / width, (top - y/scale) / height
end

-- getID --
local function getID(x, y)
	if not x or not y then return nil end
	return round(x * 10000, 0) + round(y * 10000, 0) * 10000
end
Notes.getID = getID

-- getXY --
local function getXY(id)
	if not id then return nil, nil end
	return (id % 10000) / 10000, math_floor(id / 10000) / 10000
end
Notes.getXY = getXY

-- getColorID --
local function getColorID(r, g, b)
	if not r then r = 1 end
	if not g then g = 0.82 end
	if not b then b = 0 end
	r = round(r*255, 0)
	g = round(g*255, 0)
	b = round(b*255, 0)
	if r > 255 then
		r = 255
	elseif r < 0 then
		r = 0
	end
	if g > 255 then
		g = 255
	elseif g < 0 then
		g = 0
	end
	if b > 255 then
		b = 255
	elseif b < 0 then
		b = 0
	end
	return r*65536 + g*256 + b
end
Notes.getColorID = getColorID
local goldColorID = getColorID(1, 0.82, 0)

-- getRGB --
local function getRGB(id)
	if not id then
		return 1, 0.82, 0
	end
	local r = math_floor(id/65536)/255
	local g = (math_floor(id/256) % 256)/255
	local b = (id % 256)/255
	return r, g, b
end
Notes.getRGB = getRGB


-- getrawpoi --
local function getrawpoi(zone, id, creator)
	if creator then
		local v = Notes.externalDBs[creator]
		if v and rawget(v, zone) and rawget(v[zone], id) then
			return v[zone][id], creator
		end
		return
	end
	for k,v in pairs(Notes.externalDBs) do
		if rawget(v, zone) and rawget(v[zone], id) then
			return v[zone][id], k
		end
	end
	if rawget(Notes.db.global.pois, zone) then
		local t = rawget(Notes.db.global.pois[zone], id)
		if t then
			return t, nil
		end
	end
end

-- getpoi --
local function getpoi(zone, id, creator)
	if creator then
		local v = Notes.externalDBs[creator]
		if v and v[zone] and v[zone][id] then
			return v[zone][id], creator
		end
		return
	end
	for k,v in pairs(Notes.externalDBs) do
		if v[zone] and v[zone][id] then
			return v[zone][id], k
		end
	end
	local t = Notes.db.global.pois[zone][id]
	return t, nil
end

-- checkParameter --
local function checkParameter(method, parameter, position, datatype, ...)
	local datatype2, datatype3, datatype4, datatype5 = ...
	if datatype2 then
		if datatype2 == "nil" then
			if type(parameter) ~= datatype and parameter ~= nil then error(("Bad argument #%d to 'Notes:%s'. Expected %q or nil, got %q."):format(position, method, datatype, type(parameter)), 2) end
		else
			if type(parameter) ~= datatype and type(parameter) ~= datatype2 then error(("Bad argument #%d to 'Notes:%s'. Expected %q or %q, got %q."):format(position, method, datatype, datatype2, type(parameter)), 2) end
		end
	else
		if type(parameter) ~= datatype then error(("Bad argument #%d to 'Notes:%s'. Expected %q, got %q."):format(position, method, datatype, type(parameter)), 2) end
	end
end


local NOTES_ICONPATH = "Interface\\AddOns\\MapsterEnhanced\\Textures\\"
local function RegisterAllIcons()
	local iconName = {
		"Square 1", "Square 2", "Square 3", "Square 4", "Square 5", "Square 6", "Square 7", "Square 8", "Square 9", "Square 10", "Square 11", "Square 12",
		"Cross 1", "Cross 2", "Cross 3", "Cross 4", "Cross 5", "Cross 6", "Cross 7", "Cross 8", "Cross 9", "Cross 10", "Cross 11", "Cross 12",
		"Diamond 1", "Diamond 2", "Diamond 3", "Diamond 4", "Diamond 5", "Diamond 6", "Diamond 7", "Diamond 8",
		"Star 1", "Star 2", "Star 3", "Star 4", "Star 5", "Star 6", "Star 7", "Star 8", "Star 9", "Star 10",
		"Circle 1", "Circle 2", "Circle 3", "Circle 4", "Circle 5", "Circle 6", "Circle 7", "Circle 8"
	}
	local iconPath = {
		"Icon1", "Icon2", "Icon3", "Icon4", "Icon5", "Icon6", "Icon7", "Icon8", "Icon9", "Icon10", "Icon11", "Icon12",
		"Icon1c", "Icon2c", "Icon3c", "Icon4c", "Icon5c", "Icon6c", "Icon7c", "Icon8c", "Icon9c", "Icon10c", "Icon11c", "Icon12c",
		"Diamond1", "Diamond2", "Diamond3", "Diamond4", "Diamond5", "Diamond6", "Diamond7", "Diamond8",
		"Star1", "Star2", "Star3", "Star4", "Star5", "Star6", "Star7", "Star8", "Star9", "Star10",
		"Circle1", "Circle2", "Circle3", "Circle4", "Circle5", "Circle6", "Circle7", "Circle8"
	}

	local iconid, iconCategory
	Notes:RegisterIconCategory("1")
	Notes:RegisterIconCategory("2")
	Notes:RegisterIconCategory("3")
	Notes:RegisterIconCategory("4")
	Notes:RegisterIconCategory("5")
	Notes:RegisterIconCategory("6")
	Notes:RegisterIconCategory("7")
	Notes:RegisterIconCategory("8")
	for i = 1, 50 do
		iconid = iconName[i]
		if string_sub(iconName[i], -2, -2) == " " then
			iconid = string_gsub(iconid, "(%w+)%s(%d)", "%10%2")
		else
			iconid = string_gsub(iconid, "(%w+)%s(%d+)", "%1%2")
		end

		if string_find(iconName[i], "Square", 1, true) then
			iconid = "1" .. iconid
			iconCategory = "1"
		elseif string_find(iconName[i], "Cross", 1, true) then
			iconid = "2" .. iconid
			iconCategory = "2"
		elseif string_find(iconName[i], "Diamond", 1, true) then
			iconid = "3" .. iconid
			iconCategory = "3"
		elseif string_find(iconName[i], "Star", 1, true) then
			iconid = "4" .. iconid
			iconCategory = "4"
		elseif string_find(iconName[i], "Circle", 1, true) then
			iconid = "5" .. iconid
			iconCategory = "5"
		end
		Notes:RegisterIcon(iconName[i], {
			text = iconName[i],
			id = iconid,
			cat = iconCategory,
			r = 1,
			g = 0.82,
			b = 0,
			path = NOTES_ICONPATH .. "Notes\\" .. iconPath[i],
			showToUser = true,
		})
	end

	local iconName = { "ObjectIcons_Blue", "ObjectIcons_Blue2", "ObjectIcons_Green", "ObjectIcons_Green2", "ObjectIcons_Red", "ObjectIcons_Red2", "ObjectIcons_Yellow", "ObjectIcons_Yellow2" }
	for i = 1, 8 do
		local text = string_gsub(iconName[i], "ObjectIcons_", "")
		Notes:RegisterIcon(iconName[i], {
			text = text,
			cat = "6",
			r = 1,
			g = 0.82,
			b = 0,
			width = 14,
			height = 14,
			path = NOTES_ICONPATH .. iconName[i],
			showToUser = true,
		})
	end

	local iconName = { RAID_TARGET_1, RAID_TARGET_2, RAID_TARGET_3, RAID_TARGET_4, RAID_TARGET_5, RAID_TARGET_6, RAID_TARGET_7, RAID_TARGET_8 }
	for i = 1, 8 do
		local t = UnitPopupButtons["RAID_TARGET_" .. i]
		Notes:RegisterIcon(iconName[i], {
			text = t.text,
			cat = "7",
			r = t.color.r,
			g = t.color.g,
			b = t.color.b,
			path = "Interface\\TargetingFrame\\UI-RaidTargetingIcons",
			cLeft = t.tCoordLeft,
			cRight = t.tCoordRight,
			cTop = t.tCoordTop,
			cBottom = t.tCoordBottom,
		})
	end

	Notes:RegisterIcon("Unknown", {
		text = UNKNOWN,
		path = "Interface\\Icons\\INV_Misc_QuestionMark",
	})

    local iconPath = "Interface\\MINIMAP\\Tracking"
    Notes:RegisterIcon("TrackAmmunition", {
		text = MINIMAP_TRACKING_VENDOR_AMMO,
		path = iconPath .. "\\Ammunition",
		cat = "8",
		showToUser = true,
    })
    Notes:RegisterIcon("TrackAuctioneer", {
		text = MINIMAP_TRACKING_AUCTIONEER,
		path = iconPath .. "\\Auctioneer",
		cat = "8",
		showToUser = true,
    })
    Notes:RegisterIcon("TrackBanker", {
		text = MINIMAP_TRACKING_BANKER,
		path = iconPath .. "\\Banker",
		cat = "8",
		showToUser = true,
    })
    Notes:RegisterIcon("TrackBattleMaster", {
		text = MINIMAP_TRACKING_BATTLEMASTER,
		path = iconPath .. "\\BattleMaster",
		cat = "8",
		showToUser = true,
    })
    Notes:RegisterIcon("TrackClass", {
		text = MINIMAP_TRACKING_TRAINER_CLASS,
		path = iconPath .. "\\Class",
		cat = "8",
		showToUser = true,
    })
    Notes:RegisterIcon("TrackFlightMaster", {
		text = MINIMAP_TRACKING_FLIGHTMASTER,
		path = iconPath .. "\\FlightMaster",
		cat = "8",
	    showToUser = true,
    })
    Notes:RegisterIcon("TrackFood", {
		text = MINIMAP_TRACKING_VENDOR_FOOD,
		path = iconPath .. "\\Food",
		cat = "8",
		showToUser = true,
    })
    Notes:RegisterIcon("TrackInnkeeper", {
		text = MINIMAP_TRACKING_INNKEEPER,
		path = iconPath .. "\\Innkeeper",
		cat = "8",
		showToUser = true,
    })
    Notes:RegisterIcon("TrackMailbox", {
		text = MINIMAP_TRACKING_MAILBOX,
		path = iconPath .. "\\Mailbox",
		cat = "8",
		showToUser = true,
    })
    Notes:RegisterIcon("TrackPoisons", {
		text = MINIMAP_TRACKING_VENDOR_POISON,
		path = iconPath .. "\\Poisons",
		cat = "8",
		showToUser = true,
    })
    Notes:RegisterIcon("TrackProfession", {
		text = MINIMAP_TRACKING_TRAINER_PROFESSION,
		path = iconPath .. "\\Profession",
		cat = "8",
		showToUser = true,
    })
    Notes:RegisterIcon("TrackReagents", {
		text = MINIMAP_TRACKING_VENDOR_REAGENT,
		path = iconPath .. "\\Reagents",
		cat = "8",
		showToUser = true,
    })
    Notes:RegisterIcon("TrackRepair", {
		text = MINIMAP_TRACKING_REPAIR,
		path = iconPath .. "\\Repair",
		cat = "8",
		showToUser = true,
    })
    Notes:RegisterIcon("TrackStableMaster", {
		text = MINIMAP_TRACKING_STABLEMASTER,
		path = iconPath .. "\\StableMaster",
		cat = "8",
		showToUser = true,
    })
end

------------------------------------
-- FRAME INITIALIZATION FUNCTIONS --
------------------------------------

-- CreateNoteFrame --
local function CreateNoteFrame()
	if noteFrame then return noteFrame end

	noteFrame = CreateFrame("Frame", "MapsterNotes_NewNoteFrame", WorldMapFrame)
	noteFrame:SetPoint("CENTER", WorldMapFrame, "CENTER")
	noteFrame:SetWidth(500)
	noteFrame:SetHeight(330)
	noteFrame:SetFrameStrata("DIALOG")
	noteFrame:SetFrameLevel(9)
	noteFrame:EnableMouse(true)
	noteFrame:SetMovable(true)
	noteFrame:RegisterForDrag("LeftButton")
	noteFrame:SetScript("OnDragStart", function(self)
		self:StartMoving()
	end)
	noteFrame:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
	end)
	noteFrame:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true,
		insets = {
			left = 5,
			right = 5,
			top = 5,
			bottom = 5,
		},
		tileSize = 16,
		edgeSize = 16,
	})
	noteFrame:SetBackdropColor(0.1, 0.1, 0.1, 1)

	local texture = noteFrame:CreateTexture(nil, "ARTWORK")
	texture:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header")
	texture:SetWidth(256)
	texture:SetHeight(64)
	texture:SetPoint("TOP", noteFrame, "TOP", 0, 12)

	local header = noteFrame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	noteFrame.header = header
	header:SetText(L["New note"])
	header:SetPoint("TOP", texture, "TOP", 0, -14)

	local okayButton = CreateFrame("Button", "MapsterNotes_NewNoteFrameOkay", noteFrame, "UIPanelButtonTemplate2")
	okayButton:SetPoint("BOTTOMLEFT", noteFrame, "BOTTOMLEFT", 20, 20)
	okayButton:SetText(OKAY or "Okay")
	okayButton:SetWidth(noteFrame:GetWidth()/2 - 25)

	local cancelButton = CreateFrame("Button", "MapsterNotes_NewNoteFrameCancel", noteFrame, "UIPanelButtonTemplate2")
	cancelButton:SetPoint("BOTTOMRIGHT", noteFrame, "BOTTOMRIGHT", -20, 20)
	cancelButton:SetText(CANCEL or "Cancel")
	cancelButton:SetWidth(noteFrame:GetWidth()/2 - 25)
	cancelButton:SetScript("OnClick", function()
		noteFrame:Hide()
	end)

	local last
	local isGood

	local OnEscapePressed = function(self)
		self:ClearFocus()
	end

	local OnTextChanged = function(self)
		if isGood() then
			okayButton:Enable()
		else
			okayButton:Disable()
		end
	end

	-- CreateNoteFrame -- make() --
	local function make(text, colorful)
		local editBox = CreateFrame("EditBox", nil, noteFrame)
		editBox:SetFontObject(ChatFontNormal)
		editBox:SetWidth(colorful and 210 or 240)
		editBox:SetHeight(12)
		if not last then
			editBox:SetPoint("TOPRIGHT", noteFrame, "TOPRIGHT", -25, -35)
		else
			editBox:SetPoint("TOPLEFT", last, "BOTTOMLEFT", 0, -20)
		end
		last = editBox
		editBox:SetAutoFocus(false)
		editBox:SetScript("OnEscapePressed", OnEscapePressed)
		editBox:SetScript("OnTextChanged", OnTextChanged)

		local backdrop = CreateFrame("Frame", nil, noteFrame)
		backdrop:SetPoint("CENTER", editBox, "CENTER")
		backdrop:SetWidth(colorful and 230 or 260)
		backdrop:SetHeight(24)
		backdrop:SetBackdrop({
			bgFile = "Interface\\TutorialFrame\\TutorialFrameBackground",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			tile = true,
			insets = {
				left = 0,
				right = 0,
				top = 0,
				bottom = 0,
			},
			tileSize = 12,
			edgeSize = 12,
		})

		local label = noteFrame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		label:SetPoint("RIGHT", editBox, "LEFT", -20, 0)
		label:SetPoint("LEFT", noteFrame, "LEFT", 20, 0)
		label:SetPoint("TOP", editBox, "TOP")
		label:SetPoint("BOTTOM", editBox, "BOTTOM")
		label:SetJustifyH("RIGHT")
		label:SetText(text)

		if colorful then
			local button = CreateFrame("Button", nil, editBox)
			button:SetPoint("LEFT", editBox, "RIGHT", 20, 0)
			button:SetWidth(16)
			button:SetHeight(16)
			local texture = button:CreateTexture("ARTWORK")
			texture:SetAllPoints(button)
			texture:SetTexture(1, 1, 1)
			local function changeColor(r, g, b)
				texture:SetTexture(r, g, b)
				editBox:SetTextColor(r, g, b)
				Dewdrop:Close()
			end
			function editBox.resetColor(r, g, b)
				if not r then
					r, g, b = 1, 0.82, 0
				end
				texture:SetTexture(r, g, b)
				editBox:SetTextColor(r, g, b)
			end
			button:SetScript("OnClick", function()
				Dewdrop:Register(button,
					'children', function()
						local r,g,b = editBox:GetTextColor()
						Dewdrop:AddLine(
							'text', L["Custom Color"],
							'textR', r,
							'textG', g,
							'textB', b,
							'hasColorSwatch', true,
							'r', r,
							'g', g,
							'b', b,
							'colorFunc', function(r, g, b)
								texture:SetTexture(r, g, b)
								editBox:SetTextColor(r, g, b)
							end
						)
						Dewdrop:AddLine(
							'text', L["Gold"],
							'textR', 1,
							'textG', 0.82,
							'textB', 0,
							'func', changeColor,
							'arg1', 1,
							'arg2', 0.82,
							'arg3', 0
						)
						Dewdrop:AddLine(
							'text', L["White"],
							'textR', 1,
							'textG', 1,
							'textB', 1,
							'func', changeColor,
							'arg1', 1,
							'arg2', 1,
							'arg3', 1
						)
						Dewdrop:AddLine(
							'text', L["Grey"],
							'textR', 0.8,
							'textG', 0.8,
							'textB', 0.8,
							'func', changeColor,
							'arg1', 0.8,
							'arg2', 0.8,
							'arg3', 0.8
						)
						Dewdrop:AddLine(
							'text', L["Blue"],
							'textR', 0,
							'textG', 0.7,
							'textB', 1,
							'func', changeColor,
							'arg1', 0,
							'arg2', 0.7,
							'arg3', 1
						)
						Dewdrop:AddLine(
							'text', L["Pale blue"],
							'textR', 0.7,
							'textG', 0.82,
							'textB', 0.88,
							'func', changeColor,
							'arg1', 0.7,
							'arg2', 0.82,
							'arg3', 0.88
						)
						Dewdrop:AddLine(
							'text', L["Dark blue"],
							'textR', 0,
							'textG', 0.2,
							'textB', 0.7,
							'func', changeColor,
							'arg1', 0,
							'arg2', 0.2,
							'arg3', 0.7
						)
						Dewdrop:AddLine(
							'text', L["Cyan"],
							'textR', 0,
							'textG', 1,
							'textB', 1,
							'func', changeColor,
							'arg1', 0,
							'arg2', 1,
							'arg3', 1
						)
						Dewdrop:AddLine(
							'text', L["Red"],
							'textR', 1,
							'textG', 0,
							'textB', 0,
							'func', changeColor,
							'arg1', 1,
							'arg2', 0,
							'arg3', 0
						)
						Dewdrop:AddLine(
							'text', L["Orange"],
							'textR', 0.98,
							'textG', 0.57,
							'textB', 0,
							'func', changeColor,
							'arg1', 0.98,
							'arg2', 0.57,
							'arg3', 0
						)
						Dewdrop:AddLine(
							'text', L["Yellow"],
							'textR', 1,
							'textG', 0.92,
							'textB', 0,
							'func', changeColor,
							'arg1', 1,
							'arg2', 0.92,
							'arg3', 0
						)
						Dewdrop:AddLine(
							'text', L["Green"],
							'textR', 0,
							'textG', 0.7,
							'textB', 0,
							'func', changeColor,
							'arg1', 0,
							'arg2', 0.7,
							'arg3', 0
						)
						Dewdrop:AddLine(
							'text', L["Light green"],
							'textR', 0.04,
							'textG', 0.95,
							'textB', 0,
							'func', changeColor,
							'arg1', 0.04,
							'arg2', 0.95,
							'arg3', 0
						)
						Dewdrop:AddLine(
							'text', L["Pink"],
							'textR', 1,
							'textG', 0.62,
							'textB', 0.59,
							'func', changeColor,
							'arg1', 1,
							'arg2', 0.62,
							'arg3', 0.59
						)
						Dewdrop:AddLine(
							'text', L["Purple"],
							'textR', 0.83,
							'textG', 0.22,
							'textB', 0.9,
							'func', changeColor,
							'arg1', 0.83,
							'arg2', 0.22,
							'arg3', 0.9
						)
					end,
					'dontHook', true,
					'point', "TOPRIGHT",
					'relativePoint', "BOTTOMRIGHT"
				)
				button:SetScript("OnClick", function()
					if Dewdrop:IsOpen(button) then
						Dewdrop:Close()
					else
						Dewdrop:Open(button)
					end
					PlaySound("igMainMenuOptionCheckBoxOn")
				end)
				button:GetScript("OnClick")()
			end)
		end

		return editBox
	end

	noteFrame.xEditBox = make(L["X position"])
	noteFrame.yEditBox = make(L["Y position"])
	noteFrame.zone = make(ZONE or "Zone")
	noteFrame.zone:SetScript("OnEditFocusGained", function(self)
		self:ClearFocus()
	end)
	noteFrame.title = make(L["Title"], true)
	noteFrame.info1 = make(L["Info line 1 (optional)"], true)
	noteFrame.info2 = make(L["Info line 2 (optional)"], true)
	noteFrame.creator = make(L["Creator (optional)"])

	noteFrame.xEditBox:SetScript("OnTabPressed", function(self)
		self:ClearFocus()
		noteFrame.yEditBox:SetFocus()
	end)
	noteFrame.yEditBox:SetScript("OnTabPressed", function(self)
		self:ClearFocus()
		noteFrame.title:SetFocus()
	end)
	noteFrame.zone:SetScript("OnTabPressed", noteFrame.yEditBox:GetScript("OnTabPressed"))
	noteFrame.title:SetScript("OnTabPressed", function(self)
		self:ClearFocus()
		noteFrame.info1:SetFocus()
	end)
	noteFrame.info1:SetScript("OnTabPressed", function(self)
		self:ClearFocus()
		noteFrame.info2:SetFocus()
	end)
	noteFrame.info2:SetScript("OnTabPressed", function(self)
		self:ClearFocus()
		noteFrame.creator:SetFocus()
	end)
	noteFrame.creator:SetScript("OnTabPressed", function(self)
		self:ClearFocus()
		noteFrame.xEditBox:SetFocus()
	end)
	-- CreateNoteFrame -- isGood() --
	function isGood()
		local x = tonumber(noteFrame.xEditBox:GetText())
		if not x then
			return false
		elseif x < 0 or x > 100 then
			return false
		end
		local y = tonumber(noteFrame.yEditBox:GetText())
		if not y then
			return false
		elseif y < 0 or y > 100 then
			return false
		end
		return true
	end

	local icon = CreateFrame("Frame", "MapsterNotes_NewNoteFrameIcon", noteFrame)
	noteFrame.icon = icon
	icon:SetPoint("TOPLEFT", noteFrame.creator, "BOTTOMLEFT", -20, -10)
	icon:SetWidth(noteFrame:GetWidth()/2 - 25)
	icon:SetHeight(30)

	local texture = icon:CreateTexture(nil, "ARTWORK")
	texture:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
	texture:SetWidth(25)
	texture:SetHeight(64)
	texture:SetPoint("TOPLEFT", icon, "TOPLEFT", 0, 17)
	texture:SetTexCoord(0, 0.1953125, 0, 1)

	local texture2 = icon:CreateTexture(nil, "ARTWORK")
	texture2:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
	texture2:SetWidth(115)
	texture2:SetHeight(64)
	texture2:SetPoint("LEFT", texture, "RIGHT")
	texture2:SetTexCoord(0.1953125, 0.8046875, 0, 1)

	local texture3 = icon:CreateTexture(nil, "ARTWORK")
	texture3:SetTexture("Interface\\Glues\\CharacterCreate\\CharacterCreate-LabelFrame")
	texture3:SetWidth(25)
	texture3:SetHeight(64)
	texture3:SetPoint("LEFT", texture2, "RIGHT")
	texture3:SetTexCoord(0.8046875, 1, 0, 1)

	local fontstring = icon:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
	fontstring:SetJustifyH("RIGHT")
	fontstring:SetWidth(0)
	fontstring:SetHeight(10)
	fontstring:SetPoint("RIGHT", texture3, "RIGHT", -43, 2)

	local image = icon:CreateTexture(nil, "OVERLAY")
	image:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	image:SetWidth(16)
	image:SetHeight(16)
	image:SetTexCoord(0, 0.25, 0, 0.25)
	image:SetPoint("LEFT", texture, "LEFT", 20, 0)

	local button = CreateFrame("Button", "MapsterNotes_NewNoteFrameIconButton", icon)
	button:SetWidth(24)
	button:SetHeight(24)
	button:SetPoint("TOPRIGHT", texture3, "TOPRIGHT", -16, -18)
	button:SetNormalTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up")
	button:SetPushedTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Down")
	button:SetDisabledTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Disabled")
	button:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight")
	-- CreateNoteFrame -- iconSelector --
	local iconSelector = function(k)
		icon.value = k
		local t = icons[k]
		if string_find(k, "^Interface\\") then
			if noteFrame.title:GetText() == fontstring:GetText() then
				noteFrame.title:SetText(L["Custom icon"])
			end
			fontstring:SetText(L["Custom icon"])
			fontstring:SetTextColor(1, 0.82, 0)
			image:SetTexture(k)
			if string_find(k, "^Interface\\Icons\\") then
				image:SetTexCoord(0.05, 0.95, 0.05, 0.95)
			else
				image:SetTexCoord(0, 1, 0, 1)
			end
			Dewdrop:Close()
			return
		end
		if not t then
			t = icons.Unknown
		end
		if noteFrame.title:GetText() == fontstring:GetText() then
			noteFrame.title:SetText(t.text)
		end
		fontstring:SetText(t.text)
		fontstring:SetTextColor(t.r or 1, t.g or 0.82, t.b or 0)
		image:SetTexture(t.path)
		image:SetTexCoord(t.cLeft or 0, t.cRight or 1, t.cTop or 0, t.cBottom or 1)
		Dewdrop:Close()
	end
	noteFrame.iconSelector = iconSelector
	button:SetScript("OnClick", function()
		local t = {}
		local cats = {}
		-- CreateNoteFrame -- mysort --
		local mysort = function(alpha, bravo)
			if not alpha or not bravo then
				return false
			end
			local alpha_text = icons[alpha].id or alpha
			local bravo_text = icons[bravo].id or bravo
			return alpha_text < bravo_text
		end
		Dewdrop:Register(icon,
			'children', function(level, value)
				for k, v in pairs(icons) do
					if v.showToUser then
						table_insert(t, k)
					end
				end
				table_sort(t, mysort)
				if level == 1 then
					Dewdrop:AddLine(
						'text', L["Squares"],
						'tooltipTitle', L["Squares"],
						'hasArrow', true,
						'value', "Squares"
					)
					Dewdrop:AddLine(
						'text', L["Crosses"],
						'tooltipTitle', L["Crosses"],
						'hasArrow', true,
						'value', "Crosses"
					)
					Dewdrop:AddLine(
						'text', L["Diamonds"],
						'tooltipTitle', L["Diamonds"],
						'hasArrow', true,
						'value', "Diamonds"
					)
					Dewdrop:AddLine(
						'text', L["Stars"],
						'tooltipTitle', L["Stars"],
						'hasArrow', true,
						'value', "Stars"
					)
					Dewdrop:AddLine(
						'text', L["Circles"],
						'tooltipTitle', L["Circles"],
						'hasArrow', true,
						'value', "Circles"
					)
					Dewdrop:AddLine(
						'text', L["Object icons"],
						'tooltipTitle', L["Object icons"],
						'hasArrow', true,
						'value', "ObjectIcons"
					)
					if icons["Star"] and icons["Star"].showToUser then
						Dewdrop:AddLine(
							'text', L["Raid icons"],
							'tooltipTitle', L["Raid icons"],
							'hasArrow', true,
							'value', "RaidIcons"
						)
					end
					Dewdrop:AddLine(
						'text', L["Tracking icons"],
						'tooltipTitle', L["Tracking icons"],
						'hasArrow', true,
						'value', "TrackingIcons"
					)
					local customIcons
					for _, k in ipairs(t) do
						if icons[k].cat and icons[k].showToUser then
							if not tonumber(icons[k].cat) then cats[icons[k].cat] = true end
							customIcons = true
						end
					end
					if customIcons then
						for k, v in pairs(cats) do
							Dewdrop:AddLine(
								'text', k,
								'tooltipTitle', k,
								'hasArrow', true,
								'value', "CustomIcons_"..k
							)
						end
					end
				elseif level == 2 then
					if value == "Squares" then
						for _, k in ipairs(t) do
							if string_find(k, "^Square ") and icons[k].cat == "1" then
								local v = icons[k]
								Dewdrop:AddLine(
									'text', v.text or k,
									'textR', v.r or 1,
									'textG', v.g or 0.82,
									'textB', v.b or 0,
									'icon', v.path,
									'iconCoordLeft', v.cLeft or 0,
									'iconCoordRight', v.cRight or 1,
									'iconCoordTop', v.cTop or 0,
									'iconCoordBottom', v.cBottom or 1,
									'func', iconSelector,
									'arg1', k
								)
							end
						end
					elseif value == "Crosses" then
						for _, k in ipairs(t) do
							if string_find(k, "^Cross ") and icons[k].cat == "2" then
								local v = icons[k]
								Dewdrop:AddLine(
									'text', v.text or k,
									'textR', v.r or 1,
									'textG', v.g or 0.82,
									'textB', v.b or 0,
									'icon', v.path,
									'iconCoordLeft', v.cLeft or 0,
									'iconCoordRight', v.cRight or 1,
									'iconCoordTop', v.cTop or 0,
									'iconCoordBottom', v.cBottom or 1,
									'func', iconSelector,
									'arg1', k
								)
							end
						end
					elseif value == "Diamonds" then
						for _, k in ipairs(t) do
							if string_find(k, "^Diamond ") and icons[k].cat == "3" then
								local v = icons[k]
								Dewdrop:AddLine(
									'text', v.text or k,
									'textR', v.r or 1,
									'textG', v.g or 0.82,
									'textB', v.b or 0,
									'icon', v.path,
									'iconCoordLeft', v.cLeft or 0,
									'iconCoordRight', v.cRight or 1,
									'iconCoordTop', v.cTop or 0,
									'iconCoordBottom', v.cBottom or 1,
									'func', iconSelector,
									'arg1', k
								)
							end
						end
					elseif value == "Stars" then
						for _, k in ipairs(t) do
							if string_find(k, "^Star ") and icons[k].cat == "4" then
								local v = icons[k]
								Dewdrop:AddLine(
									'text', v.text or k,
									'textR', v.r or 1,
									'textG', v.g or 0.82,
									'textB', v.b or 0,
									'icon', v.path,
									'iconCoordLeft', v.cLeft or 0,
									'iconCoordRight', v.cRight or 1,
									'iconCoordTop', v.cTop or 0,
									'iconCoordBottom', v.cBottom or 1,
									'func', iconSelector,
									'arg1', k
								)
							end
						end
					elseif value == "Circles" then
						for _, k in ipairs(t) do
							if string_find(k, "^Circle ") and icons[k].cat == "5" then
								local v = icons[k]
								Dewdrop:AddLine(
									'text', v.text or k,
									'textR', v.r or 1,
									'textG', v.g or 0.82,
									'textB', v.b or 0,
									'icon', v.path,
									'iconCoordLeft', v.cLeft or 0,
									'iconCoordRight', v.cRight or 1,
									'iconCoordTop', v.cTop or 0,
									'iconCoordBottom', v.cBottom or 1,
									'func', iconSelector,
									'arg1', k
								)
							end
						end
					elseif value == "ObjectIcons" then
						for _, k in ipairs(t) do
							if string_find(k, "^ObjectIcons_") and icons[k].cat == "6" then
								local v = icons[k]
								Dewdrop:AddLine(
									'text', v.text or k,
									'textR', v.r or 1,
									'textG', v.g or 0.82,
									'textB', v.b or 0,
									'icon', v.path,
									'iconCoordLeft', v.cLeft or 0,
									'iconCoordRight', v.cRight or 1,
									'iconCoordTop', v.cTop or 0,
									'iconCoordBottom', v.cBottom or 1,
									'func', iconSelector,
									'arg1', k
								)
							end
						end
					elseif value == "RaidIcons" then
						for _, k in ipairs(t) do
							if ((k == RAID_TARGET_1) or (k == RAID_TARGET_2) or (k == RAID_TARGET_3) or (k == RAID_TARGET_4) or (k == RAID_TARGET_5) or (k == RAID_TARGET_6) or (k == RAID_TARGET_7) or (k == RAID_TARGET_8)) and icons[k].cat == "7" then
								local v = icons[k]
								Dewdrop:AddLine(
									'text', v.text or k,
									'textR', v.r or 1,
									'textG', v.g or 0.82,
									'textB', v.b or 0,
									'icon', v.path,
									'iconCoordLeft', v.cLeft or 0,
									'iconCoordRight', v.cRight or 1,
									'iconCoordTop', v.cTop or 0,
									'iconCoordBottom', v.cBottom or 1,
									'func', iconSelector,
									'arg1', k
								)
							end
						end
					elseif value == "TrackingIcons" then
						for _, k in ipairs(t) do
							if string_find(k, "^Track") and icons[k].cat == "8" then
								local v = icons[k]
								Dewdrop:AddLine(
									'text', v.text or k,
									'textR', v.r or 1,
									'textG', v.g or 0.82,
									'textB', v.b or 0,
									'icon', v.path,
									'iconCoordLeft', v.cLeft or 0,
									'iconCoordRight', v.cRight or 1,
									'iconCoordTop', v.cTop or 0,
									'iconCoordBottom', v.cBottom or 1,
									'func', iconSelector,
									'arg1', k
								)
							end
						end
					elseif string_find(value, "^CustomIcons") then
						for _, k in ipairs(t) do
							if "CustomIcons_"..icons[k].cat == value then
								local v = icons[k]
								Dewdrop:AddLine(
									'text', v.text or k,
									'textR', v.r or 1,
									'textG', v.g or 0.82,
									'textB', v.b or 0,
									'icon', v.path,
									'iconCoordLeft', v.cLeft or 0,
									'iconCoordRight', v.cRight or 1,
									'iconCoordTop', v.cTop or 0,
									'iconCoordBottom', v.cBottom or 1,
									'func', iconSelector,
									'arg1', k
								)
							end
						end
					end
				end
				for i = 1, #t do
					t[i] = nil
				end
				for k, v in pairs(cats) do
					cats[k] = nil
				end
			end,
			'dontHook', true,
			'point', "TOPLEFT",
			'relativePoint', "BOTTOMLEFT"
		)
		button:SetScript("OnClick", function()
			if Dewdrop:IsOpen(icon) then
				Dewdrop:Close()
			else
				Dewdrop:Open(icon)
			end
			PlaySound("igMainMenuOptionCheckBoxOn")
		end)
		button:GetScript("OnClick")()
	end)
	icon:SetScript("OnHide", function()
		if Dewdrop:IsOpen(icon) then
			Dewdrop:Close()
		end
	end)

	okayButton:SetScript("OnClick", function()
		local r, g, b = noteFrame.title:GetTextColor()
		local r2, g2, b2 = noteFrame.info1:GetTextColor()
		local r3, g3, b3 = noteFrame.info2:GetTextColor()
		local x, y = tonumber(noteFrame.xEditBox:GetText())/100, tonumber(noteFrame.yEditBox:GetText())/100
		x,y, r,g,b, r2,g2,b2, r3,g3,b3 = round(x, 3),round(y, 3), round(r, 2),round(g, 2),round(b, 2), round(r2, 2),round(g2, 2),round(b2, 2), round(r3, 2),round(g3, 2),round(b3, 2)
		Notes:SetNote(noteFrame.zonename, x, y, noteFrame.icon.value, noteFrame.creator:GetText(),
			'title', noteFrame.title:GetText(),
			'titleR', r,
			'titleG', g,
			'titleB', b,
			'info', noteFrame.info1:GetText(),
			'infoR', r2,
			'infoG', g2,
			'infoB', b2,
			'info2', noteFrame.info2:GetText(),
			'info2R', r3,
			'info2G', g3,
			'info2B', b3,
			'manual', true,
			'oldId', noteFrame.id)
		noteFrame:Hide()
	end)
	noteFrame:Hide()

	return noteFrame
end

------------------------------
-- INITIALIZATION FUNCTIONS --
------------------------------

-- Notes:OnInitialize() --
function Notes:OnInitialize()
	self.db = Mapster.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile
	global_db = self.db.global

	self.externalDBs = {}
	self.handlers = {}
	self.cantDelete = {}
	RegisterAllIcons()

	self:SetEnabledState(Mapster:GetModuleEnabled(MODNAME))
	Mapster:RegisterModuleOptions(MODNAME, getOptions, L["Notes"])

	-- Register options table and slash command
	self:RegisterChatCommand("note", function(command) self:AddSlashCommand("note", command) end)
	self:RegisterChatCommand("addnote", function(command) self:AddSlashCommand("note", command) end)
	self:RegisterChatCommand("qnote", function(command) self:AddSlashCommand("qnote", command) end)
	self:RegisterChatCommand("quicknote", function(command) self:AddSlashCommand("qnote", command) end)
	self:RegisterChatCommand("tnote", function(command) self:AddSlashCommand("tnote", command) end)
	self:RegisterChatCommand("targetnote", function(command) self:AddSlashCommand("tnote", command) end)
	Mapster:GetPrint(MODNAME)

	hooksecurefunc("WorldMapUnit_OnEnter", function() WorldMapTooltip:SetFrameStrata("TOOLTIP") end)

	self:InitializeZones()
end

-- Notes:OnEnable() --
function Notes:OnEnable()
	if not self:IsEnabled() then return end
	Waypoints = Mapster:GetModule("Waypoints")
	Coords = Mapster:GetModule("Coords")
	if not noteFrame then
		CreateNoteFrame()
	end
	self:ConfigureUnitPOIs()

	if not self.hooked then
		self:RawHookScript(WorldMapButton, "OnMouseUp", "WorldMapButton_OnClick")
		self:SecureHook("ToggleFrame", "ToggleFrame")
		self:RegisterEvent("WORLD_MAP_UPDATE", function()
			self:RefreshMap(false)
			if string_match(self:GetRealZoneTexture() or self:GetCurrentMapTexture(), "Orgrimmar") then
				self.zoneUpdateFrame:Show()
			end
		end)
		self.hooked = true
	end
	if WorldMapFrame:IsShown() then
		self:RefreshMap(false)
	end

	local func = function(button, mouseButton)
		self:WorldMapButton_OnClick(WorldMapButton, mouseButton)
	end
	WorldMapPlayer:SetScript("OnMouseUp", func)
	for i = 1, 4 do
		_G["WorldMapParty" .. i]:SetScript("OnMouseUp", func)
	end
	for i = 1, 40 do
		_G["WorldMapRaid" .. i]:SetScript("OnMouseUp", func)
	end
	WorldMapFlag1:SetScript("OnMouseUp", func)
	WorldMapFlag2:SetScript("OnMouseUp", func)
	WorldMapCorpse:SetScript("OnMouseUp", func)
	WorldMapDeathRelease:SetScript("OnMouseUp", func)

	self:RegisterComm(COMM_PREFIX, "OnCommReceived")

	if CUSTOM_CLASS_COLORS then
		RAID_CLASS_COLORS = CUSTOM_CLASS_COLORS
	end

	if db.minimapIcons then
		if not self.minimapIconsUpdateFrame then
			self.minimapIconsUpdateFrame = CreateFrame("Frame", "MapsterNotes_MinimapIcons")
		end
		self.minimapIconsUpdateFrame:SetScript("OnUpdate", function(self, elapsed) Notes:UpdateMinimapIcons(elapsed) end)
		self.minimapIconsUpdateFrame:Show()

		if not self.zoneUpdateFrame then
			self.zoneUpdateFrame = CreateFrame("Frame", "MapsterNotes_ZoneUpdate")
		end
		self.zoneUpdateFrame.lastUpdate = 0
		self.zoneUpdateFrame:SetScript("OnUpdate", function(self, elapsed)
			self.lastUpdate = self.lastUpdate + elapsed
			if self.lastUpdate >= 0.1 then
				self:Hide()
				self.lastUpdate = 0
				Notes:ZONE_CHANGED()
			end
		end)

		self:RegisterEvent("MINIMAP_UPDATE_ZOOM")
		self:RegisterEvent("ZONE_CHANGED")
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		self:RegisterEvent("PLAYER_LEAVING_WORLD")
		-- Notes: Do not call self:MINIMAP_UPDATE_ZOOM() here because the CVARs aren't applied yet.
		-- MINIMAP_UPDATE_ZOOM gets fired automatically by wow when it applies the CVARs.
		--self:MINIMAP_UPDATE_ZOOM()
		self:MINIMAP_ZONE_CHANGED()
		self:ZONE_CHANGED_NEW_AREA()
	end
	rotateMinimap = tostring(GetCVar("rotateMinimap")) == "1" -- delay CVar check otherwise compass won't rotate after exitting
	self:RegisterEvent("CVAR_UPDATE")

	if not self.UnitPOIs.hooked then
		self:RawHook("WorldMapUnit_OnEnter", true)
		self:RawHook("WorldMapUnit_OnLeave", true)
		self.UnitPOIs.hooked = true
		self.UnitPOIs.lastUpdate = 0
		self.UnitPOIs:SetScript("OnUpdate", function(self, elapsed) Notes.UnitPOIs:UpdateTooltip(elapsed) end)
	end

	self:configureYards()
	self:Refresh()
end

-- Notes:OnDisable() --
function Notes:OnDisable()
	self.minimapIconsUpdateFrame:SetScript("OnUpdate", nil)
	self.minimapIconsUpdateFrame:Hide()
	self.zoneUpdateFrame:SetScript("OnUpdate", nil)
	self.zoneUpdateFrame:Hide()
	self:UnregisterComm(COMM_PREFIX)
	self:ClearMap()
	WorldMapPlayer:SetScript("OnMouseUp", nil)
	for i = 1, 4 do
		_G["WorldMapParty" .. i]:SetScript("OnMouseUp", nil)
	end
	for i = 1, 40 do
		_G["WorldMapRaid" .. i]:SetScript("OnMouseUp", nil)
	end
	WorldMapFlag1:SetScript("OnMouseUp", nil)
	WorldMapFlag2:SetScript("OnMouseUp", nil)
	WorldMapCorpse:SetScript("OnMouseUp", nil)
	WorldMapDeathRelease:SetScript("OnMouseUp", nil)

	self.UnitPOIs.hooked = nil
	self.UnitPOIs:SetScript("OnUpdate", nil)
	if self.hooked then
		self:Unhook(WorldMapButton, "OnMouseUp")
		self:Unhook("ToggleFrame")
		self:UnregisterEvent("WORLD_MAP_UPDATE")
		self.hooked = nil
	end
	for id in pairs(minimapPois) do
		minimapPois:del(id)
	end
end

-- Notes:Refresh() --
function Notes:Refresh()
	db = self.db.profile
	global_db = self.db.global
	if not self:IsEnabled() then return end
	yards = (db.metric and L["%.0f m"] or L["%.0f yd"])
end


----------------------------
-- GLOBAL FUNCTIONS (API) --
----------------------------

-- Notes:RegisterIcon --
function Notes:RegisterIcon(name, data)
	checkParameter("RegisterIcon", name, 1, "string")
	checkParameter("RegisterIcon", data, 2, "table", "nil")
	if icons[name] then error(("Icon %q already registered"):format(name), 3) end
	if type(data.text) ~= "string" then error("Text in data table must be a string", 3) end
	if type(data.path) ~= "string" then error("Path in data table must be a string", 3) end
	if data.showToUser and not data.cat then error("Category must be specified", 3) end
	if data.cat and not iconCats[data.cat] then	error(("Category %q not registered"):format(data.cat), 3) end

	if not data.cLeft then
		if string_find(data.path, "^Interface\\Icons\\") then
			data.cLeft = 0.05
			data.cRight = 0.95
			data.cTop = 0.05
			data.cBottom = 0.95
		else
			data.cLeft = 0
			data.cRight = 1
			data.cTop = 0
			data.cBottom = 1
		end
	end
	if not data.alpha then
		data.alpha = 1
	end
	if not data.width then
		data.width = 16
	end
	if not data.height then
		data.height = 16
	end
	icons[name] = data
	if noteFrame then
		noteFrame:Hide()
		noteFrame = nil
	end
	self:RefreshMap(false)
end
MapsterNotes.RegisterIcon = Notes.RegisterIcon

-- Notes:RegisterIconCategory --
function Notes:RegisterIconCategory(cat)
	checkParameter("RegisterCategory", cat, 1, "string")
	if iconCats[cat] then error(("Category %q already registered"):format(cat), 3) end

	iconCats[cat] = true
end
MapsterNotes.RegisterIconCategory = Notes.RegisterIconCategory

-- support for icon remapping
local icon_cache = {}
function Notes:GetIconList()
	local list = {}
	for k,v in pairs(icons) do
		local t = next(icon_cache) or {}
		icon_cache[t] = nil
		t.name = v.text
		t.path = v.path
		list[k] = t
	end
	return list
end
MapsterNotes.GetIconList = Notes.GetIconList

function Notes:OverrideIconGraphic(name, newPath)
	checkParameter("OverrideIconGraphic", name, 1, "string")
	checkParameter("OverrideIconGraphic", newPath, 2, "string")
	if not icons[name] then error(("Icon %q not registered"):format(name), 3) end

	local oldPath = icons[name].path
	icons[name].path = newPath
	return oldPath
end
MapsterNotes.OverrideIconGraphic = Notes.OverrideIconGraphic

-- Added a way to see if an icon was registered
function Notes:IsIconRegistered(name)
	checkParameter("IsIconRegistered", name, 1, "string")
	return icons[name] and true
end
MapsterNotes.IsIconRegistered = Notes.IsIconRegistered

function Notes:UnregisterIcon(name)
	checkParameter("UnregisterIcon", name, 1, "string")
	if not icons[name] then error(("Icon %q not registered"):format(name), 3) end
	icons[name] = nil
	if noteFrame then
		noteFrame:Hide()
		noteFrame = nil
	end
	self:RefreshMap(false)
end
MapsterNotes.UnregisterIcon = Notes.UnregisterIcon

function Notes:UnregisterIconCategory(cat)
	checkParameter("UnregisterIconCategory", cat, 1, "string")
	if not iconCats[cat] then error(("Category %q not registered"):format(cat), 3) end
	for k,v in pairs(icons) do
		if v.cat == cat then self:UnregisterIcon(k) end
	end
	iconCats[cat] = nil
end
MapsterNotes.UnregisterIconCategory = Notes.UnregisterIconCategory


----------------------
-- MODULE FUNCTIONS --
----------------------

local MapZones, MapZoneTextures
function Notes:InitializeZones()
	if not MapZones then
		MapZones = {}
		local continents = Astrolabe:GetMapVirtualContinents()
		for C, continentName in pairs(continents) do
			if C < 99 then
				MapZones[Astrolabe:GetMapTexture(C, 0)] = continentName
				local zones = Astrolabe:GetMapVirtualZones(C)
				for Z, zoneName in pairs(zones) do
					MapZones[Astrolabe:GetMapTexture(C, Z)] = zoneName
				end
			else
				MapZones[Astrolabe:GetMapTexture(C, 0)] = continentName --BZL[continentName]
			end
		end
		MapZones[Astrolabe:GetMapTexture(WORLDMAP_COSMIC_ID, 0)] = BZL["Cosmic map"]
		MapZones[Astrolabe:GetMapTexture(0, 0)] = BZL["Azeroth"]
		MapZones["TheArgentColiseum"] = BZL["Trial of the Champion"]
		MapZones["TheArgentColiseumRaid"] = BZL["Trial of the Crusader"]
	end
	if not MapZoneTextures then
		MapZoneTextures = {}
		for mapTexture, mapName in pairs(MapZones) do
			MapZoneTextures[mapName] = mapTexture
		end
		MapZoneTextures["City of Ironforge"] = "Ironforge"
	end
end

local function retNormal(t, position)
	return (next(t, position))
end

local zonesInstances
local function initZonesInstances()
	if not zonesInstances then
		zonesInstances = {}
		for zone, v in pairs(MapZones) do
			zonesInstances[v] = true
		end
	end
	initZonesInstances = nil
end

function Notes:IterateZonesAndInstances()
	if initZonesInstances then
		initZonesInstances()
	end
	return retNormal, zonesInstances, nil
end

function Notes:GetCurrentZoneName()
	local map = GetMapInfo()
	if not map then
		if GetCurrentMapZone() == 0 then
			if GetCurrentMapContinent() == 0 then
				return BZL["Azeroth"]
			elseif GetCurrentMapContinent() == -1 then
				return BZL["Cosmic map"]
			end
		end
	end
	if map == "TheArgentColiseum" then
		if GetCurrentMapAreaID() == 542 then
			return BZL["Trial of the Crusader"]
		else
			return BZL["Trial of the Champion"]
		end
	end
	map = string_match(map, "^(.+)_terrain(%d+)$") or map;
	return MapZones[map]
end

function Notes:GetCurrentEnglishZoneName()
	local map = GetMapInfo()
	if not map then
		if GetCurrentMapZone() == 0 then
			if GetCurrentMapContinent() == 0 then
				return "Azeroth"
			elseif GetCurrentMapContinent() == -1 then
				return "Cosmic map"
			end
		end
	end
	if map == "TheArgentColiseum" then
		if GetCurrentMapAreaID() == 542 then
			return "Trial of the Crusader"
		else
			return "Trial of the Champion"
		end
	end
	map = string_match(map, "^(.+)_terrain(%d+)$") or map;
	return BZR[MapZones[map]]
end

function Notes:GetCurrentMapTexture()
	local mapTexture, mapFloor = GetMapInfo(), GetCurrentMapDungeonLevel()
	if not mapTexture then
		if GetCurrentMapZone() == 0 then
			if GetCurrentMapContinent() == 0 then
				return "World"
			elseif GetCurrentMapContinent() == -1 then
				return "Cosmic"
			end
		end
	end
	if mapTexture == "TheArgentColiseum" then
		if GetCurrentMapAreaID() == 542 then
			mapTexture = "TheArgentColiseumRaid"
		end
	end
	if mapFloor > 0 then
		if mapTexture == "Dalaran" or mapTexture == "Orgrimmar" then
			if mapFloor == 1 then
				return mapTexture
			end
		end
		return mapTexture..":"..mapFloor
	end
	mapTexture = string_match(mapTexture, "^(.+)_terrain(%d+)$") or mapTexture;
	return mapTexture
end

function Notes:GetRealZoneTexture()
	local mapTexture = self:GetMapTexture(GetRealZoneText())
	local mapFloor = GetCurrentMapDungeonLevel()
	if mapTexture and mapFloor > 0 then
		if mapTexture == "Dalaran" or mapTexture == "Orgrimmar" then
			if mapFloor == 1 then
				return mapTexture
			end
		elseif mapTexture == "TheNexus" and GetCurrentMapAreaID() == 803 then
			mapTexture = "TheNexusLegendary"
		end
		local px, py = GetPlayerMapPosition("player")
		if px == 0 and py == 0 then
			return mapTexture
		end
		return mapTexture..":"..mapFloor
	end
	return mapTexture
end

function Notes:GetMapTexture(mapZone)
	if (MapZoneTextures[mapZone]) then
		return MapZoneTextures[mapZone]
	end
end

function Notes:GetZoneFromMapTexture(mapTexture)
	if (MapZones[mapTexture]) then
		return MapZones[mapTexture]
	end
	local C, Z = Astrolabe:GetMapCZ(mapTexture)
	local map = Astrolabe:GetMapTexture(C)
	return MapZones[map]
end


local currentYardWidth, currentYardHeight = 1000, 1000 * 2/3
do
	local x, y, zone
	function Notes:GetCurrentPlayerPosition()
		local px, py = GetPlayerMapPosition("player")
		local pz, myZone = self:GetCurrentMapTexture(), self:GetRealZoneTexture()
		if (px == x and py == y and pz == zone and x) then
			return x, y, zone
		end

		if (px == 0 or py == 0) then
			if WorldMapFrame:IsShown()--[[ and x]] then	-- we know there is a visible world map
				return x, y, zone
			end

			if (myZone and pz == myZone) then
				return
			end

			local lastMapID, lastFloor = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel()
			SetMapToCurrentZone()
			px, py = GetPlayerMapPosition("player")
			pz = self:GetCurrentMapTexture()
			if (px == 0 or py == 0) then
				if (GetCurrentMapContinent() > -1) then
					SetMapZoom(GetCurrentMapContinent())
					px, py = GetPlayerMapPosition("player")
					pz = self:GetCurrentMapTexture()
				end
				if (px == 0 or py == 0) then
					-- we are in an instance or otherwise off the continent map
					if x then
						return x, y, zone
					else
						return
					end
				end
			end
			x, y, zone = px, py, pz
			return x, y, zone
		end
		if not WorldMapFrame:IsShown() and not IsInInstance() and myZone and pz ~= myZone then
			local lastMapID, lastFloor = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel()
			SetMapToCurrentZone()
			px, py = GetPlayerMapPosition("player")
			pz = self:GetCurrentMapTexture()
			SetMapByID(lastMapID) -- set map zoom back to what it was before
			SetDungeonMapLevel(lastFloor)
		end

		if (px and py) then
			local myZone = self:GetRealZoneTexture()
			if (myZone and pz ~= myZone) then
				local mx, my = self:TranslateWorldMapPosition(px, py, pz, myZone)
				if (mx and my) then
					x, y, zone = mx, my, myZone
				else
					x, y, zone = px, py, pz
				end
			else
				x, y, zone = px, py, pz
			end
		end

		return x, y, zone
	end

	function Notes:GetCurrentUnitPosition(unit)
		local px, py = GetPlayerMapPosition(unit or "player")
		local pz = self:GetCurrentMapTexture()

		if (px == 0 or py == 0) then
			if WorldMapFrame:IsShown() then	-- we know there is a visible world map
				return
			end

			local lastMapID, lastFloor = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel()
			SetMapToCurrentZone()
			px, py = GetPlayerMapPosition(unit or "player")
			pz = self:GetCurrentMapTexture()
			if (px == 0 or py == 0) then
				if (GetCurrentMapContinent() > -1) then
					SetMapZoom(GetCurrentMapContinent())
					px, py = GetPlayerMapPosition(unit or "player")
					pz = self:GetCurrentMapTexture()
				end
				if (px == 0 or py == 0) then
					-- we are in an instance or otherwise off the continent map
					local M, F = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel()
					if (M ~= lastMapID or F ~= lastFloor) then
						SetMapByID(lastMapID) -- set map zoom back to what it was before
						SetDungeonMapLevel(lastFloor)
					end
					return
				end
			end
			local M, F = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel()
			if (M ~= lastMapID or F ~= lastFloor) then
				SetMapByID(lastMapID) -- set map zoom back to what it was before
				SetDungeonMapLevel(lastFloor)
			end
		end

		return px, py, pz
	end
end

function Notes:PointToYards(x, y, zone)
	local w, h
	if not zone or zone == self:GetRealZoneTexture() then
		w, h = currentYardWidth, currentYardHeight
	else
		w, h = Astrolabe:GetZoneDimensions(Astrolabe:GetMapCZ(zone))
		if not w then
			w, h = 1000, 1000 * 2/3
		end
	end
	if x then
		return x * w, y * h
	end
	return nil, nil
end

function Notes:GetDistanceToPoint(x, y, zone, px, py, pzone)
	if not px then
		px, py, pzone = self:GetCurrentPlayerPosition()
	end
	if px == 0 or py == 0 or not px or not py then
		return nil
	end
	zone = zone or self:GetRealZoneTexture()
	pzone = pzone or zone
	if not zone or not pzone then
		return nil
	end

	local c2, z2 = Astrolabe:GetMapCZ(pzone)
	local c1, z1 = Astrolabe:GetMapCZ(zone)

	local dist = Astrolabe:ComputeDistance(c1, z1, x, y, c2, z2, px, py)
	return dist
end

function Notes:GetYardsToPoint(x, y, zone, px, py, pzone)
	if not px then
		px, py, pzone = self:GetCurrentPlayerPosition()
	end
	if px == 0 or py == 0 or not px or not py then
		return nil, nil
	end
	zone = zone or self:GetRealZoneTexture()
	pzone = pzone or zone
	if not zone or not pzone then
		return nil, nil
	end

	local c2, z2 = Astrolabe:GetMapCZ(pzone)
	local c1, z1 = Astrolabe:GetMapCZ(zone)

	local dist, diffX, diffY = Astrolabe:ComputeDistance(c1, z1, x, y, c2, z2, px, py)
	return diffX, diffY
end

function Notes:GetZoneYardSize(mapZone, mapFloor)
	if not mapZone then
		mapZone, mapFloor = getZoneAndLevel(self:GetRealZoneTexture())
	end
	if not mapZone then
		return nil, nil
	end

	local yardWidth, yardHeight = Astrolabe:GetZoneDimensions(Astrolabe:GetMapCZ(mapZone, mapFloor))
	return yardWidth, yardHeight
end

function Notes:TranslateWorldMapPosition(x, y, zone, pzone)
	if not zone then
		return nil, nil
	end
	local C, Z = Astrolabe:GetMapCZ(zone)
	local nC, nZ = Astrolabe:GetMapCZ(pzone or self:GetCurrentMapTexture())
	if not C or not Z or not nC or not nZ then
		return nil, nil
	end
	x, y = Astrolabe:TranslateWorldMapPosition(C, Z, x, y, nC, nZ)
	return x, y
end

function Notes:GetDifficultyColor(level)
	if not level then return 1.0, 1.0, 0.0 end
	local c = GetQuestDifficultyColor(level)
	return c.r, c.g, c.b
end

function Notes:GetDifficultyHexColor(level)
	if not level then return "ffff00" end
	local c = GetQuestDifficultyColor(level)
	return format("%02x%02x%02x", c.r*255, c.g*255, c.b*255)
end

function Notes:ConfigureUnitPOIs()
	if setupUnitPOIs then
		return
	end
	local oldWorldMapParty_OnLeave = WorldMapParty1:GetScript("OnLeave")
	function WorldMapUnit_OnLeave(self)
		oldWorldMapParty_OnLeave(self)
	end
	local function func(self, ...)
		WorldMapUnit_OnLeave()
	end
	WorldMapPlayer:SetScript("OnLeave", func)
	local myServer = GetRealmName():trim()
	function WorldMapPlayer:OnTabletRequest()
		local kind = ""
		if self.unit == "player" then
			kind = PLAYER .. ": "
		elseif string_find(self.unit, "^party") then
			kind = PARTY .. ": "
		elseif string_find(self.unit, "^raid") then
			kind = RAID .. ": "
		end
		local name, server = self.name
		if not name then
			name, server = UnitName(self.unit)
		end
		if server and server ~= myServer and server ~= "" then
			name = name .. "-" .. server
		end
		Tablet:SetTitle(kind .. name)
		if self.unit then
			local cat = Tablet:AddCategory('columns', 2)
			local class, filename = UnitClass(self.unit)
			local color = RAID_CLASS_COLORS[filename]
			cat:AddLine(
				'text', CLASS .. ":",
				'text2', class,
				'text2R', color.r,
				'text2G', color.g,
				'text2B', color.b
			)
			local level = UnitLevel(self.unit)
			local playerlevel = UnitLevel("player")
			local r, g, b = Notes:GetDifficultyColor(level)
			cat:AddLine(
				'text', LEVEL .. ":",
				'text2', level,
				'text2R', r,
				'text2G', g,
				'text2B', b
			)
			cat:AddLine(
				'text', RACE .. ":",
				'text2', (UnitRace(self.unit)),
				'text2R', UnitRace(self.unit) == UnitRace("player") and 0 or 1,
				'text2G', 1,
				'text2B', 0
			)
			local gname, grank, granknum = GetGuildInfo(self.unit)
			if not gname then
				if LibGuild:HasMember(name) then
					gname = LibGuild:GetGuildName()
					grank = LibGuild:GetRank(name)
					granknum = LibGuild:GetRankIndex(name)
				end
			end
			if gname then
				local mygname, mygrank, mygranknum = GetGuildInfo("player")
				local r, g, b = 1, 1, 0
				if mygname == gname then
					r, g, b = 0, 1, 0
				end
				cat:AddLine(
					'text', GUILD .. ":",
					'text2', "<" .. gname .. ">",
					'text2R', r,
					'text2G', g,
					'text2B', b
				)
				if mygname == gname then
					local grankdiff = granknum - mygranknum
					if grankdiff >= 2 then
						r, g, b = 0, 1, 0
					elseif grankdiff == 1 then
						r, g, b = 0.5, 1, 0
					elseif grankdiff == 0 then
						r, g, b = 1, 1, 0
					elseif grankdiff == -1 then
						r, g, b = 1, 0.5, 0
					else
						r, g, b = 1, 0, 0
					end
				end
				cat:AddLine(
					'text', RANK .. ":",
					'text2', grank,
					'text2R', r,
					'text2G', g,
					'text2B', b
				)
				if mygname == gname then
					local note, onote = LibGuild:GetNote(name), LibGuild:GetOfficerNote(name)

					if note then
						cat:AddLine(
							'text', NOTE_COLON,
							'text2', note
						)
					end

					if onote then
						cat:AddLine(
							'text', OFFICER_NOTE_COLON,
							'text2', onote
						)
					end
				end
			end
			local posString
			if Coords then
				local highPrec = Coords.db.profile.accuracy
				posString = "%." .. highPrec .. "f, %." .. highPrec .. "f"
			else
				posString = "%.1f, %.1f"
			end
			local x, y = GetPlayerMapPosition(self.unit)
			local dist = Notes:GetDistanceToPoint(x, y, Notes:GetCurrentMapTexture())

			if db.zone then
				Tablet:AddCategory(
					'columns', 2,
					'hideBlankLine', true
				):AddLine(
					'text', L["Zone:"],
					'text2', Notes:GetCurrentZoneName()
				)
			end

			if db.location and x and x > 0 and y and y > 0 then
				cat:AddLine(
					'text', L["Location:"],
					'text2', format(posString, x * 100, y * 100)
				)
			end

			if db.showDistance and dist and dist >= 1 then
				cat:AddLine(
					'text', L["Distance:"],
					'text2', format(yards, dist)
				)
			end
		end
	end
	WorldMapParty1.OnTabletRequest = WorldMapPlayer.OnTabletRequest
	WorldMapRaid1.OnTabletRequest = WorldMapPlayer.OnTabletRequest
	local tmp = {}
	function WorldMapPlayer:OnLineRequest(cat)
		if self.unit == "player" then
			tmp[#tmp+1] = PLAYER .. ": "
		elseif string_find(self.unit, "^party") then
			tmp[#tmp+1] = PARTY .. ": "
		elseif string_find(self.unit, "^raid") then
			tmp[#tmp+1] = RAID .. ": "
		end
		if self.name then
			tmp[#tmp+1] = self.name
		else
			local name, server = UnitName(self.unit)
			tmp[#tmp+1] = name
			if server and server ~= myServer and server ~= "" then
				tmp[#tmp+1] = "-"
				tmp[#tmp+1] = server
			end
			tmp[#tmp+1] = " - |cff"
			local level = UnitLevel(self.unit)
			local playerlevel = UnitLevel("player")
			tmp[#tmp+1] = Notes:GetDifficultyHexColor(level)
			tmp[#tmp+1] = tostring(level)
			tmp[#tmp+1] = "|r |cff"
			local class, filename = UnitClass(self.unit)
			local color = RAID_CLASS_COLORS[filename]
			tmp[#tmp+1] = format("%02x%02x%02x", color.r*255, color.g*255, color.b*255)
			tmp[#tmp+1] = class
			tmp[#tmp+1] = "|r"
			local gname = GetGuildInfo(self.unit)
			if not gname then
				if LibGuild:HasMember(name) then
					gname = LibGuild:GetGuildName()
				end
			end
			if gname then
				local mygname = LibGuild:GetGuildName()
				tmp[#tmp+1] = " - |cff"
				if gname == mygname then
					tmp[#tmp+1] = "00ff00"
				else
					tmp[#tmp+1] = "ffff00"
				end
				tmp[#tmp+1] = "<"
				tmp[#tmp+1] = gname
				tmp[#tmp+1] = ">|r"
			end
		end
		local text = table_concat(tmp, "")
		for i = 1, #tmp do
			tmp[i] = nil
		end
		if self.unit == "player" then
			return 'text', text,
				'hasCheck', true,
				'checked', false
		else
			local tex = _G[self:GetName() .. "Icon"]
			local left, top, _, _, _, _, right, bottom = tex:GetTexCoord()
			local r, g, b = tex:GetVertexColor()
			return 'text', text,
				'hasCheck', true,
				'checked', true,
				'checkIcon', tex:GetTexture(),
				'checkCoordLeft', left,
				'checkCoordRight', right,
				'checkCoordTop', top,
				'checkCoordBottom', bottom,
				'checkColorR', r,
				'checkColorG', g,
				'checkColorB', b
		end
	end
	WorldMapPlayer:SetFrameLevel(WorldMapPlayer:GetFrameLevel()+5)
	PlayerArrowFrame:SetFrameLevel(PlayerArrowFrame:GetFrameLevel()+5)
	PlayerArrowEffectFrame:SetFrameLevel(PlayerArrowEffectFrame:GetFrameLevel()+5)
	WorldMapParty1.OnLineRequest = WorldMapPlayer.OnLineRequest
	WorldMapRaid1.OnLineRequest = WorldMapPlayer.OnLineRequest
	for i = 1, 4 do
		local poi = _G["WorldMapParty" .. i]
		poi:SetScript("OnLeave", func)
		poi.OnTabletRequest = WorldMapParty1.OnTabletRequest
		poi.OnLineRequest = WorldMapParty1.OnLineRequest
		poi:SetFrameLevel(poi:GetFrameLevel()+3)
	end
	for i = 1, 40 do
		local poi = _G["WorldMapRaid" .. i]
		poi:SetScript("OnLeave", func)
		poi.OnTabletRequest = WorldMapRaid1.OnTabletRequest
		poi.OnLineRequest = WorldMapRaid1.OnLineRequest
		poi:SetFrameLevel(poi:GetFrameLevel()+3)
	end

	dummyFrame = CreateFrame("Frame", "MapsterNotes_dummyFrame")
	if not self.pois then
		self.pois = {}
	end
	local tmp = {}
	self.tooltipData = {}
	Tablet:Register(dummyFrame,
		'children', function()
			-- Check player
			if MouseIsOver(WorldMapPlayer) then
				tmp[#tmp+1] = WorldMapPlayer
			end

			-- Check party
			for i = 1, MAX_PARTY_MEMBERS do
				local unitButton = _G["WorldMapParty"..i]
				if unitButton:IsVisible() and MouseIsOver(unitButton) then
					tmp[#tmp+1] = unitButton
				end
			end

			--Check Raid
			for i = 1, MAX_RAID_MEMBERS do
				local unitButton = _G["WorldMapRaid"..i]
				if unitButton:IsVisible() and MouseIsOver(unitButton) then
					tmp[#tmp+1] = unitButton
				end
			end

			for frame in pairs(Notes.pois) do
				if frame:IsVisible() and MouseIsOver(frame) then
					tmp[#tmp+1] = frame
				end
			end

			if #tmp == 1 then
				tmp[1]:OnTabletRequest()
				tmp[1] = nil
			elseif #tmp > 1 then
				local cat = Tablet:AddCategory()
				for i = 1, #tmp do
					cat:AddLine(tmp[i]:OnLineRequest())
					tmp[i] = nil
				end
			end
		end,
		'point', function(parent)
			local x, y = GetCursorPosition()
			local cx, cy = GetScreenWidth()/2, GetScreenHeight()/2
			if x > cx then
				if y < cy then
					return "BOTTOMRIGHT"
				else
					return "TOPRIGHT"
				end
			else
				if y < cy then
					return "BOTTOMLEFT"
				else
					return "TOPLEFT"
				end
			end
		end,
		'relativePoint', function(parent)
			local x, y = GetCursorPosition()
			local cx, cy = GetScreenWidth()/2, GetScreenHeight()/2
			if x > cx then
				if y < cy then
					return "TOPLEFT"
				else
					return "BOTTOMLEFT"
				end
			else
				if y < cy then
					return "TOPRIGHT"
				else
					return "BOTTOMRIGHT"
				end
			end
		end,
		'dontHook', true,
		'data', self.tooltipData
	)
	self:RawHook("WorldMapUnit_OnEnter", true)
	self:RawHook("WorldMapUnit_OnLeave", true)
	self.UnitPOIs.hooked = true
	setupUnitPOIs = true

	self.UnitPOIs.lastUpdate = 0
	self.UnitPOIs:SetScript("OnUpdate", function(self, elapsed) Notes.UnitPOIs:UpdateTooltip(elapsed) end)
end

local TipTacSettings = nil
local isOpen = nil
function Notes:WorldMapUnit_OnEnter(this)
	if not Dewdrop:IsOpen(this) and db.worldmapTooltip then
		isOpen = true
		if this.minimap then
			Tablet:SetFontSizePercent(self.tooltipData, db.minimapTooltipScale)
		else
			Tablet:SetFontSizePercent(self.tooltipData, db.worldmapTooltipScale)
		end
		Tablet:Open(this, dummyFrame)
		if TipTac and not TipTacSettings then
			TipTacSettings = true
			TipTac:AddModifiedTip("Tablet20Frame")
		end
	end
end

function Notes:WorldMapUnit_OnLeave()
	isOpen = nil
	Tablet:Close()
end

local updateInterval = 1
function Notes.UnitPOIs:UpdateTooltip(elapsed)
	self.lastUpdate = self.lastUpdate + elapsed
	if (self.lastUpdate >= updateInterval) then
		self.lastUpdate = 0
		Notes:GetCurrentPlayerPosition()
		if isOpen and dummyFrame then
			Tablet:Refresh(dummyFrame)
		end
	end
end

function Notes:AddPOI(frame)
	if not self.pois then
		self.pois = {}
	end
	self.pois[frame] = true
end

--------------------
-- COMM FUNCTIONS --
--------------------

-- builtin_filter() --
local function builtin_filter(self, zone, x, y, icon, creator, data, sender, fullSender)
	-- Basically, only accept manual notes
	local commFilter = db.commFilter
	return (commFilter == "all") or (commFilter == "handmade" and type(data) == "table" and data.manual)
end
Notes.AcceptIncomingNote = builtin_filter

-- Notes:IsBuiltinIncomingNoteFilter() --
function Notes:IsBuiltinIncomingNoteFilter()
	return builtin_filter == Notes.AcceptIncomingNote
end

-- Notes:OnNoteReceive(zone, x, y, icon, creator, data, sender, fullSender) --
function Notes:OnNoteReceive(zone, x, y, icon, creator, data, sender, fullSender)
	zone = purifyZone(zone)
	local id = getID(x, y)
	local poi_data, db = getrawpoi(zone, id)
	if poi_data and (db or poi_data.creator or "") ~= creator then
		if db.showNotifications then
			self:Print(L["Error: %s sent you a note at %q: %.2f, %.2f with the title %q created by %q, but you already have a note at this location."], fullSender, self:GetZoneFromMapTexture(getZoneAndLevel(zone)), x*100, y*100, data and data.title or getIconTitle(data or icon) or UNKNOWN, creator or UNKNOWN)
		end
		-- Returning true means "note processed, stop now"
		return true
	end
end

-- Notes:OnCommReceived(prefix, message, distribution, sender) --
function Notes:OnCommReceived(prefix, message, distribution, sender)
	if not db.commActive then return end
	if prefix ~= COMM_PREFIX then return end
	if distribution ~= "GUILD" and distribution ~= "RAID" and distribution ~= "PARTY" and distribution ~= "WHISPER" then return end

	local messagetype, zone, x, y, creator, data = strmatch(message, "(%s+):(%s+):(%d+):(%d+):(%s+):(%s+)")

	-- Verify data received
	if type(messagetype) ~= "string" or messagetype ~= "NOTE" then return end
	if type(data) ~= "string" then return end
	local success, data = self:Deserialize(data)
	if not success then return end

	if type(zone) ~= "string" then return end
	if type(x) ~= "number" or x < 0 or x > 1 then return end
	if type(y) ~= "number" or y < 0 or y > 1 then return end
	if type(creator) ~= "string" and creator ~= nil then return end
	if type(data) ~= "table" and type(data) ~= "string" then return end
	if type(data) == "table" and type(data.icon) ~= "string" then return end

	zone = purifyZone(zone)

	local z, level = getZoneAndLevel(zone)
	local zoneName = self:GetZoneFromMapTexture(z)
	if not zoneName then
		self:Print(L["Error: %s sent you a note with an unknown zone: %q"], fullSender, z)
		return
	end

	local usingDB = self.externalDBs[creator] and creator or nil
	if creator and not usingDB then
		return
	end

	local real_icon = (type(data) == "string") and data or data.icon
	local real_creator = (creator or type(data) == "table") and data.creator or ""
	local real_data = (type(data) == "table") and data or nil
	local fullSender = (distribution == "GUILD") and format(L["%s from Guild"], sender) or (distribution == "RAID") and format(L["%s from Raid"], sender) or (distribution == "PARTY") and format(L["%s from Party"], sender) or sender

	if not self:AcceptIncomingNote(zone, x, y, real_icon, real_creator, real_data, sender, fullSender) then
		if db.showNotifications then
			self:Print(L["Note sent by %s ignored due to filtering settings."], fullSender)
		end
		return
	end

	-- turn off the NoteAdd Events
	setNoteFromComm = true

	-- Try handler:OnNoteReceive or overriden self:OnNoteReceive
	local handler = usingDB and self.handlers[usingDB]
	if not (handler and type(handler.OnNoteReceive) == "function" and handler:OnNoteReceive(zone, x, y, real_icon, data_creator, real_data, sender, fullSender))
		and not self:OnNoteReceive(zone, x, y, real_icon, real_creator, real_data, sender, fullSender) then

		-- Default processing
		if self:SetNote(zone, x, y, real_icon, real_creator, real_data) then
			-- if changed
			if db.showNotifications then
				self:Print(L["%s sent you a note at %q: %.2f, %.2f with the title %q created by %q."], fullSender, zoneName, x*100, y*100, (type(data) == "table" and data.title or getIconTitle(type(data) == "string") and data or data.icon), (creator or type(data) == "table") and data.creator or UNKNOWN)
			end
		end

	end
	-- turn on the note add event
	setNoteFromComm = false
end

-----------------------
-- ONCLICK FUNCTIONS --
-----------------------

-- IsModifierKeyDown() --
local function IsModifierKeyDown()
	if db.createModifier == "C" then
		if IsControlKeyDown() then
			return true
		end
	elseif db.createModifier == "A" then
		if IsAltKeyDown() then
			return true
		end
	elseif db.createModifier == "S" then
		if IsShiftKeyDown() then
			return true
		end
	elseif db.createModifier == "AC" then
		if IsControlKeyDown() and IsAltKeyDown() then
			return true
		end
	elseif db.createModifier == "CS" then
		if IsControlKeyDown() and IsShiftKeyDown() then
			return true
		end
	elseif db.createModifier == "AS" then
		if IsShiftKeyDown() and IsAltKeyDown() then
			return true
		end
	elseif db.createModifier == "ACS" then
		if IsControlKeyDown() and IsAltKeyDown() and IsShiftKeyDown() then
			return true
		end
	end
	return nil
end

-- Notes:WorldMapButton_OnClick() --
function Notes:WorldMapButton_OnClick(button, mouseButton, ...)
	if mouseButton ~= "RightButton" or not db.worldmapMenu or not IsModifierKeyDown() then
		return self.hooks[button].OnMouseUp(button, mouseButton, ...)
	end

	if not self:GetCurrentMapTexture() then
		return
	end

	return self:MapButton_OnClick(mouseButton, button)
end

-- Notes:MapButton_OnClick() --
local lastCursorX, lastCursorY
function Notes:MapButton_OnClick(mouseButton, button)
	if IsModifierKeyDown() then
		if not Dewdrop:IsRegistered(WorldMapFrame) then
			local newNoteFunc = function(creator)
				self:OpenNewNoteFrame(lastCursorX, lastCursorY, creator)
				Dewdrop:Close()
			end
			local closeFunc = function()
				Dewdrop:Close()
			end
			local function waypointFunc()
				if Waypoints then
					Waypoints:SetPointAsWaypoint(lastCursorX, lastCursorY)
					Dewdrop:Close()
				end
			end
			Dewdrop:Register(button,
				'children', function()
					Dewdrop:AddLine(
						'isTitle', true,
						'text', "Mapster"
					)
					if db.worldmapCreateClick then
						Dewdrop:AddLine(
							'text', L["Create a new note"],
							'func', newNoteFunc
						)
					end
					if Waypoints and Waypoints:IsEnabled() then
						Dewdrop:AddLine(
							'text', L["Set as waypoint"],
							'func', waypointFunc
						)
					end
					Dewdrop:AddLine()
					Dewdrop:AddLine(
						'text', CANCEL,
						'func', closeFunc
					)
				end,
				'dontHook', true,
				'cursorX', true,
				'cursorY', true
			)
		end
		lastCursorX, lastCursorY = GetCursorMapLocation(button)
		if button == WorldMapFrame then
			Dewdrop:Open(button)
		else
			Dewdrop:Open(button, WorldMapFrame)
		end
	end
end

-------------------------
-- NOTE EDIT FUNCTIONS --
-------------------------

local possibleIcons
function Notes:OpenNewNoteFrame(x, y, creator)
	if not self:GetCurrentMapTexture() then
		return
	end
	checkParameter("OpenNewNoteFrame", x, 1, "number")
	checkParameter("OpenNewNoteFrame", y, 2, "number")
	checkParameter("OpenNewNoteFrame", creator, 3, "string", "nil")
	if x < 0 or x > 1 then error(("Argument #1 to 'Notes:OpenNewNoteFrame' is expected to be [0, 1], got %s"):format(x), 1) end
	if y < 0 or y > 1 then error(("Argument #2 to 'Notes:OpenNewNoteFrame' is expected to be [0, 1], got %s"):format(y), 1) end

	local frame = CreateNoteFrame()	-- noteFrame or CreateNoteFrame()
	frame:SetFrameStrata("DIALOG")
	frame.header:SetText(L["New note"])
	frame.id = nil
	frame.xEditBox:SetText(format("%.2f", x*100))
	frame.yEditBox:SetText(format("%.2f", y*100))
	local zone, level = getZoneAndLevel(self:GetCurrentMapTexture())
	if level > 0 then
		zone = self:GetCurrentZoneName().." ("..level..")"
	else
		zone = self:GetCurrentZoneName()
	end
	frame.zone:SetText(zone)
	frame.title:SetText("")
	frame.title.resetColor()
	frame.info1:SetText("")
	frame.info1.resetColor()
	frame.info2:SetText("")
	frame.info2.resetColor()
	frame.creator:SetText(creator or UnitName("player"))
	frame:Show()
	frame.title:SetFocus()
	if not possibleIcons then
		possibleIcons = {}
	end
	for k in pairs(possibleIcons) do
		possibleIcons[k] = nil
	end
	for k,v in pairs(icons) do
		if v.showToUser then
			table_insert(possibleIcons, k)
		end
	end
	frame.iconSelector(possibleIcons[math_random(1, #possibleIcons)])
	frame.zonename = self:GetCurrentMapTexture()
end

function Notes:ShowEditNoteFrame(zone, x, y)
	checkParameter("ShowEditNoteFrame", zone, 1, "string")
	if not y then
		checkParameter("ShowEditNoteFrame", x, 2, "number")
		if x < 0 or x > 99999999 then error(("Argument #2 to 'Notes:ShowEditNoteFrame' is expected to be [0, 99999999], got %s"):format(x), 1) end
	else
		checkParameter("ShowEditNoteFrame", x, 2, "number")
		checkParameter("ShowEditNoteFrame", y, 3, "number")
		if x < 0 or x > 1 then error(("Argument #2 to 'Notes:ShowEditNoteFrame' is expected to be [0, 1], got %s"):format(x), 1) end
		if y < 0 or y > 1 then error(("Argument #3 to 'Notes:ShowEditNoteFrame' is expected to be [0, 1], got %s"):format(y), 1) end
	end

	local id
	if not y then
		id = x
	else
		id = getID(x, y)
	end

	local data, db = getrawpoi(zone, id)
	if not data then
		return
	end
	local frame = CreateNoteFrame()	-- noteFrame or CreateNoteFrame()
	frame:SetFrameStrata("DIALOG")
	frame.header:SetText(L["Edit note"])
	frame.id = id
	local x, y = getXY(id)
	frame.xEditBox:SetText(format("%.2f", x*100))
	frame.yEditBox:SetText(format("%.2f", y*100))
	local z, level = getZoneAndLevel(zone)
	if level > 0 then
		frame.zone:SetText(self:GetZoneFromMapTexture(z).." ("..level..")")
	else
		frame.zone:SetText(self:GetZoneFromMapTexture(z))
	end
	if type(data) == "table" then
		frame.title:SetText(data.title or getIconTitle(data.icon))
		frame.title.resetColor(getRGB(data.titleCol))
		frame.info1:SetText(data.info or "")
		frame.info1.resetColor(getRGB(data.infoCol))
		frame.info2:SetText(data.info2 or "")
		frame.info2.resetColor(getRGB(data.info2Col))
		frame.creator:SetText(db or data.creator or "")
		frame.iconSelector(data.icon)
	else
		frame.title:SetText(getIconTitle(data))
		frame.title.resetColor(1, 0.82, 0)
		frame.info1:SetText("")
		frame.info1.resetColor(1, 0.82, 0)
		frame.info2:SetText("")
		frame.info2.resetColor(1, 0.82, 0)
		frame.creator:SetText(db)
		frame.iconSelector(data)
	end
	frame:Show()
	frame.title:SetFocus()
	frame.zonename = zone
end


------------
-- newpoi --
------------
do
	local cache = {}

	local num_pois = 0

	local OnMouseDown, OnMouseUp, OnEnter, OnLeave, OnClick, OnTabletRequest, OnLineRequest

	local minimapClick
	local function utf8trunc(str)
		-- truncate utf8 string if > 18 characters
		local len = 0
		local needsTrunc = true
		for i = 1, 18 do
			local b = str:byte(len+1)
			if not b then
				needsTrunc = false
				break
			elseif b <= 127 then
				len = len + 1
			elseif b <= 223 then
				len = len + 2
			elseif b <= 239 then
				len = len + 3
			else
				len = len + 4
			end
		end
		if needsTrunc then
			return str:sub(1, len) .. "..."
		else
			return str
		end
	end

	-- newpoi --
	local dummy
	local function newpoi(minimap)
		local frame = next(cache)
		if frame then
			cache[frame] = nil
			frame:Show()
			frame.minimap = minimap
			if minimap then
				frame:SetPoint("CENTER", Minimap, "CENTER")
				frame:SetFrameStrata(Minimap:GetFrameStrata())
				frame:SetFrameLevel(Minimap:GetFrameLevel() + 2)
			else
				frame:SetPoint("CENTER", WorldMapButton, "CENTER")
				frame:SetFrameStrata(WorldMapButton:GetFrameStrata())
				frame:SetFrameLevel(WorldMapButton:GetFrameLevel() + 2)
			end
			frame:SetAlpha(1)
			return frame
		end

		num_pois = num_pois + 1

		local frame = CreateFrame("Button", "MapsterNotesPOI" .. num_pois, WorldMapButton)
		frame:EnableMouse(true)
		frame:SetMovable(true)
		frame:SetWidth(16)
		frame:SetHeight(16)
		frame:SetPoint("CENTER", WorldMapButton, "CENTER")
		frame:SetFrameLevel(WorldMapButton:GetFrameLevel() + 2)

		local texture = frame:CreateTexture(nil, "OVERLAY")
		frame.texture = texture
		texture:SetAllPoints(frame)
		texture:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
		texture:SetTexCoord(0, 0.25, 0, 0.25)

		frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		if not OnMouseDown then
			OnMouseDown = function(self, mouseButton)
				if mouseButton == "LeftButton" and self.manual and not self.minimap and IsAltKeyDown() then
					self.isMoving = true
					self:StartMoving()
				end
			end
		end
		frame:SetScript("OnMouseDown", OnMouseDown)

		if not OnMouseUp then
			OnMouseUp = function(self, mouseButton)
				if mouseButton == "LeftButton" then
					if self.isMoving then
						self:StopMovingOrSizing()
						self.isMoving = nil
						local x, y = self:GetCenter()
						local parent = self:GetParent()
						local left, top = parent:GetLeft(), parent:GetTop()
						local width = parent:GetWidth()
						local height = parent:GetHeight()
						x = (x - left) / width
						y = (top - y) / height

						x, y = round(x, 3), round(y, 3)
						if x > 1 then
							x = 1
						end
						if x < 0 then
							x = 0
						end
						if y > 1 then
							y = 1
						end
						if y < 0 then
							y = 0
						end
						local id = getID(x, y)
						if id ~= self.id then
							pois[id] = pois[self.id]
							pois[self.id] = nil
							minimapPois:del(self.id)
							local t, db = getpoi(self.zone, self.id)
							forceNextMinimapUpdate = true
							if db then
								local v = Notes.externalDBs[db]
								v[self.zone][id] = t
								v[self.zone][self.id] = nil
								self.id = id
								Notes:ShowNote(self.zone, id, db)
							else
								Notes.db.global.pois[self.zone][id] = t
								Notes.db.global.pois[self.zone][self.id] = nil
								self.id = id
								Notes:ShowNote(self.zone, id, t.creator or "")
							end
						end
					end
				end
			end
		end
		frame:SetScript("OnMouseUp", OnMouseUp)

		if not OnEnter then
			OnEnter = function(self)
				WorldMapUnit_OnEnter(self)
			end
		end
		frame:SetScript("OnEnter", OnEnter)

		if not OnTabletRequest then
			OnTabletRequest = function(self)
				if Notes.handlers[self.creator] and type(Notes.handlers[self.creator].OnNoteTooltipRequest) == "function" then
					local data = getrawpoi(self.zone, self.id, self.creator)
					if data then
						Notes.handlers[self.creator]:OnNoteTooltipRequest(self.zone, self.id, data, self.minimap)
						return
					end
				end
				Tablet:SetTitle(self.title)
				Tablet:SetTitleColor(getRGB(self.titleCol))

				if self.info and self.info ~= "" then
					local r,g,b = getRGB(self.infoCol)
					Tablet:AddCategory(
						'hideBlankLine', true
					):AddLine(
						'text', self.info,
						'textR', r,
						'textG', g,
						'textB', b,
						'wrap', true
					)
				end
				if self.info2 and self.info2 ~= "" then
					local r,g,b = getRGB(self.info2Col)
					Tablet:AddCategory(
						'hideBlankLine', true
					):AddLine(
						'text', self.info2,
						'textR', r,
						'textG', g,
						'textB', b,
						'wrap', true
					)
				end

				if db.zone then
					local zone, level = getZoneAndLevel(self.zone)
					if level > 0 then
						zone = Notes:GetZoneFromMapTexture(self.zone).." ("..level..")"
					else
						zone = Notes:GetZoneFromMapTexture(self.zone)
					end
					Tablet:AddCategory(
						'columns', 2,
						'hideBlankLine', true
					):AddLine(
						'text', L["Zone:"],
						'text2', zone
					)
				end

				if db.location then
					local posString
					if Coords then
						local highPrec = Coords.db.profile.accuracy
						posString = "%." .. highPrec .. "f, %." .. highPrec .. "f"
					else
						posString = "%.1f, %.1f"
					end
					local x, y = getXY(self.id)
					Tablet:AddCategory(
						'columns', 2,
						'hideBlankLine', true
					):AddLine(
						'text', L["Location:"],
						'text2', format(posString, x * 100, y * 100)
					)
				end

				if db.showDistance then
					local x, y = getXY(self.id)
					local dist = Notes:GetDistanceToPoint(x, y, self.zone)
					if dist then
						Tablet:AddCategory(
							'columns', 2,
							'hideBlankLine', true
						):AddLine(
							'text', L["Distance:"],
							'text2', format(yards, dist)
						)
					end
				end
				if db.showCreator and self.creator and self.creator ~= "" then
					Tablet:AddCategory(
						'columns', 2,
						'hideBlankLine', true
					):AddLine(
						'text', L["Created by"],
						'text2', self.creator
					)
				end
			end
		end
		frame.OnTabletRequest = OnTabletRequest

		if not OnLineRequest then
			local tmp = {}
			OnLineRequest = function(self)
				if Notes.handlers[self.creator] and type(Notes.handlers[self.creator].OnNoteTooltipLineRequest) == "function" then
					local data = getrawpoi(self.zone, self.id, self.creator)
					if data then
						local left, top, _, _, _, _, right, bottom = self.texture:GetTexCoord()
						local r, g, b = self.texture:GetVertexColor()
						return 'hasCheck', true,
							'checked', true,
							'checkIcon', self.texture:GetTexture(),
							'checkCoordLeft', left,
							'checkCoordRight', right,
							'checkCoordTop', top,
							'checkCoordBottom', bottom,
							'checkColorR', r,
							'checkColorG', g,
							'checkColorB', b,
							Notes.handlers[self.creator]:OnNoteTooltipLineRequest(self.zone, self.id, data, self.minimap)
					end
				end

				tmp[#tmp+1] = format("|cff%06x", self.titleCol)
				tmp[#tmp+1] = self.title
				tmp[#tmp+1] = "|r"

				if self.info and self.info ~= "" then
					tmp[#tmp+1] = " - "
					tmp[#tmp+1] = format("|cff%06x", self.infoCol)
					local info = self.info
					info = utf8trunc(info)
					tmp[#tmp+1] = info
					tmp[#tmp+1] = "|r"
				end

				if not self.minimap and db.showCreator and self.creator and self.creator ~= "" then
					tmp[#tmp+1] = " - "
					tmp[#tmp+1] = self.creator
				end

				local text = table_concat(tmp, "")
				for i = 1, #tmp do
					tmp[i] = nil
				end
				local left, top, _, _, _, _, right, bottom = self.texture:GetTexCoord()
				local r, g, b = self.texture:GetVertexColor()
				return 'text', text,
					'hasCheck', true,
					'checked', true,
					'checkIcon', self.texture:GetTexture(),
					'checkCoordLeft', left,
					'checkCoordRight', right,
					'checkCoordTop', top,
					'checkCoordBottom', bottom,
					'checkColorR', r,
					'checkColorG', g,
					'checkColorB', b
			end
		end
		frame.OnLineRequest = OnLineRequest

		if not OnLeave then
			OnLeave = function(self)
				WorldMapUnit_OnLeave()
			end
		end
		frame:SetScript("OnLeave", OnLeave)
		Notes:AddPOI(frame)

		if not OnClick then
			local poi
			OnClick = function(self, mouseButton)
				if self.minimap then
					if mouseButton == "LeftButton" then
						_G.self = Minimap
						local f = Minimap:GetScript("OnMouseDown")
						if f then
							f(Minimap, mouseButton)
						end
						f = Minimap:GetScript("OnMouseUp")
						if f then
							f(Minimap, mouseButton)
						end
						return
					elseif mouseButton == "RightButton" then
						minimapClick = 1
					end
				else
					minimapClick = nil
				end
				if mouseButton == "LeftButton" then
					if Notes.handlers[self.creator] and type(Notes.handlers[self.creator].OnNoteClick) == "function" then
						local data = getrawpoi(self.zone, self.id, self.creator)
						if data then
							Notes.handlers[self.creator]:OnNoteClick(self.zone, self.id, data)
						end
					end
				elseif mouseButton == "RightButton" then
					if (not self.minimap and not db.worldmapMenu) or (self.minimap and not db.minimapMenu) then
						return
					end
					if not dummy then
						dummy = CreateFrame("Frame")
						local function editNoteFunc()
							Notes:ShowEditNoteFrame(poi.zone, poi.id)
							Dewdrop:Close()
						end

						-- isWaypointFunc --
						local function isWaypointFunc()
							return Waypoints:IsNoteWaypoint(poi.id..poi.zone)
						end

						-- waypointFunc --
						local function waypointFunc()
							if isWaypointFunc() then
								Waypoints:CancelWaypoint(poi.id..poi.zone)
							else
								Waypoints:SetNoteAsWaypoint(poi.zone, poi.id)
							end
							Dewdrop:Close()
						end

						-- waypointdelFunc --
						local function waypointdelFunc()
							Waypoints:CancelWaypoint((poi.id-1)..poi.zone)
							Dewdrop:Close()
						end

						-- deleteNoteFunc --
						local function deleteNoteFunc()
							Notes:DeleteNote(poi.zone, poi.id)
							Dewdrop:Close()
						end

						-- closeFunc --
						local function closeFunc()
							Dewdrop:Close()
						end

						-- sendNoteToGroupFunc --
						local function sendNoteToGroupFunc()
							if getrawpoi(poi.zone, poi.id) then
								Notes:SendNoteToGroup(poi.zone, getXY(poi.id))
							end
						end

						-- sendNoteToGuildFunc --
						local function sendNoteToGuildFunc()
							if getrawpoi(poi.zone, poi.id) then
								Notes:SendNoteToGuild(poi.zone, getXY(poi.id))
							end
						end

						-- sendNoteToPlayerFunc --
						local function sendNoteToPlayerFunc(player)
							if getrawpoi(poi.zone, poi.id) then
								local x, y = getXY(poi.id)
								Notes:SendNoteToPlayer(poi.zone, x, y, player)
							end
						end

						local sendNoteToPlayerValidate
						if LOCALE_enUS then -- American or Australian
							-- sendNoteToPlayerValidate --
							function sendNoteToPlayerValidate(player)
								return string_find(player, "^[A-Za-z][A-Za-z]+$")
							end
						elseif GetLocale() == "enUS" or GetLocale() == "deDE" or GetLocale() == "frFR" or GetLocale() == "esES" then -- European, non-American
							local a = ('a'):byte()
							local z = ('z'):byte()
							local A = ('A'):byte()
							local Z = ('Z'):byte()
							-- sendNoteToPlayerValidate --
							function sendNoteToPlayerValidate(player)
								local i = 1
								local len = player:len()
								while i < len do
									local byte = player:byte(i)
									if byte >= a and byte <= z then
									elseif byte >= A and byte <= Z then
									elseif byte == 195 then
										i = i + 1
									else
										return false
									end
									i = i + 1
								end
								return true
							end
						else -- Asian or otherwise
							-- sendNoteToPlayerValidate --
							function sendNoteToPlayerValidate(player)
								return true
							end
						end

						-- defaultMenu --
						local function defaultMenu(level, value, level2, level3, level4)
							if level == 1 then
								if poi.title then
									Dewdrop:AddLine(
										'isTitle', true,
										'text', poi.title
									)
								end
								if Waypoints and Waypoints:IsEnabled() then
									if poi.creator == "Waypoints" then
										Dewdrop:AddLine(
											'text', L["Clear waypoint"],
											'func', waypointdelFunc
											)
									else
										Dewdrop:AddLine(
											'text', L["Set as waypoint"],
											'func', waypointFunc,
											'checked', isWaypointFunc()
										)
									end
								end
								if poi.manual and not minimapClick then
									Dewdrop:AddLine(
										'text', L["Edit note"],
										'func', editNoteFunc
									)
								end
								if (poi.manual or not Notes.cantDelete[poi.creator]) and not minimapClick then
									Dewdrop:AddLine(
										'text', L["Delete note"],
										'func', deleteNoteFunc
									)
								end
								if not minimapClick then
									Dewdrop:AddLine(
										'text', L["Send note"],
										'hasArrow', true,
										'value', 'send'
									)
								end
								Dewdrop:AddLine(
									'text', CANCEL,
									'func', closeFunc
								)
							elseif level == 2 then
								if value == 'send' then
									if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
										Dewdrop:AddLine(
											'text', L["Send to group"],
											'func', sendNoteToGroupFunc,
											'disabled', not Notes.db.profile.commActive
										)
									end
									if IsInGuild() then
										Dewdrop:AddLine(
											'text', L["Send to guild"],
											'func', sendNoteToGuildFunc,
											'disabled', not Notes.db.profile.commActive
										)
									end
									Dewdrop:AddLine(
										'text', L["Send to player"],
										'tooltipTitle', L["Send to player"],
										'disabled', not Notes.db.profile.commActive,
										'hasArrow', true,
										'hasEditBox', true,
										'editBoxText', UnitExists("target") and UnitIsPlayer("target") and UnitIsFriend("player", "target") and UnitName("target") or nil,
										'editBoxFunc', sendNoteToPlayerFunc,
										'editBoxValidateFunc', sendNoteToPlayerValidate
									)
								end
							end
						end
						Dewdrop:Register(dummy,
							'children', function(level, value, level2, level3, level4)
								if Notes.handlers[poi.creator] and type(Notes.handlers[poi.creator].OnNoteMenuRequest) == "function" then
									local data = getrawpoi(poi.zone, poi.id, poi.creator)
									if data then
										Notes.handlers[poi.creator]:OnNoteMenuRequest(poi.zone, poi.id, data, level, value, level2, level3, level4, defaultMenu)
									end
								else
									defaultMenu(level, value, level2, level3, level4)
								end
							end,
							'dontHook', true
						)
					end
					poi = self
					if Dewdrop:IsOpen(poi) then
						Dewdrop:Close()
						poi:GetScript("OnEnter")(poi)
					else
						Dewdrop:Open(poi, dummy)
						Tablet:Close()
					end
				end
				minimapClick = nil
			end
		end
		frame:SetScript("OnClick", OnClick)

		frame:Hide()
		cache[frame] = true
		return newpoi(minimap)
	end

	-- delpoi --
	local function delpoi(x)
		x:Hide()
		cache[x] = true
		return nil
	end

	-- del --
	local del = function(self, id)
		if id == 'del' then
			return
		end
		local x = rawget(self, id)
		self[id] = nil
		if x then
			delpoi(x)
		end
	end

	-- __index --
	local __index = function(self, id)
		local frame = newpoi(self == minimapPois)
		self[id] = frame
		return frame
	end

	-- newpoi --
	pois = setmetatable({ del = del }, { __index = __index })
	minimapPois = setmetatable({ del = del }, { __index = __index })
end


-- Notes:SetNote(zone, x, y, icon, creator, ...) --
local cache = {}
function Notes:SetNote(zone, x, y, icon, creator, ...)
	checkParameter("SetNote", zone, 1, "string")
	if not y then
		checkParameter("SetNote", x, 2, "number")
		if x < 0 or x > 1 then error(("Argument #2 to 'Notes:SetNote' is expected to be [0, 1], got %s"):format(x), 2) end
	else
		checkParameter("SetNote", x, 2, "number")
		checkParameter("SetNote", y, 3, "number")
		if x < 0 or x > 1 then error(("Argument #2 to 'Notes:SetNote' is expected to be [0, 1], got %s"):format(x), 2) end
		if y < 0 or y > 1 then error(("Argument #3 to 'Notes:SetNote' is expected to be [0, 1], got %s"):format(y), 2) end
	end
	checkParameter("SetNote", icon, 4, "string")
	checkParameter("SetNote", creator, 5, "string")

	local z, level = getZoneAndLevel(zone)
	local zoneName = self:GetZoneFromMapTexture(z)
	if not zoneName then
		error(("Trying to set a note with an unknown zone: %q"):format(z), 2)
	end

	local usingDB = self.externalDBs[creator] and creator or nil

	local id = getID(x, y)

	if usingDB and not self.externalDBs[creator][zone] then
		self.externalDBs[creator][zone] = {}
	end
	local zoneData = usingDB and self.externalDBs[creator][zone] or self.db.global.pois[zone]
	local oldData = rawget(zoneData, id)
	if type(oldData) == "table" then
		local tmp = oldData
		oldData = cache
		for k,v in pairs(tmp) do
			oldData[k] = v
		end
	end
	local k1 = ...
	if not k1 and usingDB then
		zoneData[id] = icon
	else
		local t
		if not zoneData[id] then
			zoneData[id] = {}
		end
		t = zoneData[id]
		for k,v in pairs(t) do
			t[k] = nil
		end
		if type(k1) ~= "table" then
			for i = 1, select('#', ...), 2 do
				local k = select(i, ...)
				if k then
					t[k] = select(i+1, ...)
				else
					break
				end
			end
		else
			for k,v in pairs(k1) do
				t[k] = v
			end
		end
		t.icon = icon
		if not usingDB then
			t.creator = creator
		end
		local defaults = usingDB and self.handlers[creator] and self.handlers[creator].noteDefaults
		if type(defaults) ~= "table" then
			defaults = nil
		end
		local oldId = t.oldId
		t.oldId = nil
		if oldId and oldId ~= id then
			local oldicon
			if rawget(zoneData, oldId) then
				oldicon = zoneData[oldId]
				zoneData[oldId] = nil
				if type(oldicon) == "table" then
					oldicon = oldicon.icon
				end
			end
			if zone == self:GetCurrentMapTexture() then
				pois[id] = pois[oldId]
				pois[oldId] = nil
			end
			if oldicon then
				local oldx, oldy = getXY(oldId)
				self:SendMessage("MAPSTER_NOTES_NOTE_DELETED", zone, oldx, oldy, oldicon, usingDB)
			end
		end
		if t.title == getIconTitle(icon) or t.title == "" then
			t.title = nil
		end
		if t.titleR or t.titleG or t.titleB then
			local r, g, b = t.titleR or 1, t.titleG or 0.82, t.titleB or 0
			t.titleR, t.titleG, t.titleB = nil
			local colorID = getColorID(r, g, b)
			t.titleCol = colorID
		end
		if t.titleCol == (defaults and defaults.titleCol or goldColorID) then
			t.titleCol = nil
		end
		if t.info == (defaults and defaults.info or "") then
			t.info = nil
		end
		if t.infoR or t.infoG or t.infoB then
			local r, g, b = t.infoR or 1, t.infoG or 0.82, t.infoB or 0
			t.infoR, t.infoG, t.infoB = nil
			local colorID = getColorID(r, g, b)
			t.infoCol = colorID
		end
		if t.infoCol == (defaults and defaults.infoCol or goldColorID) then
			t.infoCol = nil
		end
		if t.info2 == (defaults and defaults.info or "") then
			t.info2 = nil
		end
		if t.info2R or t.info2G or t.info2B then
			local r, g, b = t.info2R or 1, t.info2G or 0.82, t.info2B or 0
			t.info2R, t.info2G, t.info2B = nil
			local colorID = getColorID(r, g, b)
			t.info2Col = colorID
		end
		if t.info2Col == (defaults and defaults.info2Col or goldColorID) then
			t.info2Col = nil
		end
		if usingDB then
			t.creator = nil
			t.manual = nil
		end
		if creator == "" then
			t.creator = nil
		end
		if next(t) == "icon" and next(t, "icon") == nil then
			zoneData[id] = t.icon
		end
	end
	local different = false
	local newData = zoneData[id]
	if type(oldData) ~= type(newData) then
		different = true
	elseif type(oldData) ~= "table" then
		different = oldData ~= newData
	else
		for k,v in pairs(oldData) do
			if newData[k] ~= v then
				different = true
				break
			end
		end
		if not different then
			for k,v in pairs(newData) do
				if oldData[k] ~= v then
					different = true
					break
				end
			end
		end
	end
	for k,v in pairs(cache) do
		cache[k] = nil
	end
	if not different then
		return false
	end
	zoneData[id] = zoneData[id] -- to stop from being stored in an __index'd table.

	self:SendMessage("MAPSTER_NOTES_NOTE_SET", zone, x, y, icon, creator, setNoteFromComm)

	forceNextMinimapUpdate = true
	if zone ~= self:GetCurrentMapTexture() or not isCurrentLevel(level) then
		return true
	end
	self:ShowNote(zone, id, usingDB)
	return true
end

-- Notes:GetNote(zone, x, y) --
function Notes:GetNote(zone, x, y, levels)
	checkParameter("GetNote", zone, 1, "string")
	if not y then
		checkParameter("GetNote", x, 2, "number")
		if x < 0 or x > 1 then error(("Argument #2 to 'Notes:GetNote' is expected to be [0, 1], got %s"):format(x), 1) end
	else
		checkParameter("GetNote", x, 2, "number")
		checkParameter("GetNote", y, 3, "number")
		if x < 0 or x > 1 then error(("Argument #2 to 'Notes:GetNote' is expected to be [0, 1], got %s"):format(x), 1) end
		if y < 0 or y > 1 then error(("Argument #3 to 'Notes:GetNote' is expected to be [0, 1], got %s"):format(y), 1) end
	end

	local z, level = getZoneAndLevel(zone)
	local zoneName = self:GetZoneFromMapTexture(z)
	if not zoneName then
		error(("Trying to get a note with an unknown zone: %q"):format(z), 2)
	end

	local id = getID(x, y)
	local data, db = getrawpoi(zone, id)
	if not data and levels then
		local floors = Astrolabe:GetMapFloorsFromTexture(zone) or 0
		for i = 1, floors do
			data, db = getrawpoi(zone..":"..i, id)
			if data then
				return zone..":"..i, x, y, type(data) == "string" and data or data.icon, db, data
			end
		end
	end
	if not data then
		return
	end

	return zone, x, y, type(data) == "string" and data or data.icon, db, data
end

-- Notes:DeleteNote(zone, x, y) --
function Notes:DeleteNote(zone, x, y)
	checkParameter("DeleteNote", zone, 1, "string")
	if not y then
		checkParameter("DeleteNote", x, 2, "number")
		if x < 0 or x > 99999999 then error(("Argument #2 to 'Notes:DeleteNote' is expected to be [0, 99999999], got %s"):format(x), 2) end
	else
		checkParameter("DeleteNote", x, 2, "number")
		checkParameter("DeleteNote", y, 3, "number")
		if x < 0 or x > 1 then error(("Argument #2 to 'Notes:DeleteNote' is expected to be [0, 1], got %s"):format(x), 2) end
		if y < 0 or y > 1 then error(("Argument #3 to 'Notes:DeleteNote' is expected to be [0, 1], got %s"):format(y), 2) end
	end

	local z, level = getZoneAndLevel(zone)
	local zoneName = self:GetZoneFromMapTexture(z)
	if not zoneName then
		error(("Trying to destroy a note with an unknown zone: %q"):format(z), 2)
	end

	local id
	if not y then
		id = x
		x, y = getXY(id)
	else
		id = getID(x, y)
	end
	local t, db = getrawpoi(zone, id)
	if not t then
		error(("Cannot destroy note %q at %q if it does not exist"):format(id, zone), 2)
	end
	if db then
		self.externalDBs[db][zone][id] = nil
	else
		self.db.global.pois[zone][id] = nil
	end
	local icon = t
	if type(icon) == "table" then
		icon = icon.icon
	end
	self:SendMessage("MAPSTER_NOTES_NOTE_DELETED", zone, x, y, icon, db)
	if zone == self:GetCurrentMapTexture() then
		pois:del(id)
	end
	if rawget(minimapPois, id) then
		minimapPois:del(id)
	end
	if Waypoints and Waypoints:IsEnabled() and Waypoints:IsNoteWaypoint(zone, id) then
		Waypoints:CancelWaypoint(id..zone)
	end
	forceNextMinimapUpdate = true
end

-- Notes:ShowNote(zone, id, creator) --
function Notes:ShowNote(zone, id, creator)
	local z, level = getZoneAndLevel(zone)
	if not self.db.profile.worldmapOtherZone and zone ~= self:GetCurrentMapTexture() or not self:IsEnabled() then
		return
	end
	-- creator overrides
	local note_trans, note_scale, note_icon
	local default_titleCol, default_info, default_infoCol, default_info2, default_info2Col = goldColorID, "", goldColorID, "", goldColorID

	local localeHandler
	if creator then
		local handler = self.handlers[creator]
		localeHandler = handler and handler.noteLocaleHandler
		local defaults = handler and handler.noteDefaults
		if type(defaults) ~= "table" then
			defaults = nil
		end
		local data = getrawpoi(zone, id, creator)
		if not data then
			return
		end
		-- check note hidden or not
		if handler and handler.IsNoteHidden and handler:IsNoteHidden(zone, id, data) then
			return
		end
		-- creator DB scaling and transparency overrides
		if handler and handler.GetNoteTransparency then
			note_trans = handler:GetNoteTransparency(zone, id, data)
			-- ensure transparency withing range
			if note_trans and (note_trans < 0 or note_trans > 1) then note_trans = nil end
		end
		-- per creator scaling overrides
		if handler and handler.GetNoteScaling then
			note_scale = handler:GetNoteScaling(zone, id, data)
			if note_scale and (note_scale < 0.5 or note_scale > 2) then note_scale = nil end
		end
		-- per creator icon overrides
		if handler and handler.GetNoteIcon then
			note_icon = handler:GetNoteIcon(zone,id,data)
			if type(note_icon) ~= "string" then note_icon = nil end
		end

		if defaults then
			default_titleCol = defaults.titleCol or goldColorID
			default_info = defaults.info or ""
			default_infoCol = defaults.infoCol or goldColorID
			default_info2 = defaults.info2 or ""
			default_info2Col = defaults.info2Col or goldColorID
		end
	end
	local data, db = getpoi(zone, id)
	local poi = pois[id]
	poi.zone = zone
	local icon
	local creator = db or data.creator or ""
	if type(data) == "string" then
		poi.title = getIconTitle(data)
		poi.titleCol = default_titleCol
		poi.info = default_info
		poi.infoCol = default_infoCol
        poi.info2 = default_info2
		poi.info2Col = default_info2Col
		poi.manual = false
		icon = note_icon or data
	else
		poi.title = data.title or getIconTitle(data.icon)
		poi.titleCol = data.titleCol or default_titleCol
		poi.info = data.info or default_info
		poi.infoCol = data.infoCol or default_infoCol
		poi.info2 = data.info2 or default_info2
		poi.info2Col = data.info2Col or default_info2Col
		poi.icon = data.icon
		poi.manual = data.manual or false
		icon = note_icon or data.icon
	end
	if localeHandler then
		if poi.title ~= getIconTitle(type(data) == "string" and data or data.icon) and localeHandler:HasTranslation(poi.title) then
			poi.title = localeHandler:GetTranslation(poi.title)
		end
		if poi.info ~= "" and localeHandler:HasTranslation(poi.info) then
			poi.info = localeHandler:GetTranslation(poi.info)
		end
		if poi.info2 ~= "" and localeHandler:HasTranslation(poi.info2) then
			poi.info2 = localeHandler:GetTranslation(poi.info2)
		end
	end
	poi.icon = icon
	poi.creator = creator
	poi.id = id

	local button = WorldMapButton

	poi:SetParent(button)
	if type(icon) == "string" and string_find(icon, "^Interface\\") then
		if note_scale then
			poi:SetWidth(16*self.db.profile.worldmapIconSize*note_scale)
			poi:SetHeight(16*self.db.profile.worldmapIconSize*note_scale)
		else
			poi:SetWidth(16*self.db.profile.worldmapIconSize)
			poi:SetHeight(16*self.db.profile.worldmapIconSize)
		end
		-- Note transparency overrides
		if note_trans then
			poi:SetAlpha(note_trans)
		else
			poi:SetAlpha(1)
		end
		poi.texture:SetTexture(icon)
		if string_find(icon, "^Interface\\Icons\\") then
			poi.texture:SetTexCoord(0.05, 0.95, 0.05, 0.95)
		else
			poi.texture:SetTexCoord(0, 1, 0, 1)
		end
	else
		local t = icons[icon]
		if not t then
			t = icons.Unknown
		end
		if note_scale then
			poi:SetWidth(t.width*self.db.profile.worldmapIconSize*note_scale)
			poi:SetHeight(t.height*self.db.profile.worldmapIconSize*note_scale)
		else
			poi:SetWidth(t.width*self.db.profile.worldmapIconSize)
			poi:SetHeight(t.height*self.db.profile.worldmapIconSize)
		end
		-- Note transparency overrides
		if note_trans then
			poi:SetAlpha(note_trans)
		else
			poi:SetAlpha(t.alpha)
		end
		poi.texture:SetTexture(t.path)
		poi.texture:SetTexCoord(t.cLeft, t.cRight, t.cTop, t.cBottom)
	end
	poi:Show()

	poi:ClearAllPoints()
	local x, y = getXY(id)
	if self.db.profile.worldmapOtherZone then
		x, y = self:TranslateWorldMapPosition(x, y, zone, self:GetCurrentMapTexture())
		if not x or x < 0 or x > 1 or not y or y < 0 or y > 1 then
			pois:del(poi.id)
			return
		end
	end
	poi:SetPoint("CENTER", button, "TOPLEFT", x * button:GetWidth(), -y * button:GetHeight())
end


-- Notes:FindZone(search, destTable, iterFunc, iterTable, iterKey) --
-- search = localized/english zone name
do
	local searchResults = {}
	function Notes:FindZone(search, destTable, iterFunc, iterTable, iterKey)
		checkParameter("FindZone", search, 1, "string", "nil")
		checkParameter("FindZone", destTable, 2, "table", "nil")
		checkParameter("FindZone", iterFunc, 3, "function", "nil")

		if not destTable then
			while #searchResults > 0 do
				table_remove(searchResults)
			end

			destTable = searchResults
		end

		local exact = nil

		if not iterFunc then
			iterFunc, iterTable, iterKey = self:IterateZonesAndInstances()
		end

        local lzone, lsearch
		for zone in iterFunc, iterTable, iterKey do
			if self:GetMapTexture(zone) then
				if not search then
					table_insert(destTable, zone)
				else
					lzone = zone:lower()
					lsearch = search:lower()

					if lzone == lsearch then
						table_insert(destTable, zone)
						exact = zone
					elseif string_find(lzone, lsearch) then
						table_insert(destTable, zone)
					end
				end
            end
		end

		if exact then
			return exact
		end

		return unpack(destTable)
	end
end

-- Notes:FindNearestNote(search, zone, x, y, minDist, validate) --
-- zone = localized/english zone name
local tempZones = {}
function Notes:FindNearestNote(search, zone, x, y, minDist, matchLength, validate)
	checkParameter("FindNearestNote", search, 1, "string")
	checkParameter("FindNearestNote", zone, 2, "string", "nil")
	checkParameter("FindNearestNote", x, 3, "number", "nil")
	checkParameter("FindNearestNote", y, 4, "number", "nil")
	checkParameter("FindNearestNote", minDist, 5, "number", "nil")
	checkParameter("FindNearestNote", matchLength, 6, "boolean", "nil")
	checkParameter("FindNearestNote", validate, 7, "function", "nil")

	if not x then
		x, y = self:GetCurrentPlayerPosition()
	end

	if zone then
		local lzone = string.lower(zone)
		if lzone == L["all"] then
			zone, lzone = nil, nil
		end

		local szone, notexact = Notes:FindZone(lzone, tempZones)

        if not szone then
            return nil, nil, nil, format(L["Unknown zone: %q"], zone)
        end

		if not notexact then
			while #tempZones > 0 do
				table_remove(tempZones)
			end
			table_insert(tempZones, szone)
		end
	else
		zone = self:GetCurrentZoneName()

		while #tempZones > 0 do
			table_remove(tempZones)
		end

		table_insert(tempZones, zone)
	end

	return self:FindNotes(search, nil, tempZones, x, y, minDist, matchLength, validate)
end

-- Notes:FindNotes(search, results, zoneList, x, y, minDist, validate) --
-- zoneList = localized/english zone names
function Notes:FindNotes(search, results, zoneList, x, y, minDist, matchLength, validate)
    if not minDist then
        minDist = 0
    end
	search = string.lower(search)

	local _dist, _length = math_huge, 1024
	local _x, _y, _zone, _icon, _db, _data, _print, _key, db, value, key, matchThis

	local zone

	if type(zoneList) == "table" then
		zone = table_remove(zoneList)
	else
		zone = zoneList
		zoneList = nil
	end

	local zone2, floors
    repeat
        repeat
			zone2 = self:GetMapTexture(zone)
			if not zone2 then
                error(format(L["Unknown zone: %q"], zone))
            end
			floors = Astrolabe:GetMapFloorsFromTexture(zone2) or 0

            for i = 0, floors do
				local zones = zone2
				if i > 0 then
					zones = zone2..":"..i
				end

				for zone, x, y, icon, db, data in self:IterateNearbyNotes(zones, x, y, nil, db) do

					if not validate or validate(zone, x, y, icon, db, data) then
						local found, better, dist, length

						if type(data) == "table" then
							for k,v in pairs(data) do
								if type(v) == "number" then
									v = tostring(v)
								end

								if type(v) == "string" then
									if string_find(string.lower(v), search, 1, true) then
										dist = dist or self:GetDistanceToPoint(x, y, zone, px, py, zone) or math_huge - 1
										found = dist and dist >= minDist
										if matchLength then length = string.len(v) end
										matchThis = ((matchLength and k == "title") and (length == _length and dist < _dist or length < _length)) or (not (matchLength and k == "title") and dist < _dist)
										if found and matchThis then
											_print = v
											_key = k
											_length = length
											better = true
										end
									end
								end
							end
						elseif data then
							dist = dist or self:GetDistanceToPoint(x, y, zone, px, py, zone) or math_huge - 1
							found = dist and string_find(string.lower(data), search, 1, true) and dist >= minDist
							if found and dist < _dist then
								_print = data
								_key = "data"
								better = true
							end
						end

						if not found then
							dist = dist or self:GetDistanceToPoint(x, y, zone, px, py, zone) or math_huge - 1
							found = dist and string_find(string.lower(icon), search, 1, true) and dist >= minDist
							if found and dist < _dist then
								_print = icon
								_key = "icon"
								better = true
							end
						end

						if not found and db then
							dist = dist or self:GetDistanceToPoint(x, y, zone, px, py, zone) or math_huge - 1
							found = dist and string_find(string.lower(db), search, 1, true) and dist >= minDist
							if found and dist < _dist then
								_print = db
								_key = "db"
								better = true
							end
						end

						if found then
							if results then
								if not results[zone] then
									results[zone] = {}
								end

								table_insert(results[zone], getID(x, y))
							end

							if better then
								_zone, _x, _y, _icon, _db, _data, _dist, _length = zone, x, y, icon, db, data, dist, length
							end
						end
					end
				end
            end

            db = next(self.externalDBs, db)
		until not db

		zone = zoneList and #zoneList > 0 and table_remove(zoneList) or nil
	until not zone

	if _print then
		_zone = self:GetZoneFromMapTexture(_zone)
		return _zone, _x, _y, _print, _key
	end
end

-- Notes:ClearMap() --
local lastMap = nil
function Notes:ClearMap()
	for k,v in pairs(pois) do
		pois:del(k)
	end
	lastMap = nil
end

-- RefreshMap() --
local forceRefresh
local function RefreshMap()
	local force = forceRefresh
	forceRefresh = nil
	local zone = Notes:GetCurrentMapTexture()
	if not zone or not Notes:IsEnabled() then
		Notes:ClearMap()
		return
	end
	if zone == lastMap and force == false then
		return
	end
	Notes:ClearMap()
	if db.worldmapOtherZone and zone ~= "World" and GetCurrentMapZone() ~= 0 then
		local skip
		for k,v in pairs(Notes.externalDBs) do
			for zones in pairs(Notes.externalDBs[k]) do
				for id,data in pairs(Notes.externalDBs[k][zones]) do
					if zones ~= zone and rawget(Notes.externalDBs[k], zone) then
						for id2,data2 in pairs(Notes.externalDBs[k][zone]) do
							if data2.title == data.title then
								skip = true
							end
						end
					end
					if not skip then
						Notes:ShowNote(zones, id, k)
					end
					skip = nil
				end
			end
		end
		-- Internal database shown first
		for zones in pairs(Notes.db.global.pois) do
			for id in pairs(Notes.db.global.pois[zones]) do
				Notes:ShowNote(zones, id, false)
			end
		end
	else
		for k,v in pairs(Notes.externalDBs) do
			if rawget(v, zone) then
				for id in pairs(v[zone]) do
					Notes:ShowNote(zone, id, k)
				end
			end
		end
		-- Internal database shown first
		if rawget(Notes.db.global.pois, zone) then
			for id in pairs(Notes.db.global.pois[zone]) do
				Notes:ShowNote(zone, id, false)
			end
		end
	end
	lastMap = zone
	forceNextMinimapUpdate = true
end

function Notes:RefreshMap(value)
	-- value defaults to true and false should override forceRefresh if it is nil
	forceRefresh = (value == nil) or forceRefresh or value
	if WorldMapFrame:IsShown() then
		RefreshMap()
	end
end

function Notes:ToggleFrame(frame)
	if not frame or frame ~= WorldMapFrame then return end

	if MapsterNotes_NewNoteFrame then
		MapsterNotes_NewNoteFrame:Hide()
	end
	if not WorldMapFrame:IsShown() then
		self:ClearMap()
	else
		self:RefreshMap(false)
	end
end

function Notes:RegisterNotesDatabase(name, db, handler, cantDelete)
	checkParameter("RegisterNotesDatabase", name, 1, "string")
	checkParameter("RegisterNotesDatabase", db, 2, "table")
	checkParameter("RegisterNotesDatabase", handler, 3, "table", "nil")
	checkParameter("RegisterNotesDatabase", cantDelete, 4, "boolean", "nil")
	if self.externalDBs[name] then
		error("Cannot register a database already registered.", 2)
	end

	self.externalDBs[name] = db
	self.handlers[name] = handler
	self.cantDelete[name] = not not cantDelete

	for zone, zdata in pairs(self.db.global.pois) do
		if type(zdata) == "table" then
			for id, data in pairs(zdata) do
				if data.creator == name then
					zdata[id] = nil
					if not db[zone] then
						db[zone] = {}
					end
					if not rawget(db[zone], id) then
						db[zone][id] = data
						data.creator = nil
						data.manual = nil
						if next(data) == "icon" and next(data, "icon") == nil then
							db[zone][id] = data.icon
						end
					end
				end
			end
		end
	end

	if WorldMapFrame:IsShown() then
		local zone = self:GetCurrentMapTexture()
		if rawget(db, zone) then
			for id in pairs(db[zone]) do
				self:ShowNote(zone, id, name)
			end
		end
	end
end

function Notes:UnregisterNotesDatabase(name)
	checkParameter("UnregisterNotesDatabase", name, 1, "string")
	if not self.externalDBs[name] then
		error("Cannot unregister a database not registered.", 2)
	end

	local t = self.externalDBs[name]
	self.externalDBs[name] = nil
	self.handlers[name] = nil
	local zone = self:GetCurrentMapTexture()
	if t then
		if rawget(t, zone) then
			for id in pairs(t[zone]) do
				pois:del(id)
			end
		end
	end
end

function Notes:SendNoteToPlayer(zone, x, y, player)
	checkParameter("SendNoteToPlayer", zone, 1, "string")
	checkParameter("SendNoteToPlayer", x, 2, "number")
	checkParameter("SendNoteToPlayer", y, 3, "number")
	checkParameter("SendNoteToPlayer", player, 4, "string")
	if x < 0 or x > 1 then error(("Argument #2 to 'Notes:SendNoteToPlayer' is expected to be [0, 1], got %s"):format(x), 1) end
	if y < 0 or y > 1 then error(("Argument #3 to 'Notes:SendNoteToPlayer' is expected to be [0, 1], got %s"):format(y), 1) end

	local z, level = getZoneAndLevel(zone)
	local zoneName = self:GetZoneFromMapTexture(z)
	if not zoneName then
		error(("Trying to send a note with an unknown zone: %q"):format(z), 2)
	end
	zone = purifyZone(z .. level)
	if GetLocale() ~= "zhTW" and GetLocale() ~= "zhCN" and GetLocale() ~= "ruRU" then
		if not string_find(player, "^%a%a+$") then
			error(("Argument #5 to 'Notes:SendNoteToPlayer' in the wrong format, got %q"):format(player), 2)
		end
	end

	local data, db = getrawpoi(zone, getID(x, y))

	if not data then
		error(("Cannot send note %q(%s,%s). Does not exist."):format(tostring(zone), x*100, y*100), 2)
	end

	local message = format("%s:%s:%s:%s:%s:%s", "NOTE", zone, x, y, db, self:Serialize(data))
	self:SendCommMessage(COMM_PREFIX, message, "WHISPER", player)
	return true
end

local function sendNote(distribution, zone, x, y)
	local data, db = getrawpoi(zone, getID(x, y))

	if not data then
		error(("Cannot send note %q(%s,%s). Does not exist."):format(tostring(zone), x*100, y*100), 2)
	end

	local message = format("%s:%s:%s:%s:%s:%s", "NOTE", zone, x, y, db, Notes:Serialize(data))
	Notes:SendCommMessage(COMM_PREFIX, message, distribution)
	return true
end

function Notes:SendNoteToGuild(zone, x, y)
	checkParameter("SendNoteToGuild", zone, 1, "string")
	checkParameter("SendNoteToGuild", x, 2, "number")
	checkParameter("SendNoteToGuild", y, 3, "number")
	if x < 0 or x > 1 then error(("Argument #2 to 'Notes:SendNoteToGuild' is expected to be [0, 1], got %s"):format(x), 1) end
	if y < 0 or y > 1 then error(("Argument #3 to 'Notes:SendNoteToGuild' is expected to be [0, 1], got %s"):format(y), 1) end

	local z, level = getZoneAndLevel(zone)
	local zoneName = self:GetZoneFromMapTexture(z)
	if not zoneName then
		error(("Trying to send a note with an unknown zone: %q"):format(z), 2)
	end
	zone = purifyZone(z .. level)
	if not IsInGuild() then
		return false
	end
	return sendNote("GUILD", zone, x, y)
end

function Notes:SendNoteToGroup(zone, x, y)
	checkParameter("SendNoteToGroup", zone, 1, "string")
	checkParameter("SendNoteToGroup", x, 2, "number")
	checkParameter("SendNoteToGroup", y, 3, "number")
	if x < 0 or x > 1 then error(("Argument #2 to 'Notes:SendNoteToGroup' is expected to be [0, 1], got %s"):format(x), 1) end
	if y < 0 or y > 1 then error(("Argument #3 to 'Notes:SendNoteToGroup' is expected to be [0, 1], got %s"):format(y), 1) end

	local z, level = getZoneAndLevel(zone)
	local zoneName = self:GetZoneFromMapTexture(z)
	if not zoneName then
		error(("Trying to send a note with an unknown zone: %q"):format(z), 2)
	end
	zone = purifyZone(z .. level)
	if GetNumRaidMembers() > 0 or GetNumPartyMembers() > 0 then
		return sendNote("GROUP", zone, x, y)
	end
	return false
end


function Notes:ToggleShowingMinimapIcons(value)
	if value == nil then
		value = not self.db.profile.minimapIcons
	end
	self.db.profile.minimapIcons = value
	if self:IsEnabled() then
		if not value then
			for id in pairs(minimapPois) do
				minimapPois:del(id)
			end
			if self.minimapIconsUpdateFrame then
				self.minimapIconsUpdateFrame:SetScript("OnUpdate", nil)
				self.minimapIconsUpdateFrame:Hide()
			end
			self:UnregisterEvent("MINIMAP_UPDATE_ZOOM")
			self:UnregisterEvent("ZONE_CHANGED")
			self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
			self:UnregisterEvent("PLAYER_LEAVING_WORLD")
		else
			if not self.minimapIconsUpdateFrame then
				self.minimapIconsUpdateFrame = CreateFrame("Frame", "MapsterNotes_MinimapIcons")
			end
			self.minimapIconsUpdateFrame:SetScript("OnUpdate", function(self, elapsed) Notes:UpdateMinimapIcons(elapsed) end)
			self.minimapIconsUpdateFrame:Show()
			self:RegisterEvent("MINIMAP_UPDATE_ZOOM")
			self:RegisterEvent("ZONE_CHANGED")
			self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
			self:RegisterEvent("PLAYER_LEAVING_WORLD")
			self:MINIMAP_UPDATE_ZOOM()
			self:MINIMAP_ZONE_CHANGED()
			self:ZONE_CHANGED_NEW_AREA()
		end
	end
end

function Notes:ClearAllNotes()
	local zone = self:GetCurrentMapTexture()
	local zoneData
	zoneData = rawget(self.db.global.pois, zone)
	-- clear manual notes
	if zoneData and next(zoneData) then
		for i, data in pairs(zoneData) do
			local x, y = getXY(i)
			self:DeleteNote(zone, x, y)
		end
	end
	-- clear other db notes
	for k,v in pairs(self.externalDBs) do
		local zoneData = rawget(v, zone)
		if zoneData and next(zoneData) then
			for i,data in pairs(zoneData) do
				local x, y = getXY(i)
				self:DeleteNote(zone, x, y)
			end
		end
	end
end


------------------------
-- ITERATOR FUNCTIONS --
------------------------
local cache = {}
local function iter(t)
	t.id = t.id + 1

	local notes = t.notes

	local id = notes[t.id]
	if id then
		local data = t.zoneData[id]

		local x, y = getXY(id)

		return t.zone, x, y, type(data) == "string" and data or data.icon, t.creator, data
	end

	cache[t] = true
	for k in pairs(t) do
		t[k] = nil
	end
	cache[notes] = true
	for k in pairs(notes) do
		notes[k] = nil
	end
	return nil
end

local function retNil()
	return nil
end

local current_x, current_y
local current_using_yards
local function my_sort(alpha, bravo)
	if not alpha or not bravo then
		return false
	end
	local a_x, a_y = getXY(alpha)
	local b_x, b_y = getXY(bravo)
	a_x, a_y = a_x - current_x, a_y - current_y
	b_x, b_y = b_x - current_x, b_y - current_y
	if current_using_yards then
		a_y = a_y * 2/3
		b_y = b_y * 2/3
	end
	return a_x*a_x + a_y*a_y < b_x*b_x + b_y*b_y
end

-- Notes:IterateNearbyNotes(zone, x, y, radius, creator, max_notes, use_yards, global) --
function Notes:IterateNearbyNotes(zone, x, y, radius, creator, max_notes, use_yards, global)
	checkParameter("IterateNearbyNotes", zone, 1, "string")
	checkParameter("IterateNearbyNotes", x, 2, "number")
	checkParameter("IterateNearbyNotes", y, 3, "number")
	checkParameter("IterateNearbyNotes", radius, 4, "number", "nil")
	if radius and radius < 0 then error(("Argument #4 to 'Notes:IterateNearbyNotes' is expected to be [0, inf], got %s"):format(radius), 2) end
	checkParameter("IterateNearbyNotes", creator, 5, "string", "nil")
	checkParameter("IterateNearbyNotes", max_notes, 6, "number", "nil")
	checkParameter("IterateNearbyNotes", use_yards, 7, "boolean", "nil")

	local z, level = getZoneAndLevel(zone)
	local zoneName = self:GetZoneFromMapTexture(z)
	if not zoneName then
		error(("Trying to iterate through notes in an unknown zone: %q"):format(z), 2)
	end

	if not radius then
		radius = math_huge
	end
	if creator and not self.externalDBs[creator] then
		error(("Database %q not registered."):format(creator), 2)
	end

	local zoneData
	if creator then
		zoneData = rawget(self.externalDBs[creator], zone)
	else
		zoneData = rawget(self.db.global.pois, zone)
	end
	if not zoneData or not next(zoneData) then
		return retNil
	end

	-- Allow handlers to specify radius
	if creator then
		local handler = self.handlers[creator]
		if handler and handler["GetRadius"] then
			radius = handler:GetRadius()
		end
	end

	local radius_2 = radius*radius

	local notes = next(cache) or {}
	cache[notes] = nil
	local yardWidthMult, yardHeightMult = 1, 1
	if use_yards then
		yardWidthMult, yardHeightMult = self:GetZoneYardSize(zone)
		if not yardWidthMult then
			yardWidthMult, yardHeightMult = 1000, 1000 * 2/3
		end
	end
	for i, data in pairs(zoneData) do
		local x_p, y_p = getXY(i)
		local x_d = (x_p - x) * yardWidthMult
		local y_d = (y_p - y) * yardHeightMult
		if x_d*x_d + y_d*y_d <= radius_2*3 then
			table_insert(notes, i)
		end
	end

	current_x, current_y = x, y
	current_using_yards = use_yards
	table_sort(notes, my_sort)
	current_x, current_y = nil, nil
	current_using_yards = nil

	if max_notes and max_notes < #notes then
		for i = max_notes + 1, #notes do
			notes[i] = nil
		end
	end

	local t = next(cache) or {}
	cache[t] = nil

	t.zoneData = zoneData
	t.zone = zone
	t.creator = creator
	t.notes = notes
	t.id = 0

	return iter, t, nil
end

-- Notes:IterateNearbyNotes(zone, x, y, radius, creator, use_yards) --
function Notes:GetNearbyNote(zone, x, y, radius, creator, use_yards)
	checkParameter("GetNearbyNote", zone, 1, "string")
	checkParameter("GetNearbyNote", x, 2, "number")
	checkParameter("GetNearbyNote", y, 3, "number")
	if x < 0 or x > 1 then error(("Argument #2 to 'Notes:GetNearbyNote' is expected to be [0, 1], got %s"):format(x), 1) end
	if y < 0 or y > 1 then error(("Argument #3 to 'Notes:GetNearbyNote' is expected to be [0, 1], got %s"):format(y), 1) end
	checkParameter("GetNearbyNote", radius, 4, "number", "nil")
	if radius and radius < 0 then error(("Argument #4 to 'Notes:GetNearbyNote' is expected to be [0, inf], got %s"):format(radius), 1) end
	checkParameter("GetNearbyNote", creator, 5, "string", "nil")
	checkParameter("GetNearbyNote", use_yards, 6, "boolean", "nil")

	local z, level = getZoneAndLevel(zone)
	local zoneName = self:GetZoneFromMapTexture(z)
	if not zoneName then
		error(("Trying to get a note in an unknown zone: %q"):format(z), 2)
	end

	if not radius then
		radius = math_huge
	end
	if creator and not self.externalDBs[creator] then
		error(("Database %q not registered."):format(creator), 2)
	end

	local zoneData
	if creator then
		zoneData = rawget(self.externalDBs[creator], zone)
	else
		zoneData = rawget(self.db.global.pois, zone)
	end
	if not zoneData or not next(zoneData) then
		return
	end
	local radius_2 = radius*radius

	local close_distance = math_huge
	local close_id

	local yardWidthMult, yardHeightMult = 1, 1
	if use_yards then
		yardWidthMult, yardHeightMult = self:GetZoneYardSize(zone)
		if not yardWidthMult then
			yardWidthMult, yardHeightMult = 1000, 1000 * 2/3
		end
	end

	for id, data in pairs(zoneData) do
		local x_p, y_p = getXY(id)

		local x_d = (x_p - x) * yardWidthMult
		local y_d = (y_p - y) * yardHeightMult

		local d_2 = x_d*x_d + y_d*y_d

		if d_2 <= radius_2 and d_2 < close_distance then
			close_distance = d_2
			close_id = id
		end
	end
	if not close_id then
		return
	end
	local x_p, y_p = getXY(close_id)
	local data = zoneData[close_id]
	return zone, x_p, y_p, type(data) == "string" and data or data.icon, creator, data
end

-----------------------
-- MINIMAP FUNCTIONS --
-----------------------

local MinimapSize = { -- radius of minimap
	indoor = {
		[0] = 150,
		[1] = 120,
		[2] = 90,
		[3] = 60,
		[4] = 40,
		[5] = 25,
	},
	outdoor = {
		[0] = 233 + 1/3,
		[1] = 200,
		[2] = 166 + 2/3,
		[3] = 133 + 1/6,
		[4] = 100,
		[5] = 66 + 2/3,
	},
}

local lastX, lastY = math_huge, math_huge
local lastZoom = -1
local lastTracking = nil
local indoors = "outdoor"
local TRACK = {
	["Interface\\Icons\\Spell_Nature_Earthquake"] = "Mining",
	["Interface\\Icons\\INV_Misc_Flower_02"] = "Herbalism",
	["Interface\\Icons\\Racial_Dwarf_FindTreasure"] = "Treasure",
}
local TRACK2 = {}
local notesCache = {}
local function CheckToUpdateMinimapIcons(x_, y_, x, y, force)
	if isInInstance then
		return
	end
	if not x then
		return
	end
	local diffX = (lastX - x)
	local diffY = (lastY - y)
	local distance_2 = diffX*diffX + diffY*diffY
	if not force and not forceNextMinimapUpdate and distance_2 < 5^2 then
		return
	end
	forceNextMinimapUpdate = false
	lastX, lastY = x, y
	local radius = MinimapSize[indoors][lastZoom]*2
	local radius_2 = radius*radius

	local zone = Notes:GetRealZoneTexture() or Notes:GetCurrentMapTexture()
	if zone == "Cosmic" then
		return
	end

	for id, v in pairs(minimapPois) do
		if id ~= 'del' then
			local idX, idY = Notes:PointToYards(getXY(id))
			local diffX, diffY = (x - idX), (y - idY)
			local distance_2 = diffX*diffX + diffY*diffY
			if distance_2 > radius_2*3 or ((not db.minimapOtherZone or IsInInstance()) and not getrawpoi(zone, id)) then
				minimapPois:del(id)
			end
		end
	end

	local db = nil
	local notesPerDB = Notes.db.profile.minimapNotesPerDB
	if Notes.db.profile.minimapOtherZone and not IsInInstance() then
		local myZone = zone
		wipe(notesCache)
		while true do
			db = next(Notes.externalDBs, db)
			for zones in pairs(db and Notes.externalDBs[db] or Notes.db.global.pois) do
				local mx, my = Notes:TranslateWorldMapPosition(x_, y_, zone, zones)

				if mx and my then
					for zone, x, y, icon, creator, data in Notes:IterateNearbyNotes(zones, mx, my, radius, db, notesPerDB, true, Notes.db.profile.minimapOtherZone) do
						local id = getID(x, y)
						local note_icon
						local default_titleCol, default_info, default_infoCol, default_info2, default_info2Col = goldColorID, "", goldColorID, "", goldColorID
						local keepGoing = false

						if creator then
							local handler = Notes.handlers[creator]
							local defaults = handler and handler.noteDefaults
							if type(defaults) ~= "table" then
								defaults = nil
							end
							local data = rawget(Notes.externalDBs[creator], zone)
							if handler and handler.IsTracking and handler:IsTracking(zone, id, data) then
								TRACK2[creator] = true
							elseif TRACK2[creator] then
								TRACK2[creator] = nil
							end
							if not data then
								keepGoing = true
							else
								data = rawget(data, id)
								if not data then
									keepGoing = true
								elseif handler and handler.IsNoteHidden and handler:IsNoteHidden(zone, id, data) then
									keepGoing = true
								elseif handler and handler.IsMiniNoteHidden and handler:IsMiniNoteHidden(zone, id, data) then
									keepGoing = true
								elseif handler and handler.GetNoteIcon then
									note_icon = handler:GetNoteIcon(zone, id, data)
									if type(note_icon) ~= "string" then note_icon = nil end
								end
								if data and notesCache[creator] then
									local title = rawget(data, "title")
									if title and notesCache[creator][title] then
										keepGoing = true
									end
								end
								if data and zone ~= myZone then
									local title = rawget(data, "title")
									if title then
										local zoneData = rawget(Notes.externalDBs[creator], myZone)
										if zoneData then
											for id, v in pairs(zoneData) do
												if rawget(v, "title") == title then
													keepGoing = true
												end
											end
										end
									end
								end
							end
							if keepGoing and rawget(minimapPois, id) then
								minimapPois:del(id)
							elseif defaults then
								default_titleCol = defaults.titleCol or goldColorID
								default_info = defaults.info or ""
								default_infoCol = defaults.infoCol or goldColorID
								default_info2 = defaults.info2 or ""
								default_info2Col = defaults.info2Col or goldColorID
							end
							if not keepGoing then
								local title = rawget(data, "title")
								if title then
									if not notesCache[creator] then
										notesCache[creator] = {}
									end
									notesCache[creator][title] = true
								end
							end
						end
						if not keepGoing then
							local poi = minimapPois[id]
							poi.zone = zone
							local icon
							local creator = db or data.creator or ""
							if type(data) == "string" then
								poi.title = getIconTitle(data)
								poi.titleCol = default_titleCol
								poi.info = default_info
								poi.infoCol = default_infoCol
								poi.info2 = default_info2
								poi.info2Col = default_info2Col
								poi.manual = false
								icon = note_icon or data
							else
								poi.title = data.title or getIconTitle(data.icon)
								poi.titleCol = data.titleCol or default_titleCol
								poi.info = data.info or default_info
								poi.infoCol = data.infoCol or default_infoCol
								poi.info2 = data.info2 or default_info2
								poi.info2Col = data.info2Col or default_info2Col
								poi.manual = data.manual or false
								icon = note_icon or data.icon
							end
							poi.creator = creator
							poi.id = id
							poi.icon = icon
							if creator and (creator == TRACK[lastTracking] or TRACK2[creator]) then
								local poiX, poiY = Notes:PointToYards(x, y)
								local diffX = lastX - poiX
								local diffY = lastY - poiY
								local distance_2 = diffX*diffX + diffY*diffY
								if distance_2 <= 100^2 then
									icon = NOTES_ICONPATH .. "TrackCircle"
								end
							end

							poi:SetParent(Minimap)
							if type(icon) == "string" and string_find(icon, "^Interface\\") then
								poi:SetWidth(16*Notes.db.profile.minimapIconSize)
								poi:SetHeight(16*Notes.db.profile.minimapIconSize)
								poi:SetAlpha(1)
								poi.texture:SetTexture(icon)
								if string_find(icon, "^Interface\\Icons\\") then
									poi.texture:SetTexCoord(0.05, 0.95, 0.05, 0.95)
								else
									poi.texture:SetTexCoord(0, 1, 0, 1)
								end
							else
								local t = icons[icon]
								if not t then
									t = icons.Unknown
								end
								poi:SetWidth(t.width*Notes.db.profile.minimapIconSize)
								poi:SetHeight(t.height*Notes.db.profile.minimapIconSize)
								poi:SetAlpha(t.alpha)
								poi.texture:SetTexture(t.path)
								poi.texture:SetTexCoord(t.cLeft, t.cRight, t.cTop, t.cBottom)
							end
							poi:Show()
						end
					end
				else
					for id, v in pairs(minimapPois) do
						if id ~= 'del' then
							if rawget(v, zone) == zones then
								minimapPois:del(id)
							end
						end
					end
				end

			end
			if db == nil then
				break
			end
		end
	else
		while true do
			db = next(Notes.externalDBs, db)
			for zone, x, y, icon, creator, data in Notes:IterateNearbyNotes(zone, x_, y_, radius, db, notesPerDB, true) do
				local id = getID(x, y)
				local note_icon
				local default_titleCol, default_info, default_infoCol, default_info2, default_info2Col = goldColorID, "", goldColorID, "", goldColorID
				local keepGoing = false

				if creator then
					local handler = Notes.handlers[creator]
					local defaults = handler and handler.noteDefaults
					if type(defaults) ~= "table" then
						defaults = nil
					end
					local data = rawget(Notes.externalDBs[creator], zone)
					if handler and handler.IsTracking and handler:IsTracking(zone, id, data) then
						TRACK2[creator] = true
					elseif TRACK2[creator] then
						TRACK2[creator] = nil
					end
					if not data then
						keepGoing = true
					else
						data = rawget(data, id)
						if not data then
							keepGoing = true
						elseif handler and handler.IsNoteHidden and handler:IsNoteHidden(zone, id, data) then
							keepGoing = true
						elseif handler and handler.IsMiniNoteHidden and handler:IsMiniNoteHidden(zone, id, data) then
							keepGoing = true
						elseif handler and handler.GetNoteIcon then
							note_icon = handler:GetNoteIcon(zone,id,data)
							if type(note_icon) ~= "string" then note_icon = nil end
						end
					end
					if keepGoing and rawget(minimapPois, id) then
						minimapPois:del(id)
					elseif defaults then
						default_titleCol = defaults.titleCol or goldColorID
						default_info = defaults.info or ""
						default_infoCol = defaults.infoCol or goldColorID
						default_info2 = defaults.info2 or ""
						default_info2Col = defaults.info2Col or goldColorID
					end
				end
				if not keepGoing then
					local poi = minimapPois[id]
					poi.zone = zone
					local icon
					local creator = db or data.creator or ""
					if type(data) == "string" then
						poi.title = getIconTitle(data)
						poi.titleCol = default_titleCol
						poi.info = default_info
						poi.infoCol = default_infoCol
						poi.info2 = default_info2
						poi.info2Col = default_info2Col
						poi.manual = false
						icon = note_icon or data
					else
						poi.title = data.title or getIconTitle(data.icon)
						poi.titleCol = data.titleCol or default_titleCol
						poi.info = data.info or default_info
						poi.infoCol = data.infoCol or default_infoCol
						poi.info2 = data.info2 or default_info2
						poi.info2Col = data.info2Col or default_info2Col
						poi.manual = data.manual or false
						icon = note_icon or data.icon
					end
					poi.creator = creator
					poi.id = id
					poi.icon = icon
					if creator and (creator == TRACK[lastTracking] or TRACK2[creator]) then
						local poiX, poiY = Notes:PointToYards(x, y)
						local diffX = lastX - poiX
						local diffY = lastY - poiY
						local distance_2 = diffX*diffX + diffY*diffY
						if distance_2 <= 100^2 then
							icon = NOTES_ICONPATH .. "TrackCircle"
						end
					end

					poi:SetParent(Minimap)
					if type(icon) == "string" and string_find(icon, "^Interface\\") then
						poi:SetWidth(16*Notes.db.profile.minimapIconSize)
						poi:SetHeight(16*Notes.db.profile.minimapIconSize)
						poi:SetAlpha(1)
						poi.texture:SetTexture(icon)
						if string_find(icon, "^Interface\\Icons\\") then
							poi.texture:SetTexCoord(0.05, 0.95, 0.05, 0.95)
						else
							poi.texture:SetTexCoord(0, 1, 0, 1)
						end
					else
						local t = icons[icon]
						if not t then
							t = icons.Unknown
						end
						poi:SetWidth(t.width*Notes.db.profile.minimapIconSize)
						poi:SetHeight(t.height*Notes.db.profile.minimapIconSize)
						poi:SetAlpha(t.alpha)
						poi.texture:SetTexture(t.path)
						poi.texture:SetTexCoord(t.cLeft, t.cRight, t.cTop, t.cBottom)
					end
					poi:Show()
				end
			end
			if db == nil then
				break
			end
		end
	end
end

local Minimap = Minimap
local lastX, lastY, lastFacing = math_huge, math_huge, math_huge
function Notes:UpdateMinimapIcons(elapsed)
	if isInInstance then
		return
	end
	local x_, y_, zone = self:GetCurrentPlayerPosition()
	if not x_ or not y_ or x_ < 0 or x_ > 1 or y_ < 0 or y_ > 1 then
		for id in pairs(minimapPois) do
			minimapPois:del(id)
		end
		return
	end
	local x, y = self:PointToYards(x_, y_, zone)
	if not x then
		return
	end
	local zoom = Minimap:GetZoom()
	local diffZoom = zoom ~= lastZoom
	local tracking = nil
	local diffTracking = tracking ~= lastTracking
	local GetMinimapShape = GetMinimapShape
	local minimapShape = GetMinimapShape and GetMinimapShape() or "ROUND"
	local facing
	if rotateMinimap then
		facing = GetPlayerFacing()
	else
		facing = lastFacing
	end
	if diffZoom or diffTracking or x ~= lastX or y ~= lastY or facing ~= lastFacing or forceNextMinimapUpdate then
		lastZoom = zoom
		lastTracking = tracking
		lastFacing = facing
		local Minimap_Width = Minimap:GetWidth()/2
		lastX, lastY = x, y
		CheckToUpdateMinimapIcons(x_, y_, x, y, diffZoom)
		local radius = MinimapSize[indoors][lastZoom]

		for id, poi in pairs(minimapPois) do
			if id ~= 'del' then
				local px, py = self:PointToYards(getXY(id))
				if self.db.profile.minimapOtherZone and not IsInInstance() then
					px, py = getXY(id)
					local mx, my = self:TranslateWorldMapPosition(px, py, poi.zone, zone)
					if mx and my then
						px, py = self:PointToYards(mx, my)
					else
						px, py = nil, nil
					end
				end
				if px and py then
					local dx, dy = px - x, py - y
					if rotateMinimap then
						local sin = math_sin(facing)
						local cos = math_cos(facing)
						dx, dy = dx*cos - dy*sin, dx*sin + dy*cos
					end
					local diffX = dx / radius
					local diffY = dy / radius
					local dist
					local alpha = 1
					local round = true
					if minimapShape == "ROUND" then
						-- do nothing
					elseif minimapShape == "SQUARE" then
						round = false
					elseif minimapShape == "CORNER-TOPRIGHT" then
						if diffX < 0 or diffY > 0 then
							round = false
						end
					elseif minimapShape == "CORNER-TOPLEFT" then
						if diffX > 0 or diffY > 0 then
							round = false
						end
					elseif minimapShape == "CORNER-BOTTOMRIGHT" then
						if diffX < 0 or diffY < 0 then
							round = false
						end
					elseif minimapShape == "CORNER-BOTTOMLEFT" then
						if diffX > 0 or diffY < 0 then
							round = false
						end
					elseif minimapShape == "SIDE-LEFT" then
						if diffX > 0 then
							round = false
						end
					elseif minimapShape == "SIDE-RIGHT" then
						if diffX < 0 then
							round = false
						end
					elseif minimapShape == "SIDE-TOP" then
						if diffY > 0 then
							round = false
						end
					elseif minimapShape == "SIDE-BOTTOM" then
						if diffY < 0 then
							round = false
						end
					elseif minimapShape == "TRICORNER-TOPRIGHT" then
						if diffX < 0 and diffY > 0 then
							round = false
						end
					elseif minimapShape == "TRICORNER-TOPLEFT" then
						if diffX > 0 and diffY > 0 then
							round = false
						end
					elseif minimapShape == "TRICORNER-BOTTOMRIGHT" then
						if diffX < 0 and diffY < 0 then
							round = false
						end
					elseif minimapShape == "TRICORNER-BOTTOMLEFT" then
						if diffX > 0 and diffY < 0 then
							round = false
						end
					end
					if round then
						dist = (diffX*diffX + diffY*diffY) / 0.9^2
					else
						dist = math_max(diffX*diffX, diffY*diffY) / 0.9^2
					end
					if dist > 1 then
						dist = dist^0.5
						diffX = diffX/dist
						diffY = diffY/dist
						alpha = 2 - dist*0.4
					end
					poi:SetPoint("CENTER", Minimap, "CENTER", diffX * Minimap_Width, -diffY * Minimap_Width)
					poi:SetAlpha(alpha)
				else
					minimapPois:del(id)
				end
			end
		end
	end
end

function Notes:configureYards()
	if not WorldMapFrame:IsShown() then
		SetMapToCurrentZone()
	end
	local currentZone = self:GetCurrentMapTexture()
	currentYardWidth, currentYardHeight = self:GetZoneYardSize(currentZone)
	if not currentYardWidth then
		local currentZone = self:GetCurrentZoneName()
		currentYardWidth, currentYardHeight = 1000, 1000 * 2/3
	end

	forceNextMinimapUpdate = true
	isInInstance = self:GetMapTexture(GetRealZoneText()) == nil
	if isInInstance then
		for k,v in pairs(minimapPois) do
			minimapPois:del(k)
		end
	end
end


function Notes:MINIMAP_ZONE_CHANGED()
	self:configureYards()
end

function Notes:MINIMAP_UPDATE_ZOOM()
	forceNextMinimapUpdate = true
	local zoom = Minimap:GetZoom()
	if GetCVar("minimapZoom") == GetCVar("minimapInsideZoom") then
		Minimap:SetZoom(zoom < 2 and zoom + 1 or zoom - 1)
	end
	indoors = GetCVar("minimapZoom")+0 == Minimap:GetZoom() and "outdoor" or "indoor"
	Minimap:SetZoom(zoom)
	self:MINIMAP_ZONE_CHANGED()
end

function Notes:ZONE_CHANGED()
	self:configureYards()
end

function Notes:ZONE_CHANGED_NEW_AREA()
	self:PLAYER_LEAVING_WORLD()
	self:MINIMAP_UPDATE_ZOOM()
end

function Notes:PLAYER_LEAVING_WORLD()
	for k,v in pairs(minimapPois) do
		minimapPois:del(k)
	end
	forceNextMinimapUpdate = true
end

function Notes:CVAR_UPDATE(caller, event, cvar, value)
	if cvar == "ROTATE_MINIMAP" then
		rotateMinimap = tostring(value) == "1"
	end
end

-----------------------------
-- SLASH COMMAND FUNCTIONS --
-----------------------------

local function GetUnitReactionIndex(unit)
	if (UnitIsPlayer(unit) or UnitPlayerControlled(unit)) then
		if (UnitCanAttack(unit, "player")) then
			return UnitCanAttack("player", unit) and 2 or 3
		elseif (UnitCanAttack("player", unit)) then
			return 3
		elseif (UnitIsPVP(unit) and not UnitIsPVPSanctuary(unit) and not UnitIsPVPSanctuary("player")) then
			return 5
		else
			return 6
		end
	else
		local reaction = UnitReaction(unit, "player") or 3
		return (reaction > 5 and 5) or (reaction < 2 and 2) or reaction
	end
end

local levelMatch = "^"..TOOLTIP_UNIT_LEVEL:gsub("%%s",".+")
local classifications = { trivial = "~%s", normal = "%s", elite = "+%s", worldboss = "%s (Boss)", rare = "%s (Rare)", rareelite = "+%s (Rare)" }
function Notes:TargetInfo(unit)
	local name, realm = UnitName(unit)
	if (realm and realm ~= "") then name = name.." - "..realm end
	local level = UnitLevel(unit)
	if (level == -1) then level = "??" end
	level = format(classifications[UnitClassification(unit)] or "%s", level)
	if UnitIsPlayer(unit) then
		local class, classFixed = UnitClass(unit)
		local color = RAID_CLASS_COLORS[classFixed] or RAID_CLASS_COLORS["PRIEST"]
		local race = UnitRace(unit)
		local reaction = GetUnitReactionIndex(unit)
		return name, format("|cffffff00%s|r |cffffffff%s|r |cff%02x%02x%02x%s|r", level, race, color.r*255, color.g*255, color.b*255, class), nil, reaction
	else
		if (not MapsterNotesTip) then
			MapsterNotesTip = CreateFrame("GameTooltip")
			MapsterNotesTip:SetOwner(UIParent, "ANCHOR_NONE")
			MapsterNotesTip.left = {}
			MapsterNotesTip.right = {}
			for i = 1, 4 do
				MapsterNotesTip.left[i] = MapsterNotesTip:CreateFontString()
				MapsterNotesTip.left[i]:SetFontObject(GameFontNormal)
				MapsterNotesTip.right[i] = MapsterNotesTip:CreateFontString()
				MapsterNotesTip.right[i]:SetFontObject(GameFontNormal)
				MapsterNotesTip:AddFontStrings(MapsterNotesTip.left[i], MapsterNotesTip.right[i])
			end
		end
		MapsterNotesTip:ClearLines()
		MapsterNotesTip:SetUnit(unit)
		local title
		if (MapsterNotesTip:NumLines() > 2) then
			if (strfind(MapsterNotesTip.left[3]:GetText() or "", levelMatch)) then
				title = MapsterNotesTip.left[2]:GetText()
			end
		end
		local class = UnitCreatureFamily(unit) or UnitCreatureType(unit) or UNKNOWN
		local reaction = GetUnitReactionIndex(unit)
		return name, format("|cffffff00%s|r |cffffffff%s|r", level, class), title, reaction
	end
end

local tmp = {}
local options = {}
function Notes:AddSlashCommand(command, text)
	wipe(options)
	if command == "note" then
		text = text:gsub("[%s,]+", " "):trim()
		if text:match("^%d%d?%.%d%d? [^%d].*$") or text:match("^%d%d?%.%d%d?$") then
			text = text:gsub("^(%d%d?)%.(%d%d?)", "%1 %2", 1)
		end
		local x, y, title = (" "):split(text, 3)
		x, y = tonumber(x), tonumber(y)
		if not x or not y then
			self:Print(L["Commands are:"])
			self:Print(L["/note <0-100> <0-100> [title]"])
			self:Print(L["|cffffd200- Sets a note at the specified location, [title] is optional"])
			self:Print(L["/qnote [title]"])
			self:Print(L["|cffffd200- Sets a note at your current location, [title] is optional"])
			self:Print(L["/tnote [title]"])
			self:Print(L["|cffffd200- Sets a note at your current location (or target's location) with target info, [title] is optional"])
			return
		end
		x, y = x / 100, y / 100

		for k,v in pairs(icons) do
			if v.showToUser then
				table_insert(tmp, k)
			end
		end
		local icon = tmp[math_random(1, #tmp)]
		wipe(tmp)
		if title and title:len() == 0 then
			title = nil
		end
		if title and icons[title] then
			icon = title
			title = nil
		end
		self:SetNote(self:GetCurrentMapTexture(), x, y, icon, (UnitName("player")),
			'manual', true,
			'title', title)
		self:Print(format(L["New note %q placed at %.1f, %.1f in %s"], title or icon, x * 100, y * 100, self:GetCurrentZoneName() or UNKNOWN))
	elseif command == "qnote" then
		local x, y, zone = self:GetCurrentPlayerPosition()
		if not x or not y or x == 0 or y == 0 then
			self:Print(L["Unable to find your location."])
			return
		end
		local title, icon
		if text then
			title, icon = text:match("(%b\"\") ?(.*)") or text
			title = title:gsub("\"", "")
		end

		if not icon or not icons[icon] then
			if title and icons[title] then
				icon = title
				title = nil
			else
				for k,v in pairs(icons) do
					if v.showToUser then
						table_insert(tmp, k)
					end
				end
				icon = tmp[math_random(1, #tmp)]
				wipe(tmp)
			end
		end
		if title and title:len() == 0 then
			title = nil
		end
		self:SetNote(self:GetCurrentMapTexture(), x, y, icon, (UnitName("player")),
			'manual', true,
			'title', title)
		self:Print(format(L["New note %q placed at %.1f, %.1f in %s"], title or icon, x * 100, y * 100, self:GetCurrentZoneName() or UNKNOWN))
	elseif command == "tnote" then
		local targetName = UnitName("target")
		if not targetName or targetName == UnitName("player") or targetName == UNKNOWNOBJECT then
			self:Print(L["You don't have a target."])
			return
		end

		local x, y, zone = self:GetCurrentUnitPosition("target")
		if not x or not y or x == 0 or y == 0 then
			x, y, zone = self:GetCurrentPlayerPosition()
			if not x or not y or x == 0 or y == 0 then
				self:Print(L["Unable to find your location."])
				return
			end
		end

		local title, icon, info, info2, name, class, reaction
		if text then
			title, icon = text:match("(%b\"\") ?(.*)") or text
			title = title:gsub("\"", "")
		end
		if not icon or not icons[icon] then
			if title and icons[title] then
				icon = title
				title = nil
			else
				icon = nil
			end
		end
		name, class, info, reaction = self:TargetInfo("target")

		if not title or title == "" then
			title = name
			if info then
				info2 = class
			else
				info = class
				info2 = nil
			end
		else
			info = name
			info2 = info or class
		end
		if not icon then
			if reaction < 4 then
				icon = "ObjectIcons_Red"
			elseif reaction == 4 then
				icon = "ObjectIcons_Yellow"
			else
				icon = "ObjectIcons_Green"
			end
		end
		local r, g, b
		if reaction < 4 then
			r, g, b = 1, 0, 0
		elseif reaction == 4 then
			r, g, b = 1, 1, 0
		else
			r, g, b = 0, 1, 0
		end
		self:SetNote(self:GetCurrentMapTexture(), x, y, icon, (UnitName("player")),
			'manual', true,
			'title', title,
			'titleR', r,
			'titleG', g,
			'titleB', b,
			'info', info,
			'infoR', r,
			'infoG', g,
			'infoB', b,
			'info2', info2,
			'info2R', r,
			'info2G', g,
			'info2B', b
		)
		self:Print(format(L["New note %q placed at %.1f, %.1f in %s"], title or icon, x * 100, y * 100, self:GetCurrentZoneName() or UNKNOWN))
	end
end
