--[[
MapsterEnhanced - POI (Sindragosa)

Based on Cartographer
Credits: ckknight
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local L = LibStub("AceLocale-3.0"):GetLocale("Mapster")

local MODNAME = "POI"
local POI = Mapster:NewModule(MODNAME, "AceEvent-3.0")


local GetMapInfo = GetMapInfo
local GetCurrentMapZone = GetCurrentMapZone
local GetCurrentMapContinent = GetCurrentMapContinent
local GetCurrentMapDungeonLevel = GetCurrentMapDungeonLevel
local GetNumMapLandmarks = GetNumMapLandmarks
local GetMapLandmarkInfo = GetMapLandmarkInfo
local string_find = string.find

local Notes

local db
local defaults = {
	profile = {
		pois = {},
	}
}

local optGetter, optSetter
do
	local mod = POI
	function optGetter(info)
		local key = info[#info]
		return db[key]
	end

	function optSetter(info, value)
		local key = info[#info]
		db[key] = value
		mod:Refresh()
	end
end

local options
local function getOptions()
	if not options then
		options = {
			type = "group",
			name = L["POI"],
			arg = MODNAME,
			get = optGetter,
			set = optSetter,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["The POI module adds points of interest to the World Map frame."],
				},
				enabled = {
					order = 2,
					type = "toggle",
					name = L["Enable POI"],
					get = function() return Mapster:GetModuleEnabled(MODNAME) end,
					set = function(info, value) Mapster:SetModuleEnabled(MODNAME, value) end,
				},
			},
		}
	end

	return options
end

local path = "Interface\\AddOns\\MapsterEnhanced\\Textures\\"
local icon = {
	text = L["POI"],
	path = path .. "ObjectIcons_Blue",
	width = 14,
	height = 14,
}

local exceptions = {
	["PvP Activity"] = 1,
	["Shadowsight Tower"] = 2,
	["Flamewatch Tower"] = 2,
	["Winter's Edge Tower"] = 2,
	["Broken Temple Vehicle Workshop"] = 2,
	["Sunken Ring Vehicle Workshop"] = 2,
	["Westspark Vehicle Workshop"] = 2,
	["Eastspark Vehicle Workshop"] = 2,
	["Fortress Vehicle Workshop (W)"] = 2,
	["Fortress Vehicle Workshop (E)"] = 2,
	["Wintergrasp Fortress Tower (NW)"] = 2,
	["Wintergrasp Fortress Tower (NE)"] = 2,
	["Wintergrasp Fortress Tower (SW)"] = 2,
	["Wintergrasp Fortress Tower (SE)"] = 2,
	["Wintergrasp Fortress Wall"] = 1,
	["Wintergrasp Fortress Gate"] = 1,
	["Wintergrasp Fortress"] = 2,
	["Fortress Graveyard"] = 2,
	["Southwest Graveyard"] = 2,
	["Southeast Graveyard"] = 2,
	["Broken Temple Graveyard"] = 2,
	["Sunken Ring Graveyard"] = 2,
	["Baradin Hold"] = 2,
	["Ironclad Garrison"] = 2,
	["Warden's Vigil"] = 2,
	["Slagworks"] = 2,
	["West Spire"] = 2,
	["East Spire"] = 2,
	["South Spire"] = 2,
	["Wyvern Camp"] = 1,
	--["West Beacon"] = 1,
	--["East Beacon"] = 1,
	--["Twinspire Graveyard"] = 1,
	--["Horde Field Scout"] = 1,
	--["Alliance Field Scout"] = 1,
	["Broken Hill"] = 1,
	["The Overlook"] = 1,
	["The Stadium"] = 1,
	["Spirit Tower"] = 1,
}

function POI:OnInitialize()
	self.db = Mapster.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	self:SetEnabledState(Mapster:GetModuleEnabled(MODNAME))
	Mapster:RegisterModuleOptions(MODNAME, getOptions, L["POI"])
end

function POI:OnEnable()
	Notes = Mapster:GetModule("Notes")
	if Notes and Notes:IsEnabled() then
		Notes:RegisterNotesDatabase("POI", self.db.profile.pois, self)
		Notes:RegisterIcon("POI", icon)
	else
		Mapster:SetModuleEnabled(MODNAME, false)
		return
	end

	self:RegisterEvent("WORLD_MAP_UPDATE")
	self:Refresh()
end

function POI:OnDisable()
	if Notes and Notes:IsEnabled() then
		Notes:UnregisterIcon("POI")
		Notes:UnregisterNotesDatabase("POI")
	end
	self:UnregisterEvent("WORLD_MAP_UPDATE")
end

function POI:Refresh()
	db = self.db.profile
	if not self:IsEnabled() then return end

	Notes:RefreshMap()
end

function POI:WORLD_MAP_UPDATE()
	if not Notes:IsEnabled() then
		return
	end

	local continentIndex, zoneIndex = GetCurrentMapContinent(), GetCurrentMapZone()
	if (continentIndex < 1) or (zoneIndex == 0) then
		return
	end

	local zone = Notes:GetCurrentMapTexture()
	if zone then
		local name, desc, tex, x, y
		for landmark = 1, GetNumMapLandmarks(), 1 do
			name, desc, tex, x, y = GetMapLandmarkInfo(landmark)
			if not string_find(name, "PvP Activity") and not string_find(name, ", ") and not exceptions[name] then
				Notes:SetNote(
					zone, x, y,
					"POI",
					"POI",
					"title", name,
					"titleR", 1.0,
					"titleG", 0.5,
					"titleB", 0.0
				)
			end
		end
	end
end
