--[[
MapsterEnhanced - SatMap (Sindragosa)

Based on Cartographer
Credits: ckknight
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local MapsterEnhanced = LibStub("AceAddon-3.0"):GetAddon("MapsterEnhanced")
local L = LibStub("AceLocale-3.0"):GetLocale("Mapster")
local MODNAME = "SatMap"
local SatMap = Mapster:NewModule(MODNAME, "AceEvent-3.0", "AceHook-3.0")

local LSM3 = LibStub("LibSharedMedia-3.0")

local GetCursorPosition = GetCursorPosition
local WorldMapDetailFrame = WorldMapDetailFrame
local GetTime = GetTime
local GetMapInfo = GetMapInfo
local GetCurrentMapZone = GetCurrentMapZone
local GetCurrentMapContinent = GetCurrentMapContinent
local GetCurrentMapDungeonLevel = GetCurrentMapDungeonLevel
local GetMapContinents = GetMapContinents
local GetMapZones = GetMapZones
local UpdateMapHighlight = UpdateMapHighlight
local IsControlKeyDown = IsControlKeyDown
local IsAltKeyDown = IsAltKeyDown

-- local --
local _G = getfenv(0)
local table_sort = table.sort
local string_find = string.find
local string_match = string.match
local format = format
local floor = floor
local ipairs = ipairs
local pairs = pairs
local next = next
local setmetatable = setmetatable
local tonumber = tonumber
local tostring = tostring
local unpack = unpack
local math = math
local math_random = math.random
local math_sin = math.sin
local math_cos = math.cos
local math_min = math.min
local math_max = math.max
local math_atan2 = math.atan2
local math_abs = math.abs

local TEXTURE_PATH = "Interface\\AddOns\\MapsterEnhanced\\Textures\\"
local updateInterval = 0
local mapView
local Notes

local zoneSearchData = {}
local mapTextures = {}
local mapTextureFrames = {}
local poiTypeScales = { 12, 12, 18, 18 }
local shownPOIs = {}
local showMinimapStyle
local createMapTextureSection
local initializeContinents = true
local initializeMapHolder = true
local initializePoints = true
local initializeInstances = true
local initializeZoneSearch = true
local inQuelThalas = true
local QuelThalasZones = { EversongWoods = true, Ghostlands = true, Sunwell = true, SilvermoonCity = true, Sunwell5Man = true, SunwellPlateau = true, ZulAman = true }
local QuelThalasMaps = {
	[4019] = { 1 },
	[4119] = { 1 },
	[4120] = { 1 },
	[4219] = { 1 },
	[4220] = { 1 },
	[4320] = { 1 },
	[4420] = { 1 },
	[4520] = { 1 },
	[4620] = { 1 },
	[4720] = { 1 },
	[4820] = { 1 },
	[3723] = { 2 },
	[3823] = { 2 },
	[3824] = { 2 },
	[3825] = { 2 },
	[3923] = { 2 },
	[3924] = { 2 },
	[3925] = { 2 },
	[4023] = { 2 },
	[4024] = { 2 },
	[4025] = { 2 },
	[4123] = { 2 },
	[4124] = { 2 },
	[4125] = { 2 },
	[4224] = { 2 },
}

SatMap.data = {}
SatMap.data.mapViewX = 0
SatMap.data.mapViewY = 0
SatMap.data.mapViewZoom = 1
SatMap.data.poiTypeScales = poiTypeScales
SatMap.data.shownPOIs = shownPOIs

SatMap.data.ZONE_DATA = {}

local MAP_HEIGHT_TO_WIDTH_RATIO = 2/3
local MINIMAP_TILE_YARD_SIZE = 533 + 1/3
local YARDS_PER_PIXEL = 100
local ITERATIVE_SECONDS = 1/300


local db
local defaults = {
	profile = {
		overlayAlpha = 1,
		poiLabels = true,
		poiLabelsOnlyCtrl = true,
		pois = {
		}
	}
}

local optGetter, optSetter
do
	local mod = SatMap
	function optGetter(info)
		local key = info[#info]
		return db[key]
	end

	function optSetter(info, value)
		local key = info[#info]
		db[key] = value
		mod:Refresh()
	end
end

local options
local function getOptions()
	if not options then
		options = {
			type = "group",
			name = L["SatMap"],
			arg = MODNAME,
			get = optGetter,
			set = optSetter,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["The SatMap module adds minimap style satellite images to the World Map."],
				},
				enabled = {
					order = 2,
					type = "toggle",
					width = "full",
					name = L["Enable SatMap"],
					get = function() return Mapster:GetModuleEnabled(MODNAME) end,
					set = function(info, value) Mapster:SetModuleEnabled(MODNAME, value) end,
				},
				desc = {
					order = 3,
					type = "description",
					name = "\n" .. L["You can control the display of the map images."],
				},
				mapView = {
					order = 4,
					type = "toggle",
					name = L["Show map images"],
					desc = L["Display of the map images."],
					get = function(info) return db.mapView end,
					set = function(info, value)
						SatMap:ToggleMap()
					end,
				},
				overlayAlpha = {
					order = 5,
					type = "range",
					name = L["Alpha"],
					desc = L["The transparency of the map overlays."],
					min = 0, max = 1, step = 0.01,
					isPercent = true,
					get = function(info) return db.overlayAlpha end,
					set = function(info, value)
						db.overlayAlpha = value
						SatMap.mapHolder:SetAlpha(value)
					end,
				},
				desc = {
					order = 6,
					type = "description",
					name = "",
				},
				poiLabels = {
					order = 7,
					type = "toggle",
					name = L["POI Labels"],
					desc = L["Display labels next to POI icons."],
					get = function(info) return db.poiLabels end,
					set = function(info, value)
						db.poiLabels = value
						if value and showMinimapStyle then
							if initializePoints then
								SatMap:InitializePoints()
							elseif not db.poiLabelsOnlyCtrl then
								if SatMap.data.currentZoneID == 0 and not SatMap.data.currentZoneData then
									SatMap.poiContFrame:Hide()
									SatMap.poiZoneFrame:Hide()
								else
									SatMap.poiContFrame:Show()
									SatMap.poiZoneFrame:Show()
								end
							end
						elseif showMinimapStyle then
							if not initializePoints and not db.poiLabelsOnlyCtrl then
								SatMap.poiContFrame:Hide()
								SatMap.poiZoneFrame:Hide()
							end
						end
						SatMap:Refresh()
					end,
				},
				poiLabelsOnlyCtrl = {
					order = 8,
					type = "toggle",
					name = L["Only on Ctrl"],
					desc = L["Display labels next to POI icons only when holding Control."],
					get = function(info) return db.poiLabelsOnlyCtrl end,
					set = function(info, value)
						db.poiLabelsOnlyCtrl = value
						if value then
							if db.poiLabels then
								SatMap.poiContFrame:Hide()
								SatMap.poiZoneFrame:Hide()
							end
						elseif db.poiLabels then
							SatMap.poiContFrame:Show()
							SatMap.poiZoneFrame:Show()
						end
						SatMap:Refresh()
					end,
					disabled = function(info)
						return not db.poiLabels;
					end,
				},
			},
		}
	end

	return options
end

function SatMap:OnInitialize()
	self.db = Mapster.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	self:SetEnabledState(Mapster:GetModuleEnabled(MODNAME))
	Mapster:RegisterModuleOptions(MODNAME, getOptions, L["SatMap"])
	Mapster:GetPrint(MODNAME)
	LSM3:Register("font", "Myriad", TEXTURE_PATH.."Myriad.ttf");
	self.fontMyriad = LSM3:Fetch("font", "Myriad")

	if not self.mapHolder then
		self.scrollFrame = CreateFrame("ScrollFrame", "SatMap_ScrollFrame", WorldMapFrame)
		self.scrollFrame:SetPoint("TOPLEFT", WorldMapDetailFrame, "TOPLEFT")
		self.scrollFrame:SetPoint("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT")
		self.scrollFrame:SetFrameStrata(WorldMapDetailFrame:GetFrameStrata())
		self.scrollFrame:Hide()
		self.mapHolder = CreateFrame("Button", "SatMap_MapHolder", self.scrollFrame)
		self.mapHolder:SetPoint("TOPLEFT", WorldMapDetailFrame, "TOPLEFT")
		self.mapHolder:SetPoint("BOTTOMRIGHT", WorldMapDetailFrame, "BOTTOMRIGHT")
		self.mapHolder:SetFrameStrata(WorldMapDetailFrame:GetFrameStrata())
		self.mapHolder:Hide()
	end
	self.mapHolder:SetAlpha(db.overlayAlpha or 1)
	mapView = self.mapHolder

	self.mapButton = CreateFrame("Button", "SatMap_MapButton", WorldMapFrame, "UIPanelButtonTemplate")
	self.mapButton:SetWidth(80)
	self.mapButton:SetText("SatMap")
	self.mapButton:ClearAllPoints()
	self.mapButton:SetPoint("LEFT", WorldMapZoomOutButton, "RIGHT", 6, 0)
	self.mapButton:SetScript("OnClick", function(self) SatMap:ToggleMap() end)
	self.mapButton:Hide()
end


-- round --
local function round(num, digits)
	if not digits then digits = 0 end
	local mantissa = 10^digits
	local norm = num*mantissa
	norm = norm + 0.5
	local norm_f = floor(norm)
	return norm_f/mantissa
end

-- getID --
local function getID(x, y)
	if not x or not y then return nil end
	return round(x * 10000, 0) + round(y * 10000, 0) * 10000
end

-- getXY --
local function getXY(id)
	if not id then return nil, nil end
	return (id % 10000) / 10000, floor(id / 10000) / 10000
end

local function InitializeMapHolder()
	if not initializeMapHolder then return end
	initializeMapHolder = nil

	local mapView = SatMap.mapHolder
	SatMap.scrollFrame:SetScrollChild(mapView)
	SatMap.mapHolder:SetWidth(10000)
	SatMap.mapHolder:SetHeight(10000)
	SatMap:ReadjustView()
	createMapTextureSection = true
end

local function InitializeContinentData()
	if not initializeContinents then return end
	initializeContinents = nil
	local continentNames = { GetMapContinents() }
	for i,v in ipairs(continentNames) do
		local data = SatMap.data.CONTINENT_DATA[i]
		if not data then
			data = {
				x = 0,
				y = 375,
				height = 20000 * 2/3,
				width = 20000,
				rect = { 0, 1, 0, 1 },
				mapTextureCenterX = 0,
				mapTextureCenterY = 0,
				mapTextureOffsetX = 0,
				mapTextureOffsetY = 0,
				mapTextureBackground = "water1",
			}
			SatMap.data.CONTINENT_DATA[i] = data
		end
		data.name = v
	end

	for continentID, data in ipairs(SatMap.data.CONTINENT_DATA) do
		local currentContinentData = {}
		SatMap.data.CONTINENT_DATA[continentID] = currentContinentData
		SetMapZoom(continentID)
		currentContinentData.texture = SatMap.data.currentMapTexture
		SatMap.data.CONTINENT_DATA[currentContinentData.texture] = currentContinentData
		currentContinentData.id = continentID

		currentContinentData.localizedName = data.name

		local continentCenterX = data.x
		currentContinentData.fullCenterX = continentCenterX

		local continentCenterY = data.y
		currentContinentData.fullCenterY = continentCenterY

		local continentWidth = data.width / YARDS_PER_PIXEL
		currentContinentData.fullWidth = continentWidth

		local continentHeight = continentWidth * 2/3	-- data.height / YARDS_PER_PIXEL
		currentContinentData.fullHeight = continentHeight

		local rect = data.rect
		currentContinentData.visibleLeft = continentCenterX + continentWidth * (rect[1] - 0.5)
		currentContinentData.visibleRight = continentCenterX + continentWidth * (rect[2] - 0.5)
		currentContinentData.visibleTop = continentCenterY - continentHeight * (rect[3] - 0.5)
		currentContinentData.visibleBottom = continentCenterY - continentHeight * (rect[4] - 0.5)

		local zoneNames = { GetMapZones(continentID) }
		if continentID == 1 then
			tinsert(zoneNames, "Orgrimmar 2")
		elseif continentID == 4 then
			tinsert(zoneNames, "Dalaran 2")
			tinsert(zoneNames, "Isle of Conquest")
		end
		local zones = {}
		currentContinentData.zones = zones
		local mapData = SatMap.data.MAP_DATA[continentID]

		for i = 1, #zoneNames do
			local x, y
			local name, mapName, texPctX, texPctY, texX, texY, scrollX, scrollY

			local scansDone = 0
			repeat
				scansDone = scansDone + 1
				if scansDone >= 10000 then
					mapName = nil
					break
				end
				x, y = math_random(), math_random()
				name, mapName, texPctX, texPctY, texX, texY, scrollX, scrollY = UpdateMapHighlight(x, y)
				if mapName then
					mapName = string_match(mapName, "^(.+)_terrain(%d+)$") or mapName
				end
			until mapName and not zones[mapName]
			if not mapName then
				for index, zoneData in pairs(mapData) do
					if not zones[index] then
						name, mapName, texPctX, texPctY, texX, texY, scrollX, scrollY = unpack(mapData[index])
						break
					end
				end
				if not mapName then
					break
				end
			end

			local currentZoneData = {}
			currentZoneData.continentID = continentID
			for index, zoneName in ipairs(zoneNames) do
				if zoneName == name then
					currentZoneData.id = index
				end
			end
			currentZoneData.texture = mapName
			currentZoneData.localizedName = name
			SatMap.data.ZONE_DATA[mapName] = currentZoneData
			zones[mapName] = currentZoneData

			if mapData[mapName] then
				currentZoneData.highlightLeft = scrollX
				currentZoneData.highlightRight = scrollX + texX
				currentZoneData.highlightBottom = scrollY
				currentZoneData.highlightTop = scrollY + texY
			else
				for x = scrollX, scrollX+texX, 0.0025 do
					for y = scrollY, scrollY+texY, 0.0025 do
						if UpdateMapHighlight(x, y) == name then
							if not currentZoneData.highlightLeft then
								currentZoneData.highlightLeft = x
								currentZoneData.highlightRight = x
								currentZoneData.highlightBottom = y
								currentZoneData.highlightTop = y
							else
								if x < currentZoneData.highlightLeft then
									currentZoneData.highlightLeft = x
								elseif x > currentZoneData.highlightRight then
									currentZoneData.highlightRight = x
								end
								if y < currentZoneData.highlightBottom then
									currentZoneData.highlightBottom = y
								elseif y > currentZoneData.highlightTop then
									currentZoneData.highlightTop = y
								end
							end
						end
					end
				end
			end

			currentZoneData.highlightLeft = currentZoneData.highlightLeft-0.001
			currentZoneData.highlightRight = currentZoneData.highlightRight+0.001
			currentZoneData.highlightBottom = currentZoneData.highlightBottom-0.001
			currentZoneData.highlightTop = currentZoneData.highlightTop+0.001

			if mapName == "EversongWoods" or mapName == "Ghostlands" or mapName == "Sunwell" or mapName == "SilvermoonCity" then
				scrollX = scrollX - 0.00168
				scrollY = scrollY + 0.01
				currentZoneData.highlightLeft = currentZoneData.highlightLeft - 0.00168
				currentZoneData.highlightRight = currentZoneData.highlightRight - 0.00168
				currentZoneData.highlightBottom = currentZoneData.highlightBottom + 0.01
				currentZoneData.highlightTop = currentZoneData.highlightTop + 0.01

				currentZoneData.continentOffsetX = -0.00168 * continentWidth
				currentZoneData.continentOffsetY = -0.01 * continentHeight
			else
				currentZoneData.continentOffsetX = 0
				currentZoneData.continentOffsetY = 0
			end

			currentZoneData.fullWidth = texX * continentWidth
			currentZoneData.fullHeight = texY * continentHeight

			currentZoneData.fullCenterX = (scrollX - 0.5 + texX/2) * continentWidth + continentCenterX
			currentZoneData.fullCenterY = (0.5 - scrollY - texY/2) * continentHeight + continentCenterY

			currentZoneData.highlightLeft = currentZoneData.highlightLeft * continentWidth - continentWidth/2 + continentCenterX
			currentZoneData.highlightRight = currentZoneData.highlightRight * continentWidth - continentWidth/2 + continentCenterX
			currentZoneData.highlightBottom, currentZoneData.highlightTop = 1 - currentZoneData.highlightTop, 1 - currentZoneData.highlightBottom
			currentZoneData.highlightBottom = currentZoneData.highlightBottom * continentHeight - continentHeight/2 + continentCenterY
			currentZoneData.highlightTop = currentZoneData.highlightTop * continentHeight - continentHeight/2 + continentCenterY

			if SatMap.data.CITIES[mapName] then
				currentZoneData.visibleLeft = currentZoneData.fullCenterX - currentZoneData.fullWidth/2
				currentZoneData.visibleRight = currentZoneData.fullCenterX + currentZoneData.fullWidth/2
				currentZoneData.visibleBottom = currentZoneData.fullCenterY - currentZoneData.fullHeight/2
				currentZoneData.visibleTop = currentZoneData.fullCenterY + currentZoneData.fullHeight/2
			else
				currentZoneData.visibleLeft = math_max(currentZoneData.highlightLeft, currentZoneData.fullCenterX - currentZoneData.fullWidth/2)
				currentZoneData.visibleRight = math_min(currentZoneData.highlightRight, currentZoneData.fullCenterX + currentZoneData.fullWidth/2)
				currentZoneData.visibleBottom = math_max(currentZoneData.highlightBottom, currentZoneData.fullCenterY - currentZoneData.fullHeight/2)
				currentZoneData.visibleTop = math_min(currentZoneData.highlightTop, currentZoneData.fullCenterY + currentZoneData.fullHeight/2)
			end
		end

		currentContinentData.mapTextureCenterX = data.mapTextureCenterX
		currentContinentData.mapTextureCenterY = data.mapTextureCenterY
		currentContinentData.mapTextureOffsetX = data.mapTextureOffsetX
		currentContinentData.mapTextureOffsetY = data.mapTextureOffsetY
		currentContinentData.mapTexturePath = data.mapTexturePath
		currentContinentData.mapTextureBackground = data.mapTextureBackground
		local extraMapTextureAreas = {}
		currentContinentData.extraMapTextureAreas = extraMapTextureAreas
		if data.extraMapTextureAreas then
			for k,v in pairs(data.extraMapTextureAreas) do
				extraMapTextureAreas[#extraMapTextureAreas + 1] = {
					k,
					v.mapTextureCenterX,
					v.mapTextureCenterY,
					v.mapTextureOffsetX,
					v.mapTextureOffsetY,
					v.mapType,
					v.mapScale,
					v.mapTransparency,
					v.mapTexturePath,
					v.mapTextureBackground,
					v.mapSize,
				}
			end
		end

		local bounds = data.bounds
		currentContinentData.boundsLeft = continentCenterX + continentWidth * (bounds[1] - 0.5)
		currentContinentData.boundsRight = continentCenterX + continentWidth * (bounds[2] - 0.5)
		currentContinentData.boundsTop = continentCenterY - continentHeight * (bounds[3] - 0.5)
		currentContinentData.boundsBottom = continentCenterY - continentHeight * (bounds[4] - 0.5)
		currentContinentData.rectLeft = rect[1]
		currentContinentData.rectRight = rect[2]
		currentContinentData.rectTop = rect[3]
		currentContinentData.rectBottom = rect[4]
	end
	SetMapToCurrentZone()
end

local function InitializeInstanceData()
	if not initializeInstances then return end
	initializeInstances = nil

	local continentCount = #SatMap.data.CONTINENT_DATA
	for continentID, data in ipairs(SatMap.data.INSTANCE_DATA) do
		local currentContinentData = {}
		SatMap.data.CONTINENT_DATA[continentID + continentCount] = currentContinentData
		currentContinentData.texture = data.name
		SatMap.data.CONTINENT_DATA[currentContinentData.texture] = currentContinentData
		currentContinentData.id = continentID

		currentContinentData.localizedName = data.name

		local continentCenterX = data.x
		currentContinentData.fullCenterX = continentCenterX

		local continentCenterY = data.y
		currentContinentData.fullCenterY = continentCenterY

		local continentWidth = data.width / YARDS_PER_PIXEL
		currentContinentData.fullWidth = continentWidth

		local continentHeight = continentWidth * 2/3	-- data.height / YARDS_PER_PIXEL
		currentContinentData.fullHeight = continentHeight

		local rect = data.rect
		currentContinentData.visibleLeft = continentCenterX + continentWidth * (rect[1] - 0.5)
		currentContinentData.visibleRight = continentCenterX + continentWidth * (rect[2] - 0.5)
		currentContinentData.visibleTop = continentCenterY - continentHeight * (rect[3] - 0.5)
		currentContinentData.visibleBottom = continentCenterY - continentHeight * (rect[4] - 0.5)

		local zones = {}
		currentContinentData.zones = zones
		local mapData = SatMap.data.MAP_DATA[continentID + continentCount]

		for index, zoneData in pairs(mapData) do
			local x, y
			local name, mapName, texPctX, texPctY, texX, texY, scrollX, scrollY = unpack(mapData[index])
			if not mapName then
				break
			end

			local currentZoneData = {}
			currentZoneData.continentID = continentID + continentCount
			currentZoneData.texture = mapName
			currentZoneData.localizedName = name
			SatMap.data.ZONE_DATA[mapName] = currentZoneData
			zones[mapName] = currentZoneData

			if mapData[mapName] then
				currentZoneData.highlightLeft = scrollX
				currentZoneData.highlightRight = scrollX + texX
				currentZoneData.highlightBottom = scrollY
				currentZoneData.highlightTop = scrollY + texY
			end

			currentZoneData.highlightLeft = currentZoneData.highlightLeft-0.001
			currentZoneData.highlightRight = currentZoneData.highlightRight+0.001
			currentZoneData.highlightBottom = currentZoneData.highlightBottom-0.001
			currentZoneData.highlightTop = currentZoneData.highlightTop+0.001

			currentZoneData.continentOffsetX = 0
			currentZoneData.continentOffsetY = 0

			currentZoneData.fullWidth = texX * continentWidth
			currentZoneData.fullHeight = texY * continentHeight

			currentZoneData.fullCenterX = (scrollX - 0.5 + texX/2) * continentWidth + continentCenterX
			currentZoneData.fullCenterY = (0.5 - scrollY - texY/2) * continentHeight + continentCenterY

			currentZoneData.highlightLeft = currentZoneData.highlightLeft * continentWidth - continentWidth/2 + continentCenterX
			currentZoneData.highlightRight = currentZoneData.highlightRight * continentWidth - continentWidth/2 + continentCenterX
			currentZoneData.highlightBottom, currentZoneData.highlightTop = 1 - currentZoneData.highlightTop, 1 - currentZoneData.highlightBottom
			currentZoneData.highlightBottom = currentZoneData.highlightBottom * continentHeight - continentHeight/2 + continentCenterY
			currentZoneData.highlightTop = currentZoneData.highlightTop * continentHeight - continentHeight/2 + continentCenterY

			if SatMap.data.CITIES[mapName] then
				currentZoneData.visibleLeft = currentZoneData.fullCenterX - currentZoneData.fullWidth/2
				currentZoneData.visibleRight = currentZoneData.fullCenterX + currentZoneData.fullWidth/2
				currentZoneData.visibleBottom = currentZoneData.fullCenterY - currentZoneData.fullHeight/2
				currentZoneData.visibleTop = currentZoneData.fullCenterY + currentZoneData.fullHeight/2
			else
				currentZoneData.visibleLeft = math_max(currentZoneData.highlightLeft, currentZoneData.fullCenterX - currentZoneData.fullWidth/2)
				currentZoneData.visibleRight = math_min(currentZoneData.highlightRight, currentZoneData.fullCenterX + currentZoneData.fullWidth/2)
				currentZoneData.visibleBottom = math_max(currentZoneData.highlightBottom, currentZoneData.fullCenterY - currentZoneData.fullHeight/2)
				currentZoneData.visibleTop = math_min(currentZoneData.highlightTop, currentZoneData.fullCenterY + currentZoneData.fullHeight/2)
			end
		end

		currentContinentData.mapTextureCenterX = data.mapTextureCenterX
		currentContinentData.mapTextureCenterY = data.mapTextureCenterY
		currentContinentData.mapTextureOffsetX = data.mapTextureOffsetX
		currentContinentData.mapTextureOffsetY = data.mapTextureOffsetY
		currentContinentData.mapTexturePath = data.mapTexturePath
		currentContinentData.mapTextureBackground = data.mapTextureBackground
		local extraMapTextureAreas = {}
		currentContinentData.extraMapTextureAreas = extraMapTextureAreas
		if data.extraMapTextureAreas then
			for k,v in pairs(data.extraMapTextureAreas) do
				extraMapTextureAreas[#extraMapTextureAreas + 1] = {
					k,
					v.mapTextureCenterX,
					v.mapTextureCenterY,
					v.mapTextureOffsetX,
					v.mapTextureOffsetY,
					v.mapType,
					v.mapScale,
					v.mapTransparency,
					v.mapTexturePath,
					v.mapTextureBackground,
					v.mapSize,
				}
			end
		end

		local bounds = data.bounds
		currentContinentData.boundsLeft = continentCenterX + continentWidth * (bounds[1] - 0.5)
		currentContinentData.boundsRight = continentCenterX + continentWidth * (bounds[2] - 0.5)
		currentContinentData.boundsTop = continentCenterY - continentHeight * (bounds[3] - 0.5)
		currentContinentData.boundsBottom = continentCenterY - continentHeight * (bounds[4] - 0.5)
		currentContinentData.rectLeft = rect[1]
		currentContinentData.rectRight = rect[2]
		currentContinentData.rectTop = rect[3]
		currentContinentData.rectBottom = rect[4]
	end
end

local function InitializeZoneSearchData()
	if not initializeZoneSearch then return end
	initializeZoneSearch = nil
	local function zoneSearchData_func(alpha, bravo)
		if SatMap.data.CITIES[bravo] then
			return false
		elseif SatMap.data.CITIES[alpha] then
			return true
		end

		local zoneData_alpha = SatMap.data.ZONE_DATA[alpha]
		local zoneData_bravo = SatMap.data.ZONE_DATA[bravo]
		local area_alpha = (zoneData_alpha.highlightRight - zoneData_alpha.highlightLeft) * (zoneData_alpha.highlightTop - zoneData_alpha.highlightBottom)
		local area_bravo = (zoneData_bravo.highlightRight - zoneData_bravo.highlightLeft) * (zoneData_bravo.highlightTop - zoneData_bravo.highlightBottom)
		return area_alpha < area_bravo
	end
	for continentID, currentContinentData in ipairs(SatMap.data.CONTINENT_DATA) do
		local t = {}
		zoneSearchData[continentID] = t
		local zones = currentContinentData.zones

		local continentLeft = currentContinentData.visibleLeft
		local continentWidth = currentContinentData.visibleRight - currentContinentData.visibleLeft
		local continentBottom = currentContinentData.visibleBottom
		local continentHeight = currentContinentData.visibleTop - currentContinentData.visibleBottom

		for mapName, zoneData in pairs(zones) do
			local zoneLeft = (zoneData.highlightLeft - continentLeft) / continentWidth
			local zoneRight = (zoneData.highlightRight - continentLeft) / continentWidth
			local zoneBottom = (zoneData.highlightBottom - continentBottom) / continentHeight
			local zoneTop = (zoneData.highlightTop - continentBottom) / continentHeight
			for x = 0, 9 do
				if zoneLeft < (x+1)/10 and zoneRight > x/10 then
					for y = 0, 9 do
						if zoneBottom < (y+1)/10 and zoneTop > y/10 then
							local num = x*10 + y
							local u = t[num]
							if not u then
								u = {}
								t[num] = u
							end
							u[#u+1] = mapName
						end
					end
				end
			end
		end
		for num = 0, 99 do
			if t[num] then
				table_sort(t[num], zoneSearchData_func)
			end
		end
	end
end
local function QuelThalasTextures(currentMapTexture)
	if QuelThalasZones[currentMapTexture] then
		if inQuelThalas == 1 then return end
		inQuelThalas = 1
	elseif currentMapTexture == "Azeroth" then
		if inQuelThalas == 2 then return end
		inQuelThalas = 2
	else
		if not inQuelThalas then return end
		inQuelThalas = nil
	end

	if inQuelThalas then
		for mapGrid, mapData in pairs(QuelThalasMaps) do
			if mapData[2] and mapData[1] == 1 then
				mapData[2]:Show()
			elseif mapData[2] and mapData[1] == 2 then
				mapData[2]:Hide()
			end
		end
	else
		for mapGrid, mapData in pairs(QuelThalasMaps) do
			if mapData[2] and mapData[1] == 1 then
				mapData[2]:Hide()
			elseif mapData[2] and mapData[1] == 2 then
				mapData[2]:Show()
			end
		end
	end

	if inQuelThalas == 2 then
		QuelThalasMaps[4019][2]:Hide()
		QuelThalasMaps[4119][2]:Hide()
		QuelThalasMaps[4120][2]:Hide()
		QuelThalasMaps[4220][2]:Hide()
		QuelThalasMaps[4320][2]:Hide()
		QuelThalasMaps[4420][2]:Hide()
		QuelThalasMaps[4720][2]:Hide()
		QuelThalasMaps[3825][2]:Show()
		QuelThalasMaps[3925][2]:Show()
		QuelThalasMaps[4025][2]:Show()
		QuelThalasMaps[4820][2]:Show()
	end
end

local function UpdateMapTextureSection(zone)
	for mapName, mapTextureFrame in pairs(mapTextureFrames) do
		if not SatMap.data.INDOOR_MAPS[mapName] then
			mapTextureFrame:Show()
		end
	end
	showMinimapStyle = 1

	SatMap.scrollFrame:SetFrameLevel(Mapster.tileHolder:GetFrameLevel() + 1)
	SatMap.mapHolder:SetFrameLevel(Mapster.tileHolder:GetFrameLevel() + 1)

	if db.poiLabels then
		if initializePoints then SatMap:InitializePoints() end
	end

	local currentMapTexture = GetMapInfo()
	QuelThalasTextures(currentMapTexture)
	for mapName, mapTexture in pairs(SatMap.data.INDOOR_MAPS) do
		if currentMapTexture == mapName then
			mapTextureFrames[mapTexture]:Show()
		else
			mapTextureFrames[mapTexture]:Hide()
		end
	end
	SatMap.data.currentMapTexture = nil
	SatMap:WORLD_MAP_UPDATE()

	if db.mapView then
		WorldMapBlobFrame:SetFrameLevel(WorldMapButton:GetFrameLevel() + 1)
		WorldMapArchaeologyDigSites:SetFrameLevel(WorldMapButton:GetFrameLevel() + 1)
	else
		WorldMapBlobFrame:SetFrameLevel(WorldMapButton:GetFrameLevel())
		WorldMapArchaeologyDigSites:SetFrameLevel(WorldMapButton:GetFrameLevel())
	end
end

local CreateMapTextureSection
do
	local doneContinents = {}
	local tempContinent, tempExtraArea, tempTextureDataPosition
	local first = true
	local mapTextureFrame

	function CreateMapTextureSection()
		if not createMapTextureSection then return end
		if first then
			first = nil
			mapTextureFrame = CreateFrame("Frame", "SatMap_MapHolder_MapTextureFrame_World", SatMap.mapHolder)
			mapTextureFrames["World"] = mapTextureFrame
			mapTextureFrame:SetPoint("CENTER", SatMap.mapHolder, "CENTER", 0, 0)
			mapTextureFrame:SetWidth(10000)
			mapTextureFrame:SetHeight(10000)
			local tex = mapTextureFrame:CreateTexture(mapTextureFrame:GetName() .. "_Background", "BACKGROUND")
			tex:SetTexture(0, 0.114, 0.157)
			tex:SetPoint("BOTTOMLEFT", mapTextureFrame, "BOTTOMLEFT")
			tex:SetPoint("TOPRIGHT", mapTextureFrame, "TOPRIGHT")
			mapTextureFrames["World"].tex = tex
			for i, currentContinentData in ipairs(SatMap.data.CONTINENT_DATA) do
				local currentContinentData_texture = currentContinentData.texture
				mapTextureFrame = CreateFrame("Frame", "SatMap_MapHolder_MapTextureFrame_" .. currentContinentData_texture, SatMap.mapHolder)
				mapTextureFrames[currentContinentData_texture] = mapTextureFrame
				local offsetX = -currentContinentData.mapTextureOffsetX + currentContinentData.fullCenterX
				local offsetY = currentContinentData.mapTextureOffsetY + currentContinentData.fullCenterY
				mapTextureFrame:SetPoint("CENTER", SatMap.mapHolder, "CENTER", offsetX, offsetY)
				mapTextureFrame:SetWidth(1e-10)
				mapTextureFrame:SetHeight(1e-10)
				if currentContinentData.mapTextureBackground then
					local tex = mapTextureFrame:CreateTexture(mapTextureFrame:GetName() .. "_Background", "BACKGROUND")
					tex:SetTexture(format(TEXTURE_PATH.."SatMap\\%s", currentContinentData.mapTextureBackground))
					tex:SetPoint("BOTTOMLEFT", mapTextureFrame, "CENTER", currentContinentData.boundsLeft - offsetX - 5, currentContinentData.boundsBottom - offsetY - 5)
					tex:SetPoint("TOPRIGHT", mapTextureFrame, "CENTER", currentContinentData.boundsRight - offsetX + 5, currentContinentData.boundsTop - offsetY + 5)
				end
				mapTextureFrame:Hide()

				if currentContinentData.extraMapTextureAreas then
					for k,v in pairs(currentContinentData.extraMapTextureAreas) do
						if v[6] == "city" or v[6] == "overlay" then
							mapTextureFrame = CreateFrame("Frame", "SatMap_MapHolder_MapTextureFrame_" .. v[1], SatMap.mapHolder)
							mapTextureFrames[v[1]] = mapTextureFrame
							local scale = v[7] or 1
							local currentZoneData = SatMap.data.ZONE_DATA[v[1]]
							local offsetX = (-v[4] + currentZoneData.fullCenterX) / scale
							local offsetY = (v[5] + currentZoneData.fullCenterY) / scale
							mapTextureFrame:SetPoint("CENTER", SatMap.mapHolder, "CENTER", offsetX, offsetY)
							mapTextureFrame:SetWidth(1e-10)
							mapTextureFrame:SetHeight(1e-10)
							mapTextureFrame:SetScale(scale)
							local tex = mapTextureFrame:CreateTexture(mapTextureFrame:GetName() .. "_Background", "BACKGROUND")
							tex:SetTexture(format("%s\\%s", v[9] or TEXTURE_PATH.."SatMap", v[10] or "water4"))
							if v[8] then tex:SetAlpha(v[8]) end
							if v[11] then
								tex:SetPoint("BOTTOMLEFT", mapTextureFrame, "CENTER", v[11][1], v[11][2])
								tex:SetPoint("TOPRIGHT", mapTextureFrame, "CENTER", v[11][3], v[11][4])
							else
								tex:SetPoint("BOTTOMLEFT", mapTextureFrame, "CENTER", -10, -10)
								tex:SetPoint("TOPRIGHT", mapTextureFrame, "CENTER", 10, 10)
							end
							mapTextureFrame:Hide()
						end
					end
				end
			end
			return
		end

		if not tempContinent then
			if SatMap.data.CONTINENT_DATA[SatMap.data.currentMapTexture] then
				tempContinent = SatMap.data.CONTINENT_DATA[SatMap.data.currentMapTexture].id
			elseif SatMap.data.currentZoneData then
				tempContinent = SatMap.data.currentZoneData.continentID
			end

			local isDone
			if tempContinent then
				local continentTexture = SatMap.data.CONTINENT_DATA[tempContinent].texture
				isDone = doneContinents[continentTexture]
			end
			if isDone or not tempContinent then
				tempContinent = nil
				for continentID, currentContinentData in ipairs(SatMap.data.CONTINENT_DATA) do
					if not doneContinents[currentContinentData.texture] then
						tempContinent = continentID
					end
				end
			end
			if not tempContinent then
				SatMap.data.TEXTURE_DATA = nil
				createMapTextureSection = nil
				UpdateMapTextureSection()
				if not SatMap:IsEnabled() then
					SatMap.updateMap:SetScript("OnUpdate", nil)
					SatMap.updateMap:Hide()
					SatMap:UnregisterEvent("WORLD_MAP_UPDATE")
				end
				return
			end
		end

		local currentContinentData = SatMap.data.CONTINENT_DATA[tempContinent]
		local currentContinentData_texture = currentContinentData.texture
		local mapTextureFrame = mapTextureFrames[currentContinentData_texture]
		local mapName = currentContinentData_texture

		local size = MINIMAP_TILE_YARD_SIZE / YARDS_PER_PIXEL
		local centerX, centerY

		local currentTextureData
		local currentTexturePath
		if not tempExtraArea then
			currentTextureData = SatMap.data.TEXTURE_DATA[currentContinentData_texture]
			currentTexturePath = currentContinentData.mapTexturePath
			centerX, centerY = currentContinentData.mapTextureCenterX, currentContinentData.mapTextureCenterY
		else
			local extraMapTextureArea = currentContinentData.extraMapTextureAreas[tempExtraArea]
			currentTextureData = SatMap.data.TEXTURE_DATA[extraMapTextureArea[1]]
			currentTexturePath = extraMapTextureArea[9]
			centerX, centerY = extraMapTextureArea[2], extraMapTextureArea[3]
		end

		if not currentTextureData then
			doneContinents[currentContinentData.texture] = true
			if not tempExtraArea then
				tempExtraArea = 1
			else
				tempExtraArea = tempExtraArea + 1
			end

			if not currentContinentData.extraMapTextureAreas[tempExtraArea] then
				-- done continent
				tempContinent = nil
				tempExtraArea = nil
				-- no longer using minimap texture data, clear it, refresh the table
				currentContinentData.mapTextureCenterX = nil
				currentContinentData.mapTextureCenterY = nil
				currentContinentData.mapTextureOffsetX = nil
				currentContinentData.mapTextureOffsetY = nil
				currentContinentData.mapTextureBackground = nil
				currentContinentData.extraMapTextureAreas = nil
			end
			return
		end

		local overlayTile
		if tempExtraArea then
			local mapType = currentContinentData.extraMapTextureAreas[tempExtraArea][6]
			if mapType == "city" or mapType == "overlay" then
				mapTextureFrame = mapTextureFrames[currentContinentData.extraMapTextureAreas[tempExtraArea][1]]
				mapName = currentContinentData.extraMapTextureAreas[tempExtraArea][1]
			end
			if mapType == "overlay" then
				overlayTile = true
			end
		end

		local finish = 0
		while finish < 10000 do
			finish = finish + 1
			local texturePath
			tempTextureDataPosition, texturePath = next(currentTextureData, tempTextureDataPosition)
			if not tempTextureDataPosition then
				break
			end

			local QuelThalasTile
			if currentContinentData.texture == "Azeroth" and QuelThalasMaps[tempTextureDataPosition] then
				QuelThalasTile = true
			end
			if not mapTextures[mapName] then
				mapTextures[mapName] = {}
			end
			local x, y = floor(tempTextureDataPosition / 100), tempTextureDataPosition % 100
			local chunkNum = floor(x - centerX) * 100 + floor(y - centerY) + 10000
			local tex = mapTextureFrame:CreateTexture(mapTextureFrame:GetName() .. "_Texture" .. chunkNum, (QuelThalasTile or overlayTile) and "OVERLAY" or "ARTWORK")
			mapTextures[mapName][chunkNum] = tex
			if QuelThalasTile and QuelThalasMaps[tempTextureDataPosition] then
				QuelThalasMaps[tempTextureDataPosition][2] = tex
			end
			if tempExtraArea and SatMap.data.INDOOR_MAPS[currentContinentData.extraMapTextureAreas[tempExtraArea][1]] then
				tex:SetTexture(format(TEXTURE_PATH.."SatMap\\%s", texturePath))
			elseif tempExtraArea and currentContinentData.extraMapTextureAreas[tempExtraArea][1] == "Vashjir" then
				tex:SetTexture(format("%smap%02d_%02d", currentTexturePath, x, y))
			else
				tex:SetTexture(format("%s\\map%02d_%02d", currentTexturePath, x, y))
			end
			tex:SetWidth(size)
			tex:SetHeight(size)
			tex:SetAlpha(1)
			tex:SetPoint("BOTTOMLEFT", mapTextureFrame, "CENTER", (x - centerX) * size, -(y - centerY) * size)
			QuelThalasTile = nil
		end
		if not tempTextureDataPosition then
			SatMap.data.TEXTURE_DATA[currentContinentData.texture] = nil
			doneContinents[currentContinentData.texture] = true
			if not tempExtraArea then
				tempExtraArea = 1
			else
				tempExtraArea = tempExtraArea + 1
			end

			if not currentContinentData.extraMapTextureAreas[tempExtraArea] then
				-- done continent
				tempContinent = nil
				tempExtraArea = nil
				-- no longer using minimap texture data, clear it, refresh the table
				currentContinentData.mapTextureCenterX = nil
				currentContinentData.mapTextureCenterY = nil
				currentContinentData.mapTextureOffsetX = nil
				currentContinentData.mapTextureOffsetY = nil
				currentContinentData.mapTextureBackground = nil
				currentContinentData.extraMapTextureAreas = nil
			end
		end
	end
end

local function POI_Resize(this)
	if this then
		local currentContinentID = SatMap.data.currentContinentID
		local currentZoneID = SatMap.data.currentZoneID
		local size = poiTypeScales[this.poiType]
		if currentZoneID == 0 then size = 12 end

		local mapViewZoom = SatMap.data.mapViewZoom
		local font, height, flags = this:GetFont()
		this:SetFont(font, size / mapViewZoom, flags)
		this:SetShadowOffset(0.8 / mapViewZoom, -0.8 / mapViewZoom)
		this:ClearAllPoints()
		if this.poiType == 1 then
			this:SetPoint("CENTER", SatMap.poiZoneFrame, "CENTER", this.x, this.y + (10 / mapViewZoom))
			if SatMap.data.CITIES[this.poiZone] then
				if SatMap.data.CITIES[SatMap.data.currentMapTexture] then
					this:Show()
				else
					this:Hide()
				end
			end
		else
			this:SetPoint("CENTER", SatMap.poiContFrame, "CENTER", this.x, this.y + (12 / mapViewZoom))
		end
	end
end

function SatMap:InitializePoints()
	if not initializePoints then return end
	initializePoints = nil

	local poiZoneFrame
	poiZoneFrame = CreateFrame("Frame", "SatMap_MapHolder_MapPOI_ZoneFrame", self.mapHolder)
	poiZoneFrame:SetPoint("CENTER", self.mapHolder, "CENTER", 0, 0)
	poiZoneFrame:SetWidth(10000)
	poiZoneFrame:SetHeight(10000)
	self.poiZoneFrame = poiZoneFrame
	poiZoneFrame:Hide()
	local poiContFrame
	poiContFrame = CreateFrame("Frame", "SatMap_MapHolder_MapPOI_ContFrame", self.mapHolder)
	poiContFrame:SetPoint("CENTER", self.mapHolder, "CENTER", 0, 0)
	poiContFrame:SetWidth(10000)
	poiContFrame:SetHeight(10000)
	self.poiContFrame = poiContFrame
	poiContFrame:Hide()

	local count = 0
	local data = self.data.POI_DATA
	for zone, pois in pairs(data) do
		for id, points in pairs(pois) do
			local x, y = getXY(id)
			local px, py = self:ConvertZoneCoordinateToUniverseCoordinate(zone, x, y)
			if px and py then
				count = count + 1
				local poiFrame
				if points[1] == 1 then
					poiFrame = poiZoneFrame
				else
					poiFrame = poiContFrame
				end
				local text = poiFrame:CreateFontString("SatMap_POI_Text"..count, "OVERLAY", "GameFontNormal")
				text:SetFont(self.fontMyriad, 12)
				text:SetTextColor(1, 0.82, 0)
				text:SetText(points[2] or "")
				text:SetShadowOffset(0.8, -0.8)
				text:SetPoint("CENTER", poiFrame, "CENTER", px, py)
				text.poiType = points[1]
				text.poiText = points[2]
				text.x = px
				text.y = py
				text.poiZone = zone
				text.poiX = x
				text.poiY = y
				shownPOIs[count] = text
			end
		end
	end
	for i, poi in ipairs(shownPOIs) do
		POI_Resize(poi)
	end
	if db.poiLabels and not db.poiLabelsOnlyCtrl then
		poiContFrame:Show()
		if self.data.currentZoneID > 0 or self.data.currentZoneData then
			poiZoneFrame:Show()
		end
	end
end


function SatMap:OnEnable()
	if not self:IsEnabled() then return end
	self.initializing = true
	Notes = Mapster:GetModule("Notes")
	if not Notes then
		Mapster:SetModuleEnabled(MODNAME, false)
		return
	end

	self:SecureHook("WorldMapButton_OnClick")
	self:SecureHookScript(WorldMapButton, "OnMouseUp", "WorldMapButton_OnClick")
	self:RegisterEvent("MODIFIER_STATE_CHANGED")
	self:RegisterEvent("WORLD_MAP_UPDATE")

	if showMinimapStyle then
		if db.mapView then
			self.mapShown = true
			self.scrollFrame:Show()
			self.mapHolder:Show()
		end
		self:WORLD_MAP_UPDATE()
	end

	if initializeContinents then InitializeContinentData() end
	if initializeInstances then InitializeInstanceData() end
	if initializeZoneSearch then InitializeZoneSearchData() end
	if initializeMapHolder then InitializeMapHolder() end

	WorldMapBlobFrame:SetFrameLevel(WorldMapButton:GetFrameLevel() + 1)
	WorldMapArchaeologyDigSites:SetFrameLevel(WorldMapButton:GetFrameLevel() + 1)

	self:SecureHook(MapsterEnhanced, "PLAYER_REGEN_ENABLED")
	if not self.updateMap then
		self.updateMap = CreateFrame("Frame", "Mapster_SatMap_UpdateMap")
		self.lastUpdate = 0
	end
	self.updateMap:SetScript("OnUpdate", function(self, elapsed) SatMap:UpdateMap(elapsed) end)
	self.updateMap:Show()
	self.mapButton:Show()

	self.initializing = nil
	self:Refresh()
end

function SatMap:OnDisable()
	self.mapButton:Hide()
	self.mapShown = nil
	self.scrollFrame:Hide()
	self.mapHolder:Hide()
	if self.initializing then return end

	WorldMapBlobFrame:SetFrameLevel(WorldMapButton:GetFrameLevel())
	WorldMapArchaeologyDigSites:SetFrameLevel(WorldMapButton:GetFrameLevel())

	self:Unhook(MapsterEnhanced, "PLAYER_REGEN_ENABLED")
	if not createMapTextureSection then
		if self.updateMap then
			self.updateMap:SetScript("OnUpdate", nil)
			self.updateMap:Hide()
		end
		self:UnregisterEvent("WORLD_MAP_UPDATE")
	end
	self:Unhook("WorldMapButton_OnClick")
	self:Unhook(WorldMapButton, "OnClick")
end

function SatMap:Refresh()
	db = self.db.profile
	if not self:IsEnabled() then return end

	self.miniMap = Mapster.miniMap
	self:UpdateMapsize(self.miniMap)
end

function SatMap:UpdateMapsize(mini)
	-- map was minimized, fix display position
	if mini then
		self.mapButton:ClearAllPoints()
		self.mapButton:SetPoint("RIGHT", Mapster.optionsButton, "LEFT", -4, 0)
		self.mapButton:SetHeight(18)
	else
		self.mapButton:ClearAllPoints()
		self.mapButton:SetPoint("LEFT", WorldMapZoomOutButton, "RIGHT", 6, 0)
		self.mapButton:SetHeight(22)
	end
	if showMinimapStyle then
		self:ZoomToZone(self.data.currentMapTexture)
	end
end

function SatMap:ToggleMap()
	if not self:IsEnabled() then return end
	if db.mapView then
		WorldMapBlobFrame:SetFrameLevel(WorldMapButton:GetFrameLevel())
		WorldMapArchaeologyDigSites:SetFrameLevel(WorldMapButton:GetFrameLevel())
		db.mapView = nil
		self.mapShown = nil
		self.scrollFrame:Hide()
		self.mapHolder:Hide()
		if self.data.currentContinentID == 0 then
			for i = 1, 12 do
				_G["WorldMapDetailTile"..i]:SetTexture("Interface\\WorldMap\\World\\World"..i)
			end
		end
	else
		WorldMapBlobFrame:SetFrameLevel(WorldMapButton:GetFrameLevel() + 1)
		WorldMapArchaeologyDigSites:SetFrameLevel(WorldMapButton:GetFrameLevel() + 1)
		db.mapView = true
		if self.data.currentContinentID == 0 then
			for i = 1, 12 do
				_G["WorldMapDetailTile"..i]:SetTexture(TEXTURE_PATH.."SatMap\\World"..i)
			end
		elseif self.data.currentContinentID > 0 or self.data.currentZoneData then
			self.mapShown = true
			self.scrollFrame:Show()
			self.mapHolder:Show()
		end
	end
end

function SatMap:PLAYER_REGEN_ENABLED()
	WorldMapBlobFrame:SetFrameLevel(WorldMapButton:GetFrameLevel() + 1)
	WorldMapArchaeologyDigSites:SetFrameLevel(WorldMapButton:GetFrameLevel() + 1)
end

function SatMap:WORLD_MAP_UPDATE()
	if not WorldMapFrame:IsShown() and showMinimapStyle then return end

	local currentMapTextureWithoutLevel = GetMapInfo()
	if currentMapTextureWithoutLevel then
		currentMapTextureWithoutLevel = string_match(currentMapTextureWithoutLevel, "^(.+)_terrain(%d+)$") or currentMapTextureWithoutLevel
	end
	local level = GetCurrentMapDungeonLevel()
	local currentMapTexture = currentMapTextureWithoutLevel
	if currentMapTextureWithoutLevel and level > 1 then
		currentMapTexture = currentMapTexture .. ":" .. level
	end

	if self.data.currentMapTexture == currentMapTexture then
		return
	end
	self.data.currentContinentID = GetCurrentMapContinent()
	self.data.currentZoneID = GetCurrentMapZone()
	if not currentMapTexture then
		if self.data.currentContinentID == -1 then
			currentMapTexture = "Cosmic"
		else
			currentMapTexture = "World"
		end
		currentMapTextureWithoutLevel = currentMapTexture
	end
	self.data.currentMapTextureWithoutLevel = currentMapTextureWithoutLevel
	self.data.currentMapTexture = currentMapTexture
	self.data.currentZoneData = self.data.ZONE_DATA[currentMapTexture]
	self.data.currentContinentData = self.data.CONTINENT_DATA[self.data.currentContinentID]
	if showMinimapStyle then
		if db.poiLabels and not db.poiLabelsOnlyCtrl then
			if self.data.currentZoneID == 0 and not self.data.currentZoneData then
				self.poiZoneFrame:Hide()
			else
				self.poiZoneFrame:Show()
			end
		end
		QuelThalasTextures(currentMapTexture)
		local showMapTexture
		for mapName, mapTexture in pairs(self.data.INDOOR_MAPS) do
			if currentMapTexture == mapName then
				mapTextureFrames[mapTexture]:Show()
				showMapTexture = mapTexture
			elseif mapTexture ~= showMapTexture then
				mapTextureFrames[mapTexture]:Hide()
			end
		end
		if currentMapTexture == "TheMaelstrom" then
			mapTextureFrames["TheMaelstrom"]:Show()
		else
			mapTextureFrames["TheMaelstrom"]:Hide()
		end
		if currentMapTexture == "Deepholm" then
			mapTextureFrames["Deepholm"]:Show()
		else
			mapTextureFrames["Deepholm"]:Hide()
		end
		if currentMapTexture == "TolBarad" then
			_G["SatMap_MapHolder_MapTextureFrame_Azeroth_Texture8900"]:Hide()
			_G["SatMap_MapHolder_MapTextureFrame_Azeroth_Texture8901"]:Hide()
			_G["SatMap_MapHolder_MapTextureFrame_Azeroth_Texture8902"]:Hide()
		else
			_G["SatMap_MapHolder_MapTextureFrame_Azeroth_Texture8900"]:Show()
			_G["SatMap_MapHolder_MapTextureFrame_Azeroth_Texture8901"]:Show()
			_G["SatMap_MapHolder_MapTextureFrame_Azeroth_Texture8902"]:Show()
		end
		if currentMapTexture == "VashjirDepths" or currentMapTexture == "VashjirKelpForest" or currentMapTexture == "VashjirRuins" then
			mapTextureFrames["Vashjir"]:Show()
		else
			mapTextureFrames["Vashjir"]:Hide()
		end
		if self.data.currentContinentID == 1 then
			mapTextureFrames["World"].tex:SetTexture(0, 0.114, 0.157)
			if self.data.currentZoneID > 0 then
				mapTextureFrames["Northrend"]:Hide()
			else
				mapTextureFrames["Northrend"]:Show()
			end
			mapTextureFrames["Kalimdor"]:Show()
			mapTextureFrames["Azeroth"]:Hide()
			mapTextureFrames["Expansion01"]:Hide()
			mapTextureFrames["TolBarad"]:Hide()
			if self.data.currentZoneID == 0 then
				mapTextureFrames["TheMaelstromContinent"]:Show()
			else
				mapTextureFrames["TheMaelstromContinent"]:Hide()
			end
		elseif self.data.currentContinentID == 2 then
			mapTextureFrames["World"].tex:SetTexture(0, 0.114, 0.157)
			if self.data.currentZoneID > 0 then
				mapTextureFrames["Northrend"]:Hide()
			else
				mapTextureFrames["Northrend"]:Show()
			end
			mapTextureFrames["Kalimdor"]:Hide()
			mapTextureFrames["Azeroth"]:Show()
			mapTextureFrames["Expansion01"]:Hide()
			mapTextureFrames["TolBarad"]:Show()
			if self.data.currentZoneID == 0 then
				mapTextureFrames["TheMaelstromContinent"]:Show()
			else
				mapTextureFrames["TheMaelstromContinent"]:Hide()
			end
			for i = 11487, 11515 do
				mapTextures["Northrend"][i]:Hide()
			end
		elseif self.data.currentContinentID == 3 then
			mapTextureFrames["World"].tex:SetTexture(0, 0, 0)
			mapTextureFrames["Kalimdor"]:Hide()
			mapTextureFrames["Azeroth"]:Hide()
			mapTextureFrames["Expansion01"]:Show()
			mapTextureFrames["Northrend"]:Hide()
			mapTextureFrames["TheMaelstromContinent"]:Hide()
			mapTextureFrames["TolBarad"]:Hide()
		elseif self.data.currentContinentID == 4 then
			mapTextureFrames["World"].tex:SetTexture(0, 0.114, 0.157)
			mapTextureFrames["Kalimdor"]:Hide()
			mapTextureFrames["Azeroth"]:Hide()
			mapTextureFrames["Expansion01"]:Hide()
			mapTextureFrames["Northrend"]:Show()
			mapTextureFrames["TheMaelstromContinent"]:Hide()
			mapTextureFrames["TolBarad"]:Hide()
			for i = 11487, 11515 do
				mapTextures["Northrend"][i]:Show()
			end
			mapTextures["IsleofConquest"][10002]:Hide()
			for i = 10098, 10102 do
				mapTextures["IsleofConquest"][i]:Hide()
			end
		elseif self.data.currentContinentID == 5 then
			mapTextureFrames["World"].tex:SetTexture(0, 0.114, 0.157)
			mapTextureFrames["Kalimdor"]:Hide()
			mapTextureFrames["Azeroth"]:Hide()
			mapTextureFrames["Expansion01"]:Hide()
			mapTextureFrames["Northrend"]:Hide()
			mapTextureFrames["TheMaelstromContinent"]:Show()
			mapTextureFrames["TolBarad"]:Hide()
			if currentMapTexture == "TheMaelstrom" then
				mapTextureFrames["TheMaelstromContinent"]:Hide()
				self.poiZoneFrame:Hide()
			else
				mapTextureFrames["TheMaelstromContinent"]:Show()
			end
		elseif self.data.currentZoneData then
			mapTextureFrames["World"].tex:SetTexture(0, 0.114, 0.157)
			mapTextureFrames["Kalimdor"]:Hide()
			mapTextureFrames["Azeroth"]:Hide()
			mapTextureFrames["Expansion01"]:Hide()
			mapTextureFrames["Northrend"]:Hide()
			mapTextureFrames["TheMaelstromContinent"]:Hide()
			mapTextureFrames["TolBarad"]:Hide()
			if mapTextureFrames[currentMapTexture] then
				for mapName, zoneData in pairs(self.data.ZONE_DATA) do
					if zoneData.continentID == 6 and mapTextureFrames[mapName] then
						mapTextureFrames[mapName]:Hide()
					end
				end
				mapTextureFrames[currentMapTexture]:Show()
				if currentMapTexture == "IsleofConquest" then
					mapTextures["IsleofConquest"][10002]:Show()
					for i = 10098, 10102 do
						mapTextures["IsleofConquest"][i]:Show()
					end
				end
			end
		else
			mapTextureFrames["World"].tex:SetTexture(0, 0.114, 0.157)
			mapTextureFrames["Kalimdor"]:Hide()
			mapTextureFrames["Azeroth"]:Hide()
			mapTextureFrames["Expansion01"]:Hide()
			mapTextureFrames["Northrend"]:Hide()
			mapTextureFrames["TheMaelstromContinent"]:Hide()
			mapTextureFrames["TolBarad"]:Hide()
		end
		if currentMapTexture and self.data.currentContinentID > 0 or self.data.currentZoneData then
			self:ZoomToZone(currentMapTexture)
			if db.mapView and not self.mapShown then
				self.mapShown = true
				self.scrollFrame:Show()
				self.mapHolder:Show()
			end
		else
			if db.mapView and self.mapShown then
				self.mapShown = nil
				self.scrollFrame:Hide()
				self.mapHolder:Hide()
			end
		end
	end
	if db.mapView and self.data.currentContinentID == 0 then
		for i = 1, 12 do
			_G["WorldMapDetailTile"..i]:SetTexture(TEXTURE_PATH.."SatMap\\World"..i)
		end
	end
end

function SatMap:MODIFIER_STATE_CHANGED(event, key, state)
	if not self:IsEnabled() or not self.mapShown or not db.poiLabels or not db.poiLabelsOnlyCtrl then return end
	if key == "LCTRL" or key == "RCTRL" then
		if state == 1 then
			self.poiContFrame:Show()
			if self.data.currentZoneID > 0 or self.data.currentZoneData then
				self.poiZoneFrame:Show()
			end
		else
			self.poiContFrame:Hide()
			self.poiZoneFrame:Hide()
		end
	end
end

function SatMap:UpdateMap(elapsed)
	self.lastUpdate = self.lastUpdate + elapsed
	if (self.lastUpdate >= updateInterval) then
		self.lastUpdate = 0

		if createMapTextureSection then
			CreateMapTextureSection()
		end
	end
end

function SatMap:WorldMapButton_OnClick(button, mouseButton)
	if not self.mapShown or not WorldMapFrame:IsVisible() then return end
	if (mouseButton == "LeftButton") then
		if self.data.currentMapTexture ~= "Azeroth" and self.data.currentMapTexture ~= "Kalimdor" then return end
		local left, top = WorldMapDetailFrame:GetLeft(), WorldMapDetailFrame:GetTop()
		local width, height = WorldMapDetailFrame:GetWidth(), WorldMapDetailFrame:GetHeight()
		local scale = WorldMapDetailFrame:GetEffectiveScale()
		local x, y = GetCursorPosition()
		local cx, cy = (x/scale - left) / width, (top - y/scale) / height

		if not cx or not cy or cx < 0 or cx > 1 or cy < 0 or cy > 1 then return end
		if self.data.currentMapTexture == "Azeroth" then
			if cx and cx >= 0 and cx <= 0.2 and cy and cy >= 0 and cy <= 0.2 then
				SetMapZoom(4)
			elseif cx and cx > 0.2 and cx <= 0.34 and cy and cy >= 0 and cy <= 0.3 then
				SetMapZoom(4)
			end
		elseif self.data.currentMapTexture == "Kalimdor" then
			if cx and cx >= 0.7 and cx <= 1 and cy and cy >= 0 and cy <= 0.14 then
				SetMapZoom(4)
			elseif cx and cx > 0.95 and cx <= 1 and cy and cy > 0.14 and cy <= 0.24 then
				SetMapZoom(4)
			end
		end
	end
end

function SatMap:ZoomToCurrentZone(instant)
	SetMapToCurrentZone()
	self:ZoomToZone(self.data.currentMapTexture, instant)
end

function SatMap:ZoomToZone(texture, instant)
	if not texture then return end
	texture = string_match(texture, "^(.+)_terrain%d+$") or texture
	local data = self.data.CONTINENT_DATA[texture] or self.data.ZONE_DATA[texture]
	if data and data.fullCenterX then
		self:WarpMap(
			data.fullCenterX,
			data.fullCenterY,
			math_min(
				self.scrollFrame:GetWidth() / data.fullWidth,
				self.scrollFrame:GetHeight() / data.fullHeight
			)
		)
	end
end

function SatMap:ReadjustView()
	local mapView = self.mapHolder
	local scrollFrame = self.scrollFrame

	local previousScale = mapView:GetScale()
	if previousScale ~= self.data.mapViewZoom then
		if self.data.mapViewZoom < 0.0005 then
			self.data.mapViewZoom = 0.0005
		elseif self.data.mapViewZoom > 420 then
			self.data.mapViewZoom = 420
		end
		mapView:SetScale(self.data.mapViewZoom)

		for i, poi in ipairs(shownPOIs) do
			POI_Resize(poi)
		end
	end
	scrollFrame:SetHorizontalScroll((mapView:GetWidth() - scrollFrame:GetWidth() / self.data.mapViewZoom) / 2 + self.data.mapViewX)
	scrollFrame:SetVerticalScroll((mapView:GetHeight() - scrollFrame:GetHeight() / self.data.mapViewZoom) / 2 - self.data.mapViewY)
end

function SatMap:WarpMap(x, y, zoom)
	if x or y then
		if not x or not y then
			x = x or self.data.mapViewX
			y = y or self.data.mapViewY
		end
	elseif zoom then
		x = self.data.mapViewX
		y = self.data.mapViewY
	end
	zoom = zoom or self.data.mapViewZoom
	self.data.mapViewX = x
	self.data.mapViewY = y
	self.data.mapViewZoom = zoom
	self:ReadjustView()
end

function SatMap:ConvertUniverseCoordinateToZoneCoordinate(x, y)
	if not x or not y then
		return nil, nil, nil
	end
	for continentID, currentContinentData in ipairs(self.data.CONTINENT_DATA) do
		local continentLeft = currentContinentData.visibleLeft
		local continentRight = currentContinentData.visibleRight
		local continentBottom = currentContinentData.visibleBottom
		local continentTop = currentContinentData.visibleTop
		if x >= continentLeft and x <= continentRight and y >= continentBottom and y <= continentTop then
			local x_grid = floor((x - continentLeft) / (continentRight - continentLeft) * 10)
			local y_grid = floor((y - continentBottom) / (continentTop - continentBottom) * 10)
			local num = x_grid*10 + y_grid
			local searchData = zoneSearchData[continentID][num]
			local bestZone
			if searchData then
				for _, zoneTexture in ipairs(searchData) do
					local zoneData = self.data.ZONE_DATA[zoneTexture]

					if x >= zoneData.highlightLeft and x <= zoneData.highlightRight and y >= zoneData.highlightBottom and y <= zoneData.highlightTop then
						bestZone = zoneTexture
						break
					end
				end
			end

			if bestZone then
				local zoneData = currentContinentData.zones[bestZone]
				return bestZone,
					(x - zoneData.fullCenterX) / zoneData.fullWidth + 0.5,
					0.5 - (y - zoneData.fullCenterY) / zoneData.fullHeight
			else
				return currentContinentData.texture,
					(x - currentContinentData.fullCenterX) / currentContinentData.fullWidth + 0.5,
					0.5 - (y - currentContinentData.fullCenterY) / currentContinentData.fullHeight
			end
		end
	end

	return nil, nil, nil
end

function SatMap:ConvertZoneCoordinateToUniverseCoordinate(zone, x, y)
	if (x == 0 and y == 0) or not x or not y or not zone then
		return nil, nil
	end
	local data = self.data.ZONE_DATA[zone]
	if data and data.fullCenterX then
		y = 1 - y
		x = (x - 1/2) * data.fullWidth + data.fullCenterX
		y = (y - 1/2) * data.fullHeight + data.fullCenterY
		return x, y
	end
	data = self.data.CONTINENT_DATA[zone]
	if data then
		y = 1 - y
		x = (x - 1/2) * data.fullWidth + data.fullCenterX
		y = (y - 1/2) * data.fullHeight + data.fullCenterY

		local zone_z, zone_x, zone_y = self:ConvertUniverseCoordinateToZoneCoordinate(x, y)
		local zoneData = self.data.ZONE_DATA[zone_z]
		if zoneData then
			-- This is done because Quel'Thalas' position is further northeast than it actually is.
			return x + zoneData.continentOffsetX, y + zoneData.continentOffsetY
		end

		return x, y
	end

	return nil, nil
end
