--[[
MapsterEnhanced - SatMap (Sindragosa)

Based on Cartographer
Credits: ckknight
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local L = LibStub("AceLocale-3.0"):GetLocale("Mapster")
local BZ = LBZ and LBZ:GetUnstrictLookupTable() or setmetatable({}, {__index = function(t,k) return k end})

local MODNAME = "SatMap"
local SatMap = Mapster:GetModule(MODNAME)

local data = SatMap.data

data.CONTINENT_DATA = {
	{ -- Kalimdor [1]
		x = -150,
		y = -20,
		height = 24533.025279205,
		width = 36800.210572494,
		bounds = { 0.0120, 0.9880, 0.0200, 0.9800 },
		rect = { 0.2606, 0.6898, 0.0647, 0.9485 },
		mapTextureCenterX = 34.5,
		mapTextureCenterY = 30,
		mapTextureOffsetX = 0,
		mapTextureOffsetY = 0,
		mapTexturePath = "World\\Minimaps\\Kalimdor",
		mapTextureBackground = "water1",
		extraMapTextureAreas = {
			Azuremyst = {
				mapTextureCenterX = 67.5,
				mapTextureCenterY = 49,
				mapTexturePath = "World\\Minimaps\\Expansion01",
			},
			TheExodar = {
				mapTextureCenterX = 2,
				mapTextureCenterY = 2,
				mapTextureOffsetX = 0.61,
				mapTextureOffsetY = -0.159,
				mapType = "city",
				mapScale = 0.24,
				mapTransparency = 0.5,
				mapSize = { -20, -17, 25, 17 },
				mapTextureBackground = "water1",
			},
			["Orgrimmar:2"] = {
				mapTextureCenterX = 1.29,
				mapTextureCenterY = -0.03,
				mapTextureOffsetX = 0,
				mapTextureOffsetY = 0,
				mapType = "city",
				mapScale = 0.25,
				mapTransparency = 0.8,
				mapTextureBackground = "nether",
			},
		},
	},
	{ -- Eastern Kingdoms [2]
		x = 141.0325,
		y = -12.6260,
		height = 27149.795290881,
		width = 40741.175327834,
		bounds = { 0.0120, 0.9880, 0.0142, 0.9857 },
		rect = { 0.3526, 0.6432, 0.0142, 0.9657 },
		mapTextureCenterX = 36.125,
		mapTextureCenterY = 35.5,
		mapTextureOffsetX = -0.01,
		mapTextureOffsetY = -0.03,
		mapTexturePath = "World\\Minimaps\\Azeroth",
		mapTextureBackground = "water1",
		extraMapTextureAreas = {
			QuelThalas = {
				mapTextureCenterX = 40.75,
				mapTextureCenterY = 30.5,
				mapTexturePath = "World\\Minimaps\\Expansion01",
			},
			Ironforge = {
				mapTextureCenterX = 2,
				mapTextureCenterY = 2,
				mapTextureOffsetX = 0.644,
				mapTextureOffsetY = -0.32,
				mapType = "city",
				mapScale = 0.24,
				mapTransparency = 0.5,
				mapSize = { -14, -10, 20, 14 },
				mapTextureBackground = "water1",
			},
			Undercity = {
				mapTextureCenterX = 2,
				mapTextureCenterY = 2,
				mapTextureOffsetX = 1.215,
				mapTextureOffsetY = -0.665,
				mapType = "city",
				mapScale = 0.24,
				mapSize = { -15, -11, 26, 17 },
				mapTextureBackground = "nether",
			},
			TolBarad = {
				mapTextureCenterX = 30.12,
				mapTextureCenterY = 33.31,
				mapTextureOffsetX = 0,
				mapTextureOffsetY = 0,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\TolBarad",
			},
			Vashjir = {
				mapTextureCenterX = 22.0985,
				mapTextureCenterY = 42.3189,
				mapTextureOffsetX = 0,
				mapTextureOffsetY = 0,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\Azeroth\\noLiquid_",
			},
			SilvermoonCity = {
				mapTextureCenterX = 2,
				mapTextureCenterY = 2,
				mapTextureOffsetX = -2.27,
				mapTextureOffsetY = -0.809,
				mapType = "city",
				mapScale = 0.244,
				mapTransparency = 0,
				mapTextureBackground = "water1",
				mapSize = { -36, -20, 20, 20 },
			},
		}
	},
	{ -- Outland [3]
		x = 0,
		y = -300,
		height = 11642.355227091,
		width = 17463.987300595,
		bounds = { 0.0222, 0.9771, 0.0453, 0.9830 },
		rect = { 0.1022, 0.7971, 0.0453, 0.9830 },
		mapTextureCenterX = 24,
		mapTextureCenterY = 31,
		mapTextureOffsetX = 0,
		mapTextureOffsetY = 0,
		mapTexturePath = "World\\Minimaps\\Expansion01",
		mapTextureBackground = "nether",
	},
	{ -- Northrend [4]
		x = 0.8699,
		y = 95.2377,
		height = 11834.3119870532,
		width = 17751.3962441049,
		bounds = { 0.020, 0.940, 0.028, 0.968 },
		rect = { 0.066, 0.911, 0.028, 0.968 },
		mapTextureCenterX = 31.5,
		mapTextureCenterY = 22,
		mapTextureOffsetX = -0.8,
		mapTextureOffsetY = 1.2,
		mapTexturePath = "World\\Minimaps\\Northrend",
		mapTextureBackground = "water4",
		extraMapTextureAreas = {
			Dalaran = {
				mapTextureCenterX = 2,
				mapTextureCenterY = 2,
				mapTextureOffsetX = 1.269407,
				mapTextureOffsetY = -0.184856,
				mapType = "city",
				mapScale = 0.24,
				mapTransparency = 0.5,
				mapTextureBackground = "water4",
				mapSize = { -20, -17, 25, 17 },
			},
			["Dalaran:2"] = {
				mapTextureCenterX = 1,
				mapTextureCenterY = 1,
				mapTextureOffsetX = 1.12,
				mapTextureOffsetY = -0.73,
				mapType = "overlay",
				mapScale = 0.24,
				mapTransparency = 0.8,
				mapTextureBackground = "nether",
				mapSize = { -10, -10, 15, 11 },
			},
			IsleofConquest = {
				mapTextureCenterX = 34,
				mapTextureCenterY = 30,
				mapTextureOffsetX = -2.65,
				mapTextureOffsetY = -2.93,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\IsleofConquest",
				mapSize = { -200, -200, 200, 200 },
				mapTextureBackground = "nether",
			},
		}
	},
	{ -- Maelstrom [5]
		x = 0,
		y = -24,
		height = 8662,
		width = 12993,
		bounds = { 0.026, 0.971, 0.028, 0.968 },
		rect = { 0.066, 0.911, 0.028, 0.968 },
		mapTextureCenterX = 30.5,
		mapTextureCenterY = 34,
		mapTextureOffsetX = 0,
		mapTextureOffsetY = 0,
		mapTexturePath = "World\\Minimaps\\LostIsles",
		mapTextureBackground = "water1",
		extraMapTextureAreas = {
			Kezan = {
				mapTextureCenterX = 32.3,
				mapTextureCenterY = 44.2,
				mapTexturePath = "World\\Minimaps\\LostIsles",
			},
			TheLostIsles = {
				mapTextureCenterX = 37.3,
				mapTextureCenterY = 28.2,
				mapTexturePath = "World\\Minimaps\\LostIsles",
			},
			TheMaelstrom = {
				mapTextureCenterX = 30.530,
				mapTextureCenterY = 29.385,
				mapTextureOffsetX = 0,
				mapTextureOffsetY = 0,
				mapType = "overlay",
				mapScale = 3,
				mapTexturePath = "World\\Minimaps\\MaelstromZone",
			},
			Deepholm = {
				mapTextureCenterX = 31.06,
				mapTextureCenterY = 28.95,
				mapTextureOffsetX = 0,
				mapTextureOffsetY = 0,
				mapType = "overlay",
				mapScale = 0.918,
				mapTexturePath = "World\\Minimaps\\Deephome",
			},
		}
	},
}


data.INSTANCE_DATA = {
	{ -- Instances [1]
		name = "Instances",
		x = 0,
		y = 400,
		height = 40000,
		width = 60000,
		bounds = { 0, 1, 0, 1 },
		rect = { 0, 1, 0, 1 },
		mapTextureCenterX = 0,
		mapTextureCenterY = 0,
		mapTextureOffsetX = 0,
		mapTextureOffsetY = 0,
		mapTexturePath = "World\\Minimaps\\Expansion01",
		mapTextureBackground = "nether",
		extraMapTextureAreas = {
			MoltenFront = {
				mapTextureCenterX = 31.367,
				mapTextureCenterY = 28.553,
				mapTextureOffsetX = 0,
				mapTextureOffsetY = 0,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\FirelandsDailies",
			},
			TempestKeep = {
				mapTextureCenterX = 1.5,
				mapTextureCenterY = 0.6,
				mapTextureOffsetX = 0.12,
				mapTextureOffsetY = -0.5,
				mapType = "overlay",
				mapScale = 0.74,
				mapSize = { -12, -10, 12, 10 },
				mapTextureBackground = "nether",
			},
			Firelands = {
				mapTextureCenterX = 32,
				mapTextureCenterY = 31,
				mapTextureOffsetX = 0.76,
				mapTextureOffsetY = 1.04,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\Firelands1",
			},
			["Firelands:2"] = {},
			["Firelands:3"] = {},
			BlackwingDescent = {
				mapTextureCenterX = 2,
				mapTextureCenterY = 1,
				mapTextureOffsetX = -0.22,
				mapTextureOffsetY = 1.31,
				mapType = "overlay",
				mapScale = 0.44,
				mapSize = { -12, -12, 12, 12 },
				mapTextureBackground = "nether",
			},
			["BlackwingDescent:2"] = {},
			TheBastionofTwilight = {
				mapTextureCenterX = 2,
				mapTextureCenterY = 1,
				mapTextureOffsetX = -1.14,
				mapTextureOffsetY = 0.77,
				mapType = "overlay",
				mapScale = 0.35,
				mapSize = { -20, -24, 14, 10 },
				mapTextureBackground = "nether",
			},
			["TheBastionofTwilight:2"] = {},
			["TheBastionofTwilight:3"] = {
				mapTextureCenterX = 1.5,
				mapTextureCenterY = 0.5,
				mapTextureOffsetX = -0.23,
				mapTextureOffsetY = -0.64,
				mapType = "overlay",
				mapScale = 0.48,
				mapSize = { -12, -10, 12, 10 },
				mapTextureBackground = "nether",
			},
			Skywall = {
				mapTextureCenterX = 32,
				mapTextureCenterY = 32,
				mapTextureOffsetX = -0.98,
				mapTextureOffsetY = 2.5,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\SkywallDungeon",
			},
			ThroneoftheFourWinds = {
				mapTextureCenterX = 30.5,
				mapTextureCenterY = 31,
				mapTextureOffsetX = 0.26,
				mapTextureOffsetY = 0.5,
				mapType = "overlay",
				mapScale = 1.025,
				mapTexturePath = "World\\Minimaps\\SkywallRaid",
			},
			TheEyeofEternity = {
				mapTextureCenterX = 29.5,
				mapTextureCenterY = 29.5,
				mapTextureOffsetX = 0.807,
				mapTextureOffsetY = 0.44,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\NexusRaid",
				mapTextureBackground = "water4",
			},
			SunwellPlateau = {
				mapTextureCenterX = 31,
				mapTextureCenterY = 27.5,
				mapTextureOffsetX = -2.2,
				mapTextureOffsetY = 1.55,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\SunwellPlateau",
			},
			["SunwellPlateau:2"] = {
				mapTextureCenterX = 1,
				mapTextureCenterY = 0,
				mapTextureOffsetX = 0.22,
				mapTextureOffsetY = -0.05,
				mapType = "overlay",
				mapScale = 0.24,
				mapTextureBackground = "nether",
			},
			MagistersTerrace = {
				mapTextureCenterX = 1.5,
				mapTextureCenterY = 0,
				mapTextureOffsetX = 0.212,
				mapTextureOffsetY = -0.22,
				mapType = "overlay",
				mapScale = 0.306,
				mapTextureBackground = "nether",
			},
			["MagistersTerrace:2"] = {},
			TheObsidianSanctum = {
				mapTextureCenterX = 31,
				mapTextureCenterY = 25,
				mapTextureOffsetX = -0.18,
				mapTextureOffsetY = -0.295,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\ChamberOfAspectsBlack",
			},
			TheRubySanctum = {
				mapTextureCenterX = 31,
				mapTextureCenterY = 25,
				mapTextureOffsetX = 0.07,
				mapTextureOffsetY = 0.23,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\ChamberofAspectsRed",
			},
			ZulGurub = {
				mapTextureCenterX = 35,
				mapTextureCenterY = 53.5,
				mapTextureOffsetX = 0.72,
				mapTextureOffsetY = -0.7,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\Zul_Gurub5Man",
			},
			ZulAman = {
				mapTextureCenterX = 30,
				mapTextureCenterY = 31,
				mapTextureOffsetX = -1.514,
				mapTextureOffsetY = -1.47,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\ZulAman",
			},
			HallsofOrigination = {
				mapTextureCenterX = 1,
				mapTextureCenterY = 0,
				mapTextureOffsetX = -2.09,
				mapTextureOffsetY = 1.44,
				mapType = "overlay",
				mapScale = 0.807,
				mapSize = { -14, -10, 10, 10 },
				mapTextureBackground = "nether",
			},
			["HallsofOrigination:2"] = {
				mapTextureCenterX = 32,
				mapTextureCenterY = 32,
				mapTextureOffsetX = 4.87,
				mapTextureOffsetY = -0.17,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\Uldum",
			},
			["HallsofOrigination:3"] = {
				mapTextureCenterX = 1.5,
				mapTextureCenterY = 0.5,
				mapTextureOffsetX = 0.31,
				mapTextureOffsetY = -0.07,
				mapType = "overlay",
				mapScale = 0.403,
				mapSize = { -14, -10, 15, 10 },
				mapTextureBackground = "nether",
			},
			LostCityofTolvir = {
				mapTextureCenterX = 34,
				mapTextureCenterY = 52,
				mapTextureOffsetX = 4.24,
				mapTextureOffsetY = -2.84,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\UldumDungeon",
			},
			TheNexus = {
				mapTextureCenterX = 1.5,
				mapTextureCenterY = 0,
				mapTextureOffsetX = 0.12,
				mapTextureOffsetY = 1.08,
				mapType = "overlay",
				mapScale = 0.49,
				mapSize = { -12, -10, 12, 10 },
			},
			DarkmoonFaireIsland = {
				mapTextureCenterX = 20,
				mapTextureCenterY = 38.5,
				mapTextureOffsetX = 0.6,
				mapTextureOffsetY = -0.58,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\DarkmoonFaire",
			},
			EndTime = {
				mapTextureCenterX = 31,
				mapTextureCenterY = 24,
				mapTextureOffsetX = -0.78,
				mapTextureOffsetY = 1.36,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\COTDragonblight",
			},
			["EndTime:2"] = {},
			["EndTime:3"] = {},
			["EndTime:4"] = {},
			["EndTime:5"] = {},
			["EndTime:6"] = {},
			WellofEternity = {
				mapTextureCenterX = 42,
				mapTextureCenterY = 25,
				mapTextureOffsetX = -0.59,
				mapTextureOffsetY = -1.51,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\COTWarOfTheAncients",
			},
			HourofTwilight = {
				mapTextureCenterX = 31.5,
				mapTextureCenterY = 23,
				mapTextureOffsetX = -0.04,
				mapTextureOffsetY = -0.48,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\TheHourOfTwilight",
			},
			["HourofTwilight:2"] = {},
			DragonSoul = {
				mapTextureCenterX = 0,
				mapTextureCenterY = 0,
				mapTextureOffsetX = 0,
				mapTextureOffsetY = 0,
			},
			["DragonSoul:4"] = {},
			StrandoftheAncients = {
				mapTextureCenterX = 32,
				mapTextureCenterY = 28.5,
				mapTextureOffsetX = 0.82,
				mapTextureOffsetY = 0.29,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\NorthrendBG",
			},
			ArathiBasin = {
				mapTextureCenterX = 30,
				mapTextureCenterY = 29,
				mapTextureOffsetX = 0.87,
				mapTextureOffsetY = 1.4,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\PVPZone04",
			},
			WarsongGulch = {
				mapTextureCenterX = 29.5,
				mapTextureCenterY = 29,
				mapTextureOffsetX = -1.35,
				mapTextureOffsetY = -1.8,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\PVPZone03",
			},
			AlteracValley = {
				mapTextureCenterX = 32.5,
				mapTextureCenterY = 31.5,
				mapTextureOffsetX = 0.7,
				mapTextureOffsetY = 0.55,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\PVPZone01",
			},
			NetherstormArena = {
				mapTextureCenterX = 29,
				mapTextureCenterY = 27,
				mapTextureOffsetX = 0.77,
				mapTextureOffsetY = -0.3,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\EyeoftheStorm2.0",
			},
			GilneasBattleground2 = {
				mapTextureCenterX = 30,
				mapTextureCenterY = 29,
				mapTextureOffsetX = -0.29,
				mapTextureOffsetY = -1.04,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\Gilneas_BG_2",
			},
			TwinPeaks = {
				mapTextureCenterX = 31.5,
				mapTextureCenterY = 27.5,
				mapTextureOffsetX = -0.58,
				mapTextureOffsetY = 0.05,
				mapType = "overlay",
				mapTexturePath = "World\\Minimaps\\CataclysmCTF",
			},
		}
	},
}

data.MAP_DATA = {
	{ -- Kalimdor [1]
		["Orgrimmar"] = { "Orgrimmar", "Orgrimmar", 1, 0.6640625, 0.04724834, 0.04726589, 0.55905, 0.4204 },
		["Darnassus"] = { "Darnassus", "Darnassus", 1, 0.6640625, 0.0418546884, 0.041865025, 0.377126, 0.09522 },
		["Orgrimmar:2"] = { "Orgrimmar", "Orgrimmar:2", 1, 1, 0.00984, 0.00984, 0.57692, 0.442998 },
	},
	{ -- Eastern Kingdoms [2]
		["StormwindCity"] = { "Stormwind City", "StormwindCity", 1, 0.6640625, 0.042647267784764, 0.042664690258866, 0.40374517545301, 0.70616561562589 },
		["StranglethornJungle"] = { "Northern Stranglethorn", "StranglethornJungle", 1, 0.6640625, 0.10063528, 0.1006764, 0.40323381, 0.817431464 },
		["TheCapeOfStranglethorn"] = { "The Cape of Stranglethorn", "TheCapeOfStranglethorn", 1, 0.6640625, 0.09685122, 0.0969164, 0.394285, 0.87268072 },
		["RuinsofGilneasCity"] = { "Ruins of Gilneas City", "RuinsofGilneasCity", 1, 0.6640625, 0.02162, 0.021636, 0.3986999, 0.4599368 },
		["VashjirDepths"] = { "Abyssal Depths", "VashjirDepths", 1, 0.6640625, 0.10002165, 0.100062533, 0.24394575, 0.592367538 },
		["VashjirKelpForest"] = { "Kelp'thar Forest", "VashjirKelpForest", 1, 0.6640625, 0.06877766, 0.068831372, 0.3215699, 0.5596784 },
		["VashjirRuins"] = { "Shimmering Expanse", "VashjirRuins", 1, 0.6640625, 0.119044159961, 0.11909282594, 0.282041934692, 0.58684261241 },
	},
	{ -- Outland [3]
	},
	{ -- Northrend [4]
		["Dalaran"] = { "Dalaran", "Dalaran", 1, 1, 0.046757728, 0.046757728, 0.459943612, 0.382508348 },
		["Dalaran:2"] = { "Dalaran", "Dalaran:2", 1, 1, 0.03173, 0.03173, 0.46764, 0.38888446 },
		["IsleofConquest"] = { "Isle of Conquest", "IsleofConquest", 1, 0.6640625, 0.149284, 0.149284, 0.0675, 0.1876 },
	},
	{ -- Maelstrom [5]
		["Deepholm"] = { "Deepholm", "Deepholm", 1, 1, 0.36, 0.36, 0.33, 0.10 },
		["Kezan"] = { "Kezan", "Kezan", 1, 1, 0.1038, 0.1038, 0.323824, 0.5800 },
		["TheLostIsles"] = { "The Lost Isles", "TheLostIsles", 1, 1, 0.3456, 0.3456, -0.0538, 0.3406 },
		["TheMaelstrom"] = { "The Maelstrom", "TheMaelstrom", 1, 1, 0.36, 0.36, -0.03, 0.1 },
	},
	{ -- Instances [6]
		["MoltenFront"] = { "Molten Front", "MoltenFront", 1, 0.6640625, 0.0198263885498, 0.019843748474, 0.01, 0.01 },
		["TempestKeep"] = { "Tempest Keep", "TempestKeep", 1, 0.6640625, 0.02625, 0.02625, 0.54, 0.01 },
		["Firelands"] = { "Firelands", "Firelands", 1, 0.6640625, 0.026458332, 0.026458332, 0.05, 0.01 },
		["Firelands:2"] = { "Firelands", "Firelands:2", 1, 0.6640625, 0.00625, 0.00625, 0.05068, 0.00578 },
		["Firelands:3"] = { "Firelands", "Firelands:3", 1, 0.6640625, 0.024, 0.024, 0.0508, -0.009999 },
		["BlackwingDescent"] = { "Blackwing Descent", "BlackwingDescent", 1, 0.6640625, 0.01416, 0.01416, 0.1, 0.01 },
		["BlackwingDescent:2"] = { "Blackwing Descent", "BlackwingDescent:2", 1, 0.6640625, 0.01666, 0.01666, 0.09875, 0.0011 },
		["TheBastionofTwilight"] = { "The Bastion of Twilight", "TheBastionofTwilight", 1, 0.6640625, 0.0179722, 0.0179722, 0.15, 0.01 },
		["TheBastionofTwilight:2"] = { "The Bastion of Twilight", "TheBastionofTwilight:2", 1, 0.6640625, 0.0129724, 0.0129724, 0.1525, 0.0248 },
		["TheBastionofTwilight:3"] = { "The Bastion of Twilight", "TheBastionofTwilight:3", 1, 0.6640625, 0.017372367, 0.017372374, 0.152, 0.0231 },
		["Skywall"] = { "The Vortex Pinnacle", "Skywall", 1, 0.6640625, 0.0336454, 0.0336454, 0.26, 0.01 },
		["ThroneoftheFourWinds"] = { "Throne of the Four Winds", "ThroneoftheFourWinds", 1, 0.6640625, 0.025, 0.025, 0.2, 0.01 },
		["TheEyeofEternity"] = { "The Eye of Eternity", "TheEyeofEternity", 1, 0.6640625, 0.0071678, 0.0071678, 0.4, 0.01 },
		["SunwellPlateau"] = { "Sunwell Plateau", "SunwellPlateau", 1, 0.6640625, 0.015104, 0.015104, 0.59, 0.01 },
		["SunwellPlateau:2"] = { "Sunwell Plateau", "SunwellPlateau:2", 1, 0.6640625, 0.00775, 0.00775, 0.59, 0.01 },
		["MagistersTerrace"] = { "Magisters' Terrace", "MagistersTerrace", 1, 0.6640625, 0.00884, 0.00884, 0.59, 0.07 },
		["MagistersTerrace:2"] = { "Magisters' Terrace", "MagistersTerrace:2", 1, 0.6640625, 0.00884, 0.00884, 0.59, 0.07 },
		["ZulGurub"] = { "Zul'Gurub", "ZulGurub", 1, 0.6640625, 0.03534722, 0.035364575, 0.01, 0.08 },
		["ZulAman"] = { "Zul'Aman", "ZulAman", 1, 0.6640625, 0.02114583, 0.02114583, 0.07, 0.08 },
		["TheObsidianSanctum"] = { "The Obsidian Sanctum", "TheObsidianSanctum", 1, 0.6640625, 0.019375, 0.019375, 0.3, 0.01 },
		["TheRubySanctum"] = { "The Ruby Sanctum", "TheRubySanctum", 1, 0.6640625, 0.01253472, 0.01253472, 0.36, 0.02 },
		["HallsofOrigination"] = { "Halls of Origination", "HallsofOrigination", 1, 0.6640625, 0.025529, 0.025529, 0.17, 0.08 },
		["HallsofOrigination:2"] = { "Halls of Origination", "HallsofOrigination:2", 1, 0.6640625, 0.0212126, 0.0212126, 0.17, 0.08 },
		["HallsofOrigination:3"] = { "Halls of Origination", "HallsofOrigination:3", 1, 0.6640625, 0.0188, 0.0188, 0.17, 0.08 },
		["LostCityofTolvir"] = { "Lost City of the Tol'vir", "LostCityofTolvir", 1, 0.6640625, 0.01618, 0.016198, 0.11, 0.08 },
		["TheNexus"] = { "The Nexus", "TheNexus", 1, 0.6640625, 0.0183547, 0.0183547, 0.4, 0.07 },
		["DarkmoonFaireIsland"] = { "Darkmoon Island", "DarkmoonFaireIsland", 1, 0.6640625, 0.031, 0.031, 0.01, 0.2 },
		["EndTime"] = { "End Time", "EndTime", 1, 0.6640625, 0.05493, 0.05493, 0.05, 0.2 },
		["EndTime:2"] = { "End Time", "EndTime:2", 1, 0.6640625, 0.009375, 0.009375, 0.0744, 0.235 },
		["EndTime:3"] = { "End Time", "EndTime:3", 1, 0.6640625, 0.014427, 0.014427, 0.0643, 0.2155 },
		["EndTime:4"] = { "End Time", "EndTime:4", 1, 0.6640625, 0.0079, 0.0079, 0.06005, 0.20375 },
		["EndTime:5"] = { "End Time", "EndTime:5", 1, 0.6640625, 0.011614746, 0.011614746, 0.08088, 0.2414989 },
		["EndTime:6"] = { "End Time", "EndTime:6", 1, 0.6640625, 0.007552246, 0.007552246, 0.0909, 0.2105 },
		["WellofEternity"] = { "Well of Eternity", "WellofEternity", 1, 0.6640625, 0.02083333, 0.02083333, 0.12, 0.2 },
		["HourofTwilight"] = { "Hour of Twilight", "HourofTwilight", 1, 0.6640625, 0.050729, 0.050729, 0.17, 0.17 },
		["HourofTwilight:2"] = { "Hour of Twilight", "HourofTwilight:2", 1, 0.6640625, 0.00625, 0.00625, 0.19238, 0.21133 },
		["DragonSoul"] = { "Dragon Soul", "DragonSoul", 1, 0.6640625, 0.05076321, 0.05076321, 0.17, 0.1842 },
		["DragonSoul:4"] = { "Dragon Soul", "DragonSoul:4", 1, 0.6640625, 0.0030866, 0.0030866, 0.40112, 0.012 },
		["StrandoftheAncients"] = { "Strand of the Ancients", "StrandoftheAncients", 1, 0.6640625, 0.0290625, 0.0290625, 0.01, 0.5 },
		["ArathiBasin"] = { "Arathi Basin", "ArathiBasin", 1, 0.6640625, 0.02927, 0.02927, 0.05, 0.5 },
		["WarsongGulch"] = { "Warsong Gulch", "WarsongGulch", 1, 0.6640625, 0.0191, 0.0191, 0.1, 0.5 },
		["AlteracValley"] = { "Alterac Valley", "AlteracValley", 1, 0.6640625, 0.070625, 0.070625, 0.01, 0.55 },
		["NetherstormArena"] = { "Eye of the Storm", "NetherstormArena", 1, 0.6640625, 0.0378472, 0.0378472, 0.15, 0.5 },
		["GilneasBattleground2"] = { "The Battle for Gilneas", "GilneasBattleground2", 1, 0.6640625, 0.0217, 0.0217, 0.1, 0.55 },
		["TwinPeaks"] = { "Twin Peaks", "TwinPeaks", 1, 0.6640625, 0.02025, 0.02025, 0.2, 0.5 },
	},
}

data.CITIES = {
	["Darnassus"] = true,
	["TheExodar"] = true,
	["SilvermoonCity"] = true,
	["Undercity"] = true,
	["Ironforge"] = true,
	["StormwindCity"] = true,
	["ThunderBluff"] = true,
	["Orgrimmar"] = true,
	["ShattrathCity"] = true,
	["Dalaran"] = true,
}

data.INDOOR_MAPS = {
	["Dalaran"] = "Dalaran",
	["Ironforge"] = "Ironforge",
	["Undercity"] = "Undercity",
	["TheExodar"] = "TheExodar",
	["Dalaran:2"] = "Dalaran:2",
	["Orgrimmar:2"] = "Orgrimmar:2",
	["TempestKeep"] = "TempestKeep",
	["SilvermoonCity"] = "SilvermoonCity",
	["SunwellPlateau:2"] = "SunwellPlateau:2",
	["MagistersTerrace"] = "MagistersTerrace",
	["MagistersTerrace:2"] = "MagistersTerrace",
	["BlackwingDescent"] = "BlackwingDescent",
	["BlackwingDescent:2"] = "BlackwingDescent",
	["TheBastionofTwilight"] = "TheBastionofTwilight",
	["TheBastionofTwilight:2"] = "TheBastionofTwilight",
	["TheBastionofTwilight:3"] = "TheBastionofTwilight:3",
	["HallsofOrigination"] = "HallsofOrigination",
	["HallsofOrigination:3"] = "HallsofOrigination:3",
	["TheNexus"] = "TheNexus",
	["DragonSoul"] = "HourofTwilight",
	["DragonSoul:4"] = "TheEyeofEternity",
	["HourofTwilight:2"] = "HourofTwilight",
}

data.ZONE_EXCEPTIONS = {
	["StranglethornVale"] = true,
	["Vashjir"] = true,
	--["RuinsofGilneas"] = true,
	--["RuinsofGilneasCity"] = true,
}

data.ZONE_SORT = {
	-- Eastern Kingdoms
	"Arathi",
	"BlastedLands",
	"BurningSteppes",
	"DeadwindPass",
	"DunMorogh",
	"Ghostlands",
	"WesternPlaguelands",
	"EasternPlaguelands",
	"EversongWoods",
	"HillsbradFoothills",
	"Hinterlands",
	"LochModan",
	"Redridge",
	"Elwynn",
	"SearingGorge",
	"Badlands",
	"Silverpine",
	--"StranglethornVale",
	"Duskwood",
	"Sunwell",
	"SwampOfSorrows",
	"Tirisfal",
	"Westfall",
	"Wetlands",
	"StranglethornJungle",
	"TheCapeOfStranglethorn",
	--"Vashjir",
	"VashjirDepths",
	"VashjirKelpForest",
	"VashjirRuins",
	"RuinsofGilneas",
	--"RuinsofGilneasCity",
	"Gilneas",
	--"GilneasCity",
	"TolBarad",
	"TolBaradDailyArea",
	"TwilightHighlands",

	-- Kalimdor
	"Ashenvale",
	"Aszhara",
	"AzuremystIsle",
	"BloodmystIsle",
	"Darkshore",
	"Desolace",
	"Durotar",
	"Felwood",
	"Moonglade",
	"Mulgore",
	"Silithus",
	"Tanaris",
	"Teldrassil",
	"UngoroCrater",
	"Winterspring",
	"Barrens",
	"StonetalonMountains",
	"Dustwallow",
	"ThousandNeedles",
	"Feralas",
	"SouthernBarrens",
	"Hyjal",
	"Uldum",
	"AhnQirajTheFallenKingdom",

	-- Outland
	"Netherstorm",
	"TerokkarForest",
	"Hellfire",
	"Zangarmarsh",
	"BladesEdgeMountains",
	"Nagrand",
	"ShadowmoonValley",

	-- Northrend
	"BoreanTundra",
	"CrystalsongForest",
	"Dragonblight",
	"GrizzlyHills",
	"HowlingFjord",
	"HrothgarsLanding",
	"LakeWintergrasp",
	"TheStormPeaks",
	"IcecrownGlacier",
	"SholazarBasin",
	"ZulDrak",
}

data.TEXTURE_DATA = {	-- TextureData
	Kalimdor = { [2324]="",[2325]="",[2326]="",[2327]="",[2328]="",[2329]="",[2330]="",[2331]="",[2332]="",[2333]="",[2334]="",[2335]="",[2336]="",[2337]="",[2338]="",[2339]="",[2340]="",[2341]="",[2342]="",[2343]="",[2344]="",[2345]="",[2409]="",[2410]="",[2411]="",[2412]="",[2413]="",[2424]="",[2425]="",[2426]="",[2427]="",[2428]="",[2429]="",[2430]="",[2431]="",[2432]="",[2433]="",[2434]="",[2435]="",[2436]="",[2437]="",[2438]="",[2439]="",[2440]="",[2441]="",[2442]="",[2443]="",[2444]="",[2445]="",[2509]="",[2510]="",[2511]="",[2512]="",[2513]="",[2514]="",[2515]="",[2524]="",[2525]="",[2526]="",[2527]="",[2528]="",[2529]="",[2530]="",[2531]="",[2532]="",[2533]="",[2534]="",[2535]="",[2536]="",[2537]="",[2538]="",[2539]="",[2540]="",[2541]="",[2542]="",[2543]="",[2544]="",[2545]="",[2546]="",[2547]="",[2548]="",[2549]="",[2550]="",[2551]="",[2552]="",[2553]="",[2554]="",[2609]="",[2610]="",[2611]="",[2612]="",[2613]="",[2614]="",[2615]="",[2616]="",[2624]="",[2625]="",[2626]="",[2627]="",[2628]="",[2629]="",[2630]="",[2631]="",[2632]="",[2633]="",[2634]="",[2635]="",[2636]="",[2637]="",[2638]="",[2639]="",[2640]="",[2641]="",[2642]="",[2643]="",[2644]="",[2645]="",[2646]="",[2647]="",[2648]="",[2649]="",[2650]="",[2651]="",[2652]="",[2653]="",[2654]="",[2709]="",[2710]="",[2711]="",[2712]="",[2713]="",[2714]="",[2715]="",[2716]="",[2717]="",[2718]="",[2719]="",[2720]="",[2721]="",[2722]="",[2723]="",[2724]="",[2725]="",[2726]="",[2727]="",[2728]="",[2729]="",[2730]="",[2731]="",[2732]="",[2733]="",[2734]="",[2735]="",[2736]="",[2737]="",[2738]="",[2739]="",[2740]="",[2741]="",[2742]="",[2743]="",[2744]="",[2745]="",[2746]="",[2747]="",[2748]="",[2749]="",[2750]="",[2751]="",[2752]="",[2753]="",[2754]="",[2809]="",[2810]="",[2811]="",[2812]="",[2813]="",[2814]="",[2815]="",[2816]="",[2817]="",[2818]="",[2819]="",[2820]="",[2821]="",[2822]="",[2823]="",[2824]="",[2825]="",[2826]="",[2827]="",[2828]="",[2829]="",[2830]="",[2831]="",[2832]="",[2833]="",[2834]="",[2835]="",[2836]="",[2837]="",[2838]="",[2839]="",[2840]="",[2841]="",[2842]="",[2843]="",[2844]="",[2845]="",[2846]="",[2847]="",[2848]="",[2849]="",[2850]="",[2851]="",[2852]="",[2853]="",[2854]="",[2909]="",[2910]="",[2911]="",[2912]="",[2913]="",[2914]="",[2915]="",[2916]="",[2917]="",[2918]="",[2919]="",[2920]="",[2921]="",[2922]="",[2923]="",[2924]="",[2925]="",[2926]="",[2927]="",[2928]="",[2929]="",[2930]="",[2931]="",[2932]="",[2933]="",[2934]="",[2935]="",[2936]="",[2937]="",[2938]="",[2939]="",[2940]="",[2941]="",[2942]="",[2943]="",[2944]="",[2945]="",[2946]="",[2947]="",[2948]="",[2949]="",[2950]="",[2951]="",[2952]="",[2953]="",[2954]="",[3009]="",[3010]="",[3011]="",[3012]="",[3013]="",[3014]="",[3015]="",[3016]="",[3017]="",[3018]="",[3019]="",[3020]="",[3021]="",[3022]="",[3023]="",[3024]="",[3025]="",[3026]="",[3027]="",[3028]="",[3029]="",[3030]="",[3031]="",[3032]="",[3033]="",[3034]="",[3035]="",[3036]="",[3037]="",[3038]="",[3039]="",[3040]="",[3041]="",[3042]="",[3043]="",[3044]="",[3045]="",[3046]="",[3047]="",[3048]="",[3049]="",[3050]="",[3051]="",[3052]="",[3053]="",[3054]="",[3109]="",[3110]="",[3111]="",[3112]="",[3113]="",[3114]="",[3115]="",[3116]="",[3117]="",[3118]="",[3119]="",[3120]="",[3121]="",[3122]="",[3123]="",[3124]="",[3125]="",[3126]="",[3127]="",[3128]="",[3129]="",[3130]="",[3131]="",[3132]="",[3133]="",[3134]="",[3135]="",[3136]="",[3137]="",[3138]="",[3139]="",[3140]="",[3141]="",[3142]="",[3143]="",[3144]="",[3145]="",[3146]="",[3147]="",[3148]="",[3149]="",[3150]="",[3151]="",[3152]="",[3153]="",[3154]="",[3209]="",[3210]="",[3211]="",[3212]="",[3213]="",[3214]="",[3215]="",[3216]="",[3217]="",[3218]="",[3219]="",[3220]="",[3221]="",[3222]="",[3223]="",[3224]="",[3225]="",[3226]="",[3227]="",[3228]="",[3229]="",[3230]="",[3231]="",[3232]="",[3233]="",[3234]="",[3235]="",[3236]="",[3237]="",[3238]="",[3239]="",[3240]="",[3241]="",[3242]="",[3243]="",[3244]="",[3245]="",[3246]="",[3247]="",[3248]="",[3249]="",[3250]="",[3251]="",[3252]="",[3253]="",[3254]="",[3309]="",[3310]="",[3311]="",[3312]="",[3313]="",[3314]="",[3315]="",[3316]="",[3317]="",[3318]="",[3319]="",[3320]="",[3321]="",[3322]="",[3323]="",[3324]="",[3325]="",[3326]="",[3327]="",[3328]="",[3329]="",[3330]="",[3331]="",[3332]="",[3333]="",[3334]="",[3335]="",[3336]="",[3337]="",[3338]="",[3339]="",[3340]="",[3341]="",[3342]="",[3343]="",[3344]="",[3345]="",[3346]="",[3347]="",[3348]="",[3349]="",[3350]="",[3351]="",[3352]="",[3353]="",[3354]="",[3409]="",[3410]="",[3411]="",[3412]="",[3413]="",[3414]="",[3415]="",[3416]="",[3417]="",[3418]="",[3419]="",[3420]="",[3421]="",[3422]="",[3423]="",[3424]="",[3425]="",[3426]="",[3427]="",[3428]="",[3429]="",[3430]="",[3431]="",[3432]="",[3433]="",[3434]="",[3435]="",[3436]="",[3437]="",[3438]="",[3439]="",[3440]="",[3441]="",[3442]="",[3443]="",[3444]="",[3445]="",[3446]="",[3447]="",[3448]="",[3449]="",[3450]="",[3451]="",[3452]="",[3453]="",[3454]="",[3514]="",[3515]="",[3516]="",[3517]="",[3518]="",[3519]="",[3520]="",[3521]="",[3522]="",[3523]="",[3524]="",[3525]="",[3526]="",[3527]="",[3528]="",[3529]="",[3530]="",[3531]="",[3532]="",[3533]="",[3534]="",[3535]="",[3536]="",[3537]="",[3538]="",[3539]="",[3540]="",[3541]="",[3542]="",[3543]="",[3544]="",[3545]="",[3546]="",[3547]="",[3548]="",[3549]="",[3550]="",[3551]="",[3552]="",[3553]="",[3554]="",[3614]="",[3615]="",[3616]="",[3617]="",[3618]="",[3619]="",[3620]="",[3621]="",[3622]="",[3623]="",[3624]="",[3625]="",[3626]="",[3627]="",[3628]="",[3629]="",[3630]="",[3631]="",[3632]="",[3633]="",[3634]="",[3635]="",[3636]="",[3637]="",[3638]="",[3639]="",[3640]="",[3641]="",[3642]="",[3643]="",[3644]="",[3645]="",[3646]="",[3647]="",[3648]="",[3649]="",[3650]="",[3651]="",[3652]="",[3653]="",[3654]="",[3714]="",[3715]="",[3716]="",[3717]="",[3718]="",[3719]="",[3720]="",[3721]="",[3722]="",[3723]="",[3724]="",[3725]="",[3726]="",[3727]="",[3728]="",[3729]="",[3730]="",[3731]="",[3732]="",[3733]="",[3734]="",[3735]="",[3736]="",[3737]="",[3738]="",[3739]="",[3740]="",[3741]="",[3742]="",[3743]="",[3744]="",[3745]="",[3746]="",[3747]="",[3748]="",[3749]="",[3750]="",[3751]="",[3752]="",[3753]="",[3754]="",[3814]="",[3815]="",[3816]="",[3817]="",[3818]="",[3819]="",[3820]="",[3821]="",[3822]="",[3823]="",[3824]="",[3825]="",[3826]="",[3827]="",[3828]="",[3829]="",[3830]="",[3831]="",[3832]="",[3833]="",[3834]="",[3835]="",[3836]="",[3837]="",[3838]="",[3839]="",[3840]="",[3841]="",[3842]="",[3843]="",[3844]="",[3845]="",[3846]="",[3847]="",[3848]="",[3849]="",[3850]="",[3851]="",[3852]="",[3853]="",[3854]="",[3855]="",[3914]="",[3915]="",[3916]="",[3917]="",[3918]="",[3919]="",[3920]="",[3921]="",[3922]="",[3923]="",[3924]="",[3925]="",[3926]="",[3927]="",[3928]="",[3929]="",[3930]="",[3931]="",[3932]="",[3933]="",[3934]="",[3935]="",[3936]="",[3937]="",[3938]="",[3939]="",[3940]="",[3941]="",[3942]="",[3943]="",[3944]="",[3945]="",[3946]="",[3947]="",[3948]="",[3949]="",[3950]="",[3951]="",[3952]="",[3953]="",[3954]="",[3955]="",[4014]="",[4015]="",[4016]="",[4017]="",[4018]="",[4019]="",[4020]="",[4021]="",[4022]="",[4023]="",[4024]="",[4025]="",[4026]="",[4027]="",[4028]="",[4029]="",[4030]="",[4031]="",[4032]="",[4033]="",[4034]="",[4035]="",[4036]="",[4037]="",[4038]="",[4039]="",[4040]="",[4041]="",[4042]="",[4043]="",[4044]="",[4045]="",[4046]="",[4047]="",[4048]="",[4049]="",[4050]="",[4051]="",[4052]="",[4053]="",[4054]="",[4055]="",[4114]="",[4115]="",[4116]="",[4117]="",[4118]="",[4119]="",[4120]="",[4121]="",[4122]="",[4123]="",[4124]="",[4125]="",[4126]="",[4127]="",[4128]="",[4129]="",[4130]="",[4131]="",[4132]="",[4133]="",[4134]="",[4135]="",[4136]="",[4137]="",[4138]="",[4139]="",[4140]="",[4141]="",[4142]="",[4143]="",[4144]="",[4145]="",[4146]="",[4147]="",[4148]="",[4149]="",[4150]="",[4151]="",[4152]="",[4153]="",[4154]="",[4155]="",[4214]="",[4215]="",[4216]="",[4217]="",[4218]="",[4219]="",[4220]="",[4221]="",[4222]="",[4223]="",[4224]="",[4225]="",[4226]="",[4227]="",[4228]="",[4229]="",[4230]="",[4231]="",[4232]="",[4233]="",[4234]="",[4235]="",[4236]="",[4237]="",[4238]="",[4239]="",[4240]="",[4241]="",[4242]="",[4243]="",[4244]="",[4245]="",[4246]="",[4247]="",[4248]="",[4249]="",[4250]="",[4251]="",[4252]="",[4253]="",[4254]="",[4255]="",[4314]="",[4315]="",[4316]="",[4317]="",[4318]="",[4319]="",[4320]="",[4321]="",[4322]="",[4323]="",[4324]="",[4325]="",[4326]="",[4327]="",[4328]="",[4329]="",[4330]="",[4331]="",[4332]="",[4333]="",[4334]="",[4335]="",[4336]="",[4337]="",[4338]="",[4339]="",[4340]="",[4344]="",[4345]="",[4346]="",[4347]="",[4348]="",[4349]="",[4421]="",[4422]="",[4423]="",[4424]="",[4425]="",[4426]="",[4427]="",[4428]="",[4429]="",[4430]="",[4431]="",[4432]="",[4433]="",[4434]="",[4435]="",[4436]="",[4437]="",[4438]="",[4439]="",[4440]="",[4521]="",[4522]="",[4523]="",[4524]="",[4525]="",[4526]="",[4527]="",[4528]="",[4529]="",[4530]="",[4621]="",[4622]="",[4623]="",[4624]="",[4625]="",[4626]="",[4627]="",[4628]="",[4629]="",[4630]="",[4721]="",[4722]="",[4723]="",[4724]="",[4725]="",[4726]="",[4727]="",[4728]="",[4729]="",[4730]="",[4821]="",[4822]="",[4823]="",[4824]="",[4825]="",[4826]="",[4827]="",[4828]="",[4829]="",[4830]="", },
	Azeroth = { [1741]="1",[1742]="1",[1743]="1",[1744]="1",[1745]="1",[1746]="1",[1841]="1",[1842]="1",[1843]="1",[1844]="1",[1845]="1",[1846]="1",[1941]="1",[1942]="1",[1943]="1",[1944]="1",[1945]="1",[1946]="1",[2039]="1",[2040]="1",[2041]="1",[2042]="1",[2043]="1",[2044]="1",[2045]="1",[2046]="1",[2139]="1",[2140]="1",[2141]="1",[2142]="1",[2143]="1",[2144]="1",[2145]="1",[2146]="1",[2239]="1",[2240]="1",[2241]="1",[2242]="1",[2243]="1",[2244]="1",[2245]="1",[2246]="1",[2339]="1",[2340]="1",[2341]="1",[2342]="1",[2343]="1",[2344]="1",[2345]="1",[2346]="1",[2439]="1",[2440]="1",[2441]="1",[2442]="1",[2443]="1",[2444]="1",[2445]="1",[2446]="1",[2539]="1",[2540]="1",[2541]="1",[2542]="1",[2543]="1",[2544]="1",[2545]="1",[2546]="1",[2639]="1",[2640]="1",[2641]="1",[2642]="1",[2643]="1",[2644]="1",[2645]="1",[2624]="",[2625]="",[2626]="",[2627]="",[2628]="",[2629]="",[2630]="",[2631]="",[2632]="",[2633]="",[2634]="",[2635]="",[2636]="",[2637]="",[2638]="",[2646]="",[2647]="",[2648]="",[2649]="",[2650]="",[2651]="",[2652]="",[2653]="",[2654]="",[2655]="",[2656]="",[2657]="",[2658]="",[2659]="",[2660]="",[2724]="",[2725]="",[2726]="",[2727]="",[2728]="",[2729]="",[2730]="",[2731]="",[2732]="",[2733]="",[2734]="",[2735]="",[2736]="",[2737]="",[2738]="",[2739]="",[2740]="",[2741]="",[2742]="",[2743]="",[2744]="",[2745]="",[2746]="",[2747]="",[2748]="",[2749]="",[2750]="",[2751]="",[2752]="",[2753]="",[2754]="",[2755]="",[2756]="",[2757]="",[2758]="",[2759]="",[2760]="",[2824]="",[2825]="",[2826]="",[2827]="",[2828]="",[2829]="",[2830]="",[2831]="",[2832]="",[2833]="",[2834]="",[2835]="",[2836]="",[2837]="",[2838]="",[2839]="",[2840]="",[2841]="",[2842]="",[2843]="",[2844]="",[2845]="",[2846]="",[2847]="",[2848]="",[2849]="",[2850]="",[2851]="",[2852]="",[2853]="",[2854]="",[2855]="",[2856]="",[2857]="",[2858]="",[2859]="",[2860]="",[2924]="",[2925]="",[2926]="",[2927]="",[2928]="",[2929]="",[2930]="",[2931]="",[2932]="",[2933]="",[2934]="",[2935]="",[2936]="",[2937]="",[2938]="",[2939]="",[2940]="",[2941]="",[2942]="",[2943]="",[2944]="",[2945]="",[2946]="",[2947]="",[2948]="",[2949]="",[2950]="",[2951]="",[2952]="",[2953]="",[2954]="",[2955]="",[2956]="",[2957]="",[2958]="",[2959]="",[2960]="",[3024]="",[3025]="",[3026]="",[3027]="",[3028]="",[3029]="",[3030]="",[3031]="",[3032]="",[3033]="",[3034]="",[3035]="",[3036]="",[3037]="",[3038]="",[3039]="",[3040]="",[3041]="",[3042]="",[3043]="",[3044]="",[3045]="",[3046]="",[3047]="",[3048]="",[3049]="",[3050]="",[3051]="",[3052]="",[3053]="",[3054]="",[3055]="",[3056]="",[3057]="",[3058]="",[3059]="",[3060]="",[3124]="",[3125]="",[3126]="",[3127]="",[3128]="",[3129]="",[3130]="",[3131]="",[3132]="",[3133]="",[3134]="",[3135]="",[3136]="",[3137]="",[3138]="",[3139]="",[3140]="",[3141]="",[3142]="",[3143]="",[3144]="",[3145]="",[3146]="",[3147]="",[3148]="",[3149]="",[3150]="",[3151]="",[3152]="",[3153]="",[3154]="",[3155]="",[3156]="",[3157]="",[3158]="",[3159]="",[3160]="",[3224]="",[3225]="",[3226]="",[3227]="",[3228]="",[3229]="",[3230]="",[3231]="",[3232]="",[3233]="",[3234]="",[3235]="",[3236]="",[3237]="",[3238]="",[3239]="",[3240]="",[3241]="",[3242]="",[3243]="",[3244]="",[3245]="",[3246]="",[3247]="",[3248]="",[3249]="",[3250]="",[3251]="",[3252]="",[3253]="",[3254]="",[3255]="",[3256]="",[3257]="",[3258]="",[3259]="",[3260]="",[3324]="",[3325]="",[3326]="",[3327]="",[3328]="",[3329]="",[3330]="",[3331]="",[3332]="",[3333]="",[3334]="",[3335]="",[3336]="",[3337]="",[3338]="",[3339]="",[3340]="",[3341]="",[3342]="",[3343]="",[3344]="",[3345]="",[3346]="",[3347]="",[3348]="",[3349]="",[3350]="",[3351]="",[3352]="",[3353]="",[3354]="",[3355]="",[3356]="",[3357]="",[3358]="",[3359]="",[3360]="",[3424]="",[3425]="",[3426]="",[3427]="",[3428]="",[3429]="",[3430]="",[3431]="",[3432]="",[3433]="",[3434]="",[3435]="",[3436]="",[3437]="",[3438]="",[3439]="",[3440]="",[3441]="",[3442]="",[3443]="",[3444]="",[3445]="",[3446]="",[3447]="",[3448]="",[3449]="",[3450]="",[3451]="",[3452]="",[3453]="",[3454]="",[3455]="",[3456]="",[3457]="",[3458]="",[3459]="",[3460]="",[3524]="",[3525]="",[3526]="",[3527]="",[3528]="",[3529]="",[3530]="",[3531]="",[3532]="",[3533]="",[3534]="",[3535]="",[3536]="",[3537]="",[3538]="",[3539]="",[3540]="",[3541]="",[3542]="",[3543]="",[3544]="",[3545]="",[3546]="",[3547]="",[3548]="",[3549]="",[3550]="",[3551]="",[3552]="",[3553]="",[3554]="",[3555]="",[3556]="",[3557]="",[3558]="",[3559]="",[3560]="",[3623]="",[3624]="",[3625]="",[3626]="",[3627]="",[3628]="",[3629]="",[3630]="",[3631]="",[3632]="",[3633]="",[3634]="",[3635]="",[3636]="",[3637]="",[3638]="",[3639]="",[3640]="",[3641]="",[3642]="",[3643]="",[3644]="",[3645]="",[3646]="",[3647]="",[3648]="",[3649]="",[3650]="",[3651]="",[3652]="",[3653]="",[3654]="",[3655]="",[3656]="",[3657]="",[3658]="",[3659]="",[3660]="",[3723]="",[3724]="",[3725]="",[3726]="",[3727]="",[3728]="",[3729]="",[3730]="",[3731]="",[3732]="",[3733]="",[3734]="",[3735]="",[3736]="",[3737]="",[3738]="",[3739]="",[3740]="",[3741]="",[3742]="",[3743]="",[3744]="",[3745]="",[3746]="",[3747]="",[3748]="",[3749]="",[3750]="",[3751]="",[3752]="",[3753]="",[3754]="",[3755]="",[3756]="",[3757]="",[3758]="",[3759]="",[3760]="",[3823]="",[3824]="",[3825]="",[3826]="",[3827]="",[3828]="",[3829]="",[3830]="",[3831]="",[3832]="",[3833]="",[3834]="",[3835]="",[3836]="",[3837]="",[3838]="",[3839]="",[3840]="",[3841]="",[3842]="",[3843]="",[3844]="",[3845]="",[3846]="",[3847]="",[3848]="",[3849]="",[3850]="",[3851]="",[3852]="",[3853]="",[3854]="",[3855]="",[3856]="",[3857]="",[3858]="",[3859]="",[3860]="",[3923]="",[3924]="",[3925]="",[3926]="",[3927]="",[3928]="",[3929]="",[3930]="",[3931]="",[3932]="",[3933]="",[3934]="",[3935]="",[3936]="",[3937]="",[3938]="",[3939]="",[3940]="",[3941]="",[3942]="",[3943]="",[3944]="",[3945]="",[3946]="",[3947]="",[3948]="",[3949]="",[3950]="",[3951]="",[3952]="",[3953]="",[3954]="",[3955]="",[3956]="",[3957]="",[3958]="",[3959]="",[3960]="",[4023]="",[4024]="",[4025]="",[4026]="",[4027]="",[4028]="",[4029]="",[4030]="",[4031]="",[4032]="",[4033]="",[4034]="",[4035]="",[4036]="",[4037]="",[4038]="",[4039]="",[4040]="",[4041]="",[4042]="",[4043]="",[4044]="",[4045]="",[4046]="",[4047]="",[4048]="",[4049]="",[4050]="",[4051]="",[4052]="",[4053]="",[4054]="",[4055]="",[4056]="",[4057]="",[4058]="",[4059]="",[4060]="",[4123]="",[4124]="",[4125]="",[4126]="",[4127]="",[4128]="",[4129]="",[4130]="",[4131]="",[4132]="",[4133]="",[4134]="",[4135]="",[4136]="",[4137]="",[4138]="",[4139]="",[4140]="",[4141]="",[4142]="",[4143]="",[4144]="",[4145]="",[4146]="",[4147]="",[4148]="",[4149]="",[4150]="",[4151]="",[4152]="",[4153]="",[4154]="",[4155]="",[4156]="",[4157]="",[4158]="",[4159]="",[4160]="",[4224]="",[4225]="",[4226]="",[4227]="",[4228]="",[4229]="",[4230]="",[4231]="",[4232]="",[4233]="",[4234]="",[4235]="",[4236]="",[4237]="",[4238]="",[4239]="",[4240]="",[4241]="",[4242]="",[4243]="",[4244]="",[4245]="",[4246]="",[4247]="",[4248]="",[4249]="",[4250]="",[4251]="",[4252]="",[4253]="",[4254]="",[4255]="",[4256]="",[4257]="",[4258]="",[4259]="",[4260]="",[4325]="",[4326]="",[4327]="",[4328]="",[4329]="",[4330]="",[4425]="",[4426]="",[4427]="",[4428]="",[4429]="",[4430]="",[4336]="",[4337]="",[4338]="",[4339]="",[4340]="",[4341]="",[4342]="",[4343]="",[4436]="",[4437]="",[4438]="",[4439]="",[4440]="",[4441]="",[4442]="",[4536]="",[4537]="",[4538]="",[4539]="",[4540]="",[4541]="",[4542]="", },
	Expansion01 = { [1021]="",[1022]="",[1023]="",[1024]="",[1025]="",[1026]="",[1027]="",[1028]="",[1029]="",[1030]="",[1031]="",[1032]="",[1033]="",[1034]="",[1035]="",[1036]="",[1037]="",[1038]="",[1039]="",[1040]="",[1041]="",[1042]="",[1121]="",[1122]="",[1123]="",[1124]="",[1125]="",[1126]="",[1127]="",[1128]="",[1129]="",[1130]="",[1131]="",[1132]="",[1133]="",[1134]="",[1135]="",[1136]="",[1137]="",[1138]="",[1139]="",[1140]="",[1141]="",[1142]="",[1221]="",[1222]="",[1223]="",[1224]="",[1225]="",[1226]="",[1227]="",[1228]="",[1229]="",[1230]="",[1231]="",[1232]="",[1233]="",[1234]="",[1235]="",[1236]="",[1237]="",[1238]="",[1239]="",[1240]="",[1241]="",[1242]="",[1321]="",[1322]="",[1323]="",[1324]="",[1325]="",[1326]="",[1327]="",[1328]="",[1329]="",[1330]="",[1331]="",[1332]="",[1333]="",[1334]="",[1335]="",[1336]="",[1337]="",[1338]="",[1339]="",[1340]="",[1341]="",[1342]="",[1421]="",[1422]="",[1423]="",[1424]="",[1425]="",[1426]="",[1427]="",[1428]="",[1429]="",[1430]="",[1431]="",[1432]="",[1433]="",[1434]="",[1435]="",[1436]="",[1437]="",[1438]="",[1439]="",[1440]="",[1441]="",[1442]="",[1521]="",[1522]="",[1523]="",[1524]="",[1525]="",[1526]="",[1527]="",[1528]="",[1529]="",[1530]="",[1531]="",[1532]="",[1533]="",[1534]="",[1535]="",[1536]="",[1537]="",[1538]="",[1539]="",[1540]="",[1541]="",[1542]="",[1621]="",[1622]="",[1623]="",[1624]="",[1625]="",[1626]="",[1627]="",[1628]="",[1629]="",[1630]="",[1631]="",[1632]="",[1633]="",[1634]="",[1635]="",[1636]="",[1637]="",[1638]="",[1639]="",[1640]="",[1641]="",[1642]="",[1721]="",[1722]="",[1723]="",[1724]="",[1725]="",[1726]="",[1727]="",[1728]="",[1729]="",[1730]="",[1731]="",[1732]="",[1733]="",[1734]="",[1735]="",[1736]="",[1737]="",[1738]="",[1739]="",[1740]="",[1741]="",[1742]="",[1821]="",[1822]="",[1823]="",[1824]="",[1825]="",[1826]="",[1827]="",[1828]="",[1829]="",[1830]="",[1831]="",[1832]="",[1833]="",[1834]="",[1835]="",[1836]="",[1837]="",[1838]="",[1839]="",[1840]="",[1841]="",[1842]="",[1921]="",[1922]="",[1923]="",[1924]="",[1925]="",[1926]="",[1927]="",[1928]="",[1929]="",[1930]="",[1931]="",[1932]="",[1933]="",[1934]="",[1935]="",[1936]="",[1937]="",[1938]="",[1939]="",[1940]="",[1941]="",[1942]="",[2021]="",[2022]="",[2023]="",[2024]="",[2025]="",[2026]="",[2027]="",[2028]="",[2029]="",[2030]="",[2031]="",[2032]="",[2033]="",[2034]="",[2035]="",[2036]="",[2037]="",[2038]="",[2039]="",[2040]="",[2041]="",[2042]="",[2121]="",[2122]="",[2123]="",[2124]="",[2125]="",[2126]="",[2127]="",[2128]="",[2129]="",[2130]="",[2131]="",[2132]="",[2133]="",[2134]="",[2135]="",[2136]="",[2137]="",[2138]="",[2139]="",[2140]="",[2141]="",[2142]="",[2221]="",[2222]="",[2223]="",[2224]="",[2225]="",[2226]="",[2227]="",[2228]="",[2229]="",[2230]="",[2231]="",[2232]="",[2233]="",[2234]="",[2235]="",[2236]="",[2237]="",[2238]="",[2239]="",[2240]="",[2241]="",[2242]="",[2321]="",[2322]="",[2323]="",[2324]="",[2325]="",[2326]="",[2327]="",[2328]="",[2329]="",[2330]="",[2331]="",[2332]="",[2333]="",[2334]="",[2335]="",[2336]="",[2337]="",[2338]="",[2339]="",[2340]="",[2341]="",[2342]="",[2421]="",[2422]="",[2423]="",[2424]="",[2425]="",[2426]="",[2427]="",[2428]="",[2429]="",[2430]="",[2431]="",[2432]="",[2433]="",[2434]="",[2435]="",[2436]="",[2437]="",[2438]="",[2439]="",[2440]="",[2441]="",[2442]="",[2521]="",[2522]="",[2523]="",[2524]="",[2525]="",[2526]="",[2527]="",[2528]="",[2529]="",[2530]="",[2531]="",[2532]="",[2533]="",[2534]="",[2535]="",[2536]="",[2537]="",[2538]="",[2539]="",[2540]="",[2541]="",[2542]="",[2621]="",[2622]="",[2623]="",[2624]="",[2625]="",[2626]="",[2627]="",[2628]="",[2629]="",[2630]="",[2631]="",[2632]="",[2633]="",[2634]="",[2635]="",[2636]="",[2637]="",[2638]="",[2639]="",[2640]="",[2641]="",[2642]="",[2721]="",[2722]="",[2723]="",[2724]="",[2725]="",[2726]="",[2727]="",[2728]="",[2729]="",[2730]="",[2731]="",[2732]="",[2733]="",[2734]="",[2735]="",[2736]="",[2737]="",[2738]="",[2739]="",[2740]="",[2741]="",[2742]="",[2821]="",[2822]="",[2823]="",[2824]="",[2825]="",[2826]="",[2827]="",[2828]="",[2829]="",[2830]="",[2831]="",[2832]="",[2833]="",[2834]="",[2835]="",[2836]="",[2837]="",[2838]="",[2839]="",[2840]="",[2841]="",[2842]="",[2921]="",[2922]="",[2923]="",[2924]="",[2925]="",[2926]="",[2927]="",[2928]="",[2929]="",[2930]="",[2931]="",[2932]="",[2933]="",[2934]="",[2935]="",[2936]="",[2937]="",[2938]="",[2939]="",[2940]="",[2941]="",[2942]="",[3021]="",[3022]="",[3023]="",[3024]="",[3025]="",[3026]="",[3027]="",[3028]="",[3029]="",[3030]="",[3031]="",[3032]="",[3033]="",[3034]="",[3035]="",[3036]="",[3037]="",[3038]="",[3039]="",[3040]="",[3041]="",[3042]="",[3121]="",[3122]="",[3123]="",[3124]="",[3125]="",[3126]="",[3127]="",[3128]="",[3129]="",[3130]="",[3131]="",[3132]="",[3133]="",[3134]="",[3135]="",[3136]="",[3137]="",[3138]="",[3139]="",[3140]="",[3141]="",[3142]="",[3221]="",[3222]="",[3223]="",[3224]="",[3225]="",[3226]="",[3227]="",[3228]="",[3229]="",[3230]="",[3231]="",[3232]="",[3233]="",[3234]="",[3235]="",[3236]="",[3237]="",[3238]="",[3239]="",[3240]="",[3241]="",[3242]="",[3321]="",[3322]="",[3323]="",[3324]="",[3325]="",[3326]="",[3327]="",[3328]="",[3329]="",[3330]="",[3331]="",[3332]="",[3333]="",[3334]="",[3335]="",[3336]="",[3337]="",[3338]="",[3339]="",[3340]="",[3341]="",[3342]="",[3421]="",[3422]="",[3423]="",[3424]="",[3425]="",[3426]="",[3427]="",[3428]="",[3429]="",[3430]="",[3431]="",[3432]="",[3433]="",[3434]="",[3435]="",[3436]="",[3437]="",[3438]="",[3439]="",[3440]="",[3441]="",[3442]="",[3521]="",[3522]="",[3523]="",[3524]="",[3525]="",[3526]="",[3527]="",[3528]="",[3529]="",[3530]="",[3531]="",[3532]="",[3533]="",[3534]="",[3535]="",[3536]="",[3537]="",[3538]="",[3539]="",[3540]="",[3541]="",[3542]="",[3621]="",[3622]="",[3623]="",[3624]="",[3625]="",[3626]="",[3627]="",[3628]="",[3629]="",[3630]="",[3631]="",[3632]="",[3633]="",[3634]="",[3635]="",[3636]="",[3637]="",[3638]="",[3639]="",[3640]="",[3641]="",[3642]="",[3721]="",[3722]="",[3723]="",[3724]="",[3725]="",[3726]="",[3727]="",[3728]="",[3729]="",[3730]="",[3731]="",[3732]="",[3733]="",[3734]="",[3735]="",[3736]="",[3737]="",[3738]="",[3739]="",[3740]="",[3741]="",[3742]="",[3821]="",[3822]="",[3823]="",[3824]="",[3825]="",[3826]="",[3827]="",[3828]="",[3829]="",[3830]="",[3831]="",[3832]="",[3833]="",[3834]="",[3835]="",[3836]="",[3837]="",[3838]="",[3839]="",[3840]="",[3841]="",[3842]="", },
	Northrend = { [1409]="",[1410]="",[1411]="",[1412]="",[1413]="",[1414]="",[1415]="",[1416]="",[1417]="",[1418]="",[1419]="",[1420]="",[1421]="",[1422]="",[1423]="",[1424]="",[1425]="",[1426]="",[1427]="",[1428]="",[1429]="",[1430]="",[1431]="",[1432]="",[1433]="",[1434]="",[1509]="",[1510]="",[1511]="",[1512]="",[1513]="",[1514]="",[1515]="",[1516]="",[1517]="",[1518]="",[1519]="",[1520]="",[1521]="",[1522]="",[1523]="",[1524]="",[1525]="",[1526]="",[1527]="",[1528]="",[1529]="",[1530]="",[1531]="",[1532]="",[1533]="",[1534]="",[1609]="",[1610]="",[1611]="",[1612]="",[1613]="",[1614]="",[1615]="",[1616]="",[1617]="",[1618]="",[1619]="",[1620]="",[1621]="",[1622]="",[1623]="",[1624]="",[1625]="",[1626]="",[1627]="",[1628]="",[1629]="",[1630]="",[1631]="",[1632]="",[1633]="",[1634]="",[1709]="",[1710]="",[1711]="",[1712]="",[1713]="",[1714]="",[1715]="",[1716]="",[1717]="",[1718]="",[1719]="",[1720]="",[1721]="",[1722]="",[1723]="",[1724]="",[1725]="",[1726]="",[1727]="",[1728]="",[1729]="",[1730]="",[1731]="",[1732]="",[1733]="",[1734]="",[1735]="",[1736]="",[1737]="",[1809]="",[1810]="",[1811]="",[1812]="",[1813]="",[1814]="",[1815]="",[1816]="",[1817]="",[1818]="",[1819]="",[1820]="",[1821]="",[1822]="",[1823]="",[1824]="",[1825]="",[1826]="",[1827]="",[1828]="",[1829]="",[1830]="",[1831]="",[1832]="",[1833]="",[1834]="",[1835]="",[1836]="",[1837]="",[1909]="",[1910]="",[1911]="",[1912]="",[1913]="",[1914]="",[1915]="",[1916]="",[1917]="",[1918]="",[1919]="",[1920]="",[1921]="",[1922]="",[1923]="",[1924]="",[1925]="",[1926]="",[1927]="",[1928]="",[1929]="",[1930]="",[1931]="",[1932]="",[1933]="",[1934]="",[1935]="",[1936]="",[1937]="",[2009]="",[2010]="",[2011]="",[2012]="",[2013]="",[2014]="",[2015]="",[2016]="",[2017]="",[2018]="",[2019]="",[2020]="",[2021]="",[2022]="",[2023]="",[2024]="",[2025]="",[2026]="",[2027]="",[2028]="",[2029]="",[2030]="",[2031]="",[2032]="",[2033]="",[2034]="",[2035]="",[2036]="",[2037]="",[2109]="",[2110]="",[2111]="",[2112]="",[2113]="",[2114]="",[2115]="",[2116]="",[2117]="",[2118]="",[2119]="",[2120]="",[2121]="",[2122]="",[2123]="",[2124]="",[2125]="",[2126]="",[2127]="",[2128]="",[2129]="",[2130]="",[2131]="",[2132]="",[2133]="",[2134]="",[2135]="",[2136]="",[2137]="",[2209]="",[2210]="",[2211]="",[2212]="",[2213]="",[2214]="",[2215]="",[2216]="",[2217]="",[2218]="",[2219]="",[2220]="",[2221]="",[2222]="",[2223]="",[2224]="",[2225]="",[2226]="",[2227]="",[2228]="",[2229]="",[2230]="",[2231]="",[2232]="",[2233]="",[2234]="",[2235]="",[2236]="",[2237]="",[2309]="",[2310]="",[2311]="",[2312]="",[2313]="",[2314]="",[2315]="",[2316]="",[2317]="",[2318]="",[2319]="",[2320]="",[2321]="",[2322]="",[2323]="",[2324]="",[2325]="",[2326]="",[2327]="",[2328]="",[2329]="",[2330]="",[2331]="",[2332]="",[2333]="",[2334]="",[2335]="",[2336]="",[2337]="",[2409]="",[2410]="",[2411]="",[2412]="",[2413]="",[2414]="",[2415]="",[2416]="",[2417]="",[2418]="",[2419]="",[2420]="",[2421]="",[2422]="",[2423]="",[2424]="",[2425]="",[2426]="",[2427]="",[2428]="",[2429]="",[2430]="",[2431]="",[2432]="",[2433]="",[2434]="",[2435]="",[2436]="",[2437]="",[2509]="",[2510]="",[2511]="",[2512]="",[2513]="",[2514]="",[2515]="",[2516]="",[2517]="",[2518]="",[2519]="",[2520]="",[2521]="",[2522]="",[2523]="",[2524]="",[2525]="",[2526]="",[2527]="",[2528]="",[2529]="",[2530]="",[2531]="",[2532]="",[2533]="",[2534]="",[2535]="",[2536]="",[2537]="",[2609]="",[2610]="",[2611]="",[2612]="",[2613]="",[2614]="",[2615]="",[2616]="",[2617]="",[2618]="",[2619]="",[2620]="",[2621]="",[2622]="",[2623]="",[2624]="",[2625]="",[2626]="",[2627]="",[2628]="",[2629]="",[2630]="",[2631]="",[2632]="",[2633]="",[2634]="",[2635]="",[2636]="",[2637]="",[2709]="",[2710]="",[2711]="",[2712]="",[2713]="",[2714]="",[2715]="",[2716]="",[2717]="",[2718]="",[2719]="",[2720]="",[2721]="",[2722]="",[2723]="",[2724]="",[2725]="",[2726]="",[2727]="",[2728]="",[2729]="",[2730]="",[2731]="",[2732]="",[2733]="",[2734]="",[2735]="",[2736]="",[2737]="",[2809]="",[2810]="",[2811]="",[2812]="",[2813]="",[2814]="",[2815]="",[2816]="",[2817]="",[2818]="",[2819]="",[2820]="",[2821]="",[2822]="",[2823]="",[2824]="",[2825]="",[2826]="",[2827]="",[2828]="",[2829]="",[2830]="",[2831]="",[2832]="",[2833]="",[2834]="",[2835]="",[2836]="",[2837]="",[2909]="",[2910]="",[2911]="",[2912]="",[2913]="",[2914]="",[2915]="",[2916]="",[2917]="",[2918]="",[2919]="",[2920]="",[2921]="",[2922]="",[2923]="",[2924]="",[2925]="",[2926]="",[2927]="",[2928]="",[2929]="",[2930]="",[2931]="",[2932]="",[2933]="",[2934]="",[2935]="",[2936]="",[2937]="",[3009]="",[3010]="",[3011]="",[3012]="",[3013]="",[3014]="",[3015]="",[3016]="",[3017]="",[3018]="",[3019]="",[3020]="",[3021]="",[3022]="",[3023]="",[3024]="",[3025]="",[3026]="",[3027]="",[3028]="",[3029]="",[3030]="",[3031]="",[3032]="",[3033]="",[3034]="",[3035]="",[3036]="",[3037]="",[3109]="",[3110]="",[3111]="",[3112]="",[3113]="",[3114]="",[3115]="",[3116]="",[3117]="",[3118]="",[3119]="",[3120]="",[3121]="",[3122]="",[3123]="",[3124]="",[3125]="",[3126]="",[3127]="",[3128]="",[3129]="",[3130]="",[3131]="",[3132]="",[3133]="",[3134]="",[3135]="",[3136]="",[3137]="",[3209]="",[3210]="",[3211]="",[3212]="",[3213]="",[3214]="",[3215]="",[3216]="",[3217]="",[3218]="",[3219]="",[3220]="",[3221]="",[3222]="",[3223]="",[3224]="",[3225]="",[3226]="",[3227]="",[3228]="",[3229]="",[3230]="",[3231]="",[3232]="",[3233]="",[3234]="",[3235]="",[3236]="",[3237]="",[3309]="",[3310]="",[3311]="",[3312]="",[3313]="",[3314]="",[3315]="",[3316]="",[3317]="",[3318]="",[3319]="",[3320]="",[3321]="",[3322]="",[3323]="",[3324]="",[3325]="",[3326]="",[3327]="",[3328]="",[3329]="",[3330]="",[3331]="",[3332]="",[3333]="",[3334]="",[3335]="",[3336]="",[3337]="",[3409]="",[3410]="",[3411]="",[3412]="",[3413]="",[3414]="",[3415]="",[3416]="",[3417]="",[3418]="",[3419]="",[3420]="",[3421]="",[3422]="",[3423]="",[3424]="",[3425]="",[3426]="",[3427]="",[3428]="",[3429]="",[3430]="",[3431]="",[3432]="",[3433]="",[3434]="",[3435]="",[3436]="",[3437]="",[3509]="",[3510]="",[3511]="",[3512]="",[3513]="",[3514]="",[3515]="",[3516]="",[3517]="",[3518]="",[3519]="",[3520]="",[3521]="",[3522]="",[3523]="",[3524]="",[3525]="",[3526]="",[3527]="",[3528]="",[3529]="",[3530]="",[3531]="",[3532]="",[3533]="",[3534]="",[3535]="",[3536]="",[3537]="",[3609]="",[3610]="",[3611]="",[3612]="",[3613]="",[3614]="",[3615]="",[3616]="",[3617]="",[3618]="",[3619]="",[3620]="",[3621]="",[3622]="",[3623]="",[3624]="",[3625]="",[3626]="",[3627]="",[3628]="",[3629]="",[3630]="",[3631]="",[3632]="",[3633]="",[3634]="",[3635]="",[3636]="",[3637]="",[3709]="",[3710]="",[3711]="",[3712]="",[3713]="",[3714]="",[3715]="",[3716]="",[3717]="",[3718]="",[3719]="",[3720]="",[3721]="",[3722]="",[3723]="",[3724]="",[3725]="",[3726]="",[3727]="",[3728]="",[3729]="",[3730]="",[3731]="",[3732]="",[3733]="",[3734]="",[3735]="",[3736]="",[3737]="",[3809]="",[3810]="",[3811]="",[3812]="",[3813]="",[3814]="",[3815]="",[3816]="",[3817]="",[3818]="",[3819]="",[3820]="",[3821]="",[3822]="",[3823]="",[3824]="",[3825]="",[3826]="",[3827]="",[3828]="",[3829]="",[3830]="",[3831]="",[3832]="",[3833]="",[3834]="",[3835]="",[3836]="",[3837]="",[3909]="",[3910]="",[3911]="",[3912]="",[3913]="",[3914]="",[3915]="",[3916]="",[3917]="",[3918]="",[3919]="",[3920]="",[3921]="",[3922]="",[3923]="",[3924]="",[3925]="",[3926]="",[3927]="",[3928]="",[3929]="",[3930]="",[3931]="",[3932]="",[3933]="",[3934]="",[3935]="",[3936]="",[3937]="",[4009]="",[4010]="",[4011]="",[4012]="",[4013]="",[4014]="",[4015]="",[4016]="",[4017]="",[4018]="",[4019]="",[4020]="",[4021]="",[4022]="",[4023]="",[4024]="",[4025]="",[4026]="",[4027]="",[4028]="",[4029]="",[4030]="",[4031]="",[4032]="",[4033]="",[4034]="",[4035]="",[4036]="",[4037]="",[4109]="",[4110]="",[4111]="",[4112]="",[4113]="",[4114]="",[4115]="",[4116]="",[4117]="",[4118]="",[4119]="",[4120]="",[4121]="",[4122]="",[4123]="",[4124]="",[4125]="",[4126]="",[4127]="",[4128]="",[4129]="",[4130]="",[4131]="",[4132]="",[4133]="",[4134]="",[4135]="",[4136]="",[4137]="",[4209]="",[4210]="",[4211]="",[4212]="",[4213]="",[4214]="",[4215]="",[4216]="",[4217]="",[4218]="",[4219]="",[4220]="",[4221]="",[4222]="",[4223]="",[4224]="",[4225]="",[4226]="",[4227]="",[4228]="",[4229]="",[4230]="",[4231]="",[4232]="",[4233]="",[4234]="",[4235]="",[4236]="",[4237]="",[4309]="",[4310]="",[4311]="",[4312]="",[4313]="",[4314]="",[4315]="",[4316]="",[4317]="",[4318]="",[4319]="",[4320]="",[4321]="",[4322]="",[4323]="",[4324]="",[4325]="",[4326]="",[4327]="",[4328]="",[4329]="",[4330]="",[4331]="",[4332]="",[4333]="",[4334]="",[4335]="",[4336]="",[4337]="",[4409]="",[4410]="",[4411]="",[4412]="",[4413]="",[4414]="",[4415]="",[4416]="",[4417]="",[4418]="",[4419]="",[4420]="",[4421]="",[4422]="",[4423]="",[4424]="",[4425]="",[4426]="",[4427]="",[4428]="",[4429]="",[4430]="",[4431]="",[4432]="",[4433]="",[4434]="",[4435]="",[4436]="",[4437]="",[4509]="",[4510]="",[4511]="",[4512]="",[4513]="",[4514]="",[4515]="",[4516]="",[4517]="",[4518]="",[4519]="",[4520]="",[4521]="",[4522]="",[4523]="",[4524]="",[4525]="",[4526]="",[4527]="",[4528]="",[4529]="",[4530]="",[4531]="",[4532]="",[4533]="",[4534]="",[4535]="",[4536]="",[4537]="",[4609]="",[4610]="",[4611]="",[4612]="",[4613]="",[4614]="",[4615]="",[4616]="",[4617]="",[4618]="",[4619]="",[4620]="",[4621]="",[4622]="",[4623]="",[4624]="",[4625]="",[4626]="",[4627]="",[4628]="",[4629]="",[4630]="",[4631]="",[4632]="",[4633]="",[4634]="",[4635]="",[4636]="",[4637]="",[4709]="",[4710]="",[4711]="",[4712]="",[4713]="",[4714]="",[4715]="",[4716]="",[4717]="",[4718]="",[4719]="",[4720]="",[4721]="",[4722]="",[4723]="",[4724]="",[4725]="",[4726]="",[4727]="",[4728]="",[4729]="",[4730]="",[4731]="",[4732]="",[4733]="",[4734]="",[4735]="",[4736]="",[4737]="", },
	QuelThalas = { [4006]="",[4007]="",[4008]="",[4009]="",[4010]="",[4011]="",[4012]="",[4013]="",[4014]="",[4015]="",[4016]="",[4017]="",[4018]="",[4019]="",[4106]="",[4107]="",[4108]="",[4109]="",[4110]="",[4111]="",[4112]="",[4113]="",[4114]="",[4115]="",[4116]="",[4117]="",[4118]="",[4119]="",[4120]="",[4206]="",[4207]="",[4208]="",[4209]="",[4210]="",[4211]="",[4212]="",[4213]="",[4214]="",[4215]="",[4216]="",[4217]="",[4218]="",[4219]="",[4220]="",[4306]="",[4307]="",[4308]="",[4309]="",[4310]="",[4311]="",[4312]="",[4313]="",[4314]="",[4315]="",[4316]="",[4317]="",[4318]="",[4319]="",[4320]="",[4406]="",[4407]="",[4408]="",[4409]="",[4410]="",[4411]="",[4412]="",[4413]="",[4414]="",[4415]="",[4416]="",[4417]="",[4418]="",[4419]="",[4420]="",[4506]="",[4507]="",[4508]="",[4509]="",[4510]="",[4511]="",[4512]="",[4513]="",[4514]="",[4515]="",[4516]="",[4517]="",[4518]="",[4519]="",[4520]="",[4606]="",[4607]="",[4608]="",[4609]="",[4610]="",[4611]="",[4612]="",[4613]="",[4614]="",[4615]="",[4616]="",[4617]="",[4618]="",[4619]="",[4620]="",[4706]="",[4707]="",[4708]="",[4709]="",[4710]="",[4711]="",[4712]="",[4713]="",[4714]="",[4715]="",[4716]="",[4717]="",[4718]="",[4719]="",[4720]="",[4806]="",[4807]="",[4808]="",[4809]="",[4810]="",[4811]="",[4812]="",[4813]="",[4814]="",[4815]="",[4816]="",[4817]="",[4818]="",[4819]="",[4820]="",[4906]="",[4907]="",[4908]="",[4909]="",[4910]="",[4911]="",[4912]="",[4913]="",[4914]="",[4915]="",[4916]="",[4917]="",[4918]="",[4919]="",[4920]="",[5017]="",[5018]="",[5019]="",[5020]="", },
	Azuremyst = { [5032]="",[5033]="",[5034]="",[5035]="",[5036]="",[5037]="",[5038]="",[5039]="",[5040]="",[5041]="",[5042]="",[5132]="",[5133]="",[5134]="",[5135]="",[5136]="",[5137]="",[5138]="",[5139]="",[5140]="",[5141]="",[5142]="",[5232]="",[5233]="",[5234]="",[5235]="",[5236]="",[5237]="",[5238]="",[5239]="",[5240]="",[5241]="",[5242]="",[5332]="",[5333]="",[5334]="",[5335]="",[5336]="",[5337]="",[5338]="",[5339]="",[5340]="",[5341]="",[5342]="",[5432]="",[5433]="",[5434]="",[5435]="",[5436]="",[5437]="",[5438]="",[5439]="",[5440]="",[5441]="",[5442]="",[5532]="",[5533]="",[5534]="",[5535]="",[5536]="",[5537]="",[5538]="",[5539]="",[5540]="",[5541]="",[5542]="",[5632]="",[5633]="",[5634]="",[5635]="",[5636]="",[5637]="",[5638]="",[5639]="",[5640]="",[5641]="",[5642]="",[5733]="",[5734]="",[5735]="",[5736]="",[5737]="",[5738]="",[5739]="",[5740]="",[5741]="",[5742]="",[5835]="",[5836]="",[5837]="",[5838]="",[5839]="",[5840]="",[5841]="",[5842]="",[5936]="",[5937]="",[5938]="",[5939]="",[5940]="",[5941]="",[5942]="", },
	Dalaran = { [0000]="Dalaran0000",[0001]="Dalaran0001",[0002]="Dalaran0002",[0003]="Dalaran0003",[0004]="Dalaran0004",[0005]="Dalaran0005",[0100]="Dalaran0100",[0101]="Dalaran0101",[0102]="Dalaran0102",[0103]="Dalaran0103",[0104]="Dalaran0104",[0105]="Dalaran0105",[0200]="Dalaran0200",[0201]="Dalaran0201",[0202]="Dalaran0202",[0203]="Dalaran0203",[0204]="Dalaran0204",[0205]="Dalaran0205",[0300]="Dalaran0300",[0301]="Dalaran0301",[0302]="Dalaran0302",[0303]="Dalaran0303",[0304]="Dalaran0304",[0305]="Dalaran0305",[0400]="Dalaran0400",[0401]="Dalaran0401",[0402]="Dalaran0402",[0403]="Dalaran0403",[0404]="Dalaran0404",[0405]="Dalaran0405",[0500]="Dalaran0500",[0501]="Dalaran0501",[0502]="Dalaran0502",[0503]="Dalaran0503",[0504]="Dalaran0504",[0505]="Dalaran0505", },
	TheExodar = { [0000]="Exodar0000",[0001]="Exodar0001",[0002]="Exodar0002",[0003]="Exodar0003",[0004]="Exodar0004",[0005]="Exodar0005",[0100]="Exodar0100",[0101]="Exodar0101",[0102]="Exodar0102",[0103]="Exodar0103",[0104]="Exodar0104",[0105]="Exodar0105",[0200]="Exodar0200",[0201]="Exodar0201",[0202]="Exodar0202",[0203]="Exodar0203",[0204]="Exodar0204",[0205]="Exodar0205",[0300]="Exodar0300",[0301]="Exodar0301",[0302]="Exodar0302",[0303]="Exodar0303",[0304]="Exodar0304",[0305]="Exodar0305",[0400]="Exodar0400",[0401]="Exodar0401",[0402]="Exodar0402",[0403]="Exodar0403",[0404]="Exodar0404",[0405]="Exodar0405", },
	Ironforge = { [0000]="Ironforge0000",[0001]="Ironforge0001",[0002]="Ironforge0002",[0003]="Ironforge0003",[0004]="Ironforge0004",[0100]="Ironforge0100",[0101]="Ironforge0101",[0102]="Ironforge0102",[0103]="Ironforge0103",[0104]="Ironforge0104",[0200]="Ironforge0200",[0201]="Ironforge0201",[0202]="Ironforge0202",[0203]="Ironforge0203",[0204]="Ironforge0204",[0300]="Ironforge0300",[0301]="Ironforge0301",[0302]="Ironforge0302",[0303]="Ironforge0303",[0304]="Ironforge0304",[0400]="Ironforge0400",[0401]="Ironforge0401",[0402]="Ironforge0402",[0403]="Ironforge0403",[0404]="Ironforge0404", },
	Undercity = { [0000]="UC0000",[0001]="UC0001",[0002]="UC0002",[0003]="UC0003",[0004]="UC0004",[0100]="UC0100",[0101]="UC0101",[0102]="UC0102",[0103]="UC0103",[0104]="UC0104",[0200]="UC0200",[0201]="UC0201",[0202]="UC0202",[0203]="UC0203",[0204]="UC0204",[0300]="UC0300",[0301]="UC0301",[0302]="UC0302",[0303]="UC0303",[0304]="UC0304",[0400]="UC0400",[0401]="UC0401",[0402]="UC0402",[0403]="UC0403",[0404]="UC0404",[0500]="UC0500",[0501]="UC0501",[0502]="UC0502",[0503]="UC0503",[0504]="UC0504", },
	TheMaelstromContinent = {},
	TheLostIsles = { [2426]="",[2427]="",[2428]="",[2429]="",[2430]="",[2431]="",[2432]="",[2526]="",[2527]="",[2528]="",[2529]="",[2530]="",[2531]="",[2532]="",[2626]="",[2627]="",[2628]="",[2629]="",[2630]="",[2631]="",[2632]="",[2726]="",[2727]="",[2728]="",[2729]="",[2730]="",[2731]="",[2732]="",[2826]="",[2827]="",[2828]="",[2829]="",[2830]="",[2831]="",[2832]="",[2926]="",[2927]="",[2928]="",[2929]="",[2930]="",[2931]="",[2932]="",[3026]="",[3027]="",[3028]="",[3029]="",[3030]="",[3031]="",[3032]="",[3126]="",[3127]="",[3128]="",[3129]="",[3130]="",[3131]="", },
	Kezan = { [2648]="",[2649]="",[2650]="",[2651]="",[2746]="",[2747]="",[2748]="",[2749]="",[2750]="",[2751]="",[2846]="",[2847]="",[2848]="",[2849]="",[2850]="",[2851]="",[2946]="",[2947]="",[2948]="",[2949]="",[2950]="",[2951]="",[3046]="",[3047]="",[3048]="",[3049]="",[3050]="",[3051]="",[3148]="",[3149]="",[3150]="",[3151]="", },
	TheMaelstrom = { [2828]="",[2829]="",[2830]="",[2831]="",[2832]="",[2928]="",[2929]="",[2930]="",[2931]="",[2932]="",[3028]="",[3029]="",[3030]="",[3031]="",[3032]="",[3128]="",[3129]="",[3130]="",[3131]="",[3132]="",[3228]="",[3229]="",[3230]="",[3231]="",[3232]="", },
	Deepholm = { [2729]="",[2730]="",[2731]="",[2827]="",[2828]="",[2829]="",[2830]="",[2831]="",[2832]="",[2927]="",[2928]="",[2929]="",[2930]="",[2931]="",[2932]="",[3027]="",[3028]="",[3029]="",[3030]="",[3031]="",[3032]="",[3125]="",[3126]="",[3127]="",[3128]="",[3129]="",[3130]="",[3131]="",[3132]="",[3227]="",[3228]="",[3229]="",[3230]="",[3231]="",[3232]="",[3327]="",[3328]="",[3329]="",[3330]="",[3331]="",[3332]="",[3428]="",[3429]="",[3430]="", },
	TolBarad = { [2731]="",[2732]="",[2733]="",[2734]="",[2735]="",[2831]="",[2832]="",[2833]="",[2834]="",[2835]="",[2931]="",[2932]="",[2933]="",[2934]="",[2935]="",[3031]="",[3032]="",[3033]="",[3034]="",[3035]="",[3133]="",[3134]="",[3135]="", },
	Vashjir = { [1741]="1",[1742]="1",[1743]="1",[1744]="1",[1745]="1",[1746]="1",[1841]="1",[1842]="1",[1843]="1",[1844]="1",[1845]="1",[1846]="1",[1941]="1",[1942]="1",[1943]="1",[1944]="1",[1945]="1",[1946]="1",[2039]="1",[2040]="1",[2041]="1",[2042]="1",[2043]="1",[2044]="1",[2045]="1",[2046]="1",[2139]="1",[2140]="1",[2141]="1",[2142]="1",[2143]="1",[2144]="1",[2145]="1",[2146]="1",[2239]="1",[2240]="1",[2241]="1",[2242]="1",[2243]="1",[2244]="1",[2245]="1",[2246]="1",[2339]="1",[2340]="1",[2341]="1",[2342]="1",[2343]="1",[2344]="1",[2345]="1",[2346]="1",[2439]="1",[2440]="1",[2441]="1",[2442]="1",[2443]="1",[2444]="1",[2445]="1",[2446]="1",[2539]="1",[2540]="1",[2541]="1",[2542]="1",[2543]="1",[2544]="1",[2545]="1",[2546]="1",[2639]="1",[2640]="1",[2641]="1",[2642]="1",[2643]="1",[2644]="1",[2645]="1", },
	["Dalaran:2"] = { [0000]="Dalaran2_0000",[0001]="Dalaran2_0001",[0002]="Dalaran2_0002",[0100]="Dalaran2_0100",[0101]="Dalaran2_0101",[0102]="Dalaran2_0102",[0200]="Dalaran2_0200",[0201]="Dalaran2_0201",[0202]="Dalaran2_0202", },
	["Orgrimmar:2"] = { [0000]="Orgrimmar2_0000",[0001]="Orgrimmar2_0001",[0100]="Orgrimmar2_0100",[0101]="Orgrimmar2_0101",[0200]="Orgrimmar2_0200",[0201]="Orgrimmar2_0201", },
	MoltenFront = { [3028]="",[3029]="",[3030]="",[3031]="",[3128]="",[3129]="",[3130]="",[3131]="",[3228]="",[3229]="",[3230]="",[3231]="",[3328]="",[3329]="",[3330]="",[3331]="", },
	SilvermoonCity = { [0000]="Silvermoon0000",[0001]="Silvermoon0001",[0002]="Silvermoon0002",[0003]="Silvermoon0003",[0004]="Silvermoon0004",[0100]="Silvermoon0100",[0101]="Silvermoon0101",[0102]="Silvermoon0102",[0103]="Silvermoon0103",[0104]="Silvermoon0104",[0200]="Silvermoon0200",[0201]="Silvermoon0201",[0202]="Silvermoon0202",[0203]="Silvermoon0203",[0204]="Silvermoon0204",[0300]="Silvermoon0300",[0301]="Silvermoon0301",[0302]="Silvermoon0302",[0303]="Silvermoon0303",[0304]="Silvermoon0304",[0400]="Silvermoon0400",[0401]="Silvermoon0401",[0402]="Silvermoon0402",[0403]="Silvermoon0403",[0404]="Silvermoon0404", },
	TempestKeep = { [0000]="TempestKeep0000",[0001]="TempestKeep0001",[0002]="TempestKeep0002",[0100]="TempestKeep0100",[0101]="TempestKeep0101",[0102]="TempestKeep0102",[0200]="TempestKeep0200",[0201]="TempestKeep0201",[0202]="TempestKeep0202", },
	Firelands = { [3029]="",[3030]="",[3031]="",[3032]="",[3033]="",[3129]="",[3130]="",[3131]="",[3132]="",[3133]="",[3229]="",[3230]="",[3231]="",[3232]="",[3233]="",[3329]="",[3330]="",[3331]="",[3332]="",[3333]="", },
	BlackwingDescent = { [0000]="BlackwingDescent0000",[0001]="BlackwingDescent0001",[0002]="BlackwingDescent0002",[0003]="BlackwingDescent0003",[0100]="BlackwingDescent0100",[0101]="BlackwingDescent0101",[0102]="BlackwingDescent0102",[0103]="BlackwingDescent0103",[0200]="BlackwingDescent0200",[0201]="BlackwingDescent0201",[0202]="BlackwingDescent0202",[0203]="BlackwingDescent0203",[0300]="BlackwingDescent0300",[0301]="BlackwingDescent0301",[0302]="BlackwingDescent0302",[0303]="BlackwingDescent0303", },
	TheBastionofTwilight = { [0000]="GrimBatolRaid_0000",[0001]="GrimBatolRaid_0001",[0002]="GrimBatolRaid_0002",[0003]="GrimBatolRaid_0003",[0004]="GrimBatolRaid_0004",[0005]="GrimBatolRaid_0005",[0006]="GrimBatolRaid_0006",[0100]="GrimBatolRaid_0100",[0101]="GrimBatolRaid_0101",[0102]="GrimBatolRaid_0102",[0103]="GrimBatolRaid_0103",[0104]="GrimBatolRaid_0104",[0105]="GrimBatolRaid_0105",[0106]="GrimBatolRaid_0106",[0200]="GrimBatolRaid_0200",[0201]="GrimBatolRaid_0201",[0202]="GrimBatolRaid_0202",[0203]="GrimBatolRaid_0203",[0204]="GrimBatolRaid_0204",[0205]="GrimBatolRaid_0205",[0206]="GrimBatolRaid_0206",[0300]="GrimBatolRaid_0300",[0301]="GrimBatolRaid_0301",[0302]="GrimBatolRaid_0302",[0303]="GrimBatolRaid_0303",[0304]="GrimBatolRaid_0304",[0305]="GrimBatolRaid_0305",[0306]="GrimBatolRaid_0306",[0400]="GrimBatolRaid_0400",[0401]="GrimBatolRaid_0401",[0402]="GrimBatolRaid_0402",[0403]="GrimBatolRaid_0403",[0404]="GrimBatolRaid_0404",[0405]="GrimBatolRaid_0405",[0406]="GrimBatolRaid_0406", },
	["TheBastionofTwilight:3"] = { [0000]="GrimBatolRaid2_0000",[0001]="GrimBatolRaid2_0001",[0002]="GrimBatolRaid2_0002",[0100]="GrimBatolRaid2_0100",[0101]="GrimBatolRaid2_0101",[0102]="GrimBatolRaid2_0102",[0200]="GrimBatolRaid2_0200",[0201]="GrimBatolRaid2_0201",[0202]="GrimBatolRaid2_0202", },
	Skywall = { [2931]="",[2932]="",[2933]="",[2934]="",[2935]="",[3031]="",[3032]="",[3033]="",[3034]="",[3035]="",[3131]="",[3132]="",[3133]="",[3134]="",[3135]="",[3231]="",[3232]="",[3233]="",[3234]="",[3235]="",[3331]="",[3332]="",[3333]="",[3334]="",[3335]="", },
	ThroneoftheFourWinds = { [2931]="",[2932]="",[2933]="",[3031]="",[3032]="",[3033]="",[3131]="",[3132]="",[3133]="", },
	TheEyeofEternity = { [2829]="",[2830]="",[2831]="",[2929]="",[2930]="",[2931]="",[3029]="",[3030]="",[3031]="", },
	SunwellPlateau = { [2826]="",[2827]="",[2828]="",[2829]="",[2830]="",[2831]="",[2926]="",[2927]="",[2928]="",[2929]="",[2930]="",[2931]="",[3026]="",[3027]="",[3028]="",[3029]="",[3030]="",[3031]="",[3126]="",[3127]="",[3128]="",[3129]="",[3130]="",[3131]="",[3226]="",[3227]="",[3228]="",[3229]="",[3230]="",[3231]="", },
	["SunwellPlateau:2"] = { [0000]="SunwellPlateau2_0000",[0001]="SunwellPlateau2_0001",[0100]="SunwellPlateau2_0100",[0101]="SunwellPlateau2_0101", },
	MagistersTerrace = { [0000]="MagistersTerrace0000",[0001]="MagistersTerrace0001",[0100]="MagistersTerrace0100",[0101]="MagistersTerrace0101",[0200]="MagistersTerrace0200",[0201]="MagistersTerrace0201", },
	TheObsidianSanctum = { [2924]="",[2925]="",[2926]="",[2927]="",[3024]="",[3025]="",[3026]="",[3027]="",[3124]="",[3125]="",[3126]="",[3127]="",[3224]="",[3225]="",[3226]="",[3227]="", },
	TheRubySanctum = { [2924]="",[2925]="",[2926]="",[2927]="",[3024]="",[3025]="",[3026]="",[3027]="",[3124]="",[3125]="",[3126]="",[3127]="",[3224]="",[3225]="",[3226]="",[3227]="", },
	ZulGurub = { [3353]="",[3354]="",[3355]="",[3453]="",[3454]="",[3455]="",[3553]="",[3554]="",[3555]="",[3653]="",[3654]="",[3655]="",[3753]="",[3754]="",[3755]="", },
	ZulAman = { [2830]="",[2831]="",[2832]="",[2930]="",[2931]="",[2932]="",[3030]="",[3031]="",[3032]="", },
	HallsofOrigination = { [0000]="Uldum1_0000",[0001]="Uldum1_0001",[0002]="Uldum1_0002",[0100]="Uldum1_0100",[0101]="Uldum1_0101",[0102]="Uldum1_0102", },
	["HallsofOrigination:2"] = { [3031]="",[3032]="",[3033]="",[3034]="",[3131]="",[3132]="",[3133]="",[3134]="",[3231]="",[3232]="",[3233]="",[3234]="",[3331]="",[3332]="",[3333]="",[3334]="",[3431]="",[3432]="",[3433]="",[3434]="", },
	["HallsofOrigination:3"] = { [0000]="Uldum2_0000",[0001]="Uldum2_0001",[0002]="Uldum2_0002",[0100]="Uldum2_0100",[0101]="Uldum2_0101",[0102]="Uldum2_0102",[0200]="Uldum2_0200",[0201]="Uldum2_0201",[0202]="Uldum2_0202", },
	LostCityofTolvir = { [3351]="",[3352]="",[3353]="",[3451]="",[3452]="",[3453]="",[3551]="",[3552]="",[3553]="", },
	TheNexus = { [0000]="TheNexus0000",[0001]="TheNexus0001",[0002]="TheNexus0002",[0100]="TheNexus0100",[0101]="TheNexus0101",[0102]="TheNexus0102",[0200]="TheNexus0200",[0201]="TheNexus0201",[0202]="TheNexus0202", },
	DarkmoonFaireIsland = { [1736]="",[1737]="",[1738]="",[1739]="",[1740]="",[1741]="",[1742]="",[1836]="",[1837]="",[1838]="",[1839]="",[1840]="",[1841]="",[1842]="",[1936]="",[1937]="",[1938]="",[1939]="",[1940]="",[1941]="",[1942]="",[2036]="",[2037]="",[2038]="",[2039]="",[2040]="",[2041]="",[2042]="",[2136]="",[2137]="",[2138]="",[2139]="",[2140]="",[2141]="",[2142]="", },
	EndTime = { [2721]="",[2722]="",[2723]="",[2724]="",[2725]="",[2726]="",[2727]="",[2821]="",[2822]="",[2823]="",[2824]="",[2825]="",[2826]="",[2827]="",[2921]="",[2922]="",[2923]="",[2924]="",[2925]="",[2926]="",[2927]="",[3021]="",[3022]="",[3023]="",[3024]="",[3025]="",[3026]="",[3027]="",[3121]="",[3122]="",[3123]="",[3124]="",[3125]="",[3126]="",[3127]="",[3221]="",[3222]="",[3223]="",[3224]="",[3225]="",[3226]="",[3227]="",[3321]="",[3322]="",[3323]="",[3324]="",[3325]="",[3326]="",[3327]="", },
	WellofEternity = { [4023]="",[4024]="",[4025]="",[4026]="",[4027]="",[4123]="",[4124]="",[4125]="",[4126]="",[4127]="",[4223]="",[4224]="",[4225]="",[4226]="",[4227]="",[4323]="",[4324]="",[4325]="",[4326]="",[4327]="",[4423]="",[4424]="",[4425]="",[4426]="",[4427]="", },
	HourofTwilight = { [2822]="",[2823]="",[2824]="",[2825]="",[2826]="",[2827]="",[2828]="",[2922]="",[2923]="",[2924]="",[2925]="",[2926]="",[2927]="",[2928]="",[3022]="",[3023]="",[3024]="",[3025]="",[3026]="",[3027]="",[3028]="",[3122]="",[3123]="",[3124]="",[3125]="",[3126]="",[3127]="",[3128]="",[3222]="",[3223]="",[3224]="",[3225]="",[3226]="",[3227]="",[3228]="",[3322]="",[3323]="",[3324]="",[3325]="",[3326]="",[3327]="",[3328]="",[3422]="",[3423]="",[3424]="",[3425]="",[3426]="",[3427]="",[3428]="", },
	DragonSoul = {},
	IsleofConquest = { [3128]="",[3129]="",[3130]="",[3131]="",[3132]="",[3228]="",[3229]="",[3230]="",[3231]="",[3232]="",[3328]="",[3329]="",[3330]="",[3331]="",[3332]="",[3428]="",[3429]="",[3430]="",[3431]="",[3432]="",[3528]="",[3529]="",[3530]="",[3531]="",[3532]="", },
	StrandoftheAncients = { [3028]="",[3029]="",[3030]="",[3128]="",[3129]="",[3130]="",[3228]="",[3229]="",[3230]="",[3328]="",[3329]="",[3330]="", },
	ArathiBasin = { [2828]="",[2829]="",[2830]="",[2831]="",[2928]="",[2929]="",[2930]="",[2931]="",[3028]="",[3029]="",[3030]="",[3031]="",[3128]="",[3129]="",[3130]="",[3131]="", },
	WarsongGulch = { [2828]="",[2829]="",[2830]="",[2928]="",[2929]="",[2930]="",[3028]="",[3029]="",[3030]="", },
	AlteracValley = { [3129]="",[3130]="",[3131]="",[3132]="",[3133]="",[3134]="",[3135]="",[3229]="",[3230]="",[3231]="",[3232]="",[3233]="",[3234]="",[3235]="",[3329]="",[3330]="",[3331]="",[3332]="",[3333]="",[3334]="",[3335]="", },
	NetherstormArena = { [2726]="",[2727]="",[2728]="",[2729]="",[2826]="",[2827]="",[2828]="",[2829]="",[2926]="",[2927]="",[2928]="",[2929]="",[3026]="",[3027]="",[3028]="",[3029]="", },
	GilneasBattleground2 = { [2828]="",[2829]="",[2830]="",[2831]="",[2928]="",[2929]="",[2930]="",[2931]="",[3028]="",[3029]="",[3030]="",[3031]="",[3128]="",[3129]="",[3130]="",[3131]="", },
	TwinPeaks = { [3027]="",[3028]="",[3029]="",[3127]="",[3128]="",[3129]="",[3227]="",[3228]="",[3229]="", },
	Instances = {},
}

data.POI_DATA = {
	-- Kalimdor
	Orgrimmar = { [62004800]={4,"Orgrimmar"},[66203150]={1,"Valley of Spirits"},[45004600]={1,"Valley of Wisdom"},[41006900]={1,"Valley of Honor"},[73004870]={1,"Grommash Hold"},[56005800]={1,"The Drag"},[38005000]={1,"Portals"}, },
	ThunderBluff = { [46804680]={4,"Thunder Bluff"},[30007600]={1,"Elder Rise"},[22602600]={1,"Spirit Rise"},[82505850]={1,"Hunter Rise"}, },
	Darnassus = { [49804300]={4,"Darnassus"},[80704340]={1,"Temple of the Moon"},[34006100]={1,"Craftsmen's Terrace"},[70005900]={1,"Tradesmen's Terrace"},[50006580]={1,"Warrior's Terrace"},[32004200]={1,"Cenarion Enclave"}, },
	TheExodar = { [34005780]={4,"The Exodar"},[31003750]={1,"The Crystal Hall"},[65903750]={1,"The Vault of Lights"},[78005900]={1,"Trader's Tier"}, },
	Dalaran = { [45504970]={4,"Dalaran"},[48002700]={1,"The Violet Citadel"},[32006000]={1,"Sunreaver's Sanctuary"},[66003750]={1,"The Silver Enclave"},[68004900]={1,"The Eventide"},[51005000]={1,"Runeweaver Square"},[32004000]={1,"Magus Commerce Exchange"},[45607280]={1,"Krasus' Landing"},[68006600]={1,"The Violet Hold"}, },
	ShattrathCity = { [40005500]={4,"Shattrath City"},[77004700]={1,"Scryer's Tier"},[34003200]={1,"Aldor Rise"},[50005400]={1,"Terrace of Light"},[30007000]={1,"Lower City"}, },
	Ironforge = { [68503000]={1,"The Commons"},[19403080]={1,"Mystic Ward"},[78506300]={1,"Military Ward"},[24006600]={1,"Hall of Explorers"},[43404950]={1,"The Great Forge"},[50907000]={1,"Tinkertown"},[12005000]={1,"Forlorn Cavern"}, },
	SilvermoonCity = { [24005600]={1,"Sunfury Spire"},[70006080]={1,"Bazaar"},[38808420]={1,"Farstrider's Square"},[61008700]={1,"The Royal Exchange"},[80007900]={1,"Walk of Elders"},[38006800]={1,"Court of the Sun"}, },
	StormwindCity = { [83004950]={1,"Mage Quarter"},[56005500]={1,"Cathedral Square"},[42006480]={1,"Dwarven District"},[60007350]={1,"Old Town"},[37808080]={1,"Stormwind Keep"},[35703100]={1,"Stormwind Harbor"},[18507460]={1,"Eastern Earthshrine"}, },
	DunMorogh = { [26706260]={4,"Ironforge"},[49705280]={1,"Kharanos"},[66404240]={1,"Coldridge Pass"},[51404280]={1,"Chill Breeze Valley"},[34304460]={1,"Shimmer Ridge"},[44006200]={1,"Misty Pine Refuge"},[55006100]={1,"The Tundrid Hills"},[48207090]={1,"Amberstill Ranch"},[53608400]={1,"Helm's Bed Lake"},[58207570]={1,"Gol'Bolar Quarry"},[40008930]={1,"North Gate Outpost"},[49103120]={1,"Frostmane Hold"},[46403720]={1,"Brewnall Village"},[66503590]={1,"Anvilmar"},[55404790]={1,"The Grizzled Den"},[50009170]={1,"South Gate Outpost"},[41704120]={1,"Iceflow Lake"},[38302910]={1,"Gnomeregan"},[34705940]={1,"Gates of Ironforge"},[46805800]={1,"Steelgrill's Depot"},[22007770]={1,"Ironforge Airfield"},[74003200]={1,"Coldridge Valley"}, },
	Elwynn = { [36002400]={4,"Stormwind City"},[65004200]={1,"Goldshire"},[47004500]={1,"Northshire Valley"},[41005000]={1,"Northshire Abbey"},[82003800]={1,"Fargodeep Mine"},[77002700]={1,"Forest's Edge"},[87004700]={1,"Jerod's Landing"},[70006400]={1,"Tower of Azora"},[79006900]={1,"Brackwell Pumpkin Patch"},[66008100]={1,"Eastvale Logging Camp"},[79008400]={1,"Ridgepoint Tower"},[66005200]={1,"Crystal Lake"},[58007300]={1,"Stone Cairn Lake"},[62003200]={1,"Mirror Lake"},[53806170]={1,"Jasperlode Mine"},[94002600]={1,"Hogger Hill"}, },
	EversongWoods = { [40605720]={4,"Silvermoon City"},[27003600]={1,"Sunstrider Isle"},[40004400]={1,"Ruins of Silvermoon"},[59003500]={1,"West Sanctum"},[69003200]={1,"Sunsail Anchorage"},[53004400]={1,"North Sanctum"},[70005300]={1,"East Sanctum"},[62006000]={1,"Farstrider Retreat"},[55005500]={1,"Stillwhisper Pond"},[47006800]={1,"Duskwither Grounds"},[70504300]={1,"Fairbreeze Village"},[76007000]={1,"Tor'Watha"},[85003600]={1,"The Scorched Grove"},[43007200]={1,"Azurebreeze Coast"},[73006400]={1,"Elrendar Falls"},[78003200]={1,"Goldenbough Pass"},[84006480]={1,"Lake Elrendar"},[87704300]={1,"Runestone Falithas"},[84005500]={1,"Runestone Shan'dor"},[73003700]={1,"Saltheril's Haven"},[74002400]={1,"Golden Strand"},[60002700]={1,"Tranquil Shore"},[79006200]={1,"Zeb'Watha"},[31093066]={1,"Falthrien Academy"},[47004800]={1,"Falconwing Square"}, },
	Ashenvale = { [50003600]={2,"Astranaar"},[23001400]={1,"The Zoram Strand"},[36002700]={1,"Maestra's Post"},[53002200]={1,"The Shrine of Aessina"},[67003300]={1,"The Ruins of Stardust"},[72404960]={1,"Mystral Lake"},[36005400]={1,"The Howling Vale"},[51006100]={1,"Raynewood Retreat"},[82006600]={1,"Fallen Sky Lake"},[62007300]={1,"Splintertree Post"},[50008000]={1,"Satyrnaar"},[40009380]={1,"Bough Shadow"},[58009000]={1,"Warsong Lumber Camp"},[62005500]={1,"Greenpaw Village"},[75006600]={1,"Silverwing Outpost"},[46008700]={1,"Forest Song"},[66005000]={1,"Silverwind Refuge"},[45004900]={1,"Thunder Peak"},[34001200]={1,"Zoram'gar Outpost"}, },
	Aszhara = { [52007500]={1,"Bay of Storms"},[32208120]={1,"Tower of Eldara"},[37807230]={1,"Temple of Arkkoran"},[19005100]={1,"Legash Encampment"},[27004400]={1,"Ursolan"},[33003800]={1,"Timbermaw Hold"},[61002000]={1,"Haldarr Encampment"},[72101370]={1,"Shadowsong Shrine"},[53003600]={1,"Ruins of Eldarath"},[66004700]={1,"Southridge Beach"},[82806470]={1,"Ravencrest Monument"},[77003600]={1,"Lake Mennar"},[91505930]={1,"The Ruined Reaches"},[71003000]={1,"Forlorn Ridge"},[50905800]={1,"Bilgewater Harbor"},[55502100]={1,"Gallywix Pleasure Palace"},[12405600]={1,"Arcane Pinnacle"},[76201150]={1,"Talrendis Point"}, },
	AzuremystIsle = { [43007700]={1,"Ammen Vale"},[54006200]={1,"Ammen Ford"},[50004900]={1,"Azure Watch"},[66002600]={1,"Bristlelimb Village"},[17005800]={1,"Emberglade"},[05004700]={1,"Fairbridge Strand"},[68005900]={1,"Geezle's Camp"},[42005200]={1,"Moongraze Woods"},[70004640]={1,"Odesyus' Landing"},[59003700]={1,"Pod Cluster"},[61005300]={1,"Pod Wreckage"},[12003400]={1,"Silting Shore"},[80001300]={1,"Silvermyst Isle"},[20004500]={1,"Stillpine Hold"},[54002100]={1,"Valaar's Berth"},[77003200]={1,"Wrathscale Point"},[48808050]={1,"Crash Site"},[62708150]={1,"Nestlewood Thicket"},[38007070]={1,"The Sacred Grove"},[34682297]={1,"Wildwind Peak"}, },
	Barrens = { [12406630]={1,"Boulder Lode Mine"},[21005750]={1,"The Sludge Fen"},[15204350]={1,"The Mor'shan Rampart"},[39704290]={1,"Dreadmist Peak"},[34702880]={1,"The Dry Hills"},[47703740]={1,"The Forgotten Pools"},[40006700]={1,"Far Watch Post"},[50205780]={1,"Thorn Hill"},[58104960]={1,"The Crossroads"},[79705550]={1,"The Stagnant Oasis"},[74304120]={1,"Lushwater Oasis"},[71806750]={2,"Ratchet"}, },
	SouthernBarrens = { [12403830]={1,"Honor's Stand"},[44606750]={1,"Northwatch Hold"},[32004000]={1,"Agama'gor"},[52804520]={1,"Ruins of Taurajo"},[69204170]={1,"Blackthorn Ridge"},[87404920]={1,"Bael Modan"},[91503710]={1,"Razorfen Kraul"},[96204800]={1,"Razorfen Downs"},[67004910]={1,"Fort Triumph"},[38004800]={1,"The Overgrowth"},[20004000]={1,"Hunter's Hill"},[49005400]={1,"Forward Command"}, },
	BloodmystIsle = { [28801750]={1,"Amberweb Pass"},[34104100]={1,"Axxarien"},[87003180]={1,"Blacksilt Shore"},[45604640]={1,"Bladewood"},[51008500]={1,"Bloodcurse Isle"},[55305460]={1,"Blood Watch"},[78806610]={1,"Bristlelimb Enclave"},[76005190]={1,"Middenvale"},[84104250]={1,"Mystwood"},[79003860]={1,"Nazzivian"},[35775644]={1,"Ragefeather Ridge"},[44606170]={1,"Ruins of Loreth'Aran"},[42602510]={1,"Tel'athion's Camp"},[20903890]={1,"The Bloodwash"},[60403920]={1,"The Cryo-Core"},[36702940]={1,"The Foul Pool"},[24403170]={1,"The Hidden Reef"},[51501820]={1,"The Vector Coil"},[16005320]={1,"The Warp Piston"},[09207450]={1,"Veridian Point"},[45903020]={1,"Vindicator's Rest"},[67606900]={1,"Wrathscale Lair"},[16307140]={1,"Wyrmscar Island"}, },
	Darkshore = { [44003800]={1,"Ruins of Auberdine"},[16806190]={1,"Ruins of Mathystra"},[24505820]={1,"Tower of Althalaxx"},[31005200]={1,"Cliffspring River"},[36004400]={1,"Bashal'Aran"},[62804450]={1,"Ameth'Aran"},[73504400]={1,"Grove of the Ancients"},[86003800]={1,"The Master's Glaive"},[18805090]={1,"Lor'danel"},[19407000]={1,"Shatterspear Vale"}, },
	Desolace = { [13005400]={1,"Tethris Aran"},[07806520]={1,"Nijel's Point"},[21007600]={1,"Sargeron"},[28005500]={1,"Thunder Axe Fortress"},[39006200]={1,"Kormek's Hut"},[48007300]={1,"Kolkar Village"},[28003800]={1,"Ethel Rethor"},[58003300]={1,"Valley of Spears"},[57005000]={1,"Kodo Graveyard"},[70002400]={1,"Shadowprey Village"},[86003400]={1,"Gelkis Village"},[76005200]={1,"Mannoroc Coven"},[73007200]={1,"Magram Village"},[77007900]={1,"Shadowbreak Ravine"},[09002800]={1,"Ranazjar Isle"},[90006100]={1,"Valley of Bones"},[45505500]={1,"Cenarion Wildlands"}, },
	Dustwallow = { [50606800]={4,"Theramore"},[47006600]={1,"Theramore Isle"},[29005800]={1,"Witch Hill"},[31003640]={1,"Brackenwall Village"},[51004100]={1,"The Quagmire"},[65003800]={1,"The Den of Flame"},[73005200]={1,"Wyrmbog"},[19007200]={1,"Alcaz Island"},[76005200]={1,"Onyxia's Lair"},[72704200]={1,"Mudsprocket"},[24004600]={1,"North Point Tower"},[41005900]={1,"Sentry Point"}, },
	Durotar = { [64204420]={1,"Valley of Trials"},[74005500]={1,"Sen'jin Village"},[81006200]={1,"Echo Isles"},[56005800]={1,"Tiragarde Keep"},[42005250]={1,"Razor Hill"},[28003900]={1,"Thunder Ridge"},[23005300]={1,"Drygulch Ravine"},[13005400]={1,"Skull Rock"},[12004560]={1,"Orgrimmar"},[33125072]={1,"Razorwind Canyon"},[09905780]={1,"Bladefist Bay"}, },
	Felwood = { [02006400]={1,"Timbermaw Hold"},[10006300]={1,"Felpaw Village"},[22006300]={1,"Talonbranch Glade"},[23004800]={1,"Irontree Woods"},[16004200]={1,"Jadefire Run"},[40004200]={1,"Shatter Scar Vale"},[47004100]={1,"Bloodvenom Falls"},[60003500]={1,"Jaedenar"},[68003700]={1,"Ruins of Constellas"},[82003900]={1,"Jadefire Glen"},[81005100]={1,"Emerald Sanctuary"},[90004800]={1,"Deadwood Village"},[87005600]={1,"Morlos'Aran"},[29004400]={1,"Whisperwind Grove"}, },
	Feralas = { [46004400]={1,"Feathermoon Stronghold"},[51002900]={1,"Sardor Isle"},[48308800]={1,"Thalanaar"},[11005140]={1,"Dream Bough"},[48208000]={1,"Lower Wilds"},[16005400]={1,"Oneiros"},[12004000]={1,"Ruins of Ravenwind"},[18004600]={1,"The Twin Colossals"},[60004660]={1,"The Forgotten Coast"},[44005900]={1,"Dire Maul"},[62205510]={1,"Feral Scar Vale"},[74905370]={1,"Frayfeather Highlands"},[69005900]={1,"Ruins of Isildien"},[63607370]={1,"The Writhing Deep"},[43007520]={1,"Camp Mojache"},[40006810]={1,"Grimtotem Compound"},[31007700]={1,"Gordunni Outpost"},[79202980]={1,"Isle of Dread"},[36207720]={1,"Lariss Pavillion"},[67502620]={1,"Shalzaru's Lair"},[55007800]={1,"Shadebough"},[15404120]={1,"Camp Ataya"},[32005300]={1,"Rage Scar Hold"},[53605700]={1,"Tower of Estulan"},[21005070]={1,"Dreamer's Rest"},[11006400]={1,"Lake Dumont"}, },
	Hyjal = { [27006300]={2,"Nordrassil"},[30002960]={1,"Shrine of Goldrinn"},[36801940]={1,"Grove of Aessina"},[74707220]={1,"Gates of Sothann"},[29004700]={1,"The Circle of Cinders"},[79504750]={1,"Sulfuron Spire"},[48204240]={1,"Shrine of Aviana"},[55003200]={1,"The Regrowth"},[80003100]={1,"Sethria's Roost"},[44405380]={2,"Mount Hyjal"},[63002700]={1,"Sanctuary of Malorne"}, },
	Moonglade = { [40005000]={1,"Nighthaven"},[57606840]={1,"Stormrage Barrow Dens"},[44003600]={1,"Shrine of Remulos"},[78003400]={1,"Timbermaw Hold"},[54505450]={1,"Lake Elune'ara"},[67004790]={2,"Moonglade"}, },
	Mulgore = { [82003900]={1,"Red Cloud Mesa"},[62003400]={1,"Palemane Rock"},[58004900]={1,"Bloodhoof Village"},[66005300]={1,"Winterhoof Water Well"},[48006200]={1,"The Venture Co. Mine"},[35004900]={1,"The Golden Plains"},[45004400]={1,"Thunderhorn Water Well"},[48003200]={1,"Bael'Dun Digsite"},[21006000]={1,"Red Rocks"},[11005200]={1,"Windfury Ridge"},[14004200]={1,"Wildmane Water Well"},[75204500]={1,"Camp Narache"}, },
	Silithus = { [18503200]={1,"The Crystal Vale"},[19508110]={1,"Valor's Rest"},[36805330]={1,"Cenarion Hold"},[25304950]={1,"Hive'Ashi"},[55603280]={1,"Hive'Zora"},[74306060]={1,"Hive'Regal"},[83503630]={1,"The Scarab Wall"},[23207120]={1,"Staghelm Point"},[79104680]={1,"Bronzebeard Encampment"},[75402840]={1,"Twilight Outpost"},[31603330]={1,"Twilight Post"},[15006900]={1,"Twilight's Run"}, },
	StonetalonMountains = { [92007600]={1,"Camp Aparaje"},[86007500]={1,"Grimtotem Post"},[67705800]={1,"Webwinder Path"},[91006500]={1,"Boulderslide Ravine"},[73405580]={1,"Sishir Canyon"},[56006680]={1,"Windshear Crag"},[60004900]={1,"Sun Rock Retreat"},[69503310]={1,"The Charred Vale"},[46005100]={1,"Mirkfallon Lake"},[18204170]={1,"Stonetalon Peak"},[45707830]={1,"The Talondeep Path"},[60907280]={1,"Windshear Mine"},[24703580]={1,"The Talon Den"},[32003800]={1,"Thal'darah Overlook"},[63606600]={1,"Krom'gar Fortress"},[54005850]={1,"Windshear Hold"},[60503200]={1,"Farwatcher's Glen"}, },
	Tanaris = { [28205100]={2,"Gadgetzan"},[27003800]={1,"Sandsorrow Watch"},[25006000]={1,"Noonshade Ruins"},[23006700]={1,"Steamwheedle Port"},[32006700]={1,"Zalashji's Den"},[46007300]={1,"Lost Rigger Cove"},[36006000]={1,"Waterspring Field"},[41004500]={1,"Abyssal Sands"},[45005200]={1,"Broken Pillar"},[43003600]={1,"The Noxious Lair"},[57004000]={1,"Dunemaul Compound"},[61506670]={1,"Southbreak Shore"},[71005300]={1,"The Gaping Chasm"},[65004700]={1,"Eastmoon Ruins"},[92005400]={1,"Land's End Beach"},[72004000]={1,"Southmoon Ruins"},[80003750]={1,"Uldum"},[64002900]={1,"Thistleshrub Valley"},[16003800]={1,"Zul'Farrak"},[49006500]={1,"Caverns of Time"},[40006800]={1,"Wavestrider Beach"},[60005550]={1,"Bootlegger's Outpost"}, },
	Teldrassil = { [40305970]={1,"Shadowglen"},[51004600]={1,"Ban'ethil Hollow"},[56005520]={1,"Dolanaar"},[70404400]={1,"Gnarlpine Hold"},[68005400]={1,"Lake Al'Ameth"},[66203750]={1,"Pools of Arlithrien"},[51706430]={1,"Starbreeze Village"},[29004140]={1,"The Oracle Glade"},[38404350]={1,"Wellspring Lake"},[91005500]={1,"Rut'theran Village"}, },
	ThousandNeedles = { [24403190]={1,"The Great Lift"},[50004500]={1,"Freewind Post"},[68007500]={1,"The Shimmering Flats"},[32002000]={1,"Whitereach Post"},[63006800]={1,"Ironstone Camp"},[80708050]={1,"Mirage Raceway"},[88007800]={1,"Tahonda Ruins"},[73607770]={1,"Fizzle & Pozzik's Speedbarge"}, },
	Uldum = { [33205480]={2,"Ramkahen"},[31003300]={1,"Temple of Uldum"},[40004950]={1,"Mar'at"},[42006700]={1,"Nahom"},[41704000]={1,"Orsis"},[53607280]={1,"Halls of Origination"},[80903750]={1,"Throne of the Four Winds"},[71706200]={1,"Lost City of the Tol'vir"},[85407700]={1,"Vortex Pinnacle"},[57204560]={1,"Obelisk of the Sun"},[29806440]={1,"Obelisk of the Stars"},[07502670]={1,"Oasis of Vir'sar"},[82705050]={1,"Neferset City"},[51505480]={1,"Akhenet Fields"},[23004000]={1,"Obelisk of the Moon"},[64002240]={1,"Schnottz's Landing"}, },
	UngoroCrater = { [47005100]={1,"Fire Plume Ridge"},[50003100]={1,"Golakka Hot Springs"},[67003200]={1,"Terror Run"},[79005000]={1,"The Slithering Scar"},[64006800]={1,"The Marshlands"},[27007900]={1,"Ironstone Plateau"},[21005000]={1,"Lakkari Tar Pits"},[12004600]={1,"Marshal's Refuge"},[62405520]={1,"Marshal's Stand"},[41204380]={1,"Mossy Pile"}, },
	Winterspring = { [38004980]={1,"Starfall Village"},[49805940]={2,"Everlook"},[50403210]={1,"Frostfire Hot Springs"},[57503750]={1,"Timbermaw Post"},[55905060]={1,"Lake Kel'Theril"},[65805820]={1,"Mazthoril"},[76906470]={1,"Owl Wing Thicket"},[64706710]={1,"Ice Thistle Hills"},[51006730]={1,"Winterfall Village"},[26306340]={1,"The Hidden Grove"},[20704760]={1,"Frostsaber Rock"},[86606090]={1,"Frostwhisper Gorge"},[99905760]={1,"Darkwhisper Gorge"}, },
	RuinsofGilneas = { [50506100]={2,"Gilneas City"},[31406300]={1,"Northgate Woods"},[50002780]={1,"Greymane Manor"},[91706360]={1,"Stormglen Village"},[31507630]={1,"Emberstone Mine"},[38504200]={1,"Keel Harbor"},[53504830]={1,"Aderic's Repose"},[75006300]={1,"The Blackwald"},[21005000]={1,"Northern Headlands"},[68007800]={1,"Tempest's Reach"}, },
	Vashjir = { [36006200]={4,"Vashj'ir"}, },
	VashjirKelpForest = { [35005600]={1,"Smuggler's Scar"},[55005200]={1,"Gurboggle's Ledge"},[25005000]={1,"Seafarer's Tomb"},[34004000]={1,"Legion's Fate"},[78005800]={1,"The Clutch"},[61006100]={1,"Gnaws' Boneyard"}, },
	VashjirRuins = { [17005700]={1,"Sandy Beach"},[42004900]={1,"Silver Tide Hollow"},[57004900]={1,"Tranquil Wash"},[46006600]={1,"Ruins of Thelserai Temple"},[68003300]={1,"Ruins of Vashj'ir"},[85005700]={1,"Biel'aran Ridge"},[24005100]={1,"Shimmering Grotto"},[57005900]={1,"Nespirah"}, },
	VashjirDepths = { [34004400]={1,"Promontory Point"},[20005100]={1,"Sira'kess Front"},[55003300]={1,"L'ghorek"},[30007000]={1,"Abyssal Breach"}, },
	TolBaradDailyArea = { [60007400]={1,"Baradin Base Camp"},[80505550]={1,"Horde Base Camp"},[56005660]={1,"The Darkwood"},[44002600]={1,"Wellson Shipyard"},[26906800]={1,"Rustberg Village"},[30003750]={1,"Farson Hold"},[62004300]={1,"The Restless Front"},[78003550]={1,"Forgotten Hill"},[49008080]={1,"Largo's Overlook"},[21004900]={1,"Cape of Lost Hope"}, },
	TolBarad = { [50505100]={1,"Baradin Hold"},[69003500]={1,"Warden's Vigil"},[26705100]={1,"Ironclad Garrison"},[66906600]={1,"Slagworks"}, },
	TheLostIsles = { [73006300]={1,"Oomlot Village"},[14005100]={1,"KTC Oil Platform"},[17004300]={1,"Gallywix Docks"},[47005900]={1,"Oostan"},[27505700]={1,"Bilgewater Lumber Yard"},[86005380]={1,"Ruins of Vashj'elan"},[43006800]={1,"Volcanoth"},[44003600]={1,"Warchief's Lookout"}, },
	Kezan = { [99002000]={2,"Kezan"},[88007300]={1,"Kaja'mine"},[83005850]={1,"KTC Headquarters"},[60004760]={1,"Kajaro Field"},[33004700]={1,"Bilgewater Port"},[40001700]={1,"Gallywix's Villa"}, },
	Deepholm = { [51504950]={1,"Temple of Earth"},[14005700]={1,"Therazane's Throne"},[35002780]={1,"Needlerock Chasm"},[48002400]={1,"Needlerock Slag"},[40007200]={1,"Crimson Expanse"},[59004000]={1,"Quartzite Basin"},[41006300]={1,"Twilight Precipice"},[27505600]={1,"Crumbling Depths"},[83003500]={1,"The Quaking Fields"},[26006800]={1,"Verlok Stand"},[20004000]={1,"The Pale Roost"},[62007300]={1,"Silvermarsh"},[76505670]={1,"Storm's Fury Wreckage"},[69002750]={1,"Stonehearth"},[77006800]={1,"Twilight Overlook"},[87005500]={1,"Alabaster Shelf"}, },
	MoltenFront = { [49004500]={1,"Magma Springs"},[70004900]={1,"Ashen Fields"},[96009600]={1,"Beth'tilac's Lair"},[38002400]={1,"Fireplume Peak"},[88004900]={1,"Malfurion's Breach"},[50005800]={1,"Igneous Depths"},[25005000]={1,"Ragnaros' Reach"},[40006700]={1,"The Widow's Clutch"},[67003400]={1,"The Furnace"},[66006600]={1,"Forlorn Spire"}, },
	-- Azeroth
	Arathi = { [33801910]={1,"Circle of West Binding"},[29702660]={1,"Northfold Manor"},[48902970]={1,"Boulder'gor"},[30601360]={1,"Thoradin's Wall"},[63601890]={1,"Stromgarde Keep"},[70201100]={1,"The Tower of Arathor"},[83502580]={1,"Faldir's Cove"},[60103040]={1,"Circle of Inner Binding"},[95303840]={1,"Thandol Span"},[78204840]={1,"Boulderfist Hall"},[44802830]={1,"Boulderfist Outpost"},[47004050]={1,"Refuge Pointe"},[50005200]={1,"Circle of Outer Binding"},[68006500]={1,"Witherbark Village"},[60005670]={1,"Go'Shek Farm"},[40205000]={1,"Dabyrie's Farmstead"},[30606220]={1,"Circle of East Binding"},[36606930]={2,"Hammerfall"}, },
	Badlands = { [40001000]={2,"Kargath"},[42601880]={1,"New Kargath"},[60705820]={1,"Camp Boff"},[65001760]={1,"Camp Cagg"},[21006000]={1,"Camp Kosh"},[60004690]={1,"Agmond's End"},[69003000]={1,"Mirage Flats"},[37002700]={1,"The Dustbowl"},[53004800]={1,"Valley of Fangs"},[25504060]={1,"Angor Fortress"},[35206500]={1,"Fuselight"},[28404960]={1,"Hammertoe's Digsite"},[48007100]={1,"Lethlor Ravine"},[14004900]={1,"The Maker's Terrace"},[11604250]={1,"Uldaman"},[35306500]={1,"Dustwind Gulch"},[35004880]={1,"Dustwind Dig"},[56402200]={1,"Dragon's Mouth"},[38009000]={1,"Fuselight-by-the-Sea"}, },
	BlastedLands = { [19806110]={1,"Nethergarde Keep"},[15005450]={1,"Garrison Armory"},[52205500]={2,"The Dark Portal"},[13004300]={1,"Dreadmaul Hold"},[32006300]={1,"Serpent's Coil"},[32003900]={1,"Altar of Storms"},[43004800]={1,"Dreadmaul Post"},[68003700]={1,"The Tainted Scar"},[86204560]={1,"Surwich"},[72005000]={1,"Sunveil Excursion"},[75006300]={1,"The Red Reaches"},[40006800]={1,"Shattershore"}, },
	BurningSteppes = { [31202020]={2,"Blackrock Mountain"},[43106900]={1,"Dreadmaul Rock"},[64707310]={1,"Morgan's Vigil"},[45707980]={1,"Terror Wing Path"},[65806620]={1,"Blackrock Pass"},[40005640]={1,"Ruins of Thaurissan"},[55004900]={1,"The Pillar of Ash"},[32903300]={1,"Blackrock Stronghold"},[63002300]={1,"Draco'dar"},[30401010]={1,"Altar of Storms"},[27305510]={1,"Flame Crest"},[31008140]={1,"Slither Rock"},[51801700]={1,"Flamestar Post"},[43004600]={1,"Chiselgrip"}, },
	DeadwindPass = { [34004200]={1,"Deadman's Crossing"},[52004600]={1,"Sleeping Gorge"},[73004500]={1,"Karazhan"},[64005800]={1,"The Vice"}, },
	Duskwood = { [49000900]={1,"The Hushed Bank"},[68002100]={1,"Addle's Stead"},[55002000]={1,"Raven Hill"},[42002000]={1,"Raven Hill Cemetery"},[72003500]={1,"Vul'Gol Ogre Mound"},[44004700]={1,"Twilight Grove"},[63007600]={1,"Tranquil Gardens Cemetery"},[46007400]={1,"Darkshire"},[35007700]={1,"Manor Mistmantle"},[12005300]={1,"The Darkened Bank"},[37006400]={1,"Brightwood Grove"}, },
	EasternPlaguelands = { [57007580]={2,"Light's Hope Chapel"},[68602270]={1,"The Marris Stead"},[85002700]={1,"The Undercroft"},[68903580]={1,"Crown Guard Tower"},[48003610]={1,"The Fungal Vale"},[86003800]={1,"Darrowshire"},[64905640]={1,"Corin's Crossing"},[74505750]={1,"Lake Mereldar"},[77007700]={1,"Tyr's Hand"},[38008000]={1,"The Noxious Glade"},[42006260]={1,"Eastwall Tower"},[49005100]={1,"Blackwood Lake"},[27906800]={1,"Northdale"},[14006670]={1,"Zul'Mashar"},[24405200]={1,"Northpass Tower"},[14504840]={1,"Quel'Lithien Lodge"},[33001600]={1,"Terrordale"},[26003100]={1,"Plaguewood"},[12002600]={1,"Stratholme"},[74008700]={1,"Ruins of the Scarlet Enclave"},[48008420]={1,"Acherus: The Ebon Hold"}, },
	Ghostlands = { [32004600]={1,"Tranquillien"},[85004740]={2,"Thalassian Pass"},[12006100]={1,"Suncrown Village"},[16002600]={1,"Goldenmist Village"},[43001800]={1,"Windrunner Village"},[36003300]={1,"Sanctum of the Moon"},[48005500]={1,"Sanctum of the Sun"},[20007900]={1,"Dawnstar Spire"},[32007200]={1,"Farstrider Enclave"},[80003300]={1,"Deatholme"},[60006500]={1,"Zeb'Nowa"},[56001500]={1,"Windrunner Spire"},[11004800]={1,"Elrendar Crossing"},[63007500]={2,"Amani Pass"},[65005900]={1,"An'owyn"},[66008200]={1,"Zul'Aman"},[40617700]={1,"Zeb'Tela"},[11007700]={1,"Zeb'Sora"},[37006000]={1,"An'telas"},[15003700]={1,"An'daroth"},[25001200]={1,"Shalandris Isle"}, },
	HillsbradFoothills = { [46005700]={1,"Tarren Mill"},[71104950]={1,"Ruins of Southshore"},[53804670]={1,"Darrow Hill"},[61006850]={1,"Durnholde Keep"},[85006300]={1,"Dun Garok"},[74405870]={1,"Nethander Stead"},[81305600]={1,"Eastern Strand"},[60703780]={1,"Hillsbrad Fields"},[80004000]={1,"Western Strand"},[71703370]={1,"Azurelode Mine"},[51003600]={1,"Sludgeguard Tower"},[87802660]={1,"Purgation Isle"},[29004700]={1,"Ruins of Alterac"},[11004560]={1,"Dandred's Fold"},[25505800]={1,"Strahnbrad"},[37003060]={1,"Dalaran Crater"}, },
	Hinterlands = { [51101100]={2,"Aerie Peak"},[44001200]={1,"Wildhammer Keep"},[24006300]={1,"Seradane"},[29302300]={1,"Plaguemist Ravine"},[58003100]={1,"Hiri'watha"},[47003100]={1,"Quel'Danil Lodge"},[70003400]={1,"Shadra'Alor"},[59004000]={1,"Valorwind Lake"},[41804760]={1,"Agol'watha"},[66004800]={1,"The Altar of Zul"},[41405800]={1,"Skulk Rock"},[52307300]={1,"Shaol'watha"},[72006200]={1,"Jintha'Alor"},[65007600]={1,"The Overlook Cliffs"},[79007900]={1,"Revantusk Village"},[44006680]={1,"Stormfeather Outpost"}, },
	LochModan = { [47003400]={1,"Thelsamar"},[42004800]={1,"The Loch"},[13004700]={1,"Stonewrought Dam"},[24007000]={1,"Mo'grosh Stronghold"},[18003400]={1,"Silver Stream Mine"},[11002600]={1,"Dun Algaz"},[17001800]={1,"North Gate Pass"},[62008000]={1,"The Farstrider Lodge"},[65006800]={1,"Ironband's Excavation Site"},[74004700]={1,"Grizzlepaw Ridge"},[75003400]={1,"Stonesplinter Valley"},[77002100]={1,"Valley of Kings"},[21002500]={1,"Algaz Station"}, },
	Redridge = { [47002800]={1,"Lakeshire"},[56004400]={1,"Lake Everstill"},[65802000]={1,"Three Corners"},[68003800]={1,"Lakeridge Highway"},[23003300]={1,"Redridge Canyons"},[41405090]={1,"Alther's Mill"},[50806310]={1,"Stonewatch"},[77007300]={1,"Render's Valley"},[17004200]={1,"Render's Camp"},[56506910]={1,"Stonewatch Falls"},[36907040]={1,"Galardell Valley"},[44007170]={1,"Tower of Ilgalar"},[63007850]={1,"Shalewind Canyon"}, },
	SearingGorge = { [34302930]={1,"Firewatch Ridge"},[50004900]={1,"The Cauldron"},[80002200]={1,"Blackchar Cave"},[68005200]={1,"The Sea of Cinders"},[70007100]={1,"Tanner Camp"},[59006500]={1,"Grimesilt Dig Site"},[27007200]={1,"Dustfire Valley"},[28003800]={1,"Thorium Point"},[66804000]={1,"Iron Summit"}, },
	Silverpine = { [83004600]={1,"The Greymane Wall"},[10005700]={1,"Malden's Orchard"},[23005500]={1,"The Shining Strand"},[20004500]={1,"The Dead Field"},[16003700]={1,"The Skittering Dark"},[28003900]={1,"North Tide's Hollow"},[27006600]={1,"Fenris Isle"},[34005700]={1,"The Decrepit Ferry"},[41004300]={1,"The Sepulcher"},[46405900]={1,"Deep Elem Mine"},[53004700]={1,"Olsen's Farthing"},[64006100]={1,"Ambermill"},[68004400]={1,"Shadowfang Keep"},[73004500]={1,"Pyrewood Village"},[74006100]={1,"Beren's Peril"},[15005600]={1,"Ivar's Patch"},[98004400]={1,"Gilneas"},[23007800]={1,"Lordamere Lake"}, },
	StranglethornJungle = { [49403800]={1,"Grom'gol Base Camp"},[44008300]={2,"Zul'Gurub"},[24004400]={1,"Nesingwary's Expedition"},[11404790]={1,"Rebel Camp"},[19905860]={1,"Kurzen's Compound"},[62205460]={1,"Ziata'jai Ruins"},[56405820]={1,"Balia'mah Ruins"},[53804640]={1,"Mizjah Ruins"},[48206410]={1,"Mosh'Ogg Ogre Mound"},[34905190]={1,"Lake Nazferiti"},[36203510]={1,"Bal'lal Ruins"},[22902700]={1,"Ruins of Zul'Kunda"},[29401220]={1,"Yojamba Isle"},[38005900]={1,"Venture Co. Base Camp"},[66005280]={1,"Fort Livingston"},[41004300]={1,"Kal'ai Ruins"},[70006100]={1,"Ruins of Zul'Mamwe"}, },
	TheCapeOfStranglethorn = { [24004700]={1,"Gurubashi Arena"},[82806020]={1,"Jaguero Isle"},[50004260]={1,"Nek'mani Wellspring"},[34403400]={1,"Bloodsail Compound"},[42706110]={1,"Ruins of Aboraz"},[32406100]={1,"Crystalvein Mine"},[62703500]={1,"Janeiro's Point"},[56605150]={1,"Mistvale Valley"},[70904100]={2,"Booty Bay"},[27003500]={1,"Hardwrench Hideaway"}, },
	Sunwell = { [52005300]={1,"Sunwell"},[45004000]={1,"Sunwell Plateau"},[32004100]={1,"Dawnstar Village"},[48006300]={1,"Greengill Coast"},[30006000]={1,"Magisters' Terrace"},[36805270]={1,"Sun's Reach Harbor"},[54004200]={2,"Isle of Quel'Danas"},[84005300]={1,"The Dead Scar"}, },
	SwampOfSorrows = { [54004770]={2,"Stonard"},[68001800]={1,"Itharius's Cave"},[37801700]={1,"Misty Valley"},[33102840]={1,"The Harborage"},[51902060]={1,"Splinterspear Junction"},[42003600]={1,"The Shifting Mire"},[54006500]={1,"Pool of Tears"},[74806730]={1,"Stagalbog"},[38908400]={1,"Sorrowmurk"},[23006200]={1,"Fallow Sanctuary"},[65808890]={1,"Misty Reed Strand"},[34006800]={1,"Marshtide Watch"},[79008000]={1,"Misty Reed Post"},[12207300]={1,"Bogpaddle"}, },
	Tirisfal = { [63703300]={1,"Deathknell"},[50003600]={1,"Solliden Farmstead"},[39004800]={1,"Agamand Mills"},[52004900]={1,"Stillwater Pond"},[64004800]={1,"Nightmare Vale"},[57005300]={1,"Cold Hearth Manor"},[51005900]={1,"Brill"},[35005900]={1,"Garren's Haunt"},[45006800]={1,"Brightwater Lake"},[61007500]={1,"Balnir Farmstead"},[55007800]={1,"Crusader Outpost"},[29007900]={1,"Scarlet Watch Post"},[32008100]={1,"Whispering Gardens"},[47008400]={1,"Venomweb Vale"},[66006100]={1,"Undercity"},[34008600]={1,"Scarlet Monastery"}, },
	TwilightHighlands = { [52707550]={1,"Dragonmaw Port"},[80903600]={1,"Twilight Citadel"},[75708050]={2,"Highbank"},[58006000]={1,"Firebeard's Patrol"},[17405500]={1,"Kirthaven"},[24002800]={1,"Vermillion Redoubt"},[57004340]={1,"Victor's Point"},[78006400]={1,"Obsidian Forest"},[43005500]={1,"Bloodgulch"},[89904000]={1,"Elementium Depths"},[29704950]={2,"Thundermar"},[47406240]={1,"Gorshak War Camp"},[70004910]={1,"Dunwald Ruins"},[76404700]={1,"Crushblow"}, },
	WesternPlaguelands = { [16004400]={1,"Hearthglen"},[73006000]={1,"Darrowmere Lake"},[78006800]={1,"Caer Darrow"},[78005100]={1,"Sorrow Hill"},[69104500]={1,"Andorhal"},[57002700]={1,"The Bulwark"},[56003600]={1,"Felstone Field"},[53004600]={1,"Dalson's Tears"},[65805360]={1,"The Writhing Haunt"},[36004600]={1,"Northridge Lumber Camp"},[16004400]={1,"Hearthglen"},[58006200]={1,"Gahrron's Withering"},[40006500]={1,"The Weeping Cave"},[47006900]={1,"Thondroril River"},[84005300]={1,"Uther's Tomb"},[85004200]={1,"Chillwind Camp"},[52405370]={1,"The Mender's Stead"}, },
	Westfall = { [46605700]={1,"Sentinel Hill"},[69004300]={1,"Moonbrook"},[32005400]={1,"Saldean's Farm"},[22005100]={1,"Furlbrow's Pumpkin Farm"},[17005800]={1,"The Jansen Stead"},[25004400]={1,"Jangolode Mine"},[35004400]={1,"The Molsen Farm"},[43003200]={1,"Gold Coast Quarry"},[60006200]={1,"The Dead Acre"},[52003800]={1,"Alexston Farmstead"},[74603390]={1,"Demont's Place"},[86003000]={1,"Westfall Lighthouse"},[78004700]={1,"The Dagger Hills"},[72006400]={1,"The Dust Plains"}, },
	Wetlands = { [53001100]={2,"Menethil Harbor"},[17004900]={1,"Dun Modr"},[49002100]={1,"Black Channel Marsh"},[37001900]={1,"Bluegill Marsh"},[47003500]={1,"Whelgar's Excavation Site"},[30002800]={1,"Sundown Marsh"},[20003400]={1,"Saltspray Glen"},[27004400]={1,"Ironbeard's Tomb"},[48004700]={1,"Angerfang Encampment"},[72005200]={1,"Dun Algaz"},[34005500]={1,"The Green Belt"},[53005800]={1,"Mosshide Fen"},[37006800]={1,"Raptor Ridge"},[74007800]={1,"Grim Batol"},[41005700]={1,"Greenwarden's Grove"},[25005900]={1,"Direforge Hill"}, },
	-- Expansion01
	BladesEdgeMountains = { [64003700]={2,"Sylvanaar"},[56005200]={2,"Thunderlord Stronghold"},[12005200]={1,"Bash'ir Landing"},[53004000]={1,"Bladespire Hold"},[27005500]={1,"Bloodmaul Camp"},[77004600]={1,"Bloodmaul Outpost"},[43005300]={1,"Circle of Blood"},[67006400]={1,"Death's Door"},[41007300]={1,"Forge Camp: Anger"},[81002800]={1,"Forge Camp: Terror"},[40003600]={1,"Forge Camp: Wrath"},[20003900]={1,"Grishnath"},[24006500]={1,"Gruul's Lair"},[70004900]={1,"Jagged Ridge"},[60007200]={1,"Mok'Nathal Village"},[28003100]={1,"Raven's Wood"},[53006500]={1,"Razor Ridge"},[23007200]={1,"Skald"},[14006400]={1,"Crystal Spine"},[76003500]={1,"Veil Lashh"},[31006400]={1,"Veil Ruuan"},[64003000]={1,"Vortex Pinnacle"},[72006000]={1,"Toshley's Station"},[38006200]={1,"Evergrove"},[57002900]={1,"Ogri'la"}, },
	Hellfire = { [50008600]={2,"The Stair of Destiny"},[61002700]={1,"Falcon Watch"},[52004800]={1,"Hellfire Citadel"},[63005500]={2,"Honor Hold"},[40002300]={1,"Temple of Telhamat"},[39005400]={2,"Thrallmar"},[18006100]={1,"Throne of Kil'jaeden"},[81005400]={1,"Expedition Army"},[28003200]={1,"Mag'har Post"},[40003900]={1,"Pools of Aggonar"},[60001500]={1,"Ruins of Sha'naar"},[52007200]={1,"The Legion Front"},[72006700]={1,"Zeth'Gor"},[72002600]={1,"Den of Haal'esh"},[45001400]={1,"Fallen Sky Ridge"},[70007700]={1,"Void Ridge"},[83004500]={1,"The Warp Fields"},[31006500]={1,"Forge Camp: Mageddon"}, },
	Nagrand = { [36005600]={2,"Garadar"},[44004270]={2,"Halaa"},[72005400]={2,"Telaar"},[51501910]={1,"Forge Camp: Fear"},[81507000]={1,"Kil'sorrow Fortress"},[19404670]={1,"Laughing Skull Ruins"},[71403600]={1,"Spirit Fields"},[43103160]={1,"Sunspring Post"},[56106590]={1,"The Ring of Trials"},[23106020]={1,"Throne of the Elements"},[21102700]={1,"Warmaul Hill"},[66907440]={1,"Burning Blade Ruins"},[63806200]={1,"Clan Watch"},[35902460]={1,"Forge Camp: Hate"},[55504940]={1,"Southwind Cleft"},[43100860]={1,"The Twilight Ridge"},[36607260]={1,"Windyreed Pass"},[52607270]={1,"Windyreed Village"},[15603340]={1,"Zangar Ridge"}, },
	Netherstorm = { [65003300]={2,"Area 52"},[70002200]={1,"Manaforge B'naar"},[84004800]={1,"Manaforge Coruu"},[67005900]={1,"Manaforge Duro"},[38002600]={1,"Manaforge Ara"},[39006200]={1,"Manaforge Ultris"},[64007200]={2,"Tempest Keep"},[85005600]={1,"Kirin'Var Village"},[56003200]={1,"Ruins of Enkaat"},[78005600]={1,"Sunfury Hold"},[36004400]={2,"The Stormspire"},[13004500]={2,"Eco-Dome Farfield"},[54004500]={2,"Eco-Dome Midrealm"},[25005400]={1,"Ruins of Farahlon"},[76003100]={1,"The Heap"},[75004000]={1,"Arklon Ruins"},[40007200]={1,"Celestial Ridge"},[18004900]={1,"Netherstone"},[56002200]={1,"Gyro-Plank Bridge"},[42005500]={1,"Ethereum Staging Grounds"},[15002900]={1,"Socrethar's Seat"},[25003700]={1,"Forge Base: Oblivion"},[20004000]={1,"Forge Base: Gehenna"},[70306600]={1,"Cosmowrench"}, },
	ShadowmoonValley = { [38202330]={1,"Legion Hold"},[85407050]={2,"Netherwing Ledge"},[28702920]={1,"Shadowmoon Village"},[43807300]={2,"The Black Temple"},[59403700]={1,"Wildhammer Stronghold"},[29106170]={2,"Altar of Sha'tar"},[28304550]={1,"Coilskar Point"},[39204040]={1,"The Deathforge"},[40505000]={1,"The Hand of Gul'dan"},[48706010]={1,"Warden's Cage"},[55302950]={1,"Illidari Point"},[57606420]={1,"Netherwing Fields"},[62005600]={1,"Sanctum of the Stars"},[64007000]={1,"Dragonmaw Fortress"}, },
	TerokkarForest = { [54505750]={1,"Allerian Stronghold"},[45004900]={1,"Stonebreaker Hold"},[30405320]={2,"Tuurem"},[71003360]={2,"Auchenai Grounds"},[84007180]={2,"Skettis"},[64402020]={1,"Bleeding Hollow Ruins"},[22104370]={1,"Cenarion Thicket"},[37607110]={1,"Firewing Point"},[39604000]={1,"Grangol'var Village"},[39806130]={1,"Raastok Glade"},[11002220]={1,"The Barrier Hills"},[20005500]={1,"Razorthorn Shelf"},[53506760]={1,"Bonechewer Ruins"},[51303760]={1,"Refugee Caravan"},[65403970]={1,"Ring of Observance"},[76404390]={1,"Derelict Caravan"},[55602700]={1,"Veil Rhaze"},[67204870]={1,"Writhing Mound"},[75005900]={1,"Skethyl Mountains"}, },
	Zangarmarsh = { [64807920]={1,"Cenarion Refuge"},[48306810]={2,"Telredor"},[53904760]={1,"Twin Spire Ruins"},[30504260]={2,"Orebor Harborage"},[50803120]={2,"Zabra'jin"},[23801760]={1,"Ango'rosh Grounds"},[63604660]={1,"Feralfen Village"},[33002950]={1,"Hewn Bog"},[40702200]={1,"Marshlight Lake"},[61502920]={1,"Quagg Ridge"},[38608190]={1,"The Dead Mire"},[62105810]={1,"The Lagoon"},[82508320]={1,"Umbrafen Village"},[50401890]={1,"Sporeggar"},[07701830]={1,"Ango'rosh Stronghold"},[41106150]={1,"Bloodscale Grounds"},[62701470]={1,"The Spawning Glen"},[80107080]={1,"Darkcrest Shore"},[41005100]={1,"Coilfang Reservoir"},[44004500]={1,"Serpent Lake"},[70007500]={1,"Umbrafen Lake"}, },
	-- Northrend
	BoreanTundra = { [67805700]={2,"Valiance Keep"},[26702760]={1,"The Nexus"},[34002700]={2,"Coldarra"},[34003400]={1,"Transitus Shield"},[53304300]={2,"Warsong Hold"},[26908580]={1,"Temple City of En'kilah"},[25004900]={1,"Steeljaw's Caravan"},[77004200]={1,"Riplash Strand"},[51006440]={1,"Kaskala"},[54003200]={1,"Garrosh's Landing"},[46708420]={1,"Death's Stand"},[10005100]={1,"Bor'gorok Outpost"},[36004900]={1,"Amber Ledge"},[24006700]={1,"The Geyser Fields"},[17006800]={1,"The Dens of the Dying"},[92005050]={1,"Riplash Ruins"},[49007800]={1,"Unu'pe"},[37707750]={1,"Taunka'le Village"},[19005700]={1,"Runway"},[65503600]={1,"Plains of Nasam"}, },
	CrystalsongForest = { [57602350]={1,"The Azure Front"},[15601820]={1,"The Decrepit Flow"},[48307690]={1,"Sunreaver's Command"},[44604710]={1,"Forlorn Woods"},[80407470]={1,"Windrunner's Overlook"},[34201420]={1,"The Great Tree"},[42401570]={1,"Violet Stand"},[61806080]={1,"The Unbound Thicket"}, },
	Dragonblight = { [54205980]={2,"Wyrmrest Temple"},[40007200]={1,"Bronze Dragonshrine"},[47808730]={2,"Naxxramas"},[65405570]={1,"Azure Dragonshrine"},[34105560]={1,"Galakrond's Rest"},[67404000]={1,"Lake Indu'le"},[48704750]={1,"Ruby Dragonshrine"},[31904070]={1,"Obsidian Dragonshrine"},[78207100]={1,"New Hearthglen"},[26108400]={1,"Light's Trust"},[43202510]={1,"Icemist Village"},[73606360]={1,"Emerald Dragonshrine"},[20005500]={1,"Coldwind Heights"},[17403770]={1,"Angrathar the Wrath Gate"},[46803670]={1,"Agmar's Hammer"},[47401410]={1,"Westwind Refugee Camp"},[62007750]={1,"Venomspite"},[68408240]={1,"The Forgotten Shore"},[22206040]={1,"The Crystal Vice"},[26007250]={1,"Scarlet Point"},[50008000]={1,"Wintergarde Keep"},[54502910]={1,"Stars' Rest"},[50702640]={1,"Azjol-Nerub"},[26004000]={1,"Fordragon Hold"},[15004400]={1,"Kor'koron Vanguard"},[74704900]={1,"Moa'ki Harbor"},[62002000]={1,"Moonrest Gardens"}, },
	GrizzlyHills = { [65402140]={1,"Conquest Hold"},[28401760]={2,"Drak'Tharon Keep"},[17006650]={2,"Thor Modan"},[86101460]={2,"Venture Bay"},[36503730]={1,"Blue Sky Logging Grounds"},[26007160]={1,"Drakil'jin Ruins"},[59407780]={1,"Dun Argol"},[47701620]={1,"Granite Springs"},[43904920]={1,"Grizzlemaw"},[57805030]={1,"Rage Fang Shrine"},[74902890]={1,"Voldrune"},[59403170]={1,"Amberpine Lodge"},[47206510]={1,"Camp Oneqwah"},[30005720]={1,"Westfall Brigade Encampment"},[35002600]={1,"Silverbrook"},[26008500]={1,"Bloodmoon Isle"},[37501800]={1,"Zeb'Halak"}, },
	HowlingFjord = { [60006080]={2,"Valgarde"},[57702510]={1,"Kamagua"},[36605780]={1,"Cauldros Isle"},[10304840]={1,"Camp Winterhoof"},[24102700]={1,"Apothecary Camp"},[29507880]={1,"Vengeance Landing"},[26103080]={1,"Steel Gate"},[78503410]={1,"Scalawag Point"},[54306810]={1,"Nifflevar"},[10103660]={1,"Gjalerbron"},[50303930]={1,"Ember Clutch"},[27806890]={1,"Giant's Run"},[15306080]={1,"Fort Wildervar"},[46607830]={1,"Ivald's Ruin"},[53905000]={1,"Halgrind"},[67705290]={1,"New Agamand"},[35404510]={1,"Skorn"},[27205160]={1,"The Twisted Glade"},[46305830]={1,"Utgarde Keep"},[43303000]={1,"Westguard Keep"},[71407220]={1,"Baelgun's Excavation Site"},[39906640]={1,"Baleheim"}, },
	HrothgarsLanding = { [32005000]={2,"Hrothgar's Landing"}, },
	IcecrownGlacier = { [90005500]={2,"Icecrown Citadel"},[20007400]={1,"Argent Tournament Grounds"},[00000000]={1,"The Bombardment"},[43001000]={1,"Onslaught Harbor"},[00000000]={1,"The Broken Front"},[00000000]={1,"The Fleshwerks"},[00000000]={1,"Aldur'thar: The Desolation Gate"},[36007000]={1,"Sindragosa's Fall"},[00000000]={1,"Valhalas"},[00000000]={1,"Valley of Echoes"},[53005500]={2,"Ymirheim"},[00000000]={1,"The Conflagration"},[75005000]={1,"Corp'rethar: The Horror Gate"},[36003000]={1,"Jotunheim"},[61907730]={1,"Scourgeholme"},[23504360]={1,"The Shadow Vault"},[74008700]={1,"Argent Vanguard"}, },
	SholazarBasin = { [64404900]={1,"River's Heart"},[28404660]={1,"The Savage Thicket"},[78803660]={1,"The Mosslight Pillar"},[53808090]={1,"Makers' Overlook"},[36802800]={1,"Makers' Perch"},[56003400]={1,"The Suntouched Pillar"},[54005400]={2,"Rainspeaker Canopy"},[63606570]={1,"The Lifeblood Pillar"},[37007380]={1,"The Avalanche"},[41005000]={1,"The Glimmering Pillar"},[00000000]={1,"Kartak's Hold"},[00000000]={1,"The Stormwright's Shelf"},[83005360]={1,"The Skyreach Pillar"},[72005600]={1,"Frenzyheart Hill"},[62402680]={1,"Nesingwary Base Camp"}, },
	TheStormPeaks = { [61106460]={2,"Dun Niffelem"},[21104170]={2,"Ulduar"},[67704900]={2,"Brunnhildar Village"},[40003400]={1,"Narvir's Cradle"},[70003680]={1,"Bor's Breath"},[61302650]={1,"Valkyrion"},[42305360]={1,"Terrace of the Makers"},[86703500]={1,"Sparksocket Minefield"},[57004050]={1,"Engine of the Makers"},[46206480]={1,"Temple of Life"},[47707300]={1,"Thunderfall"},[59703350]={1,"Temple of Storms"},[43402860]={1,"Snowdrift Plains"},[80004600]={1,"Garm's Bane"},[74103010]={1,"Frosthold"},[50002500]={1,"Nidavelir"},[86204140]={1,"K3"},[51006700]={1,"Camp Tunka'lo"},[49003700]={1,"Grom'arsh Crash-Site"}, },
	LakeWintergrasp = { [22005000]={1,"Wintergrasp Fortress"},[31007100]={1,"Valiance Landing Camp"}, },
	ZulDrak = { [20708210]={2,"Gundrak"},[75305180]={1,"Drak'Sotra Fields"},[56104940]={1,"Ampitheater of Anguish"},[38604060]={1,"Altar of Sseratus"},[36805360]={1,"Altar of Rhunok"},[59207700]={1,"Altar of Quetz'lun"},[43307610]={1,"Altar of Mam'toth"},[71006390]={1,"Altar of Har'koa"},[57605970]={1,"Zim'Torga"},[76502150]={1,"Zeramas"},[46602830]={1,"Voltarus"},[58601720]={1,"Thrym's End"},[75603200]={1,"Light's Breach"},[77606200]={1,"Kolramas"}, },
	ZulGurub = { [77006000]={1,"Mandokir's Domain"},[21003100]={1,"The Devil's Terrace"},[19004800]={1,"Temple of Bethekk"},[52005000]={1,"The Coil"},[37404870]={1,"Jin'do"}, },
	ZulAman = { [50009000]={1,"Daakara"},[50007000]={1,"Hex Lord Malacrass"},[29003600]={1,"Akil'zon"},[74004000]={1,"Nalorakk"},[21005800]={1,"Halazzi"},[68005500]={1,"Jan'alai"}, },
	IsleofConquest = { [35005900]={1,"Isle of Conquest"}, },
	TheEyeofEternity = { [48003870]={1,"Malygos"}, },
	ThroneoftheFourWinds = { [22904740]={1,"Nezir"},[50003100]={1,"Anshal"},[50006360]={1,"Rohash"},[46804720]={1,"Al'Akir"}, },
	TempestKeep = { [20005000]={1,"Bridge"},[45002600]={1,"The Crucible"},[60005000]={1,"The Phoenix Hall"},[45007400]={1,"The Solarium"}, },
	SunwellPlateau = { [22007700]={1,"Terrace of the Sun"},[25003100]={1,"Sunwell Plateau"},[52003060]={1,"Apex Point"},[52006400]={1,"Sunwell"},[34006400]={1,"Witch's Sanctum"}, },
	LostCityofTolvir = { [38403660]={1,"The Warlord's Garrison"},[73005200]={1,"The Trapper's Enclave"},[78007100]={1,"Oasis of the Fallen Prophet"},[70003560]={1,"Terrace of the Augurs"},[54004140]={1,"Sanctum of the South Wind"}, },
	Skywall = { [17005400]={1,"Portal"},[43405470]={1,"Grand Vizier Ertan"},[80405200]={1,"Altairus"},[36803000]={1, "Asaad,Caliph of Zephyrs"}, },
	Firelands = { [73506590]={1,"Rhyolith Plateau"},[60004550]={1,"The Molten Fields"},[44006590]={1,"Shatterstone"},[18004870]={1,"Sulfuron Span"},[60007200]={1,"The Ridge of Ancient Flame"},[88002600]={1,"Flamebreach"},[36004940]={1,"Mortal's Demise"},[42002070]={1,"Beth'tilac's Lair"}, },
	["Firelands:2"] = { [26004800]={1,"The Anvil of Conflagration"}, },
	["Firelands:3"] = { [79905080]={1,"Sulfuron Keep Courtyard"},[27005050]={1,"Sulfuron Keep"}, },
	TheObsidianSanctum = { [48005090]={1,"Sartharion"},[50006470]={1,"Portal"}, },
	TheRubySanctum = { [53004900]={1,"Halion"},[27504950]={1,"Portal"}, },
	EndTime = { [53006000]={1,"Wyrmrest Temple"},[27008000]={1,"Bronze Dragonshrine"},[44004000]={1,"Ruby Dragonshrine"},[73005300]={1,"Azure Dragonshrine"},[87006700]={1,"Emerald Dragonshrine"},[15002500]={1,"Obsidian Dragonshrine"},[20005300]={1,"Galakrond's Rest"},[04002600]={1,"Fordragon Hold"},[67009000]={1,"Venomspite"},[40002100]={1,"Agmar's Hammer"},[04008000]={1,"Scarlet Point"},[88008000]={1,"New Hearthglen"},[89004100]={1,"Moa'ki Harbor"},[57000800]={1,"Stars' Rest"},[72002600]={1,"Indu'le Village"},[44000350]={1,"Azjol-Nerub"},[32000400]={1,"Icemist Village"},[43009200]={1,"Wintergarde Keep"}, },
	DragonSoul = { [82004200]={1,"Azure Dragonshrine"},[96005700]={1,"Emerald Dragonshrine"},[99002900]={1,"Moa'ki Harbor"},[78001400]={1,"Indu'le Village"},[75008200]={1,"Venomspite"},[98007000]={1,"New Hearthglen"}, },
	HourofTwilight = { [87005000]={1,"Wyrmrest Temple"},[60007250]={1,"Bronze Dragonshrine"},[78002750]={1,"Ruby Dragonshrine"},[48001000]={1,"Obsidian Dragonshrine"},[50004200]={1,"Galakrond's Rest"},[32005200]={1,"The Crystal Vice"},[24004000]={1,"Coldwind Heights"},[14007800]={1,"Windrunner's Overlook"},[36001100]={1,"Fordragon Hold"},[17001800]={1,"Kor'koron Vanguard"},[24001000]={1,"Angrathar the Wrath Gate"},[85009750]={1,"Naxxramas"},[36007300]={1,"Scarlet Point"},[74000700]={1,"Agmar's Hammer"},[80008500]={1,"Wintergarde Keep"}, },
	WellofEternity = { [55002200]={1,"Palace Gardens"},[33004100]={1,"Azshara's Palace"},[41006500]={1,"Well of Eternity"},[94001500]={1,"Zin-Azshari"}, },
}
